#!/bin/bash

# ---------------------------------------------------------------------------- #
#                                   变量定义                                    #
# ---------------------------------------------------------------------------- #
## 所有要清理的表
declare -a g_all_public_schema_tables=("OM_SERVICES" "OM_CLUSTERS" "OM_FSMTRANSITIONS" "OM_CLUSTER_PATCH_INFO" "TBL_SYSTEM_LICENSE" "TBL_UPGRADE_MARK" "OM_MPPDB_CLIENT_INFO" "TBL_MONITOR_CUSTOMIZE" "MPP_LOGIC_CLUSTER" "MPP_LOGIC_CLUSTER_LOOP_HOSTS" "MPP_LOGIC_CLUSTER_REDISTRIBUTION_TASK" "TBL_BASELINE" "TBL_BASELINE_TEMPLET" "OM_NODES" "OM_CLIENT_INFO" "RESOURCEPOOL" "TENANT" "TBL_BACKUP_CONFIGS" "TBL_BACKUP_TASKINFO" "TBL_RECOVERY_CONFIGS" "TBL_RECOVERY_TASKINFO" "OM_NODES_ISOLATION" "OM_CONFIGURATIONS" "OM_TOPOLOGY" "OM_INSTANCE_GROUP" "OM_ROLES" "TENANT_RESOURCES" "TENANT_SERVICES" "RESOURCES_PROPERTIES" "RESOURCEPOOL_HOSTS" "RESOURCEPOOL_SERVICE")
declare -a g_all_omm_schema_tables=("TBL_PM_ALARMTHRESHOLD" "TBL_FM_ALARM_LOG" "TBL_FM_EVENT_LOG")
declare -a g_all_redis_schema_tables=("CLUSTER" "INSTANCE" "OMINSTANCES" "SLOT")

## 清理ClusterID为Str类型的表
declare -a g_cluster_id_tables=("OM_SERVICES" "OM_CLUSTERS" "OM_FSMTRANSITIONS" "OM_CLUSTER_PATCH_INFO"  "TBL_SYSTEM_LICENSE" "TBL_UPGRADE_MARK" "OM_MPPDB_CLIENT_INFO")
declare -a g_clusterId_tables=("TBL_MONITOR_CUSTOMIZE" "MPP_LOGIC_CLUSTER" "MPP_LOGIC_CLUSTER_LOOP_HOSTS" "MPP_LOGIC_CLUSTER_REDISTRIBUTION_TASK")
## 清理ClusterID为interger类型的表
declare -a g_integer_cluster_id_tables=("OM_NODES" "OM_CLIENT_INFO" "RESOURCEPOOL" "TENANT")
## 清理以sourceID为条件的表
declare -a g_source_tables=("TBL_BASELINE" "TBL_BASELINE_TEMPLET")
## 清理omm schema表
declare -a g_source_ommschema_tables=("TBL_PM_ALARMTHRESHOLD")
## 清理备份恢复表
declare -a g_backup_tables=("TBL_BACKUP_CONFIGS" "TBL_BACKUP_TASKINFO" "TBL_RECOVERY_CONFIGS" "TBL_RECOVERY_TASKINFO")
## 清理迁移集群的告警和事件
declare -a g_alarm_event_tables=("TBL_FM_ALARM_LOG" "TBL_FM_EVENT_LOG")
## 清理Redis逻辑集群表
declare -a g_redis_logical=("CLUSTER" "INSTANCE" "OMINSTANCES" "SLOT")

declare g_curPath=""                                            # 当前脚本所在的目录
declare g_clusterId=""                                          # 集群ID

# ---------------------------------------------------------------------------- #
#                                   函数定义                                    #
# ---------------------------------------------------------------------------- #

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - > /dev/null
}

####################################################################
#  FUNCTION     : 判断表是否存在
####################################################################
isTableExist(){
    local tableSchema=$1
    local tableName=$2
    local isTableExistCommand="SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='${tableSchema}' AND TABLE_NAME='${tableName}';"
    local result=$(${DBCOMMAND} -W ${DBPASSWORD} -c "${isTableExistCommand}")
    local tableNum=$(echo ${result} | awk  '{printf $3}')
    if [ ${tableNum}X == "1"X ];then
        return 0
    else
        return 1
    fi
}

####################################################################
#  FUNCTION     : 执行sql
####################################################################
executeSql(){
    local command="${1}"
    local tableName="${2}"
    local tableSchema="${3}"
    local executeType="${4}"
    ${DBCOMMAND} -W ${DBPASSWORD} -c "${command}" >/dev/null 2>&1
    if [ $? -ne 0 ];then
        isTableExist ${tableSchema} ${tableName}
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "table ${tableSchema}.${tableName} does not exist."
        else
            showLog_error "${LINENO}" "Failed to execute command :${command}."
            exit 1
        fi
    else
        log_info "${LINENO}" "Success to execute command :${command}."
        if [ "${executeType}"X != ${EXPORT_EXECUTE_TYPE}X ]; then
            showLog_info "${LINENO}" "Delete Table : ${tableSchema}.${tableName} Successfully"
        fi
    fi
}

####################################################################
#  FUNCTION     : 备份要清理的表
####################################################################
backupPublicSchemaTables(){
    for table in "${g_all_public_schema_tables[@]}"
    do
        touch ${g_curPath}/backupTables/${PUBLIC_SCHEMA}-$table
        chmod 775 ${g_curPath}/backupTables/${PUBLIC_SCHEMA}-$table
        COMMAND="COPY (select * from $table) TO '${g_curPath}/backupTables/${PUBLIC_SCHEMA}-$table';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
    done
}

####################################################################
#  FUNCTION     : 备份要清理的表
####################################################################
backupOmmSchemaTables(){
    for table in "${g_all_omm_schema_tables[@]}"
    do
        touch ${g_curPath}/backupTables/${OMM_SCHEMA}-$table
        chmod 775 ${g_curPath}/backupTables/${OMM_SCHEMA}-$table
        COMMAND="COPY (select * from ${OMM_SCHEMA}.$table) TO '${g_curPath}/backupTables/${OMM_SCHEMA}-$table';"
        executeSql "${COMMAND}" "${table}" "${OMM_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
    done
}

backupRedisSchemaTables(){
   for table in "${g_all_redis_schema_tables[@]}"
   do
       touch ${g_curPath}/backupTables/${REDIS_SCHEMA}-$table
       chmod 775 ${g_curPath}/backupTables/${REDIS_SCHEMA}-$table
       COMMAND="COPY (select * from ${REDIS_SCHEMA}.$table) TO '${g_curPath}/backupTables/${REDIS_SCHEMA}-$table';"
       executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}" "${EXPORT_EXECUTE_TYPE}"
   done
}

####################################################################
#  FUNCTION     : 清理备份恢复表
####################################################################
deleteBackupTables(){
    for table in "${g_backup_tables[@]}"
    do
        COMMAND="delete from $table where TASKNAME='default-${g_clusterId}';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
    done
    ## 删除手动备份任务
    COMMAND="delete from TBL_BACKUP_TASKINFO where CLUSTERID='${g_clusterId}';"
    executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
}

####################################################################
#  FUNCTION     : 以clusterid进行表清理
####################################################################
deleteClusterIDTables()
{
    for table in "${g_cluster_id_tables[@]}"
    do
        COMMAND="delete from $table where CLUSTER_ID='${g_clusterId}' or CLUSTER_ID='cluster_${g_clusterId}';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="delete from $table where CLUSTER_ID='${g_clusterId}';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
            showLog_info "${LINENO}" "Delete Table : ${PUBLIC_SCHEMA}.${table} Successfully"
        fi
    done

    for table in "${g_clusterId_tables[@]}"
    do
        COMMAND="delete from $table where CLUSTERID='${g_clusterId}' or CLUSTERID='cluster_${g_clusterId}';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="delete from $table where CLUSTERID='${g_clusterId}';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
            showLog_info "${LINENO}" "Delete Table : ${PUBLIC_SCHEMA}.${table} Successfully"
        fi
    done

    for table in "${g_source_tables[@]}"
    do
        COMMAND="delete from $table where SOURCE='${g_clusterId}' or SOURCE='cluster_${g_clusterId}';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="delete from $table where SOURCE='${g_clusterId}';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
            showLog_info "${LINENO}" "Delete Table : ${PUBLIC_SCHEMA}.${table} Successfully"
        fi
    done

    for table in "${g_source_ommschema_tables[@]}"
    do
        COMMAND="delete from ${OMM_SCHEMA}.$table where SOURCE='${g_clusterId}' or SOURCE='cluster_${g_clusterId}';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="delete from ${OMM_SCHEMA}.$table where SOURCE='${g_clusterId}';"
            executeSql "${COMMAND}" "${table}" "${OMM_SCHEMA}"
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
            showLog_info "${LINENO}" "Delete Table : ${OMM_SCHEMA}.${table} Successfully"
        fi
    done

    for table in "${g_integer_cluster_id_tables[@]}"
    do
        COMMAND="delete from ${table} where CLUSTER_ID='${g_clusterId}';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="delete from ${table} where CLUSTERID='${g_clusterId}';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        else
            log_info "${LINENO}" "Success to execute command :${COMMAND}."
            showLog_info "${LINENO}" "Delete Table : ${PUBLIC_SCHEMA}.${table} Successfully"
        fi
    done
}

####################################################################
#  FUNCTION     : 告警和事件表清理
####################################################################
deleteAlarmAndEventTables(){
    COMMAND="delete from ${OMM_SCHEMA}.TBL_FM_ALARM_LOG where ISYNCNO in (select ISYNCNO from (select ISYNCNO, svAlarmId,svAlarmName,svLocationInfo,REPLACE(REGEXP_SUBSTR(svLocationInfo, '#.*#'), '#', '') as clusterId from ${OMM_SCHEMA}.TBL_FM_ALARM_LOG where svLocationInfo like '%#%') t where t.clusterId='${g_clusterId}');"
    ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
    if [ $? -ne 0 ];then
        executeSql "${COMMAND}" "TBL_FM_ALARM_LOG" "${OMM_SCHEMA}"
    else
        showLog_info "${LINENO}" "Delete Table : ${OMM_SCHEMA}.TBL_FM_ALARM_LOG Successfully"
        log_info "${LINENO}" "${COMMAND}"
    fi

    COMMAND="delete from ${OMM_SCHEMA}.TBL_FM_EVENT_LOG where svLocationInfo in (select svLocationInfo from (select svAlarmId,svLocationInfo,REPLACE(REGEXP_SUBSTR(svLocationInfo, '#.*#'), '#', '') as clusterId from ${OMM_SCHEMA}.TBL_FM_EVENT_LOG) where clusterId='${g_clusterId}');"
    ${DBCOMMAND} -W ${DBPASSWORD} -c "${COMMAND}" >/dev/null 2>&1
    if [ $? -ne 0 ];then
        executeSql "${COMMAND}" "TBL_FM_EVENT_LOG" "${OMM_SCHEMA}"
    else
        showLog_info "${LINENO}" "Delete Table : ${OMM_SCHEMA}.TBL_FM_EVENT_LOG Successfully"
        log_info "${LINENO}" "${COMMAND}"
    fi
}

####################################################################
#  FUNCTION     : redis逻辑集群表清理
####################################################################
deleteRedisLogicalTables(){
    for table in "${g_redis_logical[@]}"
    do
        COMMAND="delete from ${REDIS_SCHEMA}.${table} where OMCLUSTERID='${g_clusterId}';"
        executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}"
    done
}

####################################################################
#  FUNCTION     : 关联表清理
####################################################################
deleteRelateTables()
{
    COMMAND="delete from OM_FSMTRANSITIONS where CLUSTER_ID='${g_clusterId}' or CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_FSMTRANSITIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_FSMTRANSITIONS using OM_SERVICES where OM_SERVICES.SERVICE_ID=OM_FSMTRANSITIONS.SERVICE_ID and OM_SERVICES.CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_FSMTRANSITIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_FSMTRANSITIONS using OM_ROLES,OM_SERVICES where OM_ROLES.ROLE_ID=OM_FSMTRANSITIONS.ROLE_ID and OM_SERVICES.SERVICE_ID=OM_ROLES.SERVICE_ID and OM_SERVICES.CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_FSMTRANSITIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_FSMTRANSITIONS using OM_ROLES,OM_TOPOLOGY,OM_SERVICES where OM_TOPOLOGY.ROLE_INSTANCE_ID=OM_FSMTRANSITIONS.ROLE_INSTANCE_ID and OM_SERVICES.SERVICE_ID=OM_ROLES.SERVICE_ID and OM_TOPOLOGY.ROLE_ID=OM_ROLES.ROLE_ID and OM_SERVICES.CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_FSMTRANSITIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_NODES_ISOLATION using OM_NODES where OM_NODES.IP=OM_NODES_ISOLATION.IP and OM_NODES.CLUSTER_ID=${g_clusterId};"
    executeSql "${COMMAND}" "OM_NODES_ISOLATION" "${PUBLIC_SCHEMA}"

    # OM_CONFIGURATIONS
    COMMAND="delete from OM_CONFIGURATIONS where OWNER_CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_CONFIGURATIONS using OM_SERVICES where OM_CONFIGURATIONS.OWNER_SERVICE_ID=OM_SERVICES.SERVICE_ID and CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_CONFIGURATIONS using OM_ROLES,OM_SERVICES where OM_CONFIGURATIONS.OWNER_ROLE_ID=OM_ROLES.ROLE_ID and OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID and CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from OM_CONFIGURATIONS using OM_TOPOLOGY,OM_NODES where OM_CONFIGURATIONS.OWNER_ROLE_INSTANCE_ID=OM_TOPOLOGY.ROLE_INSTANCE_ID and OM_TOPOLOGY.NODE_ID=OM_NODES.NODE_ID and CLUSTER_ID=${g_clusterId};"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    ## 需要把 OM_TOPOLOGY 和 OM_INSTANCE_GROUP 放到OM_CONFIGURATIONS后
    # OM_TOPOLOGY
    COMMAND="delete from OM_TOPOLOGY using OM_NODES where OM_NODES.NODE_ID=OM_TOPOLOGY.NODE_ID and OM_NODES.CLUSTER_ID=${g_clusterId};"
    executeSql "${COMMAND}" "OM_TOPOLOGY" "${PUBLIC_SCHEMA}"

    # OM_INSTANCE_GROUP
    COMMAND="delete from OM_INSTANCE_GROUP using OM_ROLES,OM_SERVICES where OM_INSTANCE_GROUP.ROLE_ID=OM_ROLES.ROLE_ID and OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID and CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_INSTANCE_GROUP" "${PUBLIC_SCHEMA}"

    # OM_ROLES
    COMMAND="delete from OM_ROLES using OM_SERVICES where OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID and OM_SERVICES.CLUSTER_ID='cluster_${g_clusterId}';"
    executeSql "${COMMAND}" "OM_ROLES" "${PUBLIC_SCHEMA}"

    COMMAND="delete from TENANT_RESOURCES using TENANT where TENANT.TENANTID=TENANT_RESOURCES.TENANTID and TENANT.CLUSTERID=${g_clusterId};"
    executeSql "${COMMAND}" "TENANT_RESOURCES" "${PUBLIC_SCHEMA}"

    COMMAND="delete from TENANT_SERVICES using TENANT where TENANT.TENANTID=TENANT_SERVICES.TENANTID and TENANT.CLUSTERID=${g_clusterId};"
    executeSql "${COMMAND}" "TENANT_SERVICES" "${PUBLIC_SCHEMA}"

    COMMAND="delete from RESOURCES_PROPERTIES using TENANT_RESOURCES,TENANT where RESOURCES_PROPERTIES.RESOURCEID=TENANT_RESOURCES.RESOURCEID and TENANT_RESOURCES.TENANTID=TENANT.TENANTID and CLUSTERID=${g_clusterId};"
    executeSql "${COMMAND}" "RESOURCES_PROPERTIES" "${PUBLIC_SCHEMA}"

    COMMAND="delete from RESOURCEPOOL_HOSTS using RESOURCEPOOL where RESOURCEPOOL_HOSTS.RESOURCEPOOL_ID=RESOURCEPOOL.RESOURCEPOOL_ID and CLUSTERID='${g_clusterId}';"
    executeSql "${COMMAND}" "RESOURCEPOOL_HOSTS" "${PUBLIC_SCHEMA}"

    COMMAND="delete from RESOURCEPOOL_SERVICE using RESOURCEPOOL where RESOURCEPOOL_SERVICE.RESOURCEPOOL_ID=RESOURCEPOOL.RESOURCEPOOL_ID and CLUSTERID='${g_clusterId}';"
    executeSql "${COMMAND}" "RESOURCEPOOL_SERVICE" "${PUBLIC_SCHEMA}"
}

####################################################################
#  FUNCTION     : 表清理
####################################################################
deleteDbTables()
{
    # 从关联表进行删除
    deleteRelateTables
    # 按集群ID删除
    deleteClusterIDTables
    # 删除备份恢复表
    deleteBackupTables
    # 删除告警和事件表
    deleteAlarmAndEventTables
    # 删除redis逻辑集群
    deleteRedisLogicalTables
}

initLog()
{
    if [ ! -f ${G_LOG_FILE} ]; then
        touch ${G_LOG_FILE}
        if [ $? -ne 0 ]; then
            echo "Failed to create log file: ${G_LOG_FILE}";
            exit 1
        fi
    fi
}

initParams()
{
    # 集群 ID, 取值为 1 2 3 4...
    if [ $# != 1 ]; then
        showLog_error "${LINENO}" "Invalid parameter :$@"
        exit 1
    fi
    g_clusterId="${1}"
    validate_clusterId "${g_clusterId}"
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Invalid cluster id :${g_clusterId}"
        exit 1
    fi
}

initDBInfo()
{
    # 获取 DB 端口号及密码
    [ -f "$OMS_RUN_PATH/product_adapter.properties" ] && source $OMS_RUN_PATH/product_adapter.properties
    DBCOMMAND="gsql -U ommdba -d omm -p $OMS_GAUSS_PORT"
    DBPASSWORD=`getDBPasswd`
    if [ -z "${DBPASSWORD}" ]; then
        showLog_error "${LINENO}" "get db password failed."
        exit 1
    fi
}

backupDbTables(){
    mkdir -p ${g_curPath}/backupTables
    chmod 775 $(dirname $(dirname ${g_curPath})) -R

    backupPublicSchemaTables

    backupOmmSchemaTables

    backupRedisSchemaTables
}

main()
{
    initLog

    # 初始化入参: $1 集群ID
    initParams $@

    forbidOmsSwitch || exit 1

    initDBInfo

    backupDbTables

    # 刪除数据库表
    deleteDbTables

    restartOmsAndSyncallfile || exit 1
}

if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can delete data."
    exit 1
fi

getCurPath

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../func/functions.sh" || { echo "Failed to load functions.sh."; exit 1; }
. "${g_curPath}/../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }
. "${g_curPath}/../func/constants.sh" || { echo "Failed to load constants.sh."; exit 1; }

main $@
