#!/bin/bash

# ---------------------------------------------------------------------------- #
#                                   变量定义                                     #
# ---------------------------------------------------------------------------- #
## manager管理需要备份的表
## 全局性配置表
declare -a g_complete_tables=("OM_PACK_PACKAGES" "OM_PACKS" "OM_INSTANCE_GROUP_ID_SEQ" "OM_PACKS_PACK_ID_SEQ" "OM_AZS" "AOS_ROLE_INFO" "AOS_ROLE_USER_GROUP" "OM_USER_PRIVILEGES" "USERPRIVILAGE_ID_GEN_SEQ" "OM_PRIVILAGES_TOPOLY" "TBL_FM_HEALTHCHECK_CONFINFO" "OM_CLIENT_INFO_CLIENT_ID_SEQ" "TBL_FM_ALARM_LOCATIONINFO" "TBL_FM_EVENT_ADDITIONALINFO" "TBL_FM_EVENT_LOCATIONINFO" "TBL_FM_EVENT_RES")

## 按照集群迁移表
declare -a g_cluster_id_tables=("OM_SERVICES" "OM_CLUSTERS" "OM_CLUSTER_PATCH_INFO" "TBL_SYSTEM_LICENSE" "TBL_UPGRADE_MARK" "OM_MPPDB_CLIENT_INFO")
declare -a g_clusterId_tables=("TBL_MONITOR_CUSTOMIZE" "MPP_LOGIC_CLUSTER" "MPP_LOGIC_CLUSTER_LOOP_HOSTS" "MPP_LOGIC_CLUSTER_REDISTRIBUTION_TASK")
declare -a g_integer_cluster_id_tables=("OM_NODES" "OM_CLIENT_INFO" "RESOURCEPOOL" "TENANT")

## 告警事件 按集群迁移 通过sourceID进行区分
declare -a g_source_tables=("TBL_BASELINE" "TBL_BASELINE_TEMPLET")

## 备份恢复 按集群迁移
declare -a g_backup_tables=("TBL_BACKUP_CONFIGS" "TBL_BACKUP_TASKINFO" "TBL_RECOVERY_CONFIGS" "TBL_RECOVERY_TASKINFO")

## 组件需要备份的表,组件表备份目录带上组件名或schema,防止重名
## Redis逻辑集群迁移
declare -a g_redis_logical=("CLUSTER" "INSTANCE" "OMINSTANCES" "SLOT")

declare g_curPath=""                                            # 当前脚本所在的目录
declare g_clusterId=""                                          # 集群ID


# ---------------------------------------------------------------------------- #
#                                   函数定义                                    #
# ---------------------------------------------------------------------------- #

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - > /dev/null
}

####################################################################
#  FUNCTION     : 导出继承文件
####################################################################
backupFiles()
{
    log_info "${LINENO}" "Replace the placeholder value of the $g_curPath/backup.lst"
    BACKUP_LIST_FILE="$g_curPath/backup.lst"
    DST_FILE="$g_curPath/backupDir/backupFile.tar"
    local common_home=`dirname ${JAVA_HOME}`

    mkdir -vp $g_curPath/backupDir >/dev/null 2>&1
    chmod 775 $g_curPath/backupDir

    sed -i "s:#INSTALLCONTROLLERPATH#:${CONTROLLER_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#INSTALLTOMCATPATH#:${OM_TOMCAT_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#INSTALLOMSPATH#:${OMS_RUN_PATH}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATADATAHOME#:${BIGDATA_DATA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATAHOME#:${BIGDATA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#JAVAHOME#:${JAVA_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#OMHTTPDHOME#:${OM_HTTPD_HOME}:g" "$BACKUP_LIST_FILE"
    sed -i "s:#BIGDATA_COMMON#:${BIGDATA_COMMON}:g" "$BACKUP_LIST_FILE"
    sed -i "s/#CLUSTER_ID#/${g_clusterId}/g" "$BACKUP_LIST_FILE"
    sed -i "s:#COMMON_HOME#:${common_home}:g" "$BACKUP_LIST_FILE"
    tar -rPp -T "$BACKUP_LIST_FILE" -f "$DST_FILE" >/dev/null 2>&1
    chmod 775 "$DST_FILE"
}

####################################################################
#  FUNCTION     : 判断表是否存在
####################################################################
isTableExist(){
    local tableSchema=$1
    local tableName=$2
    local isTableExistCommand="SELECT COUNT(*) FROM INFORMATION_SCHEMA.TABLES WHERE TABLE_SCHEMA='${tableSchema}' AND TABLE_NAME='${tableName}';"
    local result=$(${DBCOMMAND} -W ${DBPASSWORD} -c "$isTableExistCommand")
    local tableNum=$(echo $result | awk  '{printf $3}')
    if [ ${tableNum}X == "1"X ];then
        return 0
    else
        return 1
    fi
}

####################################################################
#  FUNCTION     : 执行sql
####################################################################
executeSql(){
    local command="${1}"
    local tableName="${2}"
    local tableSchema="${3}"
    ${DBCOMMAND} -W ${DBPASSWORD} -c "${command}" >/dev/null 2>&1
    if [ $? -ne 0 ];then
        isTableExist "${tableSchema}" "${tableName}"
        if [ $? -ne 0 ];then
            showLog_warn "${LINENO}" "Table ${tableSchema}.${tableName} does not exist."
        else
            showLog_error "${LINENO}" "Failed to execute command :${command}."
            exit 1
        fi
    else
         log_info "${LINENO}" "Success to execute command :${command}."
         showLog_info "${LINENO}" "Export Table : ${tableSchema}.${tableName} Successfully"
    fi
}

####################################################################
#  FUNCTION     : 导出备份恢复表
####################################################################
backupBackupTables(){
    for table in "${g_backup_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table where TASKNAME='default-${g_clusterId}' or TASKNAME='default-oms') TO '$g_curPath/backupDir/tables/$table';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
    done
    ## 需要包含手动备份任务
    COMMAND="COPY (select * from TBL_BACKUP_TASKINFO where TASKNAME='default-${g_clusterId}' or TASKNAME='default-oms' or CLUSTERID='${g_clusterId}') TO '$g_curPath/backupDir/tables/TBL_BACKUP_TASKINFO';"
    executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
}

####################################################################
#  FUNCTION     : 导出告警表
####################################################################
backupAlarmTables()
{
    touch $g_curPath/backupDir/tables/TBL_PM_ALARMTHRESHOLD
    chmod 775 $g_curPath/backupDir/tables/TBL_PM_ALARMTHRESHOLD
    COMMAND="COPY (select * from OMM_1.TBL_PM_ALARMTHRESHOLD where SOURCE='-1' or SOURCE='${g_clusterId}') TO '$g_curPath/backupDir/tables/TBL_PM_ALARMTHRESHOLD';"
    executeSql "${COMMAND}" "TBL_PM_ALARMTHRESHOLD" "${OMM_SCHEMA}"

    for table in "${g_source_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table where SOURCE='-1' or SOURCE='${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
    done
}

####################################################################
#  FUNCTION     : 导出redis逻辑集群
####################################################################
backupRedisLogical()
{
    mkdir -p $g_curPath/backupDir/tables/${REDIS_SCHEMA}
    chmod 755 $g_curPath/backupDir/tables/${REDIS_SCHEMA}
    for table in "${g_redis_logical[@]}"
    do
        touch "$g_curPath/backupDir/tables/${REDIS_SCHEMA}/${table}"
        chmod 775 "$g_curPath/backupDir/tables/${REDIS_SCHEMA}/${table}"
        COMMAND="COPY (select * from redisdb.$table where OMCLUSTERID='${g_clusterId}') TO '$g_curPath/backupDir/tables/${REDIS_SCHEMA}/${table}';"
        executeSql "${COMMAND}" "${table}" "${REDIS_SCHEMA}"
    done
}

####################################################################
#  FUNCTION     : 导出全局表
####################################################################
backupCompleteTables()
{
    for table in "${g_complete_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table) TO '$g_curPath/backupDir/tables/$table';"
        executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
    done
}

####################################################################
#  FUNCTION     : 按集群ID导出表
####################################################################
backupClusterIDTables()
{
    for table in "${g_cluster_id_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table where CLUSTER_ID='${g_clusterId}' or CLUSTER_ID='cluster_${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "$COMMAND" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="COPY (select * from $table where CLUSTER_ID='${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        fi
    done

    for table in "${g_clusterId_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table where CLUSTERID='${g_clusterId}' or CLUSTERID='cluster_${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "$COMMAND" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="COPY (select * from $table where CLUSTERID='${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        fi
    done

    for table in "${g_integer_cluster_id_tables[@]}"
    do
        touch $g_curPath/backupDir/tables/$table
        chmod 775 $g_curPath/backupDir/tables/$table
        COMMAND="COPY (select * from $table where CLUSTER_ID='${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
        ${DBCOMMAND} -W ${DBPASSWORD} -c "$COMMAND" >/dev/null 2>&1
        if [ $? -ne 0 ];then
            COMMAND="COPY (select * from $table where CLUSTERID='${g_clusterId}') TO '$g_curPath/backupDir/tables/$table';"
            executeSql "${COMMAND}" "${table}" "${PUBLIC_SCHEMA}"
        fi
    done
}

####################################################################
#  FUNCTION     : 根据表关联关系导出表数据
####################################################################
backupRelateTables()
{
    # OM_FSMTRANSITIONS
    touch $g_curPath/backupDir/tables/OM_FSMTRANSITIONS
    chmod 775 $g_curPath/backupDir/tables/OM_FSMTRANSITIONS
    COMMAND="COPY (select * from OM_FSMTRANSITIONS f where f.CLUSTER_ID = 'cluster_${g_clusterId}' or exists (select 1 from OM_SERVICES s where s.CLUSTER_ID = 'cluster_${g_clusterId}' and f.SERVICE_ID = s.SERVICE_ID) or exists (select 1 from OM_ROLES r,OM_SERVICES s where r. SERVICE_ID = s.SERVICE_ID and s.CLUSTER_ID = 'cluster_${g_clusterId}' and f.ROLE_ID = r.ROLE_ID) or exists (select 1 from OM_TOPOLOGY t where exists (select 1 from OM_ROLES r, OM_SERVICES s where r.SERVICE_ID = s.SERVICE_ID and s.CLUSTER_ID = 'cluster_${g_clusterId}' and r.ROLE_ID = t.ROLE_ID) and t.ROLE_INSTANCE_ID = f.ROLE_INSTANCE_ID)) TO '$g_curPath/backupDir/tables/OM_FSMTRANSITIONS';"
    executeSql "${COMMAND}" "OM_FSMTRANSITIONS" "${PUBLIC_SCHEMA}"

    # OM_NODES_ISOLATION
    touch $g_curPath/backupDir/tables/OM_NODES_ISOLATION
    chmod 775 $g_curPath/backupDir/tables/OM_NODES_ISOLATION
    COMMAND="COPY (select OM_NODES_ISOLATION.* from OM_NODES_ISOLATION join OM_NODES on OM_NODES.IP=OM_NODES_ISOLATION.IP where OM_NODES.CLUSTER_ID=${g_clusterId}) TO '$g_curPath/backupDir/tables/OM_NODES_ISOLATION';"
    executeSql "${COMMAND}" "OM_NODES_ISOLATION" "${PUBLIC_SCHEMA}"

    # OM_ROLES
    touch $g_curPath/backupDir/tables/OM_ROLES
    chmod 775 $g_curPath/backupDir/tables/OM_ROLES
    COMMAND="COPY (select OM_ROLES.* from OM_ROLES join OM_SERVICES on OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID where OM_SERVICES.CLUSTER_ID='cluster_${g_clusterId}') TO '$g_curPath/backupDir/tables/OM_ROLES';"
    executeSql "${COMMAND}" "OM_ROLES" "${PUBLIC_SCHEMA}"

    # OM_TOPOLOGY
    touch $g_curPath/backupDir/tables/OM_TOPOLOGY
    chmod 775 $g_curPath/backupDir/tables/OM_TOPOLOGY
    COMMAND="COPY (select OM_TOPOLOGY.* from OM_TOPOLOGY join OM_NODES on OM_NODES.NODE_ID=OM_TOPOLOGY.NODE_ID where OM_NODES.CLUSTER_ID=${g_clusterId}) TO '$g_curPath/backupDir/tables/OM_TOPOLOGY';"
    executeSql "${COMMAND}" "OM_TOPOLOGY" "${PUBLIC_SCHEMA}"

    # OM_INSTANCE_GROUP
    touch $g_curPath/backupDir/tables/OM_INSTANCE_GROUP
    chmod 775 $g_curPath/backupDir/tables/OM_INSTANCE_GROUP
    COMMAND="COPY (select OM_INSTANCE_GROUP.* from (OM_INSTANCE_GROUP join OM_ROLES on OM_INSTANCE_GROUP.ROLE_ID=OM_ROLES.ROLE_ID) join OM_SERVICES on OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID where CLUSTER_ID='cluster_${g_clusterId}') TO '$g_curPath/backupDir/tables/OM_INSTANCE_GROUP';"
    executeSql "${COMMAND}" "OM_INSTANCE_GROUP" "${PUBLIC_SCHEMA}"

    # OM_CONFIGURATIONS
    touch $g_curPath/backupDir/tables/OM_CONFIGURATIONS
    local OM_CONFIGURATIONS="$g_curPath/backupDir/tables/OM_CONFIGURATIONS"
    chmod 775 $OM_CONFIGURATIONS

    touch $g_curPath/backupDir/tables/OM_CONFIGURATIONS_01
    local OM_CONFIGURATIONS_01="$g_curPath/backupDir/tables/OM_CONFIGURATIONS_01"
    chmod 775 $OM_CONFIGURATIONS_01
    COMMAND="COPY (select * from OM_CONFIGURATIONS where OWNER_CLUSTER_ID='cluster_${g_clusterId}') TO '$OM_CONFIGURATIONS_01';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    touch $g_curPath/backupDir/tables/OM_CONFIGURATIONS_02
    local OM_CONFIGURATIONS_02="$g_curPath/backupDir/tables/OM_CONFIGURATIONS_02"
    chmod 775 $OM_CONFIGURATIONS_02
    COMMAND="COPY (select OM_CONFIGURATIONS.* from OM_CONFIGURATIONS join OM_SERVICES on OM_CONFIGURATIONS.OWNER_SERVICE_ID=OM_SERVICES.SERVICE_ID where CLUSTER_ID='cluster_${g_clusterId}') TO '$OM_CONFIGURATIONS_02';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    touch $g_curPath/backupDir/tables/OM_CONFIGURATIONS_03
    local OM_CONFIGURATIONS_03="$g_curPath/backupDir/tables/OM_CONFIGURATIONS_03"
    chmod 775 $OM_CONFIGURATIONS_03
    COMMAND="COPY (select OM_CONFIGURATIONS.* from ( OM_CONFIGURATIONS join OM_ROLES on OM_CONFIGURATIONS.OWNER_ROLE_ID=OM_ROLES.ROLE_ID ) join OM_SERVICES on OM_ROLES.SERVICE_ID=OM_SERVICES.SERVICE_ID where CLUSTER_ID='cluster_${g_clusterId}') TO '$OM_CONFIGURATIONS_03';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    touch $g_curPath/backupDir/tables/OM_CONFIGURATIONS_04
    local OM_CONFIGURATIONS_04="$g_curPath/backupDir/tables/OM_CONFIGURATIONS_04"
    chmod 775 $OM_CONFIGURATIONS_04
    COMMAND="COPY (select OM_CONFIGURATIONS.* from ( OM_CONFIGURATIONS join OM_TOPOLOGY on OM_CONFIGURATIONS.OWNER_ROLE_INSTANCE_ID=OM_TOPOLOGY.ROLE_INSTANCE_ID) join OM_NODES on OM_TOPOLOGY.NODE_ID=OM_NODES.NODE_ID where CLUSTER_ID=${g_clusterId}) TO '$OM_CONFIGURATIONS_04';"
    executeSql "${COMMAND}" "OM_CONFIGURATIONS" "${PUBLIC_SCHEMA}"

    cat $OM_CONFIGURATIONS_01 >>$OM_CONFIGURATIONS
    cat $OM_CONFIGURATIONS_02 >>$OM_CONFIGURATIONS
    cat $OM_CONFIGURATIONS_03 >>$OM_CONFIGURATIONS
    cat $OM_CONFIGURATIONS_04 >>$OM_CONFIGURATIONS

    rm -rf $OM_CONFIGURATIONS_01
    rm -rf $OM_CONFIGURATIONS_02
    rm -rf $OM_CONFIGURATIONS_03
    rm -rf $OM_CONFIGURATIONS_04

    # TENANT_RESOURCES
    touch $g_curPath/backupDir/tables/TENANT_RESOURCES
    chmod 775 $g_curPath/backupDir/tables/TENANT_RESOURCES
    COMMAND="COPY (select TENANT_RESOURCES.* from TENANT_RESOURCES join TENANT on TENANT.TENANTID=TENANT_RESOURCES.TENANTID where TENANT.CLUSTERID=${g_clusterId}) TO '$g_curPath/backupDir/tables/TENANT_RESOURCES';"
    executeSql "${COMMAND}" "TENANT_RESOURCES" "${PUBLIC_SCHEMA}"

    # TENANT_SERVICES
    touch $g_curPath/backupDir/tables/TENANT_SERVICES
    chmod 775 $g_curPath/backupDir/tables/TENANT_SERVICES
    COMMAND="COPY (select TENANT_SERVICES.* from TENANT_SERVICES join TENANT on TENANT.TENANTID=TENANT_SERVICES.TENANTID where TENANT.CLUSTERID=${g_clusterId}) TO '$g_curPath/backupDir/tables/TENANT_SERVICES';"
    executeSql "${COMMAND}" "TENANT_SERVICES" "${PUBLIC_SCHEMA}"

    # RESOURCES_PROPERTIES
    touch $g_curPath/backupDir/tables/RESOURCES_PROPERTIES
    chmod 775 $g_curPath/backupDir/tables/RESOURCES_PROPERTIES
    COMMAND="COPY (select RESOURCES_PROPERTIES.* from (RESOURCES_PROPERTIES join TENANT_RESOURCES on RESOURCES_PROPERTIES.RESOURCEID=TENANT_RESOURCES.RESOURCEID) join TENANT on TENANT_RESOURCES.TENANTID=TENANT.TENANTID where CLUSTERID=${g_clusterId}) TO '$g_curPath/backupDir/tables/RESOURCES_PROPERTIES';"
    executeSql "${COMMAND}" "RESOURCES_PROPERTIES" "${PUBLIC_SCHEMA}"

    # RESOURCEPOOL_HOSTS
    touch $g_curPath/backupDir/tables/RESOURCEPOOL_HOSTS
    chmod 775 $g_curPath/backupDir/tables/RESOURCEPOOL_HOSTS
    COMMAND="COPY (select RESOURCEPOOL_HOSTS.* from RESOURCEPOOL_HOSTS join RESOURCEPOOL on RESOURCEPOOL_HOSTS.RESOURCEPOOL_ID=RESOURCEPOOL.RESOURCEPOOL_ID where CLUSTERID='${g_clusterId}')TO '$g_curPath/backupDir/tables/RESOURCEPOOL_HOSTS';"
    executeSql "${COMMAND}" "RESOURCEPOOL_HOSTS" "${PUBLIC_SCHEMA}"

    # RESOURCEPOOL_SERVICE
    touch $g_curPath/backupDir/tables/RESOURCEPOOL_SERVICE
    chmod 775 $g_curPath/backupDir/tables/RESOURCEPOOL_SERVICE
    COMMAND="COPY (select RESOURCEPOOL_SERVICE.* from RESOURCEPOOL_SERVICE join RESOURCEPOOL on RESOURCEPOOL_SERVICE.RESOURCEPOOL_ID=RESOURCEPOOL.RESOURCEPOOL_ID where CLUSTERID='${g_clusterId}')TO '$g_curPath/backupDir/tables/RESOURCEPOOL_SERVICE';"
    executeSql "${COMMAND}" "RESOURCEPOOL_SERVICE" "${PUBLIC_SCHEMA}"
}

####################################################################
#  FUNCTION     : 备份表数据
####################################################################
backupDbTables()
{
    showLog_info "${LINENO}" "Start exporting table data..."
    mkdir -vp $g_curPath/backupDir/tables >/dev/null 2>&1
    chmod 775 $(dirname ${g_curPath}) -R
    # 直接备份
    backupCompleteTables
    # 按照集群ID备份
    backupClusterIDTables
    # 从上级表查询备份
    backupRelateTables
    # 备份恢复表
    backupBackupTables
    # 备份告警表
    backupAlarmTables
    # 组件特殊场景表备份
    # 备份redis逻辑集群
    backupRedisLogical

    showLog_info "${LINENO}" "Exporting table data ends."
}

initParams()
{
    rm -rf ${g_curPath}/backupDir*
    # 集群 ID, 取值为 1 2 3 4...
    if [ $# != 2 ]; then
        showLog_error "${LINENO}" "Invalid parameter :$@"
        exit 1
    fi

    g_clusterId="${1}"
    ip_addr="${2}"

    validate_clusterId "${g_clusterId}"
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Invalid cluster id :${g_clusterId}"
        exit 1
    fi

    validate_ip "${ip_addr}"
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Invalid IP address :${ip_addr}"
        exit 1
    fi

    chmod 775 $(dirname $(dirname ${g_curPath})) -R
    find ${g_curPath} -type f | xargs  dos2unix {}  > /dev/null 2>&1
}

initDBInfo()
{
    # 获取 DB 端口号及密码
    [ -f "$OMS_RUN_PATH/product_adapter.properties" ] && source $OMS_RUN_PATH/product_adapter.properties
    DBCOMMAND="gsql -U ommdba -d omm -p $OMS_GAUSS_PORT"
    DBPASSWORD=`getDBPasswd`
    if [ -z "${DBPASSWORD}" ]; then
        showLog_error "${LINENO}" "get db password failed."
        exit 1
    fi
}

initLog()
{
    if [ ! -f ${G_LOG_FILE} ]; then
        touch ${G_LOG_FILE}
        if [ $? -ne 0 ]; then
            echo "Failed to create log file: ${G_LOG_FILE}";
            exit 1
        fi
    fi
}

precheck(){
    local toolRootPath=$(dirname ${g_curPath})
    local toolRootPathOwner=$(ls -ld ${toolRootPath} | awk '{print $3}')
    if [ ${toolRootPathOwner}X != "omm"X ];then
        echo "${toolRootPath} owner is not omm:wheel,Change the file owner first."
        exit 1
    fi
}

transfileToNewOms()
{
    # 打包数据
    cd ${g_curPath}
    tar czf backupDir.tar.gz backupDir
    # 将备份文件包拷贝至新节点
    scp backupDir.tar.gz root@${ip_addr}:/tmp
}

main()
{
    precheck

    initLog

    initDBInfo

    # 初始化入参: $1 集群ID,$2 新 OMS 节点 IP 地址
    initParams $@
    # 备份文件
    backupFiles
    # 备份数据库表
    backupDbTables
    # 打包并将备份文件包拷贝至新节点
    transfileToNewOms

    showLog_info $LINENO "Exporting data is complete。"
}

# 判断是否为 omm 用户
if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can export data."
    exit 1
fi

getCurPath

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../func/functions.sh" || { echo "Failed to load functions.sh."; exit 1; }
. "${g_curPath}/../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }
. "${g_curPath}/../func/constants.sh" || { echo "Failed to load constants.sh."; exit 1; }

main $@
