#!/bin/bash

decrpt()
{
    [ -f "/home/omm/.basrc" ] && source "/home/omm/.basrc"
    [ -z "$OMS_RUN_PATH" ] && OMS_RUN_PATH="/opt/omm/oms"

    #declare envs.
    local parmas="$1"
    local rootDir="${OMS_RUN_PATH}/workspace"
    local selfDir="${OMS_RUN_PATH}/tools/encrypt"
    local wcc="$(find "${rootDir}"/lib/ -maxdepth 1 -name 'wc2frm*.jar')"
    local log4j="$(find "${rootDir}"/lib/ -maxdepth 1 -name 'log4j*.jar')"
    local commons="$(find "${rootDir}"/lib/ -maxdepth 1 -name 'commons-codec*.jar')"
    local javaArgs="-Dbeetle.application.home.path=${WCC_PROFILE_DIR}"
    local classpath="${wcc}:${log4j}:${commons}:${selfDir}/main.jar"
    local className="com.omm.encrypt.DecryptCommandWithoutParam"
    local javaCommand="${JAVA_HOME}/bin/java -cp ${classpath} ${javaArgs} ${className}"

    #check params
    if [ -z "$parmas" ]; then
        showLog_error "${LINENO}" "strSrc is necessary parameter.";
        return 1
    fi

    chmod 550 "${selfDir}"/*.jar > /dev/null 2>&1

    ${javaCommand} 2>/dev/null << EOF
strSrc:$parmas
encType:AES256_CBC_IGNORE_KEY
EOF

    return $?
}

####################################################################
#  FUNCTION     : 获取ommdba用户的OMS数据库密码
#  INPUT        :
#  OUTPUT       :
#  RETURN       :
####################################################################
getDBPasswd()
{
    #1.判断两个工具脚本存在与否
    local getCipherPasswdTool="${OMS_RUN_PATH}/tools/getdbNameOrPasswd"
    [ -e "${getCipherPasswdTool}" ] || { showLog_error "${LINENO}" "${getCipherPasswdTool} is not exist."; return 1; }

    #2.获取密文密码
    local CipherPasswd=$(${getCipherPasswdTool} 3 | awk '{print '$1'}')
    if [ -z "${CipherPasswd}" ];then
        return 1
    fi

    #3.获取明文密码
    local DecryptPasswd=$(decrpt ${CipherPasswd} | awk '{print $1}')
    if [ -z "${DecryptPasswd}" ];then
        return 2
    fi

    echo ${DecryptPasswd}
}

####################################################################
#  FUNCTION     : 获取omm用户的OMS数据库密码
#  INPUT        :
#  OUTPUT       :
#  RETURN       :
####################################################################
function getOmmDBPassword()
{
    #1.判断两个工具脚本存在与否
    local getCipherPasswdTool="${OMS_RUN_PATH}/tools/getdbNameOrPasswd"
    [ -e "${getCipherPasswdTool}" ] || { showLog_error "${LINENO}" "${getCipherPasswdTool} is not exist."; return 1; }

    #2.获取密文密码
    local CipherPasswd=$(${getCipherPasswdTool} 2 | awk '{print '$1'}')
    if [ -z "${CipherPasswd}" ];then
        return 1
    fi

    #3.获取明文密码
    local DecryptPasswd=$(decrpt ${CipherPasswd} | awk '{print $1}')
    if [ -z "${DecryptPasswd}" ];then
        return 2
    fi

    echo ${DecryptPasswd}
}

function validate_ip()
{
    local tmpIP="${1}"

    if [ -z "${tmpIP}" ] ; then
        return 1
    fi

    echo "${tmpIP}" | grep "^[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}\.[0-9]\{1,3\}$" >/dev/null 2>&1
    if [ $? -ne 0 ] ; then
        return 1
    fi

    local -i temp=0
    for temp in $(echo ${tmpIP} | awk -F"." '{printf ("%s %s %s %s ", $1, $2, $3, $4);}'); do
        if [ ${temp} -gt 255 ] || [ ${temp} -lt 0 ] ; then
            return 1
        fi
    done

    return 0
}

function validate_clusterId()
{
    local clusterId="${1}"

    if [ -z "${clusterId}" ]; then
        return 1
    fi

    if [ "${clusterId}" -gt 0 ] 2>/dev/null;then
        return 0
    fi

    return 1
}


forbidOmsSwitch()
{
    local oms_status=$($OMS_RUN_PATH/workspace/ha/module/hacom/script/get_harole.sh | head -n 1)  >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to get OMS status."
        return 1
    fi
    local ha_mode=$($OMS_RUN_PATH/workspace/ha/module/hacom/script/config_ha.sh -a |grep "^HaMode" |awk '{ print $2 }')
    if [ ${ha_mode}X == "double"X ] && [ ${oms_status}X == "active"X ] ;then
        #禁止主备倒换return 12表示已经处于禁止状态
        $OMS_RUN_PATH/workspace/ha/module/hacom/tools/ha_client_tool --ip=127.0.0.1 --port=20013 --forbidswitch --name=product --time=120  >/dev/null 2>&1
        if [ $? -ne 0 -a $? -ne 12 ]; then
            showLog_error "${LINENO}" "Forbiden OMS switch failed."
            return 1
        fi
    elif [ ${oms_status}X == "standby"X ] ;then
        showLog_error "${LINENO}" "ImportData can be executed only on the active node."
        return 1
    elif [ ${ha_mode}X == "single"X ] ;then
        showLog_info "${LINENO}" "Ha mode is single,Do not need to forbidswitch."
    fi
    return 0
}


restartOmsAndSyncallfile()
{
    ${BIGDATA_HOME}/om-server/om/sbin/restart-oms.sh

    local oms_status=$($OMS_RUN_PATH/workspace/ha/module/hacom/script/get_harole.sh | head -n 1)
    local ha_mode=$($OMS_RUN_PATH/workspace/ha/module/hacom/script/config_ha.sh -a |grep "^HaMode" |awk '{ print $2 }')
    if [ ${ha_mode}X == "double"X ] && [ ${oms_status}X == "active"X ] ;then
        ${OMS_RUN_PATH}/workspace0/ha/module/hacom/tools/ha_client_tool --syncallfile
        ${OMS_RUN_PATH}/workspace0/ha/module/hacom/tools/ha_client_tool --getsyncfilestatus
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "File synchronization failed."
            return 1
        fi
        ${OMS_RUN_PATH}/workspace/ha/module/hacom/tools/ha_client_tool --ip=127.0.0.1 --port=20013 --cancelforbidswitch --name=product >/dev/null 2>&1
    fi
    return 0
}

stopActiveOms(){
    ${BIGDATA_HOME}/om-server/om/sbin/stop-oms.sh
    if [ $? -ne 0 ]; then
        showLog_error "${LINENO}" "stop oms failed."
        return 1
    fi
}

