#!/bin/bash
################################################################################
## Copyright:   HUAWEI Tech. Co., Ltd.
## Filename:    installcontroller.sh
## Description: install controller
## Version:     FusionInsight Hadoop V100R002C02
## Created:     Friday, May 17, 2013
################################################################################
# ---------------------------------------------------------------------------- #
#                                   变量定义                                   #
# ---------------------------------------------------------------------------- #
declare g_bigdataDataHome="$1"              # 大数据产品数据存储根目录
declare g_dbHome="$2"                       # 数据库的Home目录
declare g_omFlowIP="$3"                     # 各个节点访问主controller的浮动ip
declare g_wsFlowIP="$4"                     # 用户在web页面访问系统的浮动ip
declare g_omsHome="$5"                      # oms安装目录
declare g_logHome="$6"                      # 日志根目录
declare g_logFile="$7"                      # 日志文件
declare g_ssoServerAndMasterIpInfo="$8"
declare g_ommid="$9"                        # OMM用户ID (注：该写法并不安全，以后得改为其他方式)
declare g_localIp="${10}"                  # 本端IP
declare g_peerIp="${11}"                   # 对端IP
declare g_bigdataCommonWorkspace="${12}"      # /opt/huawei/Bigdata/common/runtime0
declare g_operatorType="${13}"                 # 当前操作的类型（升级还是普通安装）

declare g_ssoServerIp=""                    # SSO服务IP
declare g_ssoServerPort=""                  # SSO服务端口
declare g_masterldapIP=""                   # omsldap服务IP
declare g_masterldapPort=""                 # omsldap服务端口

declare g_controllerHome=""                 # controller目录
declare g_omServerWorkspace=""              # server安装的工作区目录
declare g_bigdataHome=""                    # bigdata目录
declare g_omAgentWorkspace=""               # om-agent workspace
declare g_bigdataCommon=""                  # common /opt/huawei/Bigdata/common/runtime
declare g_tomcatInstallFileName=""          # tomcat安装文件名
declare g_tomcatHome=""                     # tomcat安装目录
declare g_tomcatLogHome=""                  # tomcat日志根目录
declare g_componentsAdapterHome=""          # 组件的适配目录
declare g_curPath=""                        # 当前脚本所在的目录
declare g_scriptName=""                     # 当前脚本的脚本名称
declare -i retVal=0                         # 返回值

declare gen_cas_cert_path=""                # 生成cas证书的脚本路径
declare gen_om_cert_path=""                 # 生成om证书的脚本路径
declare gen_httpd_cert_path=""              # 生成httpd证书的脚本路径

declare -r gen_cas_cert_filename="generate_cas_cert.sh"     # 生成cas证书的脚本名称
declare -r gen_om_cert_filename="generate_om_cert.sh"       # 生成om证书的脚本名称
declare -r gen_httpd_cert_filename="generate_httpd_cert.sh" # 生成om证书的脚本名称
declare g_randon_key_dir=""                 # 随机密钥的存放目录
declare g_encrypt_tool=""                   # 加密工具
declare g_encryptDatabaseUser=""            # 加密的数据库用户
declare g_encryptDatabasePassword=""        # 加密的数据库密码
declare g_packageDistributeSubdir=""        #指定分发包子目录

declare -r INSERT_PACK_TEMPLATE="INSERT INTO OM_PACKS (PACK_NAME,PACK_DISPLAY_NAME, PACK_VERSION, PACKAGE_NAME, STATUS) VALUES('<PACK_NAME>', '<PACK_DISPLAY_NAME>', '<PACK_VERSION>', '<PACKAGE_NAME>','<STATUS>');"


# ---------------------------------------------------------------------------- #
#                                   函数定义                                   #
# ---------------------------------------------------------------------------- #
################################################################################
# Function: get_cur_path
# Description: 获取脚本所在的目录及脚本名
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_log
# Description: 初始化日志文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_log()
{
    # 确保已传入日志文件名，在此处以后好打日志。
    if [ ! -f "${g_logFile}" ] ; then
        echo "ERROR: The ${g_logFile} log file does not exist."
        return 1
    fi

    return 0
}

################################################################################
# Function: check_input_param
# Description: 检查输入参数
# Parameter:
#   input:
#   $1 -- g_greHome  GRE的HOME目录
#   $2 -- g_dbHome   数据库的Home目录
#   $3 -- g_omFlowIP 各个节点访问主controller的浮动ip
#   $4 -- g_wsFlowIP 用户在web页面访问系统的浮动ip
#   $5 -- g_omsHome  oms安装目录
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
check_input_param()
{
    if [ -z "${g_bigdataDataHome}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    if [ -z "${g_dbHome}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    if [ -z "${g_omFlowIP}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    if [ -z "${g_wsFlowIP}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    if [ -z "${g_omsHome}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    if [ -z "${g_logHome}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    return ${INS_SUCCESS}
}

################################################################################
# Function: init_global_param
# Description: 初始化全局变量
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_global_param()
{
    log_info "${LINENO}" "Initialize global parameter value."

    # 初始化g_controllerHome
    g_controllerHome="$(dirname ${g_curPath})"
    log_debug "${LINENO}" "g_controllerHome=${g_controllerHome}."
    
    # 初始化 om sever 安装的工作区目录
    g_omServerWorkspace="$(dirname "${g_controllerHome}")"
    log_debug "${LINENO}" "g_omServerWorkspace=${g_omServerWorkspace}."
    
    # 初始化g_bigdataHome
    g_bigdataHome="$(dirname ${g_omServerWorkspace})"
    log_debug "${LINENO}" "g_bigdataHome=${g_bigdataHome}."
    
    version=$(cat ${g_controllerHome}/etc/om/VERSION)
    # init agent worksapce
    g_omAgentWorkspace="${g_bigdataHome}/om-agent_${version}"
    log_debug "${LINENO}" "g_omAgentWorkspace=${g_omAgentWorkspace}."
    
    # initialize g_bigdataCommon
    g_bigdataCommon="${g_bigdataHome}/${COMMON_HOME_NAME}/${COMMON_WORKSPACE_NAME}"
    log_debug "${LINENO}" "g_bigdataCommon=${g_bigdataCommon}."
          
    # 初始化g_tomcatInstallFileName
    g_tomcatInstallFileName="$(ls ${g_controllerHome}/inst/tomcat \
                  | grep '^apache-tomcat-.*\.tar\.gz$')"
      if [ "$(echo "${g_tomcatInstallFileName}" | wc -w)" != "1" ] ; then
        showLog_error "${LINENO}" "The calculated tomcat install file name is incorrect."
        return ${INS_FILE_NOT_UNIQUE}
    fi

    g_tomcatInstallFileName="${g_controllerHome}/inst/tomcat/${g_tomcatInstallFileName}"
    log_debug "${LINENO}" "g_tomcatInstallFileName=${g_tomcatInstallFileName}."

    # 初始化g_tomcatHome
    g_tomcatHome="$(basename ${g_tomcatInstallFileName%.tar.gz})"
    g_tomcatHome="${g_omServerWorkspace}/${g_tomcatHome}"
    log_debug "${LINENO}" "g_tomcatHome=${g_tomcatHome}."

    # 初始化g_tomcatLogHome
    g_tomcatLogHome="${g_logHome}/tomcat"
    log_debug "${LINENO}" "g_tomcatLogHome=${g_tomcatLogHome}."

    # 初始化g_componentsAdapterHome
    g_componentsAdapterHome="${g_bigdataHome}/${COMPONENTS_HOME_NAME}"
    log_debug "${LINENO}" "g_componentsAdapterHome=${g_componentsAdapterHome}."
    
    local ldap_ip_list=($(echo "${g_ssoServerAndMasterIpInfo}" | awk -F '#' '{for(i=1; i<=NF; i++){print $i;};}'))
    g_ssoServerIp=${ldap_ip_list[0]}
    g_ssoServerPort=${ldap_ip_list[1]}
    g_masterldapIP=${ldap_ip_list[2]}
    g_masterldapPort=${ldap_ip_list[3]}


    # 初始化gen_cas_cert_path
    gen_cas_cert_path="${g_controllerHome}/security/cert/conf"
    # 初始化gen_om_cert_path
    gen_om_cert_path="${g_controllerHome}/security/cert/conf"
    # 初始化gen_om_cert_path
    gen_httpd_cert_path="${g_controllerHome}/security/cert/conf"

    #初始化随机密钥目录
    g_randon_key_dir="$g_controllerHome/security/wcc"

    #初始化解密工具
    g_encrypt_tool="$g_controllerHome/tools/encryptWithoutParam.sh"
    
    g_controllerProfile="${g_omServerWorkspace}/.controller_profile"
    g_commonProfile="${g_bigdataCommonWorkspace}/.common_profile"
	
    if [ -z ${BIGDATA_SUDO_HOME} ]; then
        BIGDATA_SUDO_HOME="/var/lib/sudo/Bigdata"
    fi

    return ${INS_SUCCESS}
}

################################################################################
# Function: is_jre_ok
# Description: 检查jre是否安装成功
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
is_jre_ok()
{
    log_debug "${LINENO}" "Check whether jre is effective."

    exec_cmd java -version
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install java runtime environment."
        return ${INS_INSTALL_JRE_FAILURE}
    fi

    log_debug "${LINENO}" "JRE is installed."
    return ${INS_SUCCESS}
}

################################################################################
# Function: exec_sql_file
# Description: 执行sql文件
# Parameter:
#   input:
#   $1 -- 文件名数组
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
exec_sql_file()
{
    local -a fielArray=( "$@" )

    for sqlFile in "${fielArray[@]}" ; do
        # 判断文件是否存在
        if [ ! -f "${sqlFile}" ] ; then
            showLog_error "${LINENO}" "The ${sqlFile} file does not exist."
            return ${INS_DEBUG_ERROR}
        fi

        # 执行sql文件
        local sqlCMD="${g_dbHome}/${DB_SQL_CMD}"
        log_debug "${LINENO}" "Start to execute the ${sqlFile} file."
        ${sqlCMD} -U ${DB_USER} -W ${DB_PASSWORD} -d ${DB_NAME} -h localhost -p ${PG_PORT} -f "${sqlFile}" > /dev/null 2>&1
        if [ $? -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to execute the ${sqlFile} file."
            return ${INS_EXEC_SQL_FAILURE}
        fi
        log_debug "${LINENO}" "Succeed to execute the ${sqlFile} file."
    done

    return ${INS_SUCCESS}
}

################################################################################
# Function: db_operation
# Description: 数据库操作：创建OM数据库表，创建告警表，初始化数据
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
db_operation()
{
    log_debug "${LINENO}" "Start to operate the db."

    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
       log_info "${LINENO}" "UPGRADE, upgrade oms, no need to handle db_operation."
       return 0
    fi
    # 创建OM数据库表
    log_debug "${LINENO}" "Create the db tables of omm."
    local -a initDBSql=( "${g_controllerHome}/share/om/database/postgre/CreateOMTables.sql" )
    exec_sql_file "${initDBSql[@]}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to create the db table of omm."
        return ${retVal}
    fi

    # 创建告警表
    log_debug "${LINENO}" "Create the db tables of alarm."
    local -a alarmTableSql=( ${g_controllerHome}/share/om/database/postgre/table/*.sql )
    exec_sql_file "${alarmTableSql[@]}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to create the db tables for alarm."
        return ${retVal}
    fi

    # 初始化告警数据
    log_debug "${LINENO}" "Initialize the alarm data."
    local -a alarmDataSql=( ${g_controllerHome}/share/om/database/postgre/default/*/*/*.sql )
    exec_sql_file "${alarmDataSql[@]}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to initialize the alarm data."
        return ${retVal}
    fi
    local -a iamDataSql=( ${g_controllerHome}/share/om/database/postgre/default/*/*.sql )
    exec_sql_file "${iamDataSql[@]}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to initialize the iam data."
        return ${retVal}
    fi

    sqlDir=( "${g_controllerHome}/share/om/database/postgre" )

    # 创建datafarm数据库
    log_debug "${LINENO}" "Create the datafarm db."
    local -a createDataFarmDBSql=( "${sqlDir}/CreateDataFarmDB.sql" )
    exec_sql_file "${createDataFarmDBSql[@]}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to create the datafarm db."
        return ${retVal}
    fi

    log_debug "${LINENO}" "Succeed to operate the db."
    return ${INS_SUCCESS}
}

################################################################################
# Function: config_controller
# Description: 配置controller
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
config_controller()
{
    log_debug "${LINENO}" "Config the controller."

    # 判断要配置的文件是否存在
    local controllerConfigFile="${g_controllerHome}/etc/om/controller.properties"
    if [ ! -f "${controllerConfigFile}" ] ; then
        log_error "${LINENO}" "The ${controllerConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi

    local nodeSetupConfigFile="${g_controllerHome}/etc/om/nodesetup/nodesetup.properties"
    if [ ! -f "${nodeSetupConfigFile}" ] ; then
        log_error "${LINENO}" "The ${nodeSetupConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi

    # 定义分发路径使用的软连接变量
    local distributeHome="${g_bigdataHome}/${OM_SERVER_HOME_NAME}/om"

    # 进行参数配置
    sed -i "s:<OMM_SSH_USER>:${OMM_USER}:g" "${nodeSetupConfigFile}"
    sed -i "s:<OMM_USER>:${OMM_USER}:g" "${nodeSetupConfigFile}"
    sed -i "s:<OMM_USER_ID>:${g_ommid}:g" "${nodeSetupConfigFile}"
    sed -i "s:<OMM_USER_HOME>:${OMM_USER_HOME}:g" "${nodeSetupConfigFile}"
    sed -i "s:<OMM_USER_GROUP>:${OMM_USER_GROUP}:g" "${nodeSetupConfigFile}"
    sed -i "s:<FI_COMMON_GROUP>:${FI_COMMON_GROUP}:g" "${nodeSetupConfigFile}"
    sed -i "s:<FI_COMMON_GROUP_ID>:${FI_COMMON_GROUP_ID}:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_HOME>:${g_bigdataHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_HOME>:${g_bigdataHome}:g" "${controllerConfigFile}"
    sed -i "s:<CONTROLLER_HOME>:${g_controllerHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<CONTROLLER_HOME>:${g_controllerHome}:g" "${controllerConfigFile}"
    sed -i "s:<TOMCAT_HOME>:${g_tomcatHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<NODEAGENT_HOME>:${g_omAgentWorkspace}/${NODEAGENT_HOME_NAME}:g" "${nodeSetupConfigFile}"
    sed -i "s:<OM_AGENT_WORKSPACRE>:${g_omAgentWorkspace}:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_DATA_HOME>:${g_bigdataDataHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_LOG_HOME>:${g_logHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<USE_EXTERNAL_NTP_OMM>:${USE_EXTERNAL_NTP_OMM}:g" "${nodeSetupConfigFile}"
    sed -i "s:<controller-ip>:${g_omFlowIP}:g" "${nodeSetupConfigFile}"
    sed -i "s:<controller-ip>:${g_omFlowIP}:g" "${controllerConfigFile}"
    sed -i "s:<maintain-ip>:${g_wsFlowIP}:g" "${controllerConfigFile}"
    sed -i "s:<httpd-reverse-port>:${HTTPD_REVERSE_PORT}:g" "${controllerConfigFile}"
    sed -i "s:<httpd-forward-port>:${HTTPD_FORWARD_PORT}:g" "${controllerConfigFile}"
    sed -i "s:<httpd-hue-port>:${HTTPD_HUE_PORT}:g" "${controllerConfigFile}"
    sed -i "s:<pg-port>:${PG_PORT}:g" "${controllerConfigFile}"
    sed -i "s:<db-name>:${DB_NAME}:g" "${controllerConfigFile}"
    sed -i "s:<db-user>:${DB_USER}:g" "${controllerConfigFile}"
    sed -i "s:<jre_bin_file_name>:${JRE_INSTALL_FILE_NAME}:g" "${nodeSetupConfigFile}"
    sed -i "s:<jre_install_dir>:${JRE_INSTALL_DIR}:g" "${nodeSetupConfigFile}"
    sed -i "s:<controller-home>:${g_controllerHome}:g" "${nodeSetupConfigFile}"
    sed -i "s:<controller-home>:${g_controllerHome}:g" "${controllerConfigFile}"
    sed -i "s:<distribute-home>:${distributeHome}:g" "${controllerConfigFile}"
    sed -i "s:<OMS_RUN_PATH>:${g_omsHome}:g" "${controllerConfigFile}"
    sed -i "s:<sso_server_ip>:${g_ssoServerIp}:g" "${controllerConfigFile}"
    sed -i "s:<sso_server_port>:${g_ssoServerPort}:g" "${controllerConfigFile}"
    sed -i "s:<master_ldap_ip>:${g_masterldapIP}:g" "${controllerConfigFile}"
    sed -i "s:<master_ldap_port>:${g_masterldapPort}:g" "${controllerConfigFile}"
    sed -i "s:<DATABASE_USERNAME>:${g_encryptDatabaseUser}:g" "${controllerConfigFile}"
    sed -i "s:<DATABASE_PASSWORD>:${g_encryptDatabasePassword}:g" "${controllerConfigFile}"
    sed -i "s:<GET_HA_ROLE>:${g_omsHome}/workspace/ha/module/hacom/script/get_harole.sh:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_OM_SERVER_HOME>:${g_omServerWorkspace}:g" "${controllerConfigFile}"
    sed -i "s:<BIGDATA_COMMON>:${g_bigdataCommonWorkspace}:g" "${nodeSetupConfigFile}"
    sed -i "s:<BIGDATA_SUDO_HOME>:${BIGDATA_SUDO_HOME}:g" "${nodeSetupConfigFile}"

    # 判断要配置的文件是否存在

    local DBConfigFile="${g_controllerHome}/etc/om/DBConfig.xml"
    if [ ! -f "${DBConfigFile}" ] ; then
        log_error "${LINENO}" "The ${DBConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi
    sed -i "s:#CONTROLLER_HOME#:${g_controllerHome}:g" "${DBConfigFile}"

    local openssl_subConfigFile="${g_controllerHome}/security/cert/subcert/demoCA/openssl.cnf"
    if [ ! -f "${openssl_subConfigFile}" ] ; then
        log_error "${LINENO}" "The ${openssl_subConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi
    sed -i "s:#CONTROLLER_HOME#:${g_controllerHome}:g" "${openssl_subConfigFile}"
    
    local openssl_ConfigFile="${g_controllerHome}/security/cert/om/demoCA/openssl.cnf"
    if [ ! -f "${openssl_ConfigFile}" ] ; then
        log_error "${LINENO}" "The ${openssl_ConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi
    sed -i "s:#CONTROLLER_HOME#:${g_controllerHome}:g" "${openssl_ConfigFile}"
    
    local aosConfigFile="${g_controllerHome}/share/om/aos/config/application.properties"
    if [ ! -f "${aosConfigFile}" ] ; then
        log_error "${LINENO}" "The ${aosConfigFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi
    sed -i "s:#CONTROLLER_HOME#:${g_controllerHome}:g" "${aosConfigFile}"
    sed -i "s:#BIGDATA_COMMON#:${g_bigdataCommonWorkspace}:g" "${aosConfigFile}"
    
    # 判断要配置的文件是否存在
    local applicationPropertiesFile="${g_controllerHome}/etc/om/application.properties"
    if [ ! -f "${applicationPropertiesFile}" ] ; then
        log_error "${LINENO}" "The ${applicationPropertiesFile} file does not exist."
        return ${INS_FILE_NOT_EXIST}
    fi
    sed -i "s:#BIGDATA_COMMON#:${g_bigdataCommonWorkspace}:g" "${applicationPropertiesFile}"

    return ${INS_SUCCESS}
}

################################################################################
# Function: config_trust
# Description: 配置互信
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
config_trust()
{
    # 生成公钥和密钥对
    generate_rsa_key
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
	
    # 私钥加密
    encrypt_ssh_rsa
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    # 获取公钥
    local publicKey="$(cat ${HOME}/.ssh/id_rsa.pub)"

    # 把公钥添加到authorized_keys文件中
    add_trust_host "${publicKey}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    local nodeSetupConfigFile="${g_controllerHome}/etc/om/nodesetup/nodesetup.properties"
    sed -i "s:<controller-public-key>:${publicKey}:g" "${nodeSetupConfigFile}"


    local -a filesArray=("${HOME}/.ssh/id_rsa.pub" "${HOME}/.ssh/id_rsa" "${HOME}/.ssh/authorized_keys")
    # 设置互信文件的mtime的秒值
    # 为了保证主备OMS节点互信文件mtime不一样，才能够HA进行文件同步
    local mtime_sec_diff="00"

    # 如果本端IP小，设置文件mtime的秒值为00
    # 否则，设置文件mtime的秒值为30
    if [[ "$g_localIp" < "$g_peerIp" ]]; then
        mtime_sec_diff="00"
    else
        mtime_sec_diff="30"
    fi

    for afile in "${filesArray[@]}" ; do
        update_file_mtime "${afile}" "$mtime_sec_diff"
        if [ $? -ne 0 ] ; then
            log_error "${LINENO}" "Failed to update the mtime of ${afile}."
            return 0
        fi
    done


    return ${INS_SUCCESS}
}

encrypt_ssh_rsa()
{
    log_debug "${LINENO}" "Start to encrypt ssh rsa."
    # 私钥加密
    "${g_controllerHome}/inst/encrypt_ssh_rsa.sh" "$g_logFile" >> "$g_logFile" 2>&1
}

################################################################################
# Function: deploy_webapp
# Description: 部署tomcat webapp
# Parameter:
#   input:
#   $1 -- webappFile war包名（带全路径）
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
deploy_webapp()
{
    # 带全路径的webapp的war包名
    local webappFile="$1"

    if [ -z "${webappFile}" ] ; then
        log_error "${LINENO}" "The input parameter is incorrect."
        return ${INS_DEBUG_ERROR}
    fi

    # webapp 解压后的目录
    local webappDir="$(basename ${webappFile%.war})"
    webappDir="${g_tomcatHome}/webapps/${webappDir}"

    log_debug "${LINENO}" "Deploy the ${webappFile} webapp."

    # 解压war包
    exec_cmd rm -rf "${webappDir}"/*
    exec_cmd unzip -oq "${webappFile}" -d "${webappDir}"
    retVal=$?
    # 解压完成后，删除掉war包
    rm -f "${webappFile}"
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to decompress the ${webappFile} file."
        return ${INS_DECOMPRESS_FAILURE}
    fi

    chmod 500 "${webappDir}" -R
    chmod 700 "${webappDir}/WEB-INF" -R
    if [ -d "${webappDir}/temp" ] ; then
        chmod 700 "${webappDir}/temp" -R
    fi

    #将浮动ip配置到配置文件中供java-web使用
    sed -i "s:<WS_FLOAT_IP>:${g_wsFlowIP}:g" "${g_tomcatHome}/webapps/web/WEB-INF/classes/web.properties"

    return ${INS_SUCCESS}
}

################################################################################
# Function: install_tomcat
# Description: 安装tomcat
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
install_tomcat()
{
    log_debug "${LINENO}" "Install tomcat."

    # 判断TOMCAT安装包是否存在
    if [ ! -f "${g_tomcatInstallFileName}" ] ; then
        log_error "$LINENO" "ERROR: The ${g_tomcatInstallFileName} file does not exist."
        return 1
    fi

    # 解压tomcat安装包到bigdata目录
    exec_cmd tar -xzf "${g_tomcatInstallFileName}" -C "${g_omServerWorkspace}"
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to decompress the ${g_tomcatInstallFileName} file."
        return ${INS_DECOMPRESS_FAILURE}
    fi

    # 生成OM安全证书
    source "${HOME}/.profile" > /dev/null 2>&1

    # 配置tomcat
    sed -i "s/Connector port=\"[0-9]*\"/& address=\"${g_wsFlowIP}\"/g" "${g_tomcatHome}/conf/server.xml"
    sed -i "s:#KEYSTOREFILE-HOME#:${g_tomcatHome}/conf/security/tomcat_om.keystore:g" "${g_tomcatHome}/conf/server.xml"
    sed -i "s:#BIGDATA-TOMCAT-LOG-DIR#:${g_tomcatLogHome}:g" "${g_tomcatHome}/conf/server.xml"
    sed -i "s:#BIGDATA-TOMCAT-LOG-DIR#:${g_tomcatLogHome}:g" "${g_tomcatHome}/conf/logging.properties"
    sed -i "s:#BIGDATA-TOMCAT-LOG-DIR#:${g_tomcatLogHome}:g" "${g_tomcatHome}/bin/catalina.sh"

    grep 'export JAVA_OPTS="$JAVA_OPTS -XX:PermSize=256M -XX:MaxPermSize=512M -XX:MaxHeapSize=4G"' ${g_tomcatHome}/bin/startup.sh > /dev/null
    if [ $? -ne 0 ]; then
        sed -i '/EXECUTABLE=catalina.sh/a\export JAVA_OPTS="$JAVA_OPTS -XX:MetaspaceSize=256M -XX:MaxMetaspaceSize=512M -Xss1M -XX:MaxHeapSize=4G -Djdk.tls.ephemeralDHKeySize=2048 -XX:ErrorFile=/var/log/Bigdata/tomcat/hs_error%p.log -XX:OnOutOfMemoryError=\\"kill -9 %p\\""' ${g_tomcatHome}/bin/startup.sh
    fi

    # 增加tomcat环境变量到环境变量文件(.controller_profile)
    local profileFile="${g_omServerWorkspace}/.controller_profile"
    echo "export OM_TOMCAT_HOME=${g_tomcatHome}" >> "${profileFile}" || return ${INS_WRITE_FILE_FAILURE}
    source "${profileFile}" > /dev/null 2>&1
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to source the ${profileFile} file."
        return ${retVal}
    fi

    # 改变tomcat安装目录以下所有文件的权限
    chmod 700 ${g_tomcatHome} -R
    find "${g_tomcatHome}/conf" -type f -name "*" | xargs -I{} chmod 600 "{}"

    [ -L "${g_omServerWorkspace}/tomcat" ] &&  rm -rf "${g_omServerWorkspace}/tomcat"
    ln -sf "${g_tomcatHome}" "${g_omServerWorkspace}/tomcat"
    chmod 700 "${g_omServerWorkspace}/tomcat"
    #  部署tomcat webapp
    local webFile="${g_controllerHome}/share/om/web/web.war"
    local casFile="${g_controllerHome}/packaged-distributables/${g_packageDistributeSubdir}/cas.war"
    local tmpFile=""
    for tmpFile in "${webFile}" "${casFile}" ; do
        # 判断文件存在性
        if [ ! -f "${tmpFile}" ] ; then
            showLog_error "${LINENO}" "The ${tmpFile} file does not exist."
            return ${INS_FILE_NOT_EXIST}
        fi

        # 部署webapp
        deploy_webapp "${tmpFile}"
        retVal=$?
        if [ ${retVal} -ne 0 ] ; then
            showLog_error "${LINENO}" "Failed to deploy the ${tmpFile} webapp."
            return ${retVal}
        fi
    done

    local placeholder="#OM_LOG_HOME#"
    local placedatahome="#BIGDATA_DATA_HOME#"
    local placeinstallhome="#BIGDATA_HOME#"
    local placecommonhome="#BIGDATA_COMMON#"
    local placecontrollerhome="#CONTROLLER_HOME#"
    local -a targetFileList=("${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.DEFAULT"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.FATAL"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.INFO"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.DEBUG"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.WARN"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/log4j.properties.ERROR"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/application.properties"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/auth.conf"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/DBConfig.xml"
                            "${g_tomcatHome}/webapps/web/WEB-INF/classes/config/file_management.properties"
                            "${g_tomcatHome}/webapps/cas/WEB-INF/classes/log4j.xml"
                            "${g_tomcatHome}/webapps/cas/WEB-INF/classes/config/log4j.properties"
                            "${g_tomcatHome}/webapps/cas/WEB-INF/classes/config/application.properties"
                            )

    change_file_placeholder "${placeholder}" "${g_logHome}" "${targetFileList[*]}"
    change_file_placeholder "${placedatahome}" "${g_bigdataDataHome}" "${targetFileList[*]}"
    change_file_placeholder "${placeinstallhome}" "${g_bigdataHome}" "${targetFileList[*]}"
    change_file_placeholder "${placecommonhome}" "${g_bigdataCommonWorkspace}" "${targetFileList[*]}"
    change_file_placeholder "${placecontrollerhome}" "${g_controllerHome}" "${targetFileList[*]}"
    
    # 创建tomcat日志路径
    mkdir -p ${g_tomcatLogHome}
    chmod 700 ${g_tomcatLogHome}

    # 安装完成后删除TOMCAT安装包（函数开始已判断安装包是否存在）
    rm -f "${g_tomcatInstallFileName}"
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to cleanup the ${g_tomcatInstallFileName} file."
        return 1
    fi
    return ${INS_SUCCESS}
}

################################################################################
# Function: mk_sha256_file
# Description: 为指定目录下的所有文件创建sha256文件,用于下载校验
# Parameter:
#   input:
#   $1 -- targetPath 要生成sha256文件的目标目录
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
mk_sha256_file()
{
    log_debug "${LINENO}" "Create sha256 file."

    local targetPath="$1"
    if [ -z "${targetPath}" ] ; then
        log_error "${LINENO}" "The input paramerter is empty."
        return ${INS_DEBUG_ERROR}
    fi

    # 判断传入的变量是否是目录并且是否存在
    if [ ! -d "${targetPath}" ] ; then
        showLog_error "${LINENO}" "The ${targetPath} path does not exist."
        return ${INS_DEBUG_ERROR}
    fi

    # 为targetPath目录下的所有文件生成sha256文件, 生成文件中不能带全路径否则拷贝到其他地方不能正常校验
    local sha256FileName=""
   
    cd ${targetPath}
    for fileName in $(ls ${targetPath} | grep -v "\.sha256$") ; do
    
        sha256FileName="${fileName}.sha256"

        # fileName文件如果是目录，则跳过
        if [ -d "${fileName}" ] ; then
            continue
        else
            sha256sum "${fileName}" > "${sha256FileName}"
            chmod 600 "${sha256FileName}"
        fi
    done
    cd - >/dev/null 2>&1
     
    return ${INS_SUCCESS}
}

################################################################################
# Function: install_base_pack
# Description: 生成组件配置目录：把组件的配置文件放到生成的配置目录里
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
install_base_pack()
{
    log_debug "${LINENO}" "Create the component config path."

    local basePackName="$(${g_controllerHome}/sbin/getPackInfo.sh packName ${g_controllerHome}/etc/om/${PACK_DEFINITION_FILE})" || return 1
    log_debug "${LINENO}" "basePackName=${basePackName}"
    
    local basePackComponentsPath="${g_componentsAdapterHome}/${basePackName}"
    mkdir -p "${basePackComponentsPath}" || return 1
  
    #将etc/components目录下的globalxml文件拷贝到stackConfiDir目录下
    exec_cmd /bin/cp "${g_controllerHome}/etc/om/${PACK_DEFINITION_FILE}" "${basePackComponentsPath}/" || return ${INS_COPY_FILE_FAILURE}
  
    local packagePath="${g_bigdataHome}/packaged-distributables/${g_packageDistributeSubdir}/${basePackName}"
    log_debug "${LINENO}" "exec cmd distributeAdapterFiles.sh ${packagePath} ${basePackComponentsPath} ${basePackName}"
    exec_cmd ${g_controllerHome}/sbin/distributeAdapterFiles.sh "${packagePath}" "${basePackComponentsPath}" "${basePackName}" >> "${g_logFile}" 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Distribute base components adapter files error."
        return 1
    fi
   
    #如果升级部署，不写数据库，在升级适配阶段适配数据库
    if [ "$g_operatorType" != "$UPGRADE_OPERATOR" ]; then
        local displayName="$(${g_controllerHome}/sbin/getPackInfo.sh packDisplayName ${g_controllerHome}/etc/om/${PACK_DEFINITION_FILE})" || return 1
        local version="$(${g_controllerHome}/sbin/getPackInfo.sh packVersion ${g_controllerHome}/etc/om/${PACK_DEFINITION_FILE})" || return 1
        local packageName="$(${g_controllerHome}/sbin/getPackInfo.sh packName ${g_controllerHome}/etc/om/${PACK_DEFINITION_FILE})" || return 1
        
        local sqlStatment="$(echo ${INSERT_PACK_TEMPLATE} | sed "s/<PACK_NAME>/${basePackName}/g" | sed "s/<PACK_DISPLAY_NAME>/${displayName}/g" | sed "s/<PACK_VERSION>/${version}/g" | sed "s/<PACKAGE_NAME>/${packageName}.tar.gz/g" | sed "s/<STATUS>/activated/g")"
                      
        log_info "${LINENO}" "Insert base pack record with sql statment: $sqlStatment."
        
        local execSqlTool="${g_controllerHome}/tools/execSql.sh"
        
        "${execSqlTool}" -s "${sqlStatment}"
        if [ $? -ne 0 ]; then
            log_error $LINENO "Insert data into database failed."
            return ${INS_INSERT_DATABASE_FAILED}
        fi
    fi

    return ${INS_SUCCESS}
}



################################################################################
# Function: install_httpd
# Description:安装httpd
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
install_httpd()
{
    log_debug "${LINENO}" "Start to install httpd."
    local httpdHome="$g_curPath"
    local casCertFile="$g_tomcatHome/conf/security/tomcat.crt"
    $httpdHome/installhttpd.sh "$g_wsFlowIP" "$HTTPD_REVERSE_PORT" "${g_omFlowIP}"  "$HTTPD_FORWARD_PORT"  "$g_ssoServerIp" "$g_ssoServerPort" "$casCertFile" "$g_omServerWorkspace" "$g_logHome" "$HTTPD_HUE_PORT" >> "${g_logFile}" 2>&1
    
    
}

################################################################################
# Function: change_cert_pass
# Description:生成密码文件
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
change_cert_pass()
{
    EN_CERT_PASSWORD="${1}"
    echo "password=${EN_CERT_PASSWORD}" > "${g_controllerHome}/security/cert/root/password.property"
    chmod 600 "${g_controllerHome}/security/cert/root/password.property"
    
    cp "${g_controllerHome}/security/cert/root/password.property" "${g_controllerHome}/security/cert/subcert/certFile/password.property"
    chmod 600 "${g_controllerHome}/security/cert/subcert/certFile/password.property"
    
    echo "The password is Changed" > "${g_controllerHome}/security/cert/subcert/certFile/password.changed"
    chmod 600 "${g_controllerHome}/security/cert/subcert/certFile/password.changed"
}

################################################################################
# Function: migrate_cert
# Description: 升级时继承根证书和二级证书
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
migrate_cert()
{   
    #兼容C70之前的版本OMS安装路径
    local oldVersionWorkspace="$g_bigdataHome/om-0.0.1"
    if [ ! -d "${oldVersionWorkspace}" ]; then
        oldVersionWorkspace="${g_bigdataHome}/${OM_SERVER_HOME_NAME}/om"
    fi
    
    /bin/cp -rpf "${oldVersionWorkspace}/security/cert/root/"* "${g_controllerHome}/security/cert/root/"
    if [ $? -ne 0 ] ;then
        showLog_error "${LINENO}" "Failed to cp -rpf ${oldVersionWorkspace}/security/cert/root/* ${g_controllerHome}/security/cert/root/."
        return 1
    fi
        
    rm -rf "${g_controllerHome}/security/cert/subcert/certFile/"*
    cp -rpf "${oldVersionWorkspace}/security/cert/subcert/certFile/"* "${g_controllerHome}/security/cert/subcert/certFile/"
    if [ $? -ne 0 ] ;then
        showLog_error "${LINENO}" "Failed to cp -rpf ${oldVersionWorkspace}/security/cert/subcert/certFile/* ${g_controllerHome}/security/cert/subcert/certFile/."
        return 1 
    fi
}

################################################################################
# Function: generate_cacertfiles
# Description: 依据ca.crt生成中间证书
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
generate_cacertfiles()
{
    local cert_file="$1"
    local cert_tmp_path="$2"
    local cert_num=0
    local cert_file_names=""
    
    if [ ! -d "${cert_tmp_path}" ];then
        mkdir "${cert_tmp_path}"
    fi
    
    cert_num=$(cat "${cert_file}" | grep "\-\-\-\-\-END CERTIFICATE\-\-\-\-\-" | wc -l)
    if [ ${cert_num} -le 0 ];then
        log_and_show "ERROR" "${LINENO}" "invalid cert file num ${cert_num}." 
        return 1
    fi
    
    local i=1
    for((i=1;i<=${cert_num};i++));
    do
        if [ -f "${cert_tmp_path}/ca_level$i.crt" ];then
            cat /dev/null > "${cert_tmp_path}/ca_level$i.crt"
        fi
    done
    
    local j=1
    while read line
    do
        echo "${line}" >> "${cert_tmp_path}/ca_level$j.crt"
        if [ $? -ne 0 ];then
            log_and_show "ERROR" "${LINENO}" "create ${cert_tmp_path}/ca_level$j.crt failed." 
            return 1
        fi
        
        echo "${line}" | grep "\-\-\-\-\-END CERTIFICATE\-\-\-\-\-" > /dev/null
        if [ $? -eq 0 ];then
            if [ $j -eq 1 ];then
                cert_file_names="ca_level$j.crt"
            else
                cert_file_names="${cert_file_names} ca_level$j.crt"
            fi
            j=$[ $j + 1 ]
        fi
    done < "${cert_file}"
    
    echo "${cert_file_names}"
    return 0
    
}

################################################################################
# Function: main
# Description: 主函数
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    log_debug "${LINENO}" "Start to install controller."
    umask 077

    # 如果不是omm用户，则退出
    if [ $(id -u) -ne ${g_ommid} ] ; then
        log_error "${LINENO}"  "The user is not the omm user."
        return ${INS_DEBUG_ERROR}
    fi

    # 检查输入参数
    check_input_param "$@"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    # 初始化全局变量
    init_global_param
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to initialize parameters."
        return ${retVal}
    fi

    #获取数据库用户omm的密码
    source "${g_omServerWorkspace}/.controller_profile"
    source "${g_bigdataCommonWorkspace}/.common_profile"
    
    #环境变量上下文中已经配置好分发子目录
    g_packageDistributeSubdir="${OS_TYPE}-${CPU_ARCH}"
    
    local wcc=$(find ${g_controllerHome}/share/om/controller/lib/ -name wc2frm*.jar)
    local commons=$(find ${g_controllerHome}/share/om/controller/lib -maxdepth 1 -name commons-codec*.jar)
    local log4j=$(find ${g_controllerHome}/share/om/controller/lib -maxdepth 1 -name log4j*.jar)
    local classpath="$wcc:$commons:$log4j:${g_controllerHome}/tools/main.jar"
    local javaArgs="-Dbeetle.application.home.path=${WCC_PROFILE_DIR}"
    local className="com.omm.encrypt.DecryptCommandWithoutParam"
    local javaCommand="${JAVA_HOME}/bin/java -cp ${classpath} ${javaArgs} ${className}"
    
    if [ -z "$JAVA_HOME" ]; then
        showLog_error "${LINENO}" "null env JAVA_HOME!!!"
        return 1
    fi
    
    local getCipherPasswdTool="${g_omsHome}/tools/getdbNameOrPasswd"
    if [ ! -f ${getCipherPasswdTool} ]; then
       showLog_error "${LINENO}" "${getCipherPasswdTool} is not exist."
       return 1
    fi

    local CipherPasswd=$(${getCipherPasswdTool} 2 | awk '{print $1}')
    if [ -z "${CipherPasswd}" ];then
        showLog_error "${LINENO}" "OMS's DB CipherPasswd is empty.";
        return 1;
    fi
    
    DB_PASSWORD=$(${javaCommand} <<EOF
strSrc:$CipherPasswd
encType:AES256_CBC_IGNORE_KEY
EOF
)
    
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to get omm db password."
        return ${retVal}
    fi

    #加密数据库用户名和密码
    g_encryptDatabaseUser=$($g_encrypt_tool <<EOF
--strSrc ${OMM_USER}
--encType AES256_CBC_IGNORE_KEY
EOF
)
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to encrypt omm db user ."
        return ${retVal}
    fi
    g_encryptDatabasePassword=$($g_encrypt_tool <<EOF
--strSrc ${DB_PASSWORD}
--encType AES256_CBC_IGNORE_KEY
EOF
)
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to encrypt omm db password."
        return ${retVal}
    fi

    # 数据库操作：创建OM数据库表，创建告警表，初始化告警数据，
    db_operation
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    # 修改证书密码文件
    source "${g_controllerHome}/security/cert/root/password.property"
    CERT_PASSWORD=$(${javaCommand} <<EOF
strSrc:${password}
encType:AES256
EOF
)
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to get cert password."
        return ${retVal}
    fi

    EN_CERT_PASSWORD=$($g_encrypt_tool <<EOF
--strSrc ${CERT_PASSWORD}
--encType AES256_CBC_IGNORE_KEY
EOF
)
    retVal=$?
    if [ ${retVal} -ne 0 ]; then
        showLog_error "${LINENO}" "Failed to encrypt omm db user ."
        return ${retVal}
    fi
    
    change_cert_pass ${EN_CERT_PASSWORD}
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi
    
    # 配置controller
    config_controller
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        return ${retVal}
    fi

    # 配置互信
    config_trust
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to config trust."
        return ${retVal}
    fi

    # 安装tomcat
    install_tomcat
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install tomcat."
        return ${retVal}
    fi
    
    ##################如果是升级需要把根证书和二级证书还原###########################
    if [ "$g_operatorType" = "$UPGRADE_OPERATOR" ]; then
        migrate_cert || return $?
    fi 

    if [ -f "$CONTROLLER_HOME/etc/om/jdksecurity.properties" ] ; then
            source "$CONTROLLER_HOME/etc/om/jdksecurity.properties"
            jreStorePass=$(${javaCommand} <<EOF
strSrc:${jreStorePass}
encType:AES256
EOF
)
            retVal=$?
            if [ ${retVal} -ne 0 ]; then
                showLog_error "${LINENO}" "Failed to get jreStorePass password."
                return ${retVal}
            fi

            jreStorePass=$($g_encrypt_tool <<EOF
--strSrc ${jreStorePass}
--encType AES256_CBC_IGNORE_KEY
EOF
)
            rm -f "$CONTROLLER_HOME/etc/om/jdksecurity.properties"
            echo "jreStorePass=${jreStorePass}" > "$CONTROLLER_HOME/etc/om/security.properties"
    fi
 
    
    # 将tomcat.cert导入到trustCacerts
    log_info "${LINENO}" "Installing web security certificate."
    
    # 生成ca证书中中间证书，用于证书链导入
    local meidacert_path="${CONTROLLER_HOME}/security/mediacert"
    local ca_cert="${CONTROLLER_HOME}/security/cert/root/ca.crt"
    local media_cert_names=""
    media_cert_names=$(generate_cacertfiles  "${ca_cert}" "${meidacert_path}")
    if [ 0 -ne $? ]; then
        showLog_error "${LINENO}" "Failed to generate cert files according to ${ca_cert}" 
        return 1
    fi
    
    # 生成tomcat证书
    "${gen_om_cert_path}/${gen_om_cert_filename}" "${g_wsFlowIP}" "${g_controllerProfile}" "${g_commonProfile}" "${media_cert_names}" > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install om security certificate."
        return 1
    fi

    
    # 生成cas证书
    "${gen_cas_cert_path}/${gen_cas_cert_filename}" "${g_wsFlowIP}" "${g_controllerProfile}" "${g_commonProfile}" "${media_cert_names}" > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install cas security certificate."
        return 1
    fi

    cp -arpf ${g_tomcatHome}/conf/security/tomcat.* ${g_controllerHome}/packaged-distributables/${g_packageDistributeSubdir}/ >> "${g_logFile}" 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to config tomcat."
        return ${INS_CONFIG_TOMCAT_FAILURE}
    fi

    # 为${g_controllerHome}/packaged-distributables/${g_packageDistributeSubdir}目录下的所有文件创建sha256文件，用于agent下载校验
    mk_sha256_file "${g_controllerHome}/packaged-distributables/${g_packageDistributeSubdir}"
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to create sha256 files."
        return ${retVal}
    fi

    #tomcat.crt 在安装集群-安装agent中用到, intallnodeagent.sh 从${g_controllerHome}/packaged-distributables/ 下读取tomcat.crt,  这里拷贝一份
    cp -arpf ${g_controllerHome}/packaged-distributables/${g_packageDistributeSubdir}/tomcat.* ${g_controllerHome}/packaged-distributables/ >> "${g_logFile}" 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to copy tomcat.crt"
        return ${INS_CONFIG_TOMCAT_FAILURE}
    fi

    install_httpd
    retVal=$?
    if [ ${retVal} -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install httpd."
        return ${retVal}
    fi

    # 生成httpd证书
    "${gen_httpd_cert_path}/${gen_httpd_cert_filename}" "${g_wsFlowIP}" "${g_controllerProfile}" "${g_commonProfile}" > /dev/null 2>&1
    if [ $? -ne 0 ] ; then
        showLog_error "${LINENO}" "Failed to install httpd security certificate."
        return 1
    fi

    chmod 600 ${CONTROLLER_HOME}/security/cert/om/demoCA/newcerts/*

    # 生成审计日志目录
    mkdir -p "${g_bigdataDataHome}/Manager/audit"
    chmod 700 "${g_bigdataDataHome}/Manager/audit"
    
    return ${INS_SUCCESS}
}

# ---------------------------------------------------------------------------- #
#                                 获取当前路径                                 #
# ---------------------------------------------------------------------------- #
get_cur_path
cd "${g_curPath}"

# ---------------------------------------------------------------------------- #
#                                 初始化日志文件                               #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/func/log.sh."; exit 1;}

init_log || exit 1

# ---------------------------------------------------------------------------- #
#                                 导入头文件                                   #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/func/constants.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/constants.sh."; exit 1; }
. "${g_curPath}/func/functions.sh" || { log_error "${LINENO}" "Failed to load ${g_curPath}/func/functions.sh."; exit 1; }

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #
main "$@"
retVal=$?
if [ ${retVal} -eq 0 ];then
    log_info "${LINENO}" "The controller installation is successful."
else
    log_error "${LINENO}" "Failed to install the controller."
fi
exit ${retVal}
