#!/bin/bash

# root 用户安装：不做处理
# omm 用户安装：做完 preset 后，修改 backupFile.tar 用户为 omm
declare g_curPath=""

getCurPath()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - > /dev/null
}

# 安装oms前需要保证存在备份数据,用于安装时替换与加解密有关文件
unzipBackupFile()
{
    if [ -f /tmp/backupDir.tar.gz ];then
        showEcho "${LINENO}" "INFO: Decompress the backup package /tmp/backupDir.tar.gz"
        rm -rf /tmp/backupDir
        tar xzf /tmp/backupDir.tar.gz -C /tmp >/dev/null 2>&1
        tar xf /tmp/backupDir/backupFile.tar -C /tmp/backupDir >/dev/null 2>&1
        chmod -R 750 /tmp/backupDir
        egrep "^omm:" /etc/passwd >/dev/null 2>&1
        if [ $? -eq 0 ];then
            chown omm:wheel /tmp/backupDir.tar.gz
            chown omm:wheel -R /tmp/backupDir
        fi
    else
        showEcho "${LINENO}" "ERROR: /tmp/backupDir.tar.gz does not exist"
        exit 1
    fi
    showEcho "${LINENO}" "INFO: Decompress the backup package is complete."
}

replaceInstallFile()
{
    find ${g_curPath} -type f | xargs  dos2unix {}  > /dev/null 2>&1

    local rootDir="${1}/FusionInsight_Manager"
    if [ ! -d ${rootDir} ];then
        showEcho "${LINENO}" "ERROR: intall package ${rootDir} does not exist."
        exit 1
    fi
    local installPackUserPerssion=$(ls -ld ${rootDir} | awk '{print $3}')
    showEcho "${LINENO}" "INFO: The owner of the installation package is ${installPackUserPerssion}"
    showEcho "${LINENO}" "INFO: Replacing the Installation Package File."
    cd ${1}/FusionInsight_Manager/software/om/package
    cp om.tar.gz om.bak
    tar xzf om.tar.gz
    cp -f ${g_curPath}/install.sh ${1}/FusionInsight_Manager/software/om/package/om/inst/install.sh
    if [ ${installPackUserPerssion}X == "omm"X ];then
        chown omm:wheel ${1}/FusionInsight_Manager/software/om/package/om/inst/install.sh
    fi
    tar czf om.tar.gz om
    if [ ${installPackUserPerssion}X == "omm"X ];then
        chown omm:wheel ${1}/FusionInsight_Manager/software/om/package/om.tar.gz
    fi
    rm -rf ${1}/FusionInsight_Manager/software/om/package/om
    rm -rf ${1}/FusionInsight_Manager/software/om/package/om.bak
    showEcho "${LINENO}" "INFO: Replacing the installation package file ends."
}

main()
{
    # 不再重新生成密钥，从备份文件中取
    unzipBackupFile
    # 替换安装文件
    replaceInstallFile $@
}

# 判断是否为 root 用户
if [ "$(whoami)" != "root" ]; then
    echo "Only root user can install oms."
    exit 1
fi

getCurPath

. "${g_curPath}/../../func/log.sh" || { echo "Failed to load log.sh."; exit 1; }

# $1 : 解压 FusionInsight_Manager 安装包所在目录，如：/opt
main $@