#!/bin/bash

####################################################################
## @Usage getCurPath 获取脚本所在的当前路径，并取脚本名
####################################################################
getCurPath() {
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="$(pwd)"
    cd - >/dev/null
}

initLogAndTmpDir() {
    if [ ! -f ${G_LOG_FILE} ]; then
        touch ${G_LOG_FILE}
        if [ $? -ne 0 ]; then
            echo "Failed to create log file: ${G_LOG_FILE}"
            exit 1
        fi
    fi
}

initDBInfo() {
    # 获取 DB 端口号及密码
    [ -f "$OMS_RUN_PATH/product_adapter.properties" ] && source $OMS_RUN_PATH/product_adapter.properties
    DBCOMMAND="gsql -U ommdba -d omm -p $OMS_GAUSS_PORT"
    DBPASSWORD=$(getDBPasswd)
    if [ -z "${DBPASSWORD}" ]; then
        showLog_error "${LINENO}" "get db password failed."
        exit 1
    fi
}

restart_agent() {
    local stopAgent="$BIGDATA_HOME/om-agent/nodeagent/bin/stop-agent.sh"
    local startAgent="$BIGDATA_HOME/om-agent/nodeagent/bin/start-agent.sh"

    COMMAND="select BUSINESSIP from public.om_nodes;"
    BUSINESSIPS=$(${DBCOMMAND} -W ${DBPASSWORD} -c "$COMMAND" | grep -vwE "rows|row" | sed 's/^[[:space:]]*//g;s/[[:space:]]*$//g;1,2d;/^$/d')

    touch ${g_curPath}/tmp.file
    for nodeIp in ${BUSINESSIPS}; do
        {
            showLog_info "${LINENO}" "Begin restart agents ${nodeIp}."
            ssh ${nodeIp} "source $HOME/.profile; $stopAgent; ${startAgent}" >>"${G_LOG_FILE}" 2>&1
            if [ $? -ne 0 ]; then
                rm -f ${g_curPath}/tmp.file
                showLog_error "${LINENO}" "Failed to restart agent :${nodeIp}"
            fi
        } &
    done

    wait
    if [ -f ${g_curPath}/tmp.file ]; then
        showLog_info "${LINENO}" "Success to restart agents."
        rm -f ${g_curPath}/tmp.file
        return 0
    fi

    showLog_error "${LINENO}" "Failed to restart agents."
    return 1
}

main() {
    showLog_info "${LINENO}" "Start executing restart agents."

    initLogAndTmpDir

    initDBInfo || return 1

    restart_agent || return 1
}

# 判断是否为 omm 用户
if [ "$(whoami)" != "omm" ]; then
    echo "Only omm user can export data."
    exit 1
fi

getCurPath
# ---------------------------------------------------------------------------- #
#                                 导入头文件                                     #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/../func/functions.sh" || {
    echo "Failed to load functions.sh."
    exit 1
}
. "${g_curPath}/../func/log.sh" || {
    echo "Failed to load log.sh."
    exit 1
}
. "${g_curPath}/../func/constants.sh" || {
    echo "Failed to load constants.sh."
    exit 1
}

main $@
