#!/bin/bash
declare g_rootPath=""
declare g_bigdataHome=""
declare PATCH_VERSION="6.5.1.6"
declare BACKUP_HOME="/opt/os_optimization_tool_backup_${PATCH_VERSION}"
declare WORK_HOME="/opt/os_optimization_tool_${PATCH_VERSION}"
declare agent_tar="agent-0.0.1.tar.gz"

###################################################
#
# ȡ·
#
###################################################
get_path()
{
    local path=""

    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi

    g_rootPath=$(dirname "$path")
    echo "g_rootPath=$g_rootPath."
    if [ -z "$g_rootPath" ]; then
        echo "Init parameter g_rootPath failed."
        return 1
    fi

    return 0
}

backup_sudo()
{ 
    log_and_show "INFO" "${LINENO}" "Enter backup sudoExecute file."
    
    if ! [ -d "${BACKUP_HOME}" ];then
        mkdir -p "${BACKUP_HOME}"
    elif [ -f  "${BACKUP_HOME}/sudoExecute.sh" ];then 
        log_and_show "INFO" "${LINENO}" "${BACKUP_HOME}/sudoExecute.sh exist, skip backup file."
        return 0
    fi
    
    if [ -f "/var/lib/sudo/Bigdata/sudo/runtime/sudoExecute.sh" ];then
        cp -rf "/var/lib/sudo/Bigdata/sudo/runtime/sudoExecute.sh" "${BACKUP_HOME}"
    elif [ -f "${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/sudoExecute.sh" ];then
        cp -fr "${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/sudoExecute.sh" "${BACKUP_HOME}"
        return $?
    elif [ -f "${g_bigdataHome}/om-server/om/sbin/sudo/runtime/sudoExecute.sh" ];then
        cp -fr "${g_bigdataHome}/om-server/om/sbin/sudo/runtime/sudoExecute.sh" "${BACKUP_HOME}"
        return $?
    fi
    
    log_and_show "INFO" "${LINENO}" "Leave backup sudoExecute file."
    
    return 0
}

replace_file()
{
    log_and_show "INFO" "${LINENO}" "Enter replace file."
    
    if [ -f "/var/lib/sudo/Bigdata/sudo/runtime/sudoExecute.sh" ];then
       chmod 700 "/var/lib/sudo/Bigdata/sudo/runtime/sudoExecute.sh"
       cp -fr "$g_rootPath/common/sudoExecute.sh" "/var/lib/sudo/Bigdata/sudo/runtime/"
       if [ $? -ne 0 ];then
           log_and_show "ERROR" "${LINENO}" "copy $g_rootPath/common/sudoExecute.sh to /var/lib/sudo/Bigdata/sudo/runtime/ failed."
           return 1
       fi
    fi

    if [ -f "${g_bigdataHome}/om-server/om/sbin/sudo/runtime/sudoExecute.sh" ];then
       chmod 700 "${g_bigdataHome}/om-server/om/sbin/sudo/runtime/sudoExecute.sh"
       cp -fr "$g_rootPath/common/sudoExecute.sh" "${g_bigdataHome}/om-server/om/sbin/sudo/runtime/"
       if [ $? -ne 0 ];then
           log_and_show "ERROR" "${LINENO}" "copy $g_rootPath/common/sudoExecute.sh to ${g_bigdataHome}/om-server/om/sbin/sudo/runtime/ failed."
           return 1
       fi
    fi

    if [ -f "${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/sudoExecute.sh" ];then
        chmod 700 "${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/sudoExecute.sh"
        cp -fr "$g_rootPath/common/sudoExecute.sh" "${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/"
        if [ $? -ne 0 ];then
           log_and_show "ERROR" "${LINENO}" "copy $g_rootPath/common/sudoExecute.sh to ${g_bigdataHome}/om-agent/nodeagent/bin/sudo/runtime/ failed."
           return 1
        fi
    fi
    
    log_and_show "INFO" "${LINENO}" "Leave replace file."
}

backup_agent_pkg()
{
    local pkg=$1
    local pkg_dir=$(dirname $pkg)
    local pkg_name=$(basename $pkg)
    local backup_dir="${BACKUP_HOME}/${pkg_dir}"
    local backup_file="${backup_dir}/${pkg_name}"
    
    log_and_show "INFO" "${LINENO}" "begin to backup $pkg."

    if [ ! -d "${backup_dir}" ]; then
        log_and_show "INFO" "${LINENO}" "mkdir $backup_dir."
        mkdir -p "${BACKUP_HOME}/${pkg_dir}" || { log_and_show "ERROR" "${LINENO}" "mkdir failed.";return 1;}
    fi
    
    if [ ! -f "${backup_file}" ];then
        log_and_show "INFO" "${LINENO}" "backup file $pkg to $backup_file."
        cp -af $pkg* "${BACKUP_HOME}/${pkg_dir}" || { log_and_show "ERROR" "${LINENO}" "backup $pkg to $backup_file failed.";return 1;}
    else
        log_and_show "INFO" "${LINENO}" "$backup_file already exists,skip."
    fi

    log_and_show "INFO" "${LINENO}" "succeed to backup $pkg."

    return 0
}

replace_script()
{
    local pkg=$1
    local pkg_dir=$(dirname $pkg)
    local tarDir="${WORK_HOME}/tar_tmp_agent"
    local sudo_script="sudoExecute.sh"
    local decompress_script="decompressPatch.sh"
    local installntp_script="installntp.sh"

    rm -rf $tarDir
    mkdir -p $tarDir
    tar -zxf $pkg -C $tarDir
    if [ $? -ne 0 ]; then 
        log_and_show "ERROR" "$LINENO" "Failed to unzip agent.tar,maybe the remain space of /opt is not enough."
        return 1
    fi
    
    #replace sudo
    log_and_show "INFO" "${LINENO}" "begin to replace $sudo_script in $pkg."
    local tmpSudo=$(find "${tarDir}" -name "${sudo_script}")
    cp -af "${g_rootPath}/common/${sudo_script}" "${tmpSudo}"
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Failed to replace ${g_rootPath}/common/${sudo_script} to $tmpSudo."
        return 1
    fi
    chmod 500 "${tmpSudo}"
    
    log_and_show "INFO" "${LINENO}" "succeed to replace $sudo_script in $pkg."

    #replace decompressPatch.sh
    if [ -f "${g_rootPath}/common/${decompress_script}" ] ; then 
        log_and_show "INFO" "${LINENO}" "begin to replace $decompress_script in $pkg."
        local decomscript=$(find "${tarDir}" -name "${decompress_script}")
        cp -af "${g_rootPath}/common/${decompress_script}" "${decomscript}"
        if [ $? -ne 0 ]; then
            log_and_show "ERROR" "$LINENO" "Failed to replace ${g_rootPath}/common/${decompress_script} to $decomscript."
            return 1
        fi
        log_and_show "INFO" "${LINENO}" "succeed to replace $decompress_script in $pkg."
    fi

    #replace installntp.sh
    if [ -f "${g_rootPath}/common/${installntp_script}" ] ; then 
        log_and_show "INFO" "${LINENO}" "begin to replace $installntp_script in $pkg."
        local ntpscript=$(find "${tarDir}" -name "${installntp_script}")
        cp -af "${g_rootPath}/common/${installntp_script}" "${ntpscript}"
        if [ $? -ne 0 ]; then
            log_and_show "ERROR" "$LINENO" "Failed to replace ${g_rootPath}/common/${installntp_script} to $ntpscript."
            return 1
        fi
        log_and_show "INFO" "${LINENO}" "succeed to replace $installntp_script in $pkg."
    fi

    log_and_show "INFO" "${LINENO}" "succeed to replace script,begin to compress to tar."

    cd "${tarDir}"
    tar -czf "${agent_tar}" ./* 
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Failed to package agent.tar."
        return 1
    fi
    sha256sum "${agent_tar}" > "${agent_tar}.sha256"

    cd - > /dev/null

    #replace agent pkg
    rm -f ${pkg}*
    cp -af "${tarDir}/${agent_tar}"* "${pkg_dir}"
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Failed to cp ${tarDir}/${agent_tar} to ${pkg_dir}"
        return 1
    fi

    #agentȨ޿
    chown root:wheel "${pkg}"
    chmod 640 "${pkg}"

    #УļȨ޿
    chown omm:wheel "${pkg}.sha256"
    chmod 600 "${pkg}.sha256"

    log_and_show "INFO" "${LINENO}" "succeed to replace agent.tar to $pkg."

    return 0
}

deal_with_scriptInAgentPkg()
{
    local distributeHome="${g_controllerHome}/packaged-distributables"
    local tarDir="${WORK_HOME}/tar_tmp_agent"

    agent_tarFile=($(find ${distributeHome} -name ${agent_tar}))
    
    for((i=0;i<${#agent_tarFile[@]};i++))
    do
        #backup
        backup_agent_pkg "${agent_tarFile[$i]}" 
        if [ $? -ne 0 ]; then
            rm -f "${agent_tarFile[$i]}"
            return 1
        fi

        replace_script "${agent_tarFile[$i]}" 
        if [ $? -ne 0 ]; then
            return 1
        fi

    done

    return 0
}

replace_decomScript_on_oms()
{
    log_and_show "INFO" "${LINENO}" "begin to replace decompressPatch on oms nodes."

    local decompress_script="decompressPatch.sh"

    cp -f "${g_rootPath}/common/${decompress_script}" "${g_nodeagentHome}/bin/${decompress_script}"
    if [ $? -ne 0 ];then
       log_and_show "ERROR" "${LINENO}"  "ERROR:get bigdata home failed."
       return 1
    fi

    chown omm:wheel "${g_nodeagentHome}/bin/${decompress_script}"
    if [ $? -ne 0 ];then
       log_and_show "ERROR" "${LINENO}"  "ERROR:chown to omm for ${g_nodeagentHome}/bin/${decompress_script} failed."
       return 1
    fi

    chmod 700 "${g_nodeagentHome}/bin/${decompress_script}"

    log_and_show "INFO" "${LINENO}" "succeed to replace decompressPatch on oms nodes."
    return 0
}

main()
{
    #init path constants
    get_path
    if [ $? -ne 0 ]; then        
        return 1
    fi
    
    #־
    [ -f "/home/omm/.profile" ] && source "/home/omm/.profile"
    source "${g_rootPath}/common/log.sh"
    
    if [ "$USER" != "root" ]; then
       log_and_show "ERROR" "${LINENO}" "ERROR:current user is not root."
       log_and_show "ERROR" "${LINENO}" "replace sudoExecute.sh failed."
       return 1
    fi
    
    g_bigdataHome=$(su - omm -c 'echo $BIGDATA_HOME')
    if [ $? -ne 0 ];then
       log_and_show "ERROR" "${LINENO}"  "ERROR:get bigdata home failed."
       return 1
    fi

    g_controllerHome=$(su - omm -c 'echo $CONTROLLER_HOME')
    if [ $? -ne 0 ];then
       log_and_show "ERROR" "${LINENO}"  "ERROR:get controller home failed."
       return 1
    fi

    g_nodeagentHome=$(su - omm -c 'echo $NODE_AGENT_HOME')
    if [ $? -ne 0 ];then
       log_and_show "ERROR" "${LINENO}"  "ERROR:get agent home failed."
       return 1
    fi
    
    log_and_show "INFO" "${LINENO}" "Start to replace sudoExecute.sh."
    
    backup_sudo
    if [ $? -ne 0 ];then
        log_and_show "ERROR" "${LINENO}" "backup sudoExecute file failed."
    fi

    replace_file
    if [ $? -ne 0 ];then
        log_and_show "ERROR" "${LINENO}" "replace file failed."
        return 1
    fi

    #omsڵ滻agent
    if [ -d "${g_controllerHome}" ]; then
        replace_decomScript_on_oms
        if [ $? -ne 0 ];then
            log_and_show "ERROR" "${LINENO}" "replace decompressPatch script for oms nodes failed."
            return 1
        fi

        deal_with_scriptInAgentPkg
        if [ $? -ne 0 ];then
            log_and_show "ERROR" "${LINENO}" "replace agent.tar failed."
            return 1
        fi
    fi
    
    log_and_show "INFO" "${LINENO}" "replace sudo flies successful."
    return 0
    
}

main
exit $?