#!/bin/bash

declare OMM_USER="omm"
declare OMM_WHEEL="wheel"
declare g_controllerHome=''
declare g_omsHome=''
declare PARENT_PATH=""
declare LOG_FILE=""
declare BASELINE_VERSION="6.5.1"
declare TEMP_VERSION=""
declare CLUSTER_BASELINE_VERSION=""
declare PATCH_VERSION="6.5.1.6"
declare BACKUP_HOME="/opt/os_optimization_tool_backup_${PATCH_VERSION}"
declare WORK_HOME="/opt/os_optimization_tool_${PATCH_VERSION}"

g_ifconfig="ifconfig"
standbyOMSIp=""

###################################################
#
# 获取绝对路径
#
###################################################
get_path()
{
    local path=""

    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi

    G_CURRENT_PATH="${path}"

    return 0
}

################################################################################
# Function: log
# Description: 日志打印
################################################################################
log()
{
    local curTime=$(date +%Y-%m-%d\ %T)
    local level=$1
    local lineNo=$2
    local logInfo=$3
    
    echo "[$curTime] [$level]: $logInfo(line:$lineNo)" >> "${LOG_FILE}" 2>&1
}

################################################################################
# Function: log_and_show
# Description: 日志打印
################################################################################
log_and_show()
{
    local curTime=$(date +%Y-%m-%d\ %T)
    local level=$1
    local lineNo=$2
    local logInfo=$3
    
    echo "[$curTime] [$level]: $logInfo(line:$lineNo)" >> "${LOG_FILE}" 2>&1
    echo "${logInfo}"
}

################################################################################
# Function: init
# Description: 初始化全局变量
################################################################################
init()
{   
    [ -f "/home/omm/.profile" ] && source "/home/omm/.profile"
    [ -f "/home/omm/.basrc" ] && source "/home/omm/.basrc"
    
    #init path constants
    get_path
    if [ $? -ne 0 ]; then        
        return 1
    fi

    preinstall_scp_tool="${G_CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    preinstall_cmd_tool="${G_CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    g_logHome=$(su - omm -c 'echo $BIGDATA_LOG_HOME')
    g_controllerHome=$(su - omm -c 'echo $CONTROLLER_HOME')
    g_versionFileName="${g_controllerHome}/etc/om/VERSION"
    
    local logPath="${g_logHome}/patch"
    if ! [ -d "${logPath}" ]; then
        mkdir -p "${logPath}"
        chown "${OMM_USER}:${OMM_WHEEL}" "$logPath" -R
    fi
    
    #工具运行日志
    LOG_FILE="${logPath}/optimization_patch_process.log"

    # 创建日志文件和修改文件权限
    if [ ! -f "${LOG_FILE}" ] ; then
        touch "${LOG_FILE}"
        chown "${OMM_USER}:${OMM_WHEEL}" "$LOG_FILE"
    fi

    chmod 700 "${logPath}"
    chmod 600 "${LOG_FILE}"
    
    if [ -f "/sbin/ifconfig" ]; then
        g_ifconfig="/sbin/ifconfig"
    fi

    #执行结果日志文件
    show_log="${logPath}/show_sudoPatch.log"
    if [ ! -f "${show_log}" ]; then
        touch "${show_log}"
    else
        cat /dev/null > "${show_log}"
    fi

    #远程命令执行工具
    g_optimizeTool="${WORK_HOME}/common/remote_optimize.sh"
    g_undo_optimizeTool="${WORK_HOME}/common/undo_remote_optimize.sh" 
    g_commit_optimizeTool="${WORK_HOME}/common/commit_remote_optimize.sh" 
    
    g_bigdataHome=$(su - omm -c 'echo $BIGDATA_HOME')
    g_omsHome=$(su - omm -c 'echo $OMS_RUN_PATH')
    
    PARENT_PATH=$(dirname "$G_CURRENT_PATH")
    log "INFO" "$LINENO" "PARENT_PATH=$PARENT_PATH."
    if [ -z "$PARENT_PATH" ]; then
        log "ERROR" "$LINENO" "Init parameter failed."
        return 1
    fi
    
    return 0
}

check_version()
{
    TEMP_VERSION="$(cat "$g_versionFileName")"
    
    if [[ ! "$TEMP_VERSION" =~ "6.5.1" ]]; then
        log_and_show "ERROR" "$LINENO" "Wrong version. The patch optimization tool is available on $BASELINE_VERSION only. The current version is ${TEMP_VERSION}. "
        return 1
    fi
    
    return 0
}

install_patch()
{
    log_and_show "INFO" "$LINENO" "Start to install optimization  patch,The log is kept at ${LOG_FILE}."

    install_exec_remote_batch
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Install optimization patch failed."
        return 1
    fi

    return 0
}

install_exec_remote_batch()
{
    local faillist_file="${G_CURRENT_PATH}/install_faillist.txt"
    if ! [ -f "${faillist_file}" ];then
        touch "${faillist_file}"
        chown "${OMM_USER}:${OMM_WHEEL}" "${faillist_file}"
    else
        echo "">"${faillist_file}"
    fi

    #当前目录权限控制
    chown root:root "${G_CURRENT_PATH}" -R
    chmod 700 "${G_CURRENT_PATH}" -R

    #分发工作目录
    sh ${preinstall_cmd_tool} "rm -rf ${WORK_HOME}" "${show_log}" >> "${LOG_FILE}" 2>&1
    sh ${preinstall_scp_tool} put "${G_CURRENT_PATH}" "${WORK_HOME}" "${show_log}" >> "${LOG_FILE}" 2>&1
    sh ${preinstall_cmd_tool} "chown root:root -R ${WORK_HOME}" >> "${LOG_FILE}" 2>&1
    sh ${preinstall_cmd_tool} "chmod 700 -R ${WORK_HOME}" >> "${LOG_FILE}" 2>&1

    #执行远程脚本，替换sudo
    sh ${preinstall_cmd_tool} "${g_optimizeTool}" "${show_log}" >> "${LOG_FILE}" 2>&1
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Install optimization patch failed."
        return 1
    fi

    return 0
}

uninstall_patch()
{
    log_and_show "INFO" "$LINENO" "Start to uninstall optimization  patch."

    uninstall_exec_remote_batch
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Unstall optimization patch failed."
        return 1
    fi

    sh ${preinstall_cmd_tool} "rm -rf ${WORK_HOME}" >> "${LOG_FILE}" 2>&1

    return 0
}

uninstall_exec_remote_batch()
{
    local faillist_file="${G_CURRENT_PATH}/uninstall_faillist.txt"
    if ! [ -f "${faillist_file}" ];then
        touch "${faillist_file}"
        chown "${OMM_USER}:${OMM_WHEEL}" "${faillist_file}"
    else
        echo "">"${faillist_file}"
    fi

    #卸载sudo补丁脚本
    sh ${preinstall_cmd_tool} "${g_undo_optimizeTool}" "${show_log}" >> "${LOG_FILE}" 2>&1
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Uninstall optimization patch failed."
        return 1
    fi

    return 0
}

commit_patch()
{
    log_and_show "INFO" "$LINENO" "Start to commit optimization  patch."

    if [ ! -d "${BACKUP_HOME}" ] || [ ! -d "${WORK_HOME}" ]; then
        log_and_show "ERROR" "$LINENO" "Failed to commit,because ${BACKUP_HOME} or ${WORK_HOME} does not exist."
        return 1
    fi
    commit_exec_remote_batch
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Failed to commit optimization patch."
        return 1
    fi

    return 0
}

commit_exec_remote_batch()
{
    local faillist_file="${G_CURRENT_PATH}/commit_faillist.txt"
    if ! [ -f "${faillist_file}" ];then
        touch "${faillist_file}"
    else
        echo "">"${faillist_file}"
    fi
    
    #提交操作，删除各节点备份目录/工作目录
    sh ${preinstall_cmd_tool} "${g_commit_optimizeTool}" "${show_log}" >> "${LOG_FILE}" 2>&1
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Failed to remove backup home."
        return 1
    fi

    sh ${preinstall_cmd_tool} "ls ${WORK_HOME} 2>/dev/null && rm -rf ${WORK_HOME}" "${show_log}" >> "${LOG_FILE}" 2>&1
    if [ $? -ne 0 ]; then
        log "ERROR" "$LINENO" "Failed to remove work home."
        return 1
    fi

    return 0
}

show_help()
{
    echo "Useage:" 
    echo -e "\t sh optimization_patch.sh install/uninstall(rollback)/commit."
    echo -e "\t install: install sudo patch."
    echo -e "\t uninstall/rollback: rollback sudo patch ."
    echo -e "\t commit: commit when you confirm ok."
}

################################################################################
# Function: main
# Description: 主函数入口
# Parameter: NA
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    local operation="${1}"
    
    #只有root用户才可以执行此脚本
    if [ $(id -u) -ne 0 ]; then
        echo -e "ERROR: Only the root user can execute the script."
        exit 1
    fi

    if [ "${operation}" != "install" ] && [ "${operation}" != "uninstall" ] && [ "${operation}" != "rollback" ] && [ "${operation}" != "commit" ]; then
        show_help
        return 1
    fi

    #检查 expect 工具
    which expect > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Please install expect command."
        return 1
    fi

    #初始化变量和日志文件
    init
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Initialization failed. View detailed log $LOG_FILE."
        return 1
    fi
    
    #判断下当前集群的版本号，只有支持的版本才允许安装补丁优化工具
    check_version   
    if [ $? -ne 0 ]; then
        log_and_show "ERROR" "$LINENO" "Install optimization patch failed "
        return 1
    fi
    
    case ${operation} in
        install)
            install_patch
            ret=$?
            if [ "${ret}" -ne 0 ]; then
                log_and_show "INFO" "$LINENO" "failed to get configuration ,need to check cluster.ini file."
                return 1
            else
                if [ `cat ${show_log} | wc -l` -eq 0 ]; then
                    log_and_show "INFO" "$LINENO" "Install sudo patch successfully."
                    return 0
                else
                    cat "${show_log}"
                    return 1
                fi
            fi
            ;;
        uninstall|rollback)
            uninstall_patch
            ret=$?
            if [ "${ret}" -ne 0 ]; then
                log_and_show "INFO" "$LINENO" "failed to get configuration ,need to check cluster.ini file."
                return 1
            else
                if [ `cat ${show_log} | wc -l` -eq 0 ]; then
                    log_and_show "INFO" "$LINENO" "Uninstall sudo patch successfully."
                    return 0
                else
                    cat "${show_log}"
                    return 1
                fi
            fi
            ;;
        commit)
            commit_patch
            ret=$?
            if [ "${ret}" -ne 0 ]; then
                log_and_show "INFO" "$LINENO" "failed to get configuration ,need to check cluster.ini file."
                return 1
            else
                if [ `cat ${show_log} | wc -l` -eq 0 ]; then
                    log_and_show "INFO" "$LINENO" "Commit sudo patch successfully."
                    return 0
                else
                    cat "${show_log}"
                    return 1
                fi
            fi
            ;;
    esac
    
    return 0
}



#脚本解释器 强制设置为 /bin/bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   /bin/bash $0 "$@"
   exit $?
fi

main "$@"
exit $?
