#!/bin/bash

LOG_FUNC_FILE="$(dirname "$0")"/log.sh
. "${LOG_FUNC_FILE}" || { echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [ERROR]: Failed to load ${LOG_FUNC_FILE}. [${SCRIPT_NAME}:${LINENO}]($$)"; exit 1;}

# ---------------------------------------------------------------------------- #
#                               全局变量定义                                   #
# ---------------------------------------------------------------------------- #
declare validate_ip1=$1
declare validate_ip2=$2
declare username=$3
declare password=$4
declare g_ip_model=$5

#远程获取子网掩码
function getNetmask()
{
    ip=$1
    username=$2
    password=$3

    expect -c "set timeout 60;
                spawn -noecho ssh ${username}@${ip} if \[ \-f /sbin/ifconfig \]\; then /sbin/ifconfig \-a \|grep \-w \-F ${ip}\; else /usr/bin/ifconfig \-a \|grep \-w \-F ${ip}\; fi;
                expect {
                         *yes/no* {send \"yes\r\"; exp_continue};
                         -nocase *assword:* {send \"${password}\r\"; exp_continue};
                         timeout { exit 1 };
                         FATAL {exit 2 };
                         \"*Connection refused\" { exit 3 };
                         \"*Permission denied*\" { exit 4 };
                         \"*Host key verification failed*\" { exit 5 };
                         \"*Illegal host key*\" { exit 6 };  
                         \"*No route to host\" { exit 7 };  
                         \"Received disconnect*\" { exit 9 };  
                         \"lost connection\" { exit 10 };
                       }
                expect timeout { exit 8 };
                set my_ret [wait];
                exit [lindex \$my_ret 3];"   
}

####################################################################
#  DESCRIPTION  : 判断两个IP是否为同一网段的
####################################################################
parse_ip()
{
    local cc=""
    for i in 1 2 3 4
    do
        let "aa = $(echo $1 | cut -d '.' -f $i) & $(echo $2 | cut -d '.' -f $i)"
        if [ -n "$cc" ]
        then
            cc="$cc.$aa"
        else
            cc="$aa"
        fi
    done
    echo "$cc"
}

################################################################################
# Description: 检查两个ip的合法性
################################################################################
validate_ip()
{
    ping -c 1 "${validate_ip1}" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ping $validate_ip1 failed."
        return 1
    fi
    ping -c 1 "${validate_ip2}" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ping $validate_ip2 failed."
        return 1
    fi

    #解析获取的子网掩码
    validate_ip1_Netmask=$(getNetmask "$validate_ip1" "$username" "$password" | awk -F 'inet' '{print $2}' | awk '{print $3}' | \
                         tr -d 'Mask:' | tr -d '\n' | tr -d '[:space:]')
    if [ -z "$validate_ip1_Netmask" ]; then
        log "${LINENO}" "[ERROR_DEBUG]" "get $validate_ip1_Netmask Netmask failed"
        return 1
    fi
    validate_ip2_Netmask=$(getNetmask "$validate_ip2" "$username" "$password" | awk -F 'inet' '{print $2}' | awk '{print $3}' | \
                         tr -d 'Mask:' | tr -d '\n' | tr -d '[:space:]')
    if [ -z "$validate_ip2_Netmask" ]; then
        log "${LINENO}" "[ERROR_DEBUG]" "get $validate_ip2_Netmask Netmask failed"
        return 1
    fi

    # 判断节点是否与浮动ip为同一网段的
    ret1=$(parse_ip $validate_ip1 $validate_ip1_Netmask 2>/dev/null)
    ret2=$(parse_ip $validate_ip2 $validate_ip2_Netmask 2>/dev/null)
    if [ "$ret1" == "" ] || [ "$ret1" != "$ret2" ];then
        log "[ERROR_DEBUG]" "${LINENO}" "$validate_ip1 and $validate_ip2 are not in the same network, please change."
        return 1
    fi
}

hex2dec()
{
    for i in $(echo "$@"); do
        printf "%d\n" "$(( 0x$i ))"
    done
}

################################################################################
# Description: 补全IPv6地址
################################################################################
expand_ipv6()
{
    ip=$1
    # 如果起始位置为：,补充字符 0
    echo $ip | grep -qs "^:" && ip="0${ip}"
    # 扩展 ::
    if echo $ip | grep -qs "::"; then
            colons=$(echo $ip | sed 's/[^:]//g')
            missing=$(echo ":::::::::" | sed "s/$colons//")
            expanded=$(echo $missing | sed 's/:/:0/g')
            ip=$(echo $ip | sed "s/::/$expanded/")
    fi

    blocks=$(echo $ip | grep -o "[0-9a-f]\+")
    set $blocks

    printf "%04x:%04x:%04x:%04x:%04x:%04x:%04x:%04x\n" \
        $(hex2dec $@)
}

compare_ipv6_prefix() {
    for i in 1 2 3 4
    do
        if [ $(echo $1 | cut -d ':' -f $i) != $(echo $2 | cut -d ':' -f $i) ]; then
            return 1
        fi
    done
    return 0

}

################################################################################
# Description: 检查两个ipv6的合法性
################################################################################
validate_ipv6()
{
    ping6 -c 1 "${validate_ip1}" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ping $validate_ip1 failed."
        return 1
    fi
    ping6 -c 1 "${validate_ip2}" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "ping $validate_ip2 failed."
        return 1
    fi

    # 补全IPv6
    expand_ip1=$(expand_ipv6 ${validate_ip1})
    if [ -z "$expand_ip1" ]; then
        log "${LINENO}" "[ERROR_DEBUG]" "expand ${validate_ip1} failed"
        return 1
    fi
    expand_ip2=$(expand_ipv6 ${validate_ip2})
    if [ -z "$expand_ip2" ]; then
        log "${LINENO}" "[ERROR_DEBUG]" "expand ${validate_ip2} failed"
        return 1
    fi

    # 判断节点是否与浮动ip为同一网段的
    compare_ipv6_prefix ${expand_ip1} ${expand_ip2}
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "$validate_ip1 and $validate_ip2 are not in the same network, please change."
        return 1
    fi
}

function main()
{
    log "[INFO]" "${LINENO}" "localhost: enter checking $validate_ip1 and $validate_ip2."
    if [ ${g_ip_model} == "IPv4" ]; then
        validate_ip
        if [ $? -ne 0 ]; then
            log "[ERROR_DEBUG]" "${LINENO}" "localhost: check checking $validate_ip1 and $validate_ip2 failed."
            return 1
        else
            log "[INFO]" "${LINENO}" "localhost: end checking $validate_ip1 and $validate_ip2."
            return 0
        fi
    else
        validate_ipv6
        if [ $? -ne 0 ]; then
            log "[ERROR_DEBUG]" "${LINENO}" "localhost: check checking $validate_ip1 and $validate_ip2 failed."
            return 1
        else
            log "[INFO]" "${LINENO}" "localhost: end checking $validate_ip1 and $validate_ip2."
            return 0
        fi
    fi
}

main "$@"