#!/bin/bash

#导入公共日志函数
SCRIPT_NAME="$(basename "${BASH_SOURCE-$0}")"
LOG_FUNC_FILE="$(dirname "$0")"/../../function/log.sh
. "${LOG_FUNC_FILE}" || { echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [ERROR]: Failed to load ${LOG_FUNC_FILE}. [${SCRIPT_NAME}:${LINENO}]($$)"; exit 1;}


RET_ERR=1
RET_OK=0
declare -i retVal=0  

HOST_IP=$1

function precheck_nsswitch_conf()
{
    local file_path="/etc/nsswitch.conf"
    local nss_key_string="passwd:,group:,services:,netgroup:,aliases:"
    local nss_key_list=($(echo "${nss_key_string}" | awk -F',' '{for(i=1; i<=NF; i++){print $i;};}'))
    for nss_key in "${nss_key_list[@]}"
    do
        nss_line=$(grep -w "${nss_key}" "${file_path}" | grep -v "#")
        if [ -z "${nss_line}" ];then
            log "[ERROR]" "${LINENO}" "${HOST_IP}: The configuration item "${nss_key}" cannot be found in /etc/nsswitch.conf. \
        Please update /etc/nsswitch.conf and ensure that the configuration items are correct."
            return $RET_ERR
        fi       
        ((index++))       
    done
    
    return $RET_OK
}

getPamUnixKey()
{
    local pam_unix2_key="pam_unix2.so"
    
    if [ -f "/etc/SuSE-release" ]; then
        VERSION=$(cat /etc/SuSE-release | grep "VERSION" | awk -F '=' '{print $2}')
        if [ "${VERSION}" -eq "12" ]; then
            pam_unix2_key="pam_unix.so"
        fi
    fi
    
    echo "${pam_unix2_key}"
}


function precheck_suse_ldap_pamd_conf()
{
    local pam_common_session="/etc/pam.d/common-session"
    local pam_common_password="/etc/pam.d/common-password"
    local pam_common_auth="/etc/pam.d/common-auth"
    local pam_common_account="/etc/pam.d/common-account"
    
    local pam_unix2_line=""
    local pam_unix2_key="$(getPamUnixKey)"
    
    pam_unix2_line=$(grep -w "${pam_unix2_key}" "${pam_common_session}" | grep -v "#" )
    if [ -z "${pam_unix2_line}" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The configuration item pam_unix2_key cannot be found in /etc/pam.d/common-session. \
    Please update /etc/pam.d/common-session and ensure that the configuration item is correct."            
    fi 
    
    local pam_limits_line=""
    local pam_limits_key="pam_limits.so"
    
    pam_limits_line=$(grep -w "${pam_limits_key}" "${pam_common_session}" | grep -v "#" )
    if [ -z "${pam_limits_line}" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The configuration item pam_limits_key cannot be found in /etc/pam.d/common-session. \
    Please update /etc/pam.d/common-session and ensure that the configuration item is correct."     
    fi
    
    pam_unix2_line=$(grep -w "${pam_unix2_key}" "${pam_common_password}" | grep -v "#" )
    if [ -z "${pam_unix2_line}" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The configuration item pam_unix2_key cannot be found in /etc/pam.d/common-password. \
    Please update /etc/pam.d/common-password and ensure that the configuration item is correct."           
    fi  
    
    pam_unix2_line=$(grep -w "${pam_unix2_key}" "${pam_common_auth}" | grep -v "#" )
    if [ -z "${pam_unix2_line}" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The configuration item pam_unix2_key cannot be found in /etc/pam.d/common-auth. \
    Please update /etc/pam.d/common-auth and ensure that the configuration item is correct."    
    fi  
    
    pam_unix2_line=$(grep -w "${pam_unix2_key}" "${pam_common_account}" | grep -v "#" )
    if [ -z "${pam_unix2_line}" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The configuration item pam_unix2_key cannot be found in /etc/pam.d/common-account. \
    Please update /etc/pam.d/common-account and ensure that the configuration item is correct."        
    fi    
}

function precheck_sysctl_conf()
{
    local sysctl_path="/etc/sysctl.conf"
    
    if [ ! -f "$sysctl_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /etc/sysctl.conf configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."        
    fi    
    
    local sysctl_bin_path="/sbin/sysctl"
    if [ ! -f "$sysctl_bin_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /sbin/sysctl configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."            
    fi
    
    ${CMD_CHKCTL} >/dev/null
    if [ $? -ne 0 ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: Failed to restart the sysctl service. \
    Please check whether any invalid configuration item exists in /etc/sysctl.conf."        
    fi
}

function precheck_suse_ldap_nscd_conf()
{
    local sysnscd_path="/etc/nscd.conf"
    if [ ! -f "$sysnscd_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /etc/nscd.conf configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."         
    fi    
    
    local sysnscd__bin_path="/usr/sbin/nscd"
    if [ ! -f "$sysnscd__bin_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}:   The /usr/sbin/nscd configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."        
    fi    
}

function precheck_sysldap_conf()
{
    local sysldap_path="/etc/openldap/ldap.conf"
    if [ ! -f "$sysldap_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /etc/openldap/ldap.conf configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."       
    fi    
}

function precheck_openssl_conf()
{
    local sysopenssl_path="/usr/bin/openssl"
    if [ ! -f "$sysopenssl_path" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /usr/bin/openssl configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."    
    fi    
}

function precheck_suse_ldap_env()
{
    local sysldap_file_env="/usr/lib/openldap/slapd"
    
    if [ ! -f "$sysldap_file_env" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /usr/lib/openldap/slapd configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."      
    fi    
}

function precheck_redhat_ldap_env()
{
    local sysldap_file_env="/usr/sbin/slapd"
    
    if [ ! -f "$sysldap_file_env" ];then
         log "[WARNING]" "${LINENO}" "${HOST_IP}: The /usr/sbin/slapd configuration file cannot be found. \
     Please restore the file and ensure that the file is correct."     
    fi
    
    return $RET_OK
}

function precheck_ldapclient_env()
{
    local sysldap_client_env="/usr/bin/ldapsearch"
    if [ ! -f "$sysldap_client_env" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /usr/bin/ldapsearch configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."        
    fi
    
    return $RET_OK
}

function precheck_redhat_ldap_rsyslog_conf()
{
    local sys_ldap_rsyslog_conf="/etc/rsyslog.conf"    
    if [ ! -f "$sys_ldap_rsyslog_conf" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /etc/rsyslog.conf configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."         
    fi    
    
    local sys_ldap_rsyslog_bin="/sbin/rsyslogd"
    if [ ! -f "$sys_ldap_rsyslog_bin" ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The /sbin/rsyslogd configuration file cannot be found. \
    Please restore the file and ensure that the file is correct."        
    fi    
}

precheck_redhat_ldap()
{
    precheck_redhat_ldap_env
        
    precheck_redhat_ldap_rsyslog_conf

}

precheck_suse_ldap()
{
    precheck_suse_ldap_env
    
    precheck_suse_ldap_nscd_conf
    
    precheck_suse_ldap_pamd_conf    
}

version_eq=1
version_err=3
version_compare()
{
    if [ $# -ne 2 ]; then
        log "[ERROR]" "${LINENO}" "${HOST_IP}: Version_compare need two parameter" 
        return 3
    fi
    old_version=$( echo "$1" | sed -e "s/ //g")
    new_version=$( echo "$2" | sed -e "s/ //g")
    if [ "${old_version}" != "" ] && [ "${new_version}" != "" ]; then
        old_version_array=( $(echo "${old_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        new_version_array=( $(echo "${new_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        old_version_array_length=${#old_version_array[@]}
        new_version_array_length=${#new_version_array[@]}

        length=$((${new_version_array_length}>${old_version_array_length}?${old_version_array_length}:${new_version_array_length}))
        index=0
        eq_tag=3
        while [ $index -lt "${length}" ]
        do
            v1=${old_version_array[$index]}
            v2=${new_version_array[$index]}
            if [ "${v1}" -lt "${v2}" ]; then
                return 2
            elif [ "${v1}" -gt "${v2}" ] ; then
                return 0
            elif [ "${v1}" -eq "${v2}" ] ; then
                eq_tag=1
                let index+=1
                continue
            else
                return 3
            fi
        done
        return ${eq_tag}
    else
        log "[ERROR]" "${LINENO}" "${HOST_IP}: Parameters can not be null."      
        return 3
    fi
}

version_check()
{
    local re=${version_err}
    $(version_compare "${1}" "${2}")
    re=$?
    if [ ${re} -eq ${version_err} ];then
        log "[ERROR]" "${LINENO}" "${HOST_IP}: Detect ${3} version failed. \
    Please ensure ${3} has been installed correctly."
        return ${RET_ERR}
    else
        if [ ${re} -gt ${version_eq} ];then
        log "[WARNING]" "${LINENO}" "${HOST_IP}: The ${3} version is weakly secure, \
    it cannot support TLSv1.1, TLSv1.2.  You are advised to install the recommended OS.\
        Proceeding with the installation indicates that you accepted the security risks."
        fi
    fi
    return ${RET_OK}
}

precheck_ldap_version()
{
    local system_type="$1"
    local e_ldap_version="2.4.40"
    local slapd_bin="/usr/sbin/slapd"
    if [ -n "$system_type" ];then
        slapd_bin="/usr/lib/openldap/slapd"    
    fi
    local ldapversion=$(${slapd_bin} -VV 2>&1 | grep "OpenLDAP: slapd" | tr -cd "[0-9 .]" | awk '{print $1}' | sed -e "s/ //g")
    local ldap="openldap"
    version_check "${ldapversion}" "${e_ldap_version}" "${ldap}"
    return $?
}

precheck_openssl_version()
{
    local e_ssl_version="1.0.1"
    local ssl_bin="/usr/bin/openssl"
    local sslversion=$(${ssl_bin} version 2>&1 | grep "OpenSSL" | tr -cd "[0-9 .]" | awk '{print $1}' | sed -e "s/ //g")
    local ssl="openssl"
    version_check "${sslversion}" "${e_ssl_version}" "${ssl}"
    return $?
}

main()
{
    log "[INFO]" "${LINENO}" "${HOST_IP}: start checking ldap."
    
    local system_type=$(cat /proc/version | grep "SUSE")  
    
    if [ -z "$system_type" ];then
        precheck_redhat_ldap      
    else
        precheck_suse_ldap        
    fi    
    
    precheck_ldap_version "${system_type}"
    
    precheck_nsswitch_conf    
    
    precheck_sysctl_conf    
    
    precheck_sysldap_conf   
    
    precheck_openssl_conf
    
    precheck_openssl_version
}

main
