#!/bin/bash
#------------------------------------------------------------------------------
#检查是否开启crontab非root用户禁止运行
#omm需要crontab 运行ha，如果检测开启，不合法，则报ERROR
#------------------------------------------------------------------------------

checkCrontabRight()
{
    log "[INFO]" "${LINENO}" "${ip}: start checking the omm Right of Crontab."  
    ip=$1

    CrontabRight
    if [ $? -ne 0 ]
    then 
        log "[ERROR]" "${LINENO}" "${ip}: The '/etc/cron.allow' or '/etc/cron.deny' file is configured incorrectly. User 'omm' does not have the rights to use crontab."
    fi
}

#------------------------------------------------------------------------------
#cron通过/etc/cron.allow和/etc/cron.deny来允许或禁止用户拥有自己的crontab文件。
#root用户可以创建、编辑或删除这些文件。这些文件中的条目是用户登录名，每行一个名称。
#怎么通过这两个文件进行权限界定的呢？如果以下一个条件成立，用户将不能使用crontab命令：
#1.cron.allow文件和cron.deny文件不存在（只允许root用户）。　
#2.cron.allow文件存在，但用户的登录名并不列在其中。　
#3.cron.deny文件存在，并且用户的登录名列在其中。
#4.提醒，如果cron.allow和cron.deny中用户名都存在，则用户具有运行权限。cron.allow的优先级比cron.deny高
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#   allow               0           1          2     
#------------------------------------------------------------------------------
#   deny                0           1          2
#------------------------------------------------------------------------------
#        00  允许     01都配，允许    02允许
#        10  阻止     11 阻止         12阻止
#        20  允许     21 阻止         22阻止         
#------------------------------------------------------------------------------

CrontabRight()
{
    #文件不存在
    allow=2         
    deny=2
    
    if [ -e "/etc/cron.allow" ]
    then
       #检查cron.allow 中是否配置omm
        checkResult=$(${CMD_CHKCRN} /etc/cron.allow | grep -vw '^#' | awk '{if ($1 == "omm"){print $0}}')
        if [ -z "${checkResult}" ]
        then
            log "[ERROR_DEBUG]" "${LINENO}" "${ip}: 'omm'  is not configured in '/etc/cron.allow '."
            allow=1
        else
            allow=0
        fi
    fi
    
    if [ -e "/etc/cron.deny" ]
    then
        #检查cron.deny 中是否配置omm
        checkResult=$(${CMD_CHKCRN} /etc/cron.deny | grep -vw '^#' | awk '{if ($1 == "omm"){print $0}}')
        if [ -n "${checkResult}" ]
        then
            log "[ERROR_DEBUG]" "${LINENO}" "${ip}: 'omm'  is configured in '/etc/cron.deny '."
            deny=1
        else
            deny=0
        fi
    fi    
    result=$((${allow}*${deny})) 
    
    if [ ${result} == 0 ]
    then
        #cron.allow文件存在，但用户的登录名并不列在其中。　
        if [ ${allow} == 1 ]
        then
            return 1    
        fi    
        return 0    
    else
        return 1
    fi
}


