#!/bin/bash

checkLocalHostConfigured()
{
    log "[INFO]" "${LINENO}" "${ip}: start checking the localhost configuration."  
    ip=$1
    hostNamecfg=$2
    serviceip=$3
    
    checkHostnameIp
    if [ $? -ne 0 ]
    then 
        log "[ERROR]" "${LINENO}" "${ip}: The '/etc/hosts' file is configured incorrectly."
    fi
    
}

checkHostnameIp()
{
    result=0
    #检查/etc/hosts中是否配置127.0.0.1 localhost
    checkResult=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "127.0.0.1" && ($2 == "localhost" || $3 == "localhost")){print $0}}')
    
    if [ -z "${checkResult}" ]
    then
        log "[ERROR]" "${LINENO}" "${ip}: '127.0.0.1 localhost' is not configured in '/etc/hosts'."
        result=0
    fi
    
    #检查/etc/hosts文件内容是否正确，配置不正确导致安装失败。现在LLD已经提供了hostname的配置，需要根据配置项检查
    #增加检查（LLD工具配置了就做到节点ip和主机名对应检查，做自动填写hosts文件，主机名hostname，检测未填写时不进行补充,提示错误未配置,在preinstall配置，如果已经填写则只是提示对错）
    if [ -n "${serviceip}" ] && [ -n "${hostNamecfg}" ];then
        checkResult=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${serviceip}'" && ($2 == "'${hostNamecfg}'" || $3 == "'${hostNamecfg}'")){print $0}}')
        if [ -z "${checkResult}" ];then
            checkhostip=$(cat /etc/hosts | grep -vw '^#' | grep "${serviceip}\|${hostNamecfg}")
            if [ -z "${checkhostip}" ];then
                log "[ERROR]" "${LINENO}" "${ip}: '${serviceip} ${hostNamecfg}' is not configured in '/etc/hosts'."
                result=0
            else
                log "[ERROR]" "${LINENO}" "${ip}:'${serviceip} ${hostNamecfg}' is mistakenly configured as '${checkhostip}' in '/etc/hosts'."
                result=0
            fi
        fi
    fi
    
    #检查hostname是否已经配置
    hostname -i &> /dev/null
    if [ $? -ne 0 ]
    then 
        log "[ERROR_DEBUG]" "${LINENO}" "${ip}: the hostname is not configured in '/etc/hosts'. " 
        return 1
    fi

    # 检查hostname对应的ip值是否唯一
    hostname_ip=$(hostname -i)
    ip_count=$(echo ${hostname_ip} | wc -w)
    if [ "${ip_count}" -ne 1 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "${ip}: the hostname corresponds to more than one ip in '/etc/hosts'."  
        return 1
    fi
    
    #检查hostname对应的ip值是否存在于ifconfig列表中
    if [ -f /sbin/ifconfig ]; then
        IFCONFIG="/sbin/ifconfig"
    else
        IFCONFIG="/usr/bin/ifconfig"
    fi
    [ -f ${IFCONFIG} ] && ipall=$(${IFCONFIG})
    [ -f /sbin/ip ] && ipall=$(/sbin/ip a)
    
    echo "$ipall" | grep "${hostname_ip}"  &> /dev/null
    if [ $? -ne 0 ]; then 
        log "[ERROR_DEBUG]" "${LINENO}" "${ip}: the ip ${hostname_ip} in /etc/hosts is not in ifconfig/ip list."
        return 1
    fi
    
    # 检查/etc/hosts中是否存在一个IP对应多个主机的情况
    hostname_count=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${hostname_ip}'"){print $0}}' | wc -l)
    if [ "${hostname_count}" -ne 1 ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "${ip}: the ip ${hostname_ip} corresponds to more than one hostname in '/etc/hosts'."  
        return 1
    fi
    
    return ${result}
}

