#!/bin/bash
# -*- coding:utf-8 -*-

#-----------------------------------------------------------------------------------------------------------------
# func : 检查当前操作系统版本是否为本软件支持的版本
#NAME="SLES"
#VERSION="11.4"
#VERSION_ID="11.4"
#PRETTY_NAME="SUSE Linux Enterprise Server 11 SP4"
#ID="sles"
#ANSI_COLOR="0;32"
#CPE_NAME="cpe:/o:suse:sles:11:4"
#NAME="SLES"
#VERSION="12-SP1"
#VERSION_ID="12.1"
#PRETTY_NAME="SUSE Linux Enterprise Server 12 SP1"
#ID="sles"
#NAME="Red Hat Enterprise Linux Server"
#VERSION="7.2 (Maipo)"
#ID="rhel"
#ID_LIKE="fedora"
#VERSION_ID="7.2"
#PRETTY_NAME="Red Hat Enterprise Linux Server 7.2 (Maipo)"
#-----------------------------------------------------------------------------------------------------------------

function get_osnameEX()
{
    local ole_release="/etc/oracle-release"
    local cent_release="/etc/centos-release"
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local name=""
    local os_release="/etc/os-release"
    local euler_release="/etc/euleros-release"
    local neokylin_relase="/etc/neokylin-release"
    local openeuler_release="/etc/openEuler-release"
    local kylin_relase="/etc/kylin-release"
    local linxos_release="/etc/linxos-release"
    if [ -f "${ole_release}" ]; then
        name="oracle-$(cat "${ole_release}" | awk '{print $5}')"
        echo "$name"
        return 0
    fi
    
    if [ -f "${linxos_release}" ]; then
        name=$(awk '{print $1"-"$2}' ${linxos_release})
        echo "$name"
        return 0
    fi
    
     #获取Euler的版本, Euleros中存在redhat-release和CentOS-release，最先检查    
    if [ -f "${euler_release}" ]; then
        if grep -q "release" "${euler_release}"; then
            name="$(cat $euler_release | sed 's/x86_64//g' | awk -F " |SP|)" '{print "euleros-"$3"sp"$5}')"
        else
            name="euleros-$(cat "${euler_release}" | awk '{print $2}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
        fi
        echo "$name"
        return 0
    fi
    if [ -f "${neokylin_relase}" ];then
        if grep -q "release" "${neokylin_relase}"; then
            name="$(cat "${neokylin_relase}" | awk '{print $6}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            if [ -z "${name}" ];then
                name="$(cat "${neokylin_relase}" | awk '{print $4}' | cut -d "V" -f 2 | tr 'A-Z' 'a-z')"
            fi
            name="neokylin-$(echo ${name/update/'.'})"
        fi
        echo "$name"
        return 0
    fi
    #获取kylinSecOS
    if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}" |awk -F " " '{print $1}')" ]; then
        name="kylinSecOS-$(grep "^VERSION_ID=" "${sles15_release}" | awk -F "\"" '{print $2}')"
        echo "$name"
        return 0
    fi
    if [ -f "${os_release}" ]; then
        #x86_64的suse15系统/etc目录下只有os-release，和以前的“local os_release”做下区分，先判断suse15
        if [ "suse" = "$(cat "${os_release}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${os_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
        
        raw=$(cat "${os_release}")
        eval $raw
        if [ "${ID}" = "uos" ]; then
            name="$ID-$(cat /etc/os-version | grep MinorVersion | awk -F "=" '{print $2}')"
            echo "$name"
            return 0
        fi
        #获取openEuler-SPx的版本
        if [ -f "${openeuler_release}" ] && [ "${ID}" = "openEuler" ]; then
           local MINOR_VERSION=$(cat "${openeuler_release}" | awk -F "-" '{print $2}' | awk -F ")" '{print $1}')
           #openEuler-20.03-SP1保持之前的规则提取版本号
           local MINOR_VERSION_ID="${VERSION_ID}-${MINOR_VERSION}"
           if [ -n "${MINOR_VERSION}" -a "${MINOR_VERSION_ID}" != "20.03-SP1" ] ; then
               name="$ID-${VERSION_ID}-${MINOR_VERSION}"
               echo "$name"
               return 0
           fi
        fi
        name="$ID-$VERSION_ID"
        echo "$name"
        return 0
    fi
    
    if [ -f "${cent_release}" ]; then
        name="centos-$(cat "${cent_release}" | awk '{print $3}')"
        echo "$name"
        return 0
    fi

    if [ -f "${rhel_release}" ]; then
        name="redhat-$(cat "${rhel_release}" | awk '{print $7}')"
        echo "$name"
        return 0
    fi
    
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi
    return 1
}

function is_support_os()
{
    local support="$1"
    local name=""


    if [ -z "$support" ]; then
        log "[ERROR_DEBUG]" "${LINENO}" "${ip}: support os list is null."
        return 1
    fi

    name=$(get_osnameEX|cut -f1 -d '.')

    #当前os类型不属于所支持的os类型
    if [ $? -ne 0 ]; then
        log "[ERROR]" "${LINENO}" "${ip}: The OS version is not supported. Currently support ${support}" 
        return 1
    fi

    #当前os的版本号不属于所支持的os版本号
    echo "${support}" | grep  "${name}"
    if [ $? -ne 0 ]; then
        log "[ERROR]" "${LINENO}" "${ip}: The OS version is not supported. Currently support ${support}"
        return 1
    fi
    
    echo "$name"
    return 0
}

                                      
function checkOS()
{
    ip=$1
    log "[INFO]" "${LINENO}" "${ip}: start checking operation system type." 
    
    #本软件支持的所有os版本
    support_os="fusionos-22.0.4;fusionos-23.0.2;LinxOS-6.0.99;uos-1020;uos-1050;uos-1060;kylin-V10;openEuler-20.03;openEuler-22.03;redhat-6;redhat-7;suse-11;suse-12;suse-15;centos-6;centos-7;centos-8;sles-12;sles-11;rhel-7;rhel-6;EulerOS-2;euleros-2.0sp1;neokylin-6;neokylin-7;oracle-7;bclinux-7;bclinux-21;openEuler-20.03-SP3;openEuler-22.03-SP1;kylinSecOS-3.4-4;"
    
    #检查当前os版本是否为本软件支持的os版本
    local_os="$(is_support_os "${support_os}")"
    if [ $? -ne 0 ]; then
        echo "$local_os"
        return 1
    fi
}
