#!/bin/bash

#导入公共日志函数
SCRIPT_NAME="$(basename "${BASH_SOURCE-$0}")"
LOG_FUNC_FILE="$(dirname "$0")"/../../function/log.sh
. "${LOG_FUNC_FILE}" || { echo "[$(date -d today +'%Y-%m-%d %H:%M:%S')] [ERROR]: Failed to load ${LOG_FUNC_FILE}. [${SCRIPT_NAME}:${LINENO}]($$)"; exit 1;}

declare HOST_IP=$1               ##本机IP
declare business_nic=""

getBusinessNic()
{
    local businessIp
    local interface

    businessIp="$(hostname -i)"
    if [ $? -ne 0 ]; then
        log "[ERROR_DEBUG] ${LINENO} ${HOST_IP}: The '/etc/hosts' file is configured incorrectly,the hostname is not configured in '/etc/hosts'."
        return 1
    fi

    if [ -f /sbin/ifconfig ]; then
        IFCONFIG="/sbin/ifconfig"
    else
        IFCONFIG="/usr/bin/ifconfig"
    fi
    [ -f ${IFCONFIG} ] && interface=$(${IFCONFIG} | grep -B 2 -w "$businessIp" | grep HWaddr | awk '{print $1}')
    [ -f /sbin/ip ] && interface=$(/sbin/ip a | grep -B 2 -w "$businessIp" | grep ": en\|: eth" |awk '{print $2}'|cut -f1 -d ':')
    if [ -z "${interface}" ]; then
        log "[ERROR_DEBUG] ${LINENO} ${HOST_IP}: can not get network interface with business ip."
        return 1
    fi

    if [ -f "/sys/class/net/$interface/bonding/slaves" ]; then
        # bond
        business_nic="$(cat /sys/class/net/"$interface"/bonding/slaves)"
    else
        business_nic="$interface"
    fi

    return 0
}

##############################################日志打印说明######################################################################
##需要打印的信息级别有以下几类：
## [INFO]
## [WARN]
## [ERROR_DEBUG]
## [ERROR]
## 其中 前三项是打印到log文件中的，不会显示在控制台，只有最后一项是显示在控制台的信息。
## 因此，显示在控制台的ERROR信息一定是某项检查的之中未通过的信息。对于供开发人员调试的错误信息请标成[ERROR_DEBUG]
#################################################################################################################################
checkNcRxhash()
{
    log "[INFO] ${LINENO} ${HOST_IP}: start checking network card receive-hashing feature."
	
    #获取业务平面网卡设备
    getBusinessNic
    if [ $? -ne 0 ]; then
        return 0
    fi
    
    #遍历设备检查
    for DEV in ${business_nic}
    do
        result=$(${CMD_CHKETH} -k "${DEV}" | grep "receive-hashing")
        if [ $? -ne 0 ]; then
            log "[WARN] ${LINENO} ${HOST_IP} ${DEV}: no receive-hashing feature."
            continue
        fi
        
        if [ "$result" = "receive-hashing: off" ]; then
            log "[WARNING] ${LINENO} ${HOST_IP} ${DEV}: receive-hashing feature is off. If you install Redis at this node, please run 'ethtool -K ${DEV} rxhash on' to enable."
        fi
    done
    
    log "[INFO] ${LINENO} ${HOST_IP}: checking network card receive-hashing feature finish."
}

main()
{
    ## 网卡rxhash功能检查
    checkNcRxhash
}

main
