#!/bin/bash 

###################################################
# Usage: preinstall.sh
# 功能:  进行OS适配,以满足大数据的安装  
###################################################

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   bash $0 "$@"
   exit $?
fi

export LC_ALL=C

###################################################
# 全局变量
###################################################

#支持的OS版本
g_support_os=$(cat $(dirname $0)/osList.ini)

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ "$UID" -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    g_pid_file="$HOME/fi-preinstall.pid"                   #进程PID
    g_lock_file="$HOME/fi-preinstall.lock"                 #进程锁文件
    g_work_space="$HOME/fi-preinstall"                     #各节点的工作目录
    g_ctrl_space="$HOME/fi-preinstall-ctrl"                #各节点的工作目录 

    #非root用户执行命令
    CMD_RPM_INSTALL="sudo ${INSTALL_SUDO_PATH} install_rpm_package"
    CMD_RPM_QUERY="sudo ${INSTALL_SUDO_PATH} query_rpm_package"
    CMD_RM_RF="sudo ${INSTALL_SUDO_PATH} preinstall_rm_rf"
    CMD_CREATE_SOFTLINK="sudo ${INSTALL_SUDO_PATH} preinstall_create_softlink"
    CMD_PREINSTALL_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_PREINSTALL_CHMOD_R="sudo ${INSTALL_SUDO_PATH} preinstall_chmod_r"
    CMD_PREINSTALL_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
else
    g_pid_file="/var/run/fi-preinstall.pid"                #进程PID
    g_lock_file="/var/lock/fi-preinstall.lock"             #进程锁文件
    g_work_space="/opt/fi-preinstall"                      #各节点的工作目录
    g_ctrl_space="/opt/fi-preinstall-ctrl"                 #各节点的工作目录  
    
    #root用户执行命令
    CMD_RPM_INSTALL="rpm -iv"
    CMD_RPM_QUERY="rpm -q"
    CMD_RM_RF="rm -rf"
    CMD_CREATE_SOFTLINK="ln -sf"
    CMD_PREINSTALL_CP="cp -arf"
    CMD_PREINSTALL_CHMOD_R="chmod -R"
    CMD_PREINSTALL_CHOWN_R="chown -R"
    CMD_MKDIR_P="mkdir -p"
fi

g_core_dump=0                                              #是否开启core dump
g_core_dump_dir="/var/log/core"                            #用户指定的coredump dir
g_local_os=""                                              #本机的OS版本
g_mian_path="$(dirname "$0")"                              #preinstall.sh脚本所在目录
g_preinstall_path="${g_mian_path}/script/"                 #适配脚本所在目录
g_conf_file="${g_mian_path}/preinstall.ini"                #配置文件
g_conf_file_tmp="${g_mian_path}/preinstall.tmp.$$"         #转换后的配置文件
g_remote="${g_preinstall_path}/function/remote.sh"         #运行命令或拷贝文件
g_failed_log="${g_ctrl_space}/failed.log"                  #失败日志
g_aes_rand_key="$(date +%s%N)"                             #用于保存加密密码的随机key
g_passwd_aes_file="${g_ctrl_space}/preinstall-p.bin"       #用于保存密码密钥的文件
g_check_rpm_r="${g_work_space}/add_packages/get_missing_rpms_lst.sh" #获取缺失的rpm包列表
g_preconfigured_certificates="${g_mian_path}/tools/genCert/preconfigured_certificates.sh"
g_certpasswd_file="${g_mian_path}/cacert.ini"

g_result=0

updir="$(dirname $(cd "$(dirname "$0")";pwd))"
precheck="${updir}/precheck/precheck.sh"
config="${updir}/precheck/checkNodes.Config"
key_path="$g_mian_path/partition/ini-plugin/keys"         #分区加密的访问秘钥存放目录

PRECHECK=0          #强制执行precheck
UPGRADE=0           #强制升级硬盘管理服务
NONPRECHECK=0       #强制跳过precheck

COMMIT=0
ROLL_BACK=0
ACTION=""
ARGS=0
NONINTERACTIVE=0

###################################################
#
# 捕获信号后删除中间文件
# 
###################################################
function sig_handle()
{
    #删除中间文件
    rm "${g_ctrl_space}" -rf
    rm "${g_lock_file}" -f
    rm "${g_pid_file}" -f
    rm "${g_conf_file_tmp}" -f
    stty sane
    echo ""
    syslog "preinstall" "preinstall.sh" "exit=1" "preinstall.sh, catch signal and exit."
    exit 1
}
trap "sig_handle" INT TERM QUIT

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-c                    Notes: When the -c parameter is used, the precheck.sh script will always be executed after the preinstall.sh script is executed."
    echo "-n                    Notes: When the -n parameter is used, the precheck.sh script will not be executed after the preinstall.sh script is executed."
    echo "-u                    Notes: upgrade preinstall."
    echo "-r                    Notes: roll back preinstall."
    echo "-e                    Notes: commit preinstall."
    echo "-h                    Notes: show help."
    echo "                      Notes: By default,no input parameters, after the preinstall.sh script is successfully executed, the precheck.sh script will be executed automatically."
    echo
    echo "Example:"
    echo "`basename $0`"
    echo "`basename $0` -c"
    echo "`basename $0` -n"
    echo "`basename $0` -u"
    echo "`basename $0` -u -n"
    echo "`basename $0` -h"
    echo
    return 0
}

###################################################
#
# 更新partition配置文件
# 
###################################################
function update_partition()
{
    local src_path="${g_mian_path}/partition"
    local dst_path="${g_mian_path}/script/modules/070.autopart/autopart"
    
    #删除源文件
    rm "${dst_path}"/ini/* -rf
    rm "${dst_path}"/ini-plugin/* -rf
    
    #拷贝partition ini配置文件
    cp "${src_path}"/ini  "${dst_path}" -arf
    if [ $? -ne 0 ]; then
        echo "Copy partition configuration failed."
        return 1
    fi
    
    #拷贝partition ini配置文件
    cp "${src_path}"/ini-plugin  "${dst_path}" -arf
    if [ $? -ne 0 ]; then
        echo "Copy partition configuration failed."
        return 1
    fi
    
    return 0
}

###################################################
#
# 转换文件格式
# 
###################################################
function dos2unix_files()
{
    #转化配置文件格式
    install_dos2unix
    if [ $? -ne 0 ]; then
        echo "isntall dos2unix rpm failed."
        return 1
    fi
    
    find "${g_mian_path}" -type f | xargs dos2unix >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "dos2unix files failed."
        return 1
    fi
    
    return 0
}

###################################################
#
# 引入配置文件 及 脚本
# 
###################################################
function source_file()
{
    export g_aes_rand_key="${g_aes_rand_key}"
    export g_passwd_aes_file="${g_passwd_aes_file}"
    
    #去除配置文件中的\r字符
    cat "$g_conf_file" | grep -v "^[[:space:]]*$" | \
        grep -v "^#" | tr -d '\r' > "${g_conf_file_tmp}"
    
    #引入配置文件
    source "${g_conf_file_tmp}"
    if [ $? -ne 0 ]; then
        echo "source ${g_conf_file} failed"
        return 1
    fi
    
    while read line
    do
        source "$line"
        if [ $? -ne 0 ]; then
            echo "source ${line} failed"
            return 1
        fi
    done < <(find "${g_preinstall_path}/function/" -mindepth 1 -maxdepth 1 \
             -name "[0-9][0-9][0-9].*.sh" -type f | sort)
    
    return 0
} 

function ExePrecheck()
{
    echo ""
    echo ""
    echo -e "===========FusionInsight PreCheck is starting...==========="   
    "${precheck}" "${config}"
}

###################################################
#
# 记录审计日志
# 
###################################################
function syslog()
{
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0;

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}

cp_cacertfile_to_oms()
{
    local master_node="${oms_ip1}"    #远端主oms
    local standy_node="${oms_ip2}"    #远端备oms
    local ret=0           #返回值
    
    if [ ! -z ${master_node} ];then
        if [ "$UID" -ne 0 ]; then
            bash $g_remote -i "${master_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file  -t 240 -m "ssh-cmd" -c "${CMD_MKDIR_P} /opt/cacert/; ${CMD_PREINSTALL_CHOWN_R} omm:wheel /opt/cacert/" >/dev/null
            if [ $? -ne 0 ];then
                echo "failed to create the cacert path /opt/cacert/ on the active node."
                return 1
            fi
        fi
        
        bash $g_remote -i "${master_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file -t 240 -m scp-out -s /opt/cacert -d /opt/ >/dev/null
        ret=$?
        if [ $ret -ne 0 ];then
            echo "scp cacert files to active node failed."
            return 1
        fi
        
        bash $g_remote -i "${master_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file  -t 240 -m "ssh-cmd" -c "${CMD_PREINSTALL_CHMOD_R} 644 /opt/cacert/*" >/dev/null
        if [ $? -ne 0 ];then
            echo "failed to modify the cacert file permission on the active node."
            return 1
        fi
        
    fi
    if [ ! -z ${standy_node} ];then
        if [ "$UID" -ne 0 ]; then
            bash $g_remote -i "${standy_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file  -t 240 -m "ssh-cmd" -c "${CMD_MKDIR_P} /opt/cacert/; ${CMD_PREINSTALL_CHOWN_R} omm:wheel /opt/cacert/" >/dev/null
            if [ $? -ne 0 ];then
                echo "failed to create the cacert path /opt/cacert/ on the standy node."
                return 1
            fi
        fi
        
        bash $g_remote -i "${standy_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file -t 240 -m scp-out -s /opt/cacert -d /opt/ >/dev/null
        ret=$?
        if [ $ret -ne 0 ];then
            echo "scp cacert files to standy node failed."
            return 1
        fi
        
        bash $g_remote -i "${standy_node}" -F $g_ip_model -P $g_port -u $g_user_name -a $g_passwd_aes_file  -t 240 -m "ssh-cmd" -c "${CMD_PREINSTALL_CHMOD_R} 644 /opt/cacert/*" >/dev/null
        if [ $? -ne 0 ];then
            echo "failed to modify the cacert file permission on the standy node."
            return 1
        fi
    fi
}
###################################################
#
# 入口函数
#
###################################################
function main()
{
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"

    #删除并创建工作空间
    rm "${g_ctrl_space}" -rf
    mkdir -p "${g_ctrl_space}"
    if [ $? -ne 0 ]; then
        echo "init failed"
        return 1
    fi
    
    #更新partition配置文件
    update_partition
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #引入配置文件 及 脚本
    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #检查日志文件的权限
    check_logfile
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #配置参数预检查
    pre_check_ini
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #检查os是否支持
    g_local_os="$(get_osname "${g_support_os}")"
    if [ $? -ne 0 ]; then
        echo "$g_local_os"
        return 1
    fi
    
    #检查preinstall.ini配置文件
    check_ini
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #转换preinstall文件的格式为UNIX
    dos2unix_files
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #本地功能增强(补充rpm包等)
    enhance_local
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #配置WCE开关的标记
    set_wce "${g_wce_conf}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "set wce conf failed"
        return 1
    fi
    
    #输入kms服务器的密码
    if [ ${g_kms_conf} -eq 1 ]; then
        set_kms_passwd
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "set kms password failed"
            return 1
        fi
    fi
    
    #输入集群的密码(各节点密码是一致的)
    if [ "${NONINTERACTIVE}" -eq 0 ]; then
        set_passwd "$g_passwd_aes_file" 
    else
        set_passwd "$g_passwd_aes_file" "${NONINTERACTIVE}" "${PASSWD}"
    fi
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #清除屏幕显示的内容
    echo -e "\\033[2J"
    
    echo -e "\\033[1H\\033[1G**FusionInsight PreInstall is starting..."
        
    #检查是否可以成功登陆各节点
    check_login "$g_hosts"
    if [ $? -ne 0 ]; then        
        return 1
    fi
    
    
    if [ ${ARGS} -ne 0 ]; then
        #启动preinstall升级
        start_upgrade > "${g_failed_log}" 2>&1
        if [ $? -ne 0 ]; then
            return 1
        fi
    else
        #初始化进度数组
        init_schedule_array "${#g_ipaddr_array[@]}"
        #启动OS适配
        start_preinstall > "${g_failed_log}" 2>&1 &
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi
    
    #显示适配进度
    show_schedule
    Failure=$?

    #检查输入参数控制是否执行precheck
    if [ ${NONPRECHECK} -ne 0 ]; then
        #do nothing
        #总不执行precheck
        :
    elif [ ${PRECHECK} -eq 1 ]; then
        #总执行precheck
        ExePrecheck
    else
        #成功执行precheck
        if [ ${Failure} -eq 0 ];then
            ExePrecheck
        fi
    fi
    
    if [ ! -z ${oms_ip1} ] || [ ! -z ${oms_ip2} ];then
        ${g_preconfigured_certificates} ${g_certpasswd_file}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "preconfigured certificates failed"
            return 1
        fi
        #将证书文件拷贝至主备节点
        cp_cacertfile_to_oms
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "scp certificates to OMS failed"
            return 1
        fi
    fi
    
    echo -e "You can get more information about the preinstall from $g_log_file and ${HOME}/diskmgt/autopart.log"
    
    if [ ${Failure} -ne 0 ]; then
        return 1
    fi
}


#保证单进程运行
{
    syslog "preinstall" "preinstall.sh" "begin" "preinstall.sh $* begin."
    
    flock -n 3
    if [ $? -ne 0 ]; then
        if [ -f "${g_pid_file}" ]; then
            if [ -n "$(ps -wwef | grep "$(basename $0)" | grep "$(cat ${g_pid_file})" | grep -v grep)" ]; then
                echo "Error: $(basename $0) locked by another user."
                exit 1
            fi
        fi        
        echo "Warning: lock leaks"
    fi
    
    #生成pid文件
    echo "$$" > "${g_pid_file}"
    
    #脚本输入所有参数的处理流程
    while getopts "cnurehi:" options;
    do
        case ${options} in
            c)
                PRECHECK=1          #输入挂载配置文件
                ;;
            u)
                UPGRADE=1         #是否强制安装硬盘管理服务
                ACTION="UPGRADE"
                ;;
            r)
                ROLL_BACK=1         #是否强制安装硬盘管理服务
                ACTION="ROLL_BACK"
                ;;
            e)
                COMMIT=1         #是否强制安装硬盘管理服务
                ACTION="COMMIT"
                ;;
            n)
                NONPRECHECK=1     #是否强制安装硬盘管理服务
                ;;
            i)
                PASSWD="${OPTARG}"
                NONINTERACTIVE=1     #非交互式输入密码
                ;;    
            h)
                show_help         #显示帮助
                exit $?
                ;;
            *)
                echo "ERROR invalid input!!"  #非法参数输入
                show_help
                exit 1
                ;;
        esac
    done
    
    ARGS=$((PRECHECK+NONPRECHECK))
    if [ $ARGS -gt 1 ]; then
        #非法参数输入
        echo "ERROR invalid input!!"  
        show_help
        g_result=1
    else 
        ARGS=$((UPGRADE+ROLL_BACK+COMMIT))
        if [ $ARGS -gt 1 ]; then
            #非法参数输入
            echo "ERROR invalid input!!"  
            show_help
            g_result=1
        else
            main "$@"
            g_result=$?
        fi
    fi
    
    rm "${g_ctrl_space}" -rf
    rm "${g_lock_file}" -f
    rm "${g_pid_file}" -f
    rm "${g_conf_file_tmp}" -f
    
    #执行成功后，删除加密分区的访问秘钥
    if [ "$g_result" -eq 0 ]; then
        rm -f "$key_path/fi_disk_key_1"
        rm -f "$key_path/fi_disk_key_2"
    fi
    
    syslog "preinstall" "preinstall.sh" "exit=${g_result}" "preinstall.sh $* end."
    exit "${g_result}"
} 3<>"${g_lock_file}"
