#!/bin/bash

###################################################
#
# 功能:  在远端服务器安装单个rpm包
#
###################################################
function install_rpm_r()
{
    local ipaddr="$1"          #主机名或ip地址
    local rpms_dir="$2"        #rpm包目录
    local name="$3"            #rpm包名
    local path="$4"            #拷贝rpm包的目标目录

    debug "[${ipaddr}:${rpms_dir}:${name}:${path}]Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr is null"
        return 1
    fi

    #检查rpm包是否已经安装
    exec_cmd_r "$ipaddr" "120" "rpm -q ${name}"
    if [ $? -ne 0 ]; then
        #获取rpm包文件
        file="$(get_rpm_file "${rpms_dir}" "${name}" "0")"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "get_rpm_file ${rpms_dir} ${name} failed"
            return 1
        fi

        #把rpm包拷贝至远端服务器
        cp_file_to_r "$ipaddr" "240" "$file" "${path}/$(basename "$file")"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Copy [${file}] to $ipaddr failed"
            return 1
        fi

        #在远端服务器上安装rpm包
        exec_cmd_r "$ipaddr" "120" "${CMD_RPM_INSTALL} \"${path}/$(basename "$file")\""
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "rpm -ivh $file failed(${name}) on ${ipaddr}"
            return 1
        fi

        #删除中间文件
        exec_cmd_r "$ipaddr" "120" "rm -f \"${path}/$(basename "$file")\""
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "rm -f $file failed(${name}) on ${ipaddr}"
            return 1
        fi
    fi

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

function process_ssh()
{
    local ipaddr=$1  #主机名或ip地址
    exec_cmd_r "$ipaddr" "120" "ls /etc/euleros-release"
    if [ $? -eq 0 ]
    then
        exec_cmd_r "$ipaddr" "120" "sed -i 's/^HostKeyAlgorithms/#HostKeyAlgorithms/' /etc/ssh/sshd_config"
        exec_cmd_r "$ipaddr" "120" "sed -i 's/^PubkeyAcceptedKeyTypes/#PubkeyAcceptedKeyTypes/' /etc/ssh/sshd_config"
	exec_cmd_r "$ipaddr" "120" "service sshd restart"
    fi
}

###################################################
#
# 功能:  在远端安装ssh命令
#
###################################################
function install_ssh_r()
{
    local ipaddr="$1"  #主机名或ip地址
    local ret=0        #返回码
    local osname=""    #OS名称
    local rpms_dir=""  #rpm包目录

    debug "[${ipaddr}]Enter [${FUNCNAME}:${LINENO}]"

    #入参检查
    if [ -z "$ipaddr" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Parameter error. ipaddr is null."
        return 1
    fi

    #检查远端服务器是否有ssh命令
    exec_cmd_r "$ipaddr" "120" "which ssh"
    ret=$?

    if [ $ret -eq 0 ]; then
        #对端有ssh命令,无需安装 
	process_ssh "$ipaddr"
        return 0
    fi

    #其他异常 比如 密码错误 超时 等
    if [ $ret -ne 1 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Connect [${ipaddr}] failed"
        return 1
    fi
    
    #获取远端的OS版本
    osname="$(get_osname_r "${ipaddr}")"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "get osname(${osname}) from ${ipaddr} failed"
        return 1
    fi
    
    #SuSE版本不存在此场景
    echo "${osname}" | grep "^suse" > /dev/null
    if [ $? -eq 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "${osname} in ${ipaddr}. Please install ssh artificially"
        return 1
    fi
    
    #获取rpm包目录
    rpms_dir=$(get_pkg_dir_by_name "${osname}")
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "get pkg dir by name(${osname}) failed"
        return 1
    fi
    
    debug "[${osname}:${rpms_dir}][${FUNCNAME}:${LINENO}]"

    #先安装 libedit.x86_64 (openssh-clients.x86_64依赖此包)

    if [ `uname -p` == "aarch64" ]; then
       install_rpm_r "$ipaddr" "$rpms_dir" "libedit.aarch64" "${g_work_space}"
    else
       install_rpm_r "$ipaddr" "$rpms_dir" "libedit.x86_64" "${g_work_space}"
    fi

    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "install libedit failed in ${ipaddr}"
        return 1
    fi

    #再安装 openssh-clients.x86_64 (ssh scp命令,在此包中)
    if [ `uname -p` == "aarch64" ]; then
       install_rpm_r "$ipaddr" "$rpms_dir" "openssh-clients.aarch64" "${g_work_space}"
    else
       install_rpm_r "$ipaddr" "$rpms_dir" "openssh-clients.x86_64" "${g_work_space}"
    fi

    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "install openssh-clients failed in ${ipaddr}"
        return 1
    fi

    process_ssh "$ipaddr"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
