#!/bin/bash

###################################################
# 功能: 显示适配进度
###################################################

###################################################
# g_schedule_array: 用于保存 个节点进度的数组
#                   进度范围 (0-101)
#                   0-99 表示正在运行
#                   100  表示成功完成
#                   101  表示失败
###################################################
declare -a g_schedule_array=()
g_host_succ="100"      #节点适配成功
g_host_failed="101"    #节点适配失败

g_show_result=""
g_show_fail_list_result=""
g_show_fail_log_result=""

###################################################
#
# 设置节点进度
#
###################################################
function set_node_schedule()
{
    local ipaddr="$1"
    local value="$2"
    local file="${g_ctrl_space}/sched_${ipaddr}"
    local sched=0

    debug "[${ipaddr}:${value}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "${ipaddr}" ] || [ -z "${value}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Set node schedule failed"
        return 1
    fi

    #对于已经完成(成功或失败)的任务,不能再调整其进度
    if [ -f "${file}" ]; then
        sched="$(cat ${file})"
        if [ "$sched" == "$g_host_succ" ] || [ "$sched" == "$g_host_failed" ]; then
            info "[${FUNCNAME}:${LINENO}]" "Don't update schedule ip:${ipaddr}, v:${value}"
            return 0
        fi
    fi

    #进度值大于最大值的调整为最大值
    if [ "$value" -ge "$g_host_failed" ]; then
        value=$g_host_failed
    fi

    #覆盖式写入文件(无并发写的场景)
    echo "$value" > "${file}"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 初始化进程数组的值为0
#
###################################################
function init_processor_array()
{
    local array_len="$1"
    local i=0

    debug "[${array_len}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "${array_len}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Set schedule array failed"
        return 1
    fi
    
    for ((i=0;i<${array_len};i++))
    do
        g_processor_array[${i}]=0
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 初始化进度数组的值为0
#
###################################################
function init_schedule_array()
{
    local array_len="$1"
    local i=0

    debug "[${array_len}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "${array_len}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Set schedule array failed"
        return 1
    fi
    
    for ((i=0;i<${array_len};i++))
    do
        g_schedule_array[${i}]=0
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 设置节点运行失败
#
###################################################
function set_node_run_failure()
{
    local ipaddr="$1"
    local file="${g_ctrl_space}/sched_${ipaddr}"

    debug "[${ipaddr}] Enter [${FUNCNAME}:${LINENO}]"

    if [ -z "${ipaddr}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "Set node schedule failed"
        return 1
    fi

    #覆盖式写入文件(无并发写的场景)
    echo "$g_host_failed" > "${file}"    

    #删除节点上的工作空间
    del_workspace "${g_work_space}" "${ipaddr}" > /dev/null 2>&1

    debug "[${ipaddr}] Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 组合进度字符串
#
###################################################
function combine_schedule()
{
    local loop="$1"
    local status="$2"
    local ratio=0
    local ratio_c=0
    local run_ratio=0
    local run_num=0
    local suc_num=0
    local fai_num=0
    local i=""           #循环变量
    local file=""
    local file_c=""
    local host_num=0     #节点个数
    local proc=0
    local fail_list=""
    
    local windows_x=0

    host_num=${#g_hosts_array[@]}

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"

        if [ -z "$ipaddr" ]; then
            continue
        fi
        
        #初始化变量
        ratio=${g_schedule_array[${i}]}
        ratio_c=0
        file_c="${g_ctrl_space}/sched_c_${ipaddr}"
        file="${g_ctrl_space}/sched_${ipaddr}"
        
        if [ ${g_schedule_array[${i}]} -lt 100 ]; then
            #根据客户端的进度更新进度
            if [ -e "${file_c}" ]; then
                ratio_c="$(cat ${file_c})"
            fi
            if [ -e "${file}" ]; then
                #存在并发读写的场景,正在写的时候,读出来的内容为空
                #不会出现读出一半数据的场景
                #当读出的数据为空时,以上次读取的内容为准
                ratio="$(cat ${file})"
                if [ -z "$ratio" ]; then
                    #以上次读取的内容为准
                    ratio="${g_schedule_array[${i}]}"
                fi
                if [ -n "$ratio_c" ] && [ $ratio_c -gt $ratio ]; then
                    ratio=$ratio_c
                fi
                    
                #保存本次读取的数据
                g_schedule_array[${i}]=$ratio

                if [ $ratio -lt $g_host_succ ]; then
                    #判断进程是否存在
                    if [ ! -e "/proc/${g_processor_array[${i}]}" ]; then
                        info "[${FUNCNAME}:${LINENO}]" "[${ipaddr}:${ratio}%] proc is not exist"
                        #进程不存在 把对应的适配标识为失败
                        echo "$g_host_failed" > "${file}"
                        g_schedule_array[${i}]=$g_host_failed
                        ratio=$g_host_failed
                        info "[${FUNCNAME}:${LINENO}]" "[${ipaddr}:${ratio}%] Failed"
                    fi
                fi
            else
                #没有生成进度文件时,以数组中记录的进度为准
                ratio="${g_schedule_array[${i}]}"
            fi
        fi

        if [ $ratio -eq 100 ]; then
            ((suc_num++))
            ((run_ratio+=100))
            continue
        fi

        if [ $ratio -gt 100 ]; then
            ((fai_num++))
            ((run_ratio+=100))
            fail_list="${fail_list} ${g_ipaddr_array[${i}]}"
            continue
        fi
        ((run_num++))
        ((run_ratio+=$ratio))
    done

    ratio=$((${run_ratio}/${host_num}))

    #显示汇总信息
    if [ -n "${status}" -a "${status}" == "end" ]; then
        g_show_result="
**********************************
*****FusionInsight Preinstall*****
**********************************
***** Time:${SECONDS}s
***** Running:${run_num}
***** Success:${suc_num}
***** Failure:${fai_num}
***** Total:${host_num}
***** Schedule:${ratio}%
"
    else
        windows_x=$(tput lines)
        if [ -n "${windows_x}" -a $((fai_num + 11)) -ge $windows_x ]; then
            g_show_result="\\033[2J\\033[1H\\033[1G**********************************"
        else
            g_show_result="\\033[1H\\033[1G**********************************"
        fi
        g_show_result="${g_show_result}\\033[2H\\033[1G*****FusionInsight Preinstall***** \
\\033[3H\\033[1G********************************** \
\\033[4H\\033[1G***** Time:${SECONDS}s \
\\033[5H\\033[1G***** Running:${run_num} \
\\033[6H\\033[1G***** Success:${suc_num} \
\\033[7H\\033[1G***** Failure:${fai_num} \
\\033[8H\\033[1G***** Total:${host_num} \
\\033[9H\\033[1G***** Schedule:${ratio}%
"
    fi
    
    #显示失败的ip地址列表
    if [ -n "${fail_list}" ]; then
        if [ -n "${status}" -a "${status}" == "end" ]; then
            g_show_result="${g_show_result}***** Failure List:
**${fail_list}
"
        else
            g_show_result="${g_show_result}\\033[10H\\033[K\\033[1G***** Failure List:"
            g_show_result="${g_show_result}\\033[11H\\033[K\\033[1G**${fail_list}\n"
        fi
    fi

    #失败日志
    if [ -f "${g_failed_log}" ] && [ -n "$(cat ${g_failed_log})" ]; then
            g_show_result="${g_show_result}$(cat ${g_failed_log})"
    fi
    
    if [ -z "${status}" -o "${status}" != "end" ]; then
        echo -e "$g_show_result"
    fi

    if [ $run_num -eq 0 ]; then
        if [ $fai_num -gt 0 ];then
           return 10    #执行完成,但有失败节点
        else
           return 0     #执行完成,节点全部成功
        fi        
    fi

    return 1
}

###################################################
#
# ssh到每一个执行端获取进度信息
#
###################################################
function get_schedule()
{
    local loop="$1"
    local i=""           #循环变量
    local ipaddr=""
    local ratio=0
    local ratio_c=0
    local file_c=""
    local file=""
    
    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        #5秒钟从各节点拷贝进度文件至本端
        if [ -n "$loop" ] && [ $((${loop} % 5)) -eq 0 ]; then
            ipaddr="${g_ipaddr_array[${i}]}"
        
            #初始化变量
            ratio="${g_schedule_array[${i}]}"
            if [ ${ratio} -ge 100 ]; then
                continue
            fi
        
            ratio_c=0
            file_c="${g_ctrl_space}/sched_c_${ipaddr}"
            file="${g_ctrl_space}/sched_${ipaddr}"
        
            #根据客户端的进度更新进度
            if [ -e "${file_c}" ]; then
                ratio_c="$(cat ${file_c})"
            fi
            if [ -e "${file}" ]; then
                ratio="$(cat ${file})"
            fi
        
            #那个大就使用那个
            if [ -n "$ratio_c" ] && [ $ratio_c -gt $ratio ]; then
                ratio=$ratio_c
            fi
        
            if [ -n "$ratio" -a $ratio -ge 100 ]; then
                #节点已经完成了，不需要在获取进度信息了
                continue
            fi
        
            #拷贝失败不显示日志
            cp_file_from_r "$ipaddr" "240" "${g_work_space}/modules/sched_c_${ipaddr}" "${g_ctrl_space}" "0" > /dev/null
        fi
    done
    
    return 0
}

###################################################
#
# 显示最后结果的信息，并打印到日志中，防止日志太多被冲掉
#
###################################################
function show_last_schedule()
{
    combine_schedule "1" "end"
    info "[${FUNCNAME}:${LINENO}]" "$g_show_result"
}

###################################################
#
# 启动OS适配: 共分为6个步骤
#  1:在各节点创建工作目录        /tmp/fi-preinstall
#  1:把适配脚本拷贝至各节点     拷贝至 /tmp/fi-preinstall
#  2:在各节点调用启动适配脚本   /tmp/fi-preinstall/modules/install.sh
#
###################################################
function show_schedule()
{
    local i=0
    local last_job=0
    #输出进度
    while true
    do
        #第一次刷新一下屏幕
        if [ ${i} -eq 0 ]; then
            #清除屏幕显示的内容
            echo -e "\\033[2J"
            echo -ne "\\033[1H\\033[1G**********************************"
            echo -ne "\\033[2H\\033[1G*****FusionInsight Preinstall*****"
            echo -ne "\\033[3H\\033[1G**********************************"
            echo -ne "\\033[4H\\033[1G***** Time:${SECONDS}s"
        fi
        
        if [ $last_job -eq 0 -o ! -e /proc/$last_job ]; then
            get_schedule "${i}" &
            last_job=$!
        fi

        #组合并输出字符串
        combine_schedule "${i}"
        tmp=$?
        if [ $tmp -eq 0 -o $tmp -eq 10 ]; then
            #将屏显信息打印到日志中，防止打印太多屏幕放不下被冲掉
            show_last_schedule
            wait
            return $tmp
        fi
        ((i++))

        #休眠1秒
        sleep 1
    done
}

