#!/bin/bash
###################################################
# 功能: 进行OS设置
###################################################

###################################################
# 入口函数 
# 备注:
#   模块文件夹规则: 三位的数字 + . + 名字
#                   名字只能是英文和下划线
#   例如: 050.setup 中的名字为 setup
#
#   入口函数规则:   模块文件夹的[名字] + _main
###################################################
function setup_main()
{
    local path="$1"
    local name=""
    local file=""
    local system_uname=""

    debug "[${path}]Enter [${FUNCNAME}:${LINENO}]"

    name="$(get_osname_c)"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "get_osname_c failed [${name}]"
        return 1
    fi

    debug "[${name}] [${FUNCNAME}:${LINENO}]"

    if [ "$gc_platform" != "upgrade" ]; then
        system_uname=$(uname -p)
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "'uname -p' Command execution failed"
            return 1
        fi

        if [ X"$system_uname" != X"$gc_platform" ]; then
            error "[${FUNCNAME}:${LINENO}]" "The processor type [${system_uname}] is inconsistent with [${gc_platform}] configured in preinstall.ini"
            return 1
        fi
    fi

    if [ -n "$(echo ${name} | grep -E "^redhat-|^centos-|^euleros-|^neokylin-|^kylin-|^bclinux-|^oracle-|^openEuler-|^uos-|^fusionos-|^kylinSecOS-|^LinxOS-|^hce-")" ]; then
        name="redhat"
    fi

    if [ -n "$(echo ${name} | grep "^suse-")" ]; then
        name="suse"
    fi

    file="${gc_mian_path}/${path}/${name}/setup.sh"

    if [ ! -f "${file}" ]; then
        info "[${FUNCNAME}:${LINENO}]" "Don't need to setup about ${name}"

        #设置进度
        echo "15" > "${gc_sched_file}"

        return 0
    fi

    #运行设置OS的脚本
    if [ -z "${gc_hostname}" ]; then
        bash ${file} -w "${gc_swap_off}" -v "${gc_ip_model}"
    else
        bash ${file} -w "${gc_swap_off}" -s "${gc_services_ip}" -n "${gc_hostname}" -v "${gc_ip_model}"
    fi
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "run ${file} failed"
        return 50
    fi

    #设置进度
    echo "15" > "${gc_sched_file}"

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}
