#!/bin/bash

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_REDHAT_CONFIG_LANGUAGE="sudo ${INSTALL_SUDO_PATH} redhat_config_language"
    CMD_SERVICES_CTL="sudo ${INSTALL_SUDO_PATH} services_ctl"
    CMD_CHKCONFIG_CTL="sudo ${INSTALL_SUDO_PATH} chkconfig_ctl"
    CMD_CAT_SYSCTL="sudo ${INSTALL_SUDO_PATH} cat_sysctl"
    CMD_REPLACE_SYSCTL="sudo ${INSTALL_SUDO_PATH} replace_sysctl"
    CMD_EFFECTIVE_SYSCTL="sudo ${INSTALL_SUDO_PATH} effective_sysctl"
    CMD_CONFIG_LIMITS="sudo ${INSTALL_SUDO_PATH} config_new_limits"
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
    CMD_REPLACE_FSTAB="sudo ${INSTALL_SUDO_PATH} replace_fstab"
    CMD_REMOUNT_CTL="sudo ${INSTALL_SUDO_PATH} remount_ctl"
    CMD_UPDATE_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_update_etc_hosts"
    CMD_REMOVE_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_remove_etc_hosts"
    CMD_REMOVE_HOSTNAME_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_remove_hostname_etc_hosts"
    CMD_COMPRESS_IPV6="sudo ${INSTALL_SUDO_PATH} compress_ipv6"
    CMD_UPDATE_ETC_HOSTNAME="sudo ${INSTALL_SUDO_PATH} preinstall_redhat_update_etc_hostname"
    CMD_SWAP_OFF="sudo ${INSTALL_SUDO_PATH} preinstall_swap_off"
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
    CMD_SWAPON_S="sudo ${INSTALL_SUDO_PATH} preinstall_swapon_s"
    CMD_READLINK_E="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_e"
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_SYSTEM_CTL="sudo ${INSTALL_SUDO_PATH} system_ctl"
    CMD_DISABLE_SELINUX="sudo ${INSTALL_SUDO_PATH} disable_selinux"
    CMD_CREATSOFTLINK_LIBBOOST="sudo ${INSTALL_SUDO_PATH} preinstall_creatsoftlink"
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_INVALIDATE_CGROUP="sudo ${INSTALL_SUDO_PATH} preinstall_invalidate_cgroup"
else
    CMD_REDHAT_CONFIG_LANGUAGE="redhat_config_language"
    CMD_SERVICES_CTL="service"
    CMD_CHKCONFIG_CTL="chkconfig"
    CMD_CAT_SYSCTL="cat /etc/sysctl.conf"
    CMD_REPLACE_SYSCTL="replace_sysctl"
    CMD_EFFECTIVE_SYSCTL="sysctl -q -p"
    CMD_CONFIG_LIMITS="config_new_limits"
    CMD_CAT_FSTAB="cat /etc/fstab"
    CMD_REPLACE_FSTAB="replace_fstab"
    CMD_REMOUNT_CTL="mount -o errors=panic,remount"
    CMD_UPDATE_ETC_HOSTS="preinstall_update_etc_hosts"
    CMD_REMOVE_ETC_HOSTS="preinstall_remove_etc_hosts"
    CMD_REMOVE_HOSTNAME_ETC_HOSTS="preinstall_remove_hostname_etc_hosts"
    CMD_COMPRESS_IPV6="compress_ipv6"
    CMD_UPDATE_ETC_HOSTNAME="preinstall_redhat_update_etc_hostname"
    CMD_SWAP_OFF="swapoff"
    CMD_BLKID="blkid"
    CMD_SWAPON_S="swapon -s"
    CMD_READLINK_E="readlink -e"
    CMD_FINDFS="findfs"
    CMD_SYSTEM_CTL="systemctl"
    CMD_DISABLE_SELINUX="disable_selinux"
    CMD_CREATSOFTLINK_LIBBOOST="preinstall_creatsoftlink"
    CMD_CHMOD="chmod"
    CMD_INVALIDATE_CGROUP="preinstall_invalidate_cgroup"
fi

GC_SERVICES_IP=""       #保存本机的services ip信息
GC_HOSTNAME=""          #保存本机的主机名信息
GC_SWAP_OFF=0           #是否关闭主动swap分区，默认不关闭
GC_IP_MODEL=""          #ip地址格式
gc_ipaddr=""            #日志打印的IP地址

###################################################
# 输出warning日志
###################################################
function warning()
{
    #直接使用echo 输出warning日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [WARNING]" "$@"
}

###################################################
# 输出error日志
###################################################
function error()
{
    #直接使用echo 输出error日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [ERROR]" "$@"
}

###################################################
# 输出info日志
###################################################
function info()
{
    #直接使用echo 输出info日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [INFO]" "$@"
}

###################################################
# 输出debug日志
###################################################
function debug()
{
    #直接使用echo 输出debug日志 服务端会重定向该日志
    if [ "${gc_debug}" -eq 1 ]; then
        echo "[$(date)][${gc_ipaddr}] [DEBUG]" "$@"
    fi
}

##########################################################
#  sudo封装函数
##########################################################
function redhat_config_language()
{
    local lang_file=/etc/sysconfig/i18n

    #redhat 7.X
    which localectl > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        localectl set-locale LANG=en_US.UTF-8 > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "localectl command failed."
            return 1
        fi

        return 0
    fi

    #redhat 6.X
    if [ ! -f ${lang_file} ]; then
        error "[${FUNCNAME}:${LINENO}]" "language file does not exist."
        return 1
    fi

    grep "^LANG=" ${lang_file} > /dev/null
    if [ $? -ne 0 ]; then
        sed -i "1 a\LANG=\"en_US.UTF-8\"" ${lang_file}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Insert LANG as UTF-8 failed."
            return 1
        fi
    else
        sed -i "s/^LANG=.*/LANG=\"en_US.UTF-8\"/g" ${lang_file}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Instead LANG as UTF-8 failed."
            return 1
        fi
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function replace_sysctl()
{
    local replace_path="$1"
    local target_path="/etc/sysctl.conf"

    #检查配置文件
    if [ ! -f ${replace_path} ]; then
        error "[${FUNCNAME}:${LINENO}]" "${replace_path} is not exist."
        return 1
    fi

    cat "$1" > ${target_path}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "cat $1 > ${target_path} failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function config_new_limits()
{
    local path="/etc/security/limits.d"
    local nproc="${path}/90-nproc.conf"
    local nofile="${path}/90-nofile.conf"
    local memlock="${path}/omm-memlock.conf"
    local omm_nproc="${path}/omm-nproc.conf"

    #创建limits.d目录
    if [ ! -d "${path}" ]; then
        mkdir -p ${path}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "mkdir -p ${path} failed"
            return 1
        fi
    fi

    echo "*          soft    nproc     60000" > ${nproc}
    echo "root       soft    nproc     unlimited">> ${nproc}

    info "[${FUNCNAME}:${LINENO}]" "configure nproc ok."

    echo "*          hard    nofile    640000" > ${nofile}
    echo "*          soft    nofile    640000" >> ${nofile}

    info "[${FUNCNAME}:${LINENO}]" "configure nofile ok."

    echo "omm        hard    memlock    unlimited" > ${memlock}
    echo "omm        soft    memlock    unlimited" >> ${memlock}

    info "[${FUNCNAME}:${LINENO}]" "configure memlock ok."

    echo "omm        hard    nproc     60000" > ${omm_nproc}
    echo "omm        soft    nproc     60000" >> ${omm_nproc}

    info "[${FUNCNAME}:${LINENO}]" "configure omm_nproc ok."

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function replace_fstab()
{
    local replace_path="$1"
    local target_path="/etc/fstab"

    #检查配置文件
    if [ ! -f ${replace_path} ]; then
        error "[${FUNCNAME}:${LINENO}]" "${replace_path} is not exist."
        return 1
    fi

    cat "${replace_path}" > "${target_path}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "cat ${replace_path} > ${target_path} failed"
        return 1
    fi

    return 0
}


##########################################################
#  配置服务
##########################################################
function config_service()
{
    local start_service_array=(crond sshd ipmi ipmievd sssd ntpd rngd haveged)
    local stop_service_array=(cgconfig iptables firewalld)
    local service_name=""

    for element in $(seq 0 $((${#start_service_array[@]} - 1)))
    do
        service_name="${start_service_array[$element]}"
        if [ -z "${service_name}" ]; then
            continue
        fi

        if [ -e "/usr/lib/systemd/system/${service_name}.service" ]; then
            ${CMD_SYSTEM_CTL} start "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl start ${service_name} failed."
            fi

            ${CMD_SYSTEM_CTL} enable "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl enable ${service_name} failed."
            fi

        fi

        if [ -e "/etc/init.d/${service_name}" ]; then
            ${CMD_SERVICES_CTL} "${service_name}" start > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute service ${service_name} start failed."
            fi

            ${CMD_CHKCONFIG_CTL} "${service_name}" on > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute chkconfig ${service_name} on failed."
            fi
        fi

    done


    for element in $(seq 0 $((${#stop_service_array[@]} - 1)))
    do
        service_name="${stop_service_array[$element]}"
        if [ -z "${service_name}" ]; then
            continue
        fi

        if [ -e "/usr/lib/systemd/system/${service_name}.service" ]; then
            ${CMD_SYSTEM_CTL} stop "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl stop ${service_name} failed."
            fi

            ${CMD_SYSTEM_CTL} disable "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl disable ${service_name} failed."
            fi
        fi

        if [ -e "/etc/init.d/${service_name}" ]; then
            ${CMD_SERVICES_CTL} "${service_name}" stop > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute service ${service_name} stop failed."
            fi

            ${CMD_CHKCONFIG_CTL} "${service_name}" off > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute chkconfig ${service_name} off failed."
            fi
        fi
    done

    return 0
}

##########################################################
#  修改 LANG 为 en_US.UTF-8
##########################################################
function config_language ()
{
    ${CMD_REDHAT_CONFIG_LANGUAGE}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "config language failed."
        return 1
    fi

    info "[${FUNCNAME}:${LINENO}]" "config LANG=\"en_US.UTF-8\" language ok"

    return 0
}

##########################################################
#  实际修改配置
##########################################################
function configSysctlConf()
{
    local config_name=${1};
    local config_value=${2};
    local config_file=${3};
    grep "^[[:space:]]*${config_name}[[:space:]]*=" ${config_file} > /dev/null
    if [ $? -ne 0 ]; then
        echo "${config_name} = ${config_value}" >> ${config_file}
    else
        sed -i "s/^[[:space:]]*${config_name}[[:space:]]*=.*/${config_name} = ${config_value}/g" ${config_file}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Modify ${config_file} failed for ${config_name}."
            return 1
        fi
    fi
    info "[${FUNCNAME}:${LINENO}]" "config ${config_name} = ${config_value} ok"
    return 0
}

################################################################################
# Function: invalidate cgroup
# Description: invalidate cgroup
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function preinstall_invalidate_cgroup()
{
    local config_name="DefaultInvalidateCgroup";
    local config_value="no";
    local config_file="/etc/systemd/system.conf";
    grep "^[[:space:]]*${config_name}[[:space:]]*=" ${config_file} > /dev/null
    if [ $? -ne 0 ]; then
        echo "${config_name} = ${config_value}" >> ${config_file}
    else
        sed -i "s/^[[:space:]]*${config_name}[[:space:]]*=.*/${config_name} = ${config_value}/g" ${config_file}
        if [ $? -ne 0 ]; then
            return 1
        fi
    fi
    return 0
}

##########################################################
#  配置 /etc/sysctl.conf
##########################################################
function config_sysctl()
{
    local sysctl=/etc/sysctl.conf
    local tmpSysctlFile="${HOME}/sysctl.conf.$(date +%s%N)"
    local kernelPath="/proc/sys/kernel"

    #生成临时sysctl.conf文件
    ${CMD_CAT_SYSCTL} > "${tmpSysctlFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup sysctl to ${tmpSysctlFile} failed"
        return 1
    fi

    #设置 kernel.sysrq
    if [ -f "${kernelPath}/sysrq" ]; then
        if ! configSysctlConf "kernel.sysrq" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.sysrq is not support,skip it "
    fi

    #设置 kernel.kstack_depth_to_print
    if [ -f "${kernelPath}/kstack_depth_to_print" ]; then
        if ! configSysctlConf "kernel.kstack_depth_to_print" "20" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.kstack_depth_to_print is not support,skip it "
    fi

    #设置 kernel.panic_on_oops
    if [ -f "${kernelPath}/panic_on_oops" ]; then
        if ! configSysctlConf "kernel.panic_on_oops" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.panic_on_oops is not support,skip it "
    fi

    #设置 kernel.panic
    if [ -f "${kernelPath}/panic" ]; then
        if ! configSysctlConf "kernel.panic" "5" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.panic is not support,skip it "
    fi
    

    #设置 kernel.hung_task_timeout_secs
    if [ -f "${kernelPath}/hung_task_timeout_secs" ]; then
        if ! configSysctlConf "kernel.hung_task_timeout_secs" "3600" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.hung_task_timeout_secs is not support,skip it "
    fi

    #设置 kernel.hung_task_panic
    if [ -f "${kernelPath}/hung_task_panic" ]; then
        if ! configSysctlConf "kernel.hung_task_panic" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.hung_task_panic is not support,skip it "
    fi

    #设置 vm.panic_on_oom
    if ! configSysctlConf "vm.panic_on_oom" "2" ${tmpSysctlFile};then
        return 1
    fi

    #设置 vm.oom_dump_tasks
    if ! configSysctlConf "vm.oom_dump_tasks" "1" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh1
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh1" "4096" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh2
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh2" "8192" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh3
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh3" "20480" ${tmpSysctlFile};then
        return 1
    fi

    #设置 kernel.softlockup_panic
    if [ -f "${kernelPath}/softlockup_panic" ]; then
        if ! configSysctlConf "kernel.softlockup_panic" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.softlockup_panic is not support,skip it "
    fi

    #注销 net.bridge.bridge 配置
    sed -i 's/^[[:space:]]*net.bridge.bridge.*/#&/g' $tmpSysctlFile
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "sed for bridge failedd."
        return 1
    fi

    info "[${FUNCNAME}:${LINENO}]" "config #net.bridge.bridge ok"

    #设置 vm.swappiness
    if ! configSysctlConf "vm.swappiness" "1" ${tmpSysctlFile};then
        return 1
    fi

    #设置 fs.file-max
    if ! configSysctlConf "fs.file-max" "640000" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.tcp_max_syn_backlog
    if ! configSysctlConf "net.ipv4.tcp_max_syn_backlog" "8192" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.core.somaxconn
    if ! configSysctlConf "net.core.somaxconn" "2048" ${tmpSysctlFile};then
        return 1
    fi

    #设置 fs.inotify.max_user_watches
    if ! configSysctlConf "fs.inotify.max_user_watches" "100000" ${tmpSysctlFile};then
        return 1
    fi

    #设置system.conf
    if [ -f "/etc/euleros-release" ]; then
        local versionnum=$(cat /etc/euleros-release | sed 's/x86_64//g' | awk -F " |SP|)" '{print $5}')
        if [[ $versionnum -ge 8 ]];then
            if ! ${CMD_INVALIDATE_CGROUP} ;then
                return 1
            fi
            systemctl daemon-reload
        fi
    fi  
    ${CMD_REPLACE_SYSCTL} "${tmpSysctlFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "replace sysctl ${tmpSysctlFile} failed"
        rm "${tmpSysctlFile}" -f
        return 1
    fi

    rm "${tmpSysctlFile}" -f

    #使上面的设置直接生效,不用定向到null
    ${CMD_EFFECTIVE_SYSCTL}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "run command sysctl -q -p failed"
        return 1
    fi
  

    return 0
}

##########################################################
#  配置 /etc/security/limits.d/*
##########################################################
function config_limits()
{
    ${CMD_CONFIG_LIMITS}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "config limits failed"
        return 1
    fi

    return 0
}

##########################################################
# 更新fstab
##########################################################
function update_fstab()
{
    local update=0
    local content=""
    local content2=""
    local newCont=""
    local newCont2=""
    local option=""
    local tmpFile="${HOME}/fstab.$(date +%s%N)"
    local kylin_relase="/etc/kylin-release"
    #生成临时fstab文件
    ${CMD_CAT_FSTAB} > "${tmpFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup fstab failed"
        return 1
    fi

    #只更新特定目录
    for point in / /boot /home /opt /tmp /var /var/log
    do
        if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}" |awk -F " " '{print $1}')" ]; then
            if grep -q "/data" "/etc/fstab";then
               continue
            fi
        fi
        if [ -z "${point}" ]; then
            continue
        fi

        content=$(${CMD_CAT_FSTAB} | grep -vw '^[ ]*#' | grep " ${point} ")
        if [ -z "${content}" ]; then
            point="${point}/"
            content=$(${CMD_CAT_FSTAB} | grep -vw '^[ ]*#' | grep " ${point} ")
            if [ -z "${content}" ]; then
                continue
            fi
        fi

        if ! echo "${content}" | grep -E " ext3 | ext4 "; then
            continue
        fi

        echo "${content}" | grep "errors=panic" > /dev/null
        if [ $? -eq 0 ]; then
            continue
        fi

        option=$(echo "${content}" | awk '{print $4}')
        newCont="$(echo "${content}" | sed "s/${option}/${option},errors=panic/g")"

        #把 / 替换为 \/ 即增加转义字符
        content2="$(echo "${content}" | sed 's/\//\\\//g')"
        newCont2="$(echo "${newCont}" | sed 's/\//\\\//g')"

        sed -i "s/${content2}/${newCont2}/g" "${tmpFile}"
        if [ $? -ne 0 ]; then
            rm "${tmpFile}" -f
            error "[${FUNCNAME}:${LINENO}]" "sed -i s/${content2}/${newCont2}/g ${tmpFile} failed"
            error "[${FUNCNAME}:${LINENO}]" "update /etc/fstab failed"
            return 1
        fi

        info "[${FUNCNAME}:${LINENO}]" "OLD:${content}"
        info "[${FUNCNAME}:${LINENO}]" "NEW:${newCont}"

        mount | grep " ${point} " | grep "errors=panic" > /dev/null
        if [ $? -ne 0 ]; then
            #确保实时生效
            ${CMD_REMOUNT_CTL} ${point}
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "mount -o errors=panic,remount ${point} failed"
                rm "${tmpFile}" -f
                return 1
            fi
            info "[${FUNCNAME}:${LINENO}]" "mount -o errors=panic,remount ${point}"
        fi

        ((update++))
    done

    if [ ${update} -ne 0 ]; then
        ${CMD_REPLACE_FSTAB} "${tmpFile}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "replace fstab ${tmpFile} failed"
            rm "${tmpFile}" -f
            return 1
        fi
    fi

    rm "${tmpFile}" -f

    return 0
}


##########################################################
#  sudo封装函数
##########################################################
function preinstall_update_etc_hosts()
{
    local ipaddr="$1"
    local hostname="$2"

    #检查配置文件
    if [ -z "${ipaddr}" -o -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr=${ipaddr} or hostname=${hostname} is null."
        return 1
    fi

    echo "${ipaddr} ${hostname}" >> /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "echo ${ipaddr} ${hostname} >> /etc/hosts failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function preinstall_remove_etc_hosts()
{
    local ipaddr="$1"
    local hostname="$2"

    #检查配置文件
    if [ -z "${ipaddr}" -o -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr=${ipaddr} or hostname=${hostname} is null."
        return 1
    fi

    sed -i '/^[[:space:]]*'$ipaddr' /d' /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    sed -i '/ '$hostname'[[:space:]]*$/d' /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function preinstall_redhat_update_etc_hostname()
{
    local hostname="$1"
    local hostname_conf_file="/etc/sysconfig/network"

    #检查配置文件
    if [ -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "hostname=${hostname} is null."
        return 1
    fi
    
    #高版本情况（hostnamectl命令存在）
    which hostnamectl > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        which hostname > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Command hostname doesn't exist."
            return 1
        fi
        
        #即刻生效
        hostname "${hostname}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Modify hostname failed for ${hostname}."
            return 1
        fi
        
        #修改配置文件，永久生效
        echo "${hostname}" > "/etc/hostname"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Modify /etc/hostname failed."
            return 1
        fi
        
        return 0
    fi

    #永久设置hostname
    grep "^[[:space:]]*HOSTNAME[[:space:]]*=" "${hostname_conf_file}" > /dev/null
    if [ $? -ne 0 ]; then
        echo "HOSTNAME=${hostname}" >> "${hostname_conf_file}"
    else
        sed -i "s/^[[:space:]]*HOSTNAME[[:space:]]*=.*/HOSTNAME=${hostname}/g" "${hostname_conf_file}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Modify ${hostname_conf_file} failed for HOSTNAME=${hostname}."
            return 1
        fi
    fi

    #设置临时hostname(让hostname生效)
    hostname "${hostname}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "run command hostname ${hostname} failed."
        return 1
    fi

    return 0
}


##########################################################
#  配置 /etc/hosts
##########################################################
function config_hostname()
{
    local check_result=""
    local check_host_ip=""
    local services_ip="${GC_SERVICES_IP}"
    local hostname="${GC_HOSTNAME}"

    local hostname_ip=""
    local ip_count=0
    local hostname_count=0

    if [ ${GC_IP_MODEL} == "IPv6" ]; then
         services_ip=$(${CMD_COMPRESS_IPV6} "${GC_SERVICES_IP}")
    fi

    #检查/etc/hosts文件内容是否正确，配置不正确导致安装失败。现在LLD已经提供了hostname的配置，需要根据配置项检查
    #增加检查（LLD工具配置了就做到节点ip和主机名对应检查，做自动填写hosts文件，主机名hostname，检测未填写时不进行补充,也不进行提示,在preinstall配置，如果已经填写则只是提示对错）
    if [ -n "${services_ip}" ] && [ -n "${hostname}" ]; then
        #检查/etc/hosts中是否配置127.0.0.1 localhost
        check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "127.0.0.1" && ($2 == "localhost" || $3 == "localhost")){print $0}}')
        if [ -z "${check_result}" ]; then
            error "[${FUNCNAME}:${LINENO}]" "'127.0.0.1 localhost' is not configured in '/etc/hosts'."
            return 1
        fi

        #主机名不一致的场景,找到任意一项
        check_host_ip=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" || ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
        if [ -n "${check_host_ip}" ];then
            #检查是否和期望的一致
            check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" && ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
            if [ -z "${check_result}" ];then
                #不一致，需要删除，包括冲突项
                ${CMD_REMOVE_ETC_HOSTS} "${services_ip}" "${hostname}"
                if [ $? -ne 0 ]; then
                    error "[${FUNCNAME}:${LINENO}]" "${CMD_REMOVE_ETC_HOSTS} ${services_ip} failed."
                    return 1
                fi
                ${CMD_REMOVE_HOSTNAME_ETC_HOSTS} "${hostname}"
            fi
        fi

        check_result=""
        check_host_ip=""
        check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" && ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
        if [ -z "${check_result}" ];then
            #IP地址和主机名都没有配置的场景
            check_host_ip=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" || ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
            if [ -z "${check_host_ip}" ];then
                info "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts'."
                ${CMD_UPDATE_ETC_HOSTS} "${services_ip}" "${hostname}"
                if [ $? -eq 0 ];then
                    #配置成功，后面等待检查
                    info "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts',added automatically."
                else
                    error "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts',add failed."
                    return 1
                fi
            else
                error "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is mistakenly configured as '${check_host_ip}' in '/etc/hosts'."
                return 1
            fi
        fi

        #永久设置hostname
        ${CMD_UPDATE_ETC_HOSTNAME} "${hostname}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "${CMD_UPDATE_ETC_HOSTNAME} ${hostname} cmd failed."
            return 1
        fi
        #设置完成之后对hostname进行检查
        #检查hostname是否已经配置
        hostname -i &> /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the hostname is not configured in '/etc/hosts'. "
            return 1
        fi

        # 检查hostname对应的ip值是否唯一
        hostname_ip=$(hostname -i)
        ip_count=$(echo ${hostname_ip} | wc -w)
        if [ "${ip_count}" -ne 1 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the hostname corresponds to more than one ip in '/etc/hosts'."
            return 1
        fi

        #检查hostname对应的ip值是否存在于ifconfig列表中
        /sbin/ip addr list | grep "inet ${hostname_ip}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            /sbin/ip addr list | grep "inet6 ${hostname_ip}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "the ip ${hostname_ip} in /etc/hosts is not in ifconfig list."
                return 1
            fi
        fi

        # 检查/etc/hosts中是否存在一个IP对应多个主机的情况
        hostname_count=$(cat /etc/hosts | egrep "^[[:blank:]]*$(hostname -i)[[:blank:]]" | wc -l)
        if [ "${hostname_count}" -ne 1 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the ip ${hostname_ip} corresponds to more than one hostname in '/etc/hosts'."
            return 1
        fi
    fi

    return 0
}


##########################################################
#  配置关闭swap
##########################################################
function config_swap_off() {
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
    CMD_BLKID="blkid"

    local line=""
    local update=0
    local tmpFile="${HOME}/fstab.$(date +%s%N)"

    if [ "${GC_SWAP_OFF}" -eq 0 ]; then
        return 0
    fi

    #使用swapoff命令将swap全部关闭
    ${CMD_SWAPON_S} | awk '{if(NR != 1)print $1}' | while read line
    do
        ${CMD_SWAP_OFF} "${line}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "swapoff ${line} failed."
            return 1
        fi
    done

    #生成临时fstab文件
    ${CMD_CAT_FSTAB} > "${tmpFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup fstab failed"
        return 1
    fi

    while read line
    do
        sed -i "\#${line}#d" "${tmpFile}"
        ((update++))
    done < <(${CMD_CAT_FSTAB} | grep -v "^[[:space:]]*#" | grep "[[:space:]]swap[[:space:]]" | awk '{print $1}')

    #存在fstab文件更新的情况，需要更新系统的crontab文件
    if [ ${update} -ne 0 ]; then
        ${CMD_REPLACE_FSTAB} "${tmpFile}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "replace fstab ${tmpFile} failed"
            #如果失败了，需要将swap重新打开，还原swap配置
            swapon -a > /dev/null 2>&1
            rm "${tmpFile}" -f
            return 1
        fi
    fi

    rm "${tmpFile}" -f
    
    #如果存在systemd，那么需要重新加载配置文件
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -e "/usr/lib/systemd/system/" ]; then
        ${CMD_SYSTEM_CTL} daemon-reload
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "${CMD_SYSTEM_CTL} daemon-reload failed"
            return 1
        fi
    fi

    info "[${FUNCNAME}:${LINENO}]" "turn off swap ok."
    return 0
}

##########################################################
#  配置关闭selinux
##########################################################
function disable_selinux()
{
    #检查SELinux是否存在
    which sestatus > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "SELinux does not exist. Skip shutting down selinux."
        return 0
    fi
    
    #临时关闭SELinux（不用重启）
    which getenforce > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Get current selinux mode failed."
        return 1
    fi
    
    local currentMode=$(getenforce)
    if [ X$currentMode ==  XDisabled ]; then
        info "[${FUNCNAME}:${LINENO}]" "SELinux had been disabled. Skip turning off it."
    else
        setenforce 0
        if [ $? -ne 0 ];then
            error "[${FUNCNAME}:${LINENO}]" "Turn off SELinux failed."
            return 1
        fi
    fi
    
    #修改SELinux配置文件，永久生效
    local CONFIG_FILE="/etc/selinux/config"
    if [ ! -f $CONFIG_FILE ]; then
        error "[${FUNCNAME}:${LINENO}]" "SELinux config file /etc/selinux/config doesn't exist."
        return 1 
    fi
    
    sed -i 's/^SELINUX=.*$/SELINUX=disabled/g' $CONFIG_FILE
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to modify $CONFIG_FILE."
        return 1
    fi
       
    info "[${FUNCNAME}:${LINENO}]" "Disable SELinux ok."
    return 0
}

################################################################################
# Function: preinstall_remove_hostname_etc_hosts
# Description: add config /etc/hosts
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function preinstall_remove_hostname_etc_hosts()
{
    local hostname="$1"

    #检查配置文件
    if [ -z "${hostname}" ]; then
        echo "hostname=${hostname} is null."
        return 1
    fi

    sed -i '/ '$hostname'/d' /etc/hosts
    if [ $? -ne 0 ]; then
        echo "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    return 0
}

################################################################################
# Function: expand_ipv6
# Description: expand ipv6 address
# Parameter:
#   input:
#   $1   压缩的ipv6地址
#   output:完整的ipv6地址
#   N/A
# Others: N/A
################################################################################
function expand_ipv6() {
    if [ -z "${1}" ]; then
        echo "The argument is empty of expand_ipv6: arg1=${1}"
        return 1
    fi
    local ipv6=$(echo "${1}" | tr 'A-Z' 'a-z')
    local colon_num=$(echo ${ipv6} | awk '{print gsub(/:/, "")}')
    local replace_str=""
    for (( i = 0; i <= $(( 7 - ${colon_num} )); ++ i )); do
        replace_str="${replace_str}:0"
    done
    replace_str="${replace_str}:"
    local ipv6_expanded=${ipv6/::/$replace_str}
    [[ ${ipv6_expanded} == *: ]] && ipv6_expanded="${ipv6_expanded}0"
    [[ ${ipv6_expanded} == :* ]] && ipv6_expanded="0${ipv6_expanded}"
    echo ${ipv6_expanded}
}


function hex2dec(){
    [ "$1" != "" ] && printf "%d" "$(( 0x$1 ))"
}

##########################################################
#    Description:compress_ipv6
#    Parameter:  $1 ipv6 address
#    Return:
##########################################################
function compress_ipv6() {
    if [ -z "$1" ]; then
        echo "Param is empty."
        return 1
    fi
    local tmpaddr=$(echo "${1}" | tr 'A-Z' 'a-z')
    # 先扩展，以防入参是压缩地址
    ip=$(expand_ipv6 "${tmpaddr}")
    blocks=$(echo $ip | grep -o "[0-9a-f]\+")
    set $blocks
    # 压缩前导0 0000->0
    ip=$(printf "%x:%x:%x:%x:%x:%x:%x:%x\n" \
        $(hex2dec $1) \
        $(hex2dec $2) \
        $(hex2dec $3) \
        $(hex2dec $4) \
        $(hex2dec $5) \
        $(hex2dec $6) \
        $(hex2dec $7) \
        $(hex2dec $8)
    )
    # 加上前缀：，易于匹配
    ip=:"$ip"
    # :: 压缩最长连续0，若有相同长度的，压缩左边的
    for pattern in :0:0:0:0:0:0:0:0 \
            :0:0:0:0:0:0:0 \
            :0:0:0:0:0:0 \
            :0:0:0:0:0 \
            :0:0:0:0 \
            :0:0:0 \
            :0:0; do
        if echo $ip | grep -qs $pattern; then
            ip=$(echo $ip | sed "s/$pattern/::/")
            ip=$(echo $ip | sed 's/:::/::/')
            break
        fi
    done
    echo $ip | grep -qs "^:[^:]" && ip=$(echo $ip | sed 's/://')
    echo $ip
}

###################################################
#
# 获取OS的版本是否为openEuler release
#
###################################################
function get_ostype()
{
    local euler_release="/etc/openEuler-release"
    local os_release="/etc/os-release"
    local bclinux_release="/etc/bclinux-release"
    local kylin_release="/etc/.productinfo"

    if [ -f "${euler_release}" ]; then
        cat "${euler_release}" | grep "^openEuler release" >/dev/null 2>&1
        if [ $? -eq 0 ];then
            return 0
        fi
    fi
    if [ -f "${kylin_release}" ]; then
        grep -iq "kylin" ${kylin_release}
        local isKylin=$?
        local spVersion="$(grep release ${kylin_release} | awk '{print $3}' | tr -d '(|)|[A-Z]')"
        if [ $isKylin -eq 0 -a $spVersion -ge 3 ];then
            return 0
        fi
    fi
    if [ -f "${bclinux_release}" ]; then
        versionid="$(cat ${bclinux_release} |awk '{print $7}' |awk -F '.' '{print $1}')"
        versionsp="$(cat ${bclinux_release} |awk '{print $7}' |awk -F '.' '{print $2}')"
        if [ $versionid -eq 21 -a $versionsp -ge 10 ];then
            return 0
        fi
    fi
    if [ -f "${os_release}" ]; then
        if [ "openEuler" = "$(cat "${os_release}" | grep "^NAME" | awk -F '=' '{print $2}')" ]; then
            return 0
        fi
        grep -Eqi "UnionTech|LinxOS" "${os_release}"
        if [ $? -eq 0 ];then
            return 0
        fi
    fi

    return 1
}

###################################################
# 比较版本大小
# Parameter:
#   input: 待比较版本号  目标版本号
# Return: 0 -- 当前版本高于目标版本;
#         1 -- 当前版本等于目标版本;
#         2 -- 当前版本小于目标版本;
#         3 -- 参数错误
# Others: N/A
version_compare()
{
    if [ $# -ne 2 ]; then
        echo "version_compare need two parameter"
        return 3
    fi
    old_version=$( echo "$1" | sed -e "s/ //g")
    new_version=$( echo "$2" | sed -e "s/ //g")
    if [ "${old_version}" != "" ] && [ "${new_version}" != "" ]; then
        old_version_array=( $(echo "${old_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        new_version_array=( $(echo "${new_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        old_version_array_length=${#old_version_array[@]}
        new_version_array_length=${#new_version_array[@]}

        length=$((${new_version_array_length}>${old_version_array_length}?${old_version_array_length}:${new_version_array_length}))
        index=0
        eq_tag=3
        while [ $index -lt "${length}" ]
        do
            v1=${old_version_array[$index]}
            v2=${new_version_array[$index]}
            if [ "${v1}" -lt "${v2}" ]; then
                return 2
            elif [ "${v1}" -gt "${v2}" ] ; then
                return 0
            elif [ "${v1}" -eq "${v2}" ] ; then
                eq_tag=1
                let index+=1
                continue
            else
                return 3
            fi
        done
        return ${eq_tag}
    else
        echo "parameters can not be null" 
        return 3
    fi
}

##########################################################################################
#   创建软链
##########################################################################################
function preinstall_creatsoftlink()
{
    local source_file=${1}
    local linked_file=${2}
    if [ ! -f "${source_file}" ];then
        error "[${FUNCNAME}:${LINENO}]" "${source_file} does not exist."
        return 1
    fi
    
    if [ -L "${linked_file}" -o -f "${linked_file}" ];then
        return 0
    fi
    
    ln -s "${source_file}" "${linked_file}"
    if [ $? -ne 0 ];then
        error "[${FUNCNAME}:${LINENO}]" "Failed to creatsoftlink $linked_file."
        return 1
    fi

    chmod 777 ${linked_file}
}


##########################################################################################
# 创建libboost_atomic.so软链,将高版本so文件链接到1.66版本,解决openEuler20.03安装失败问题
#       /usr/lib64/libboost_atomic.so.1.66.0 
#       /usr/lib64/libboost_system.so.1.66.0
##########################################################################################
function creatsoftlink_libboost() 
{
    local libboost_atomic_link=/usr/lib64/libboost_atomic.so.1.66.0
    local libboost_system_link=/usr/lib64/libboost_system.so.1.66.0
    
    #使用最低版本
    local libboost_atomic=`ls -lt /usr/lib64 | grep libboost_atomic.so | grep -v lrwxrwxrwx |awk '{if ($9) printf("%s\n",$9)}' | sort | head -n 1`
    local libboost_system=`ls -lt /usr/lib64 | grep libboost_system.so | grep -v lrwxrwxrwx |awk '{if ($9) printf("%s\n",$9)}' | sort | head -n 1`

    if [ ! -f "/usr/lib64/${libboost_atomic}" ] || [ ! -f "/usr/lib64/${libboost_system}" ];then
        error "[${FUNCNAME}:${LINENO}]" "${libboost_atomic} or ${libboost_system} does not exist."
        return 1
    fi
    
    local libboost_atomic_version=`echo ${libboost_atomic/libboost_atomic.so./}`
    local libboost_system_version=`echo ${libboost_system/libboost_system.so./}`
    
    #版本相同
    local version_eq=1
    local parameter_error=3
    
    version_compare "${libboost_atomic_version}" "1.66.0"
    local check_re_1=$?
    
    version_compare "${libboost_system_version}" "1.66.0"
    local check_re_2=$?
    
    echo "libboost_atomic_version: ${libboost_atomic_version};check_re_1: ${check_re_1}"
    echo "libboost_system_version: ${libboost_system_version};check_re_2: ${check_re_2}"
    
    if [ "${check_re_1}" -eq "${parameter_error}" ] ||  [ "${check_re_2}" -eq "${parameter_error}" ];then
        error "[${FUNCNAME}:${LINENO}]" "version information is abnormal."
        return 1
    fi
    
    #如果libboost_atomic版本高于1.66.0，创建软链
    if [ "${check_re_1}" -lt "${version_eq}" ];then
        ${CMD_CREATSOFTLINK_LIBBOOST} /usr/lib64/${libboost_atomic} ${libboost_atomic_link}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Failed to creatsoftlink /usr/lib64/libboost_atomic.so.1.66.0."
            return 1
        fi
    fi
    
    #如果libboost_system版本高于1.66.0，创建软链
    if [ "${check_re_2}" -lt "${version_eq}" ];then
        ${CMD_CREATSOFTLINK_LIBBOOST} /usr/lib64/${libboost_system} ${libboost_system_link}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Failed to creatsoftlink /usr/lib64/libboost_system.so.1.66.0."
            return 1
        fi
    fi
}

##########################################################################################
# Euler 2.9 : omm 用户安装集群
##########################################################################################
function process_ssh_for_local()
{
    if [ ! -f "/etc/euleros-release" ]; then
        return 0
    fi

    cat /etc/euleros-release | grep "^Euler" > /dev/null 2>&1
    if [ $? -eq 0 ]
    then
        sed -i 's/^HostKeyAlgorithms/#HostKeyAlgorithms/' /etc/ssh/sshd_config
        sed -i 's/^PubkeyAcceptedKeyTypes/#PubkeyAcceptedKeyTypes/' /etc/ssh/sshd_config
	    service sshd restart > /dev/null 2>&1
    fi
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-s                                 Notes: services ip."
    echo "-n                                 Notes: host name."
    echo "-w                                 Notes: configuration turn off swap flag."
    echo "-h                                 Notes: show help."
    echo
    echo "Example:"
    echo "`basename $0` -s 192.168.45.11 -n 192-168-45-11"
    echo "`basename $0` -h"
    echo
    return 0
}

########################################################################
# 函数描述： 修改uos-1060e上ping命令和mount命令的权限，否则omm用户无法执行这些命令
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function chmod_system_cmds()
{
    info "[${FUNCNAME}:${LINENO}]" "Start to change system commands mode."
    
    grep -i 'uos' /etc/product-info | grep -q 1060
    if [ $? -eq 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Start to chmod /usr/bin/ping and /usr/bin/mount for uos-1060."
        ${CMD_CHMOD} 755 /usr/bin/ping
        ${CMD_CHMOD} g+x /usr/bin/mount
        ${CMD_CHMOD} o+x /usr/bin/mount
    fi

    grep -iq 'linxos' /etc/os-release 
    if [ $? -eq 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Start to chmod /usr/bin/systemctl for linxos."
        ${CMD_CHMOD} o+x /usr/bin/systemctl
    fi    
}

########################################################################
# 函数描述： 启用pam_rootok.so模块，这样从root用户su到其他用户时无需输入密码
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function enable_pam_rootok()
{
    if [ $(id -u) -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Not root user. pam_rootok.so already be enabled in preset."
        return 0
    fi
    
    grep -qi 'LinxOS' /etc/os-release
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "Not LinxOS. Skip to enable pam_rootok.so."
        return 0
    fi
    
    sed -i '/pam_rootok.so/s/#//' /etc/pam.d/su
    return $?
}

##########################################################
# 入口函数
##########################################################
function main ()
{
    #脚本输入所有参数的处理流程
    while getopts "n:s:w:v:h" options;
    do
        case ${options} in
            n)
                GC_HOSTNAME="${OPTARG}"      #获取配置的本机的主机名
                ;;

            s)
                GC_SERVICES_IP="${OPTARG}"   #获取本机配置的service ip
                ;;

            w)
                GC_SWAP_OFF="${OPTARG}"      #获取是否关闭swap的标记
                ;;
            v)
                GC_IP_MODEL="${OPTARG}"      #获取ip地址格式
                ;;
            h)
                show_help                    #显示帮助
                exit $?
                ;;
            *)
                echo "invalid input!!"       #非法参数输入
                show_help
                exit 1
                ;;
        esac
    done

    if [ -z "${LOCAL_IPADDR}" ]; then
        gc_ipaddr=$(hostname -i)
    else
        gc_ipaddr="${LOCAL_IPADDR}"
    fi

    config_service
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_language
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_sysctl
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_limits
    if [ $? -ne 0 ]; then
        return 1
    fi

    update_fstab
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_hostname
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_swap_off
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "turn off swap failed."
        return 1
    fi
    
    get_ostype
    if [ $? -eq 0 ]; then
        echo "current system is openEuler, create soft link."
        creatsoftlink_libboost
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Creating Soft Link failed."
            return 1
        fi
    fi

    if [ $(id -u) -eq 0 ]; then
        process_ssh_for_local
    else
        sudo ${INSTALL_SUDO_PATH} process_ssh_for_local
    fi
    
    $CMD_DISABLE_SELINUX
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    chmod_system_cmds
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to chmod /usr/bin/ping and /usr/bin/mount for uos-1060."
        return 1
    fi
    
    enable_pam_rootok
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to enable pam_rootok.so for LinxOS."
        return 1
    fi
    
    return 0
}

main "$@"

exit $?
