#!/bin/bash

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_SUSE_CONFIG_LANGUAGE="sudo ${INSTALL_SUDO_PATH} suse_config_language"
    CMD_SERVICES_CTL="sudo ${INSTALL_SUDO_PATH} services_ctl"
    CMD_CHKCONFIG_CTL="sudo ${INSTALL_SUDO_PATH} chkconfig_ctl"
    CMD_CAT_SYSCTL="sudo ${INSTALL_SUDO_PATH} cat_sysctl"
    CMD_REPLACE_SYSCTL="sudo ${INSTALL_SUDO_PATH} replace_sysctl"
    CMD_EFFECTIVE_SYSCTL="sudo ${INSTALL_SUDO_PATH} effective_sysctl"
    CMD_CONFIG_LIMITS="sudo ${INSTALL_SUDO_PATH} config_new_limits"
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
    CMD_REPLACE_FSTAB="sudo ${INSTALL_SUDO_PATH} replace_fstab"
    CMD_REMOUNT_CTL="sudo ${INSTALL_SUDO_PATH} remount_ctl"
    CMD_UPDATE_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_update_etc_hosts"
    CMD_REMOVE_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_remove_etc_hosts"
    CMD_REMOVE_HOSTNAME_ETC_HOSTS="sudo ${INSTALL_SUDO_PATH} preinstall_remove_hostname_etc_hosts"
    CMD_COMPRESS_IPV6="sudo ${INSTALL_SUDO_PATH} compress_ipv6"
    CMD_UPDATE_ETC_HOSTNAME="sudo ${INSTALL_SUDO_PATH} preinstall_suse_update_etc_hostname"
    CMD_SWAP_OFF="sudo ${INSTALL_SUDO_PATH} preinstall_swap_off"
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
    CMD_SWAPON_S="sudo ${INSTALL_SUDO_PATH} preinstall_swapon_s"
    CMD_READLINK_E="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_e"
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_SYSTEM_CTL="sudo ${INSTALL_SUDO_PATH} system_ctl"
    CMD_DISABLE_SELINUX="sudo ${INSTALL_SUDO_PATH} disable_selinux"
    CMD_COPY_LIBBOOST="sudo ${INSTALL_SUDO_PATH} preinstall_copy_libboost"
else
    CMD_SUSE_CONFIG_LANGUAGE="suse_config_language"
    CMD_SERVICES_CTL="service"
    CMD_CHKCONFIG_CTL="chkconfig"
    CMD_CAT_SYSCTL="cat /etc/sysctl.conf"
    CMD_REPLACE_SYSCTL="replace_sysctl"
    CMD_EFFECTIVE_SYSCTL="sysctl -q -p"
    CMD_CONFIG_LIMITS="config_new_limits"
    CMD_CAT_FSTAB="cat /etc/fstab"
    CMD_REPLACE_FSTAB="replace_fstab"
    CMD_REMOUNT_CTL="mount -o errors=panic,remount"
    CMD_UPDATE_ETC_HOSTS="preinstall_update_etc_hosts"
    CMD_REMOVE_ETC_HOSTS="preinstall_remove_etc_hosts"
    CMD_REMOVE_HOSTNAME_ETC_HOSTS="preinstall_remove_hostname_etc_hosts"
    CMD_COMPRESS_IPV6="compress_ipv6"
    CMD_UPDATE_ETC_HOSTNAME="preinstall_suse_update_etc_hostname"
    CMD_SWAP_OFF="swapoff"
    CMD_BLKID="blkid"
    CMD_SWAPON_S="swapon -s"
    CMD_READLINK_E="readlink -e"
    CMD_FINDFS="findfs"
    CMD_SYSTEM_CTL="systemctl"
    CMD_DISABLE_SELINUX="disable_selinux"
    CMD_COPY_LIBBOOST="preinstall_copy_libboost"
fi

GC_SERVICES_IP=""       #保存本机的services ip信息
GC_HOSTNAME=""          #保存本机的主机名信息
GC_SWAP_OFF=0           #是否关闭主动swap分区，默认不关闭
GC_IP_MODEL=""          #ip地址格式
gc_ipaddr=""            #日志打印的IP地址

###################################################
# 输出warning日志
###################################################
function warning()
{
    #直接使用echo 输出warning日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [WARNING]" "$@"
}

###################################################
# 输出error日志
###################################################
function error()
{
    #直接使用echo 输出error日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [ERROR]" "$@"
}

###################################################
# 输出info日志
###################################################
function info()
{
    #直接使用echo 输出info日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [INFO]" "$@"
}

###################################################
# 输出debug日志
###################################################
function debug()
{
    #直接使用echo 输出debug日志 服务端会重定向该日志
    if [ "${gc_debug}" -eq 1 ]; then
        echo "[$(date)][${gc_ipaddr}] [DEBUG]" "$@"
    fi
}

##########################################################
#  sudo封装函数
##########################################################
function suse_config_language()
{
    local lang_file=/etc/sysconfig/language

    #SuSE 12.X
    which localectl > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        localectl set-locale LANG=en_US.UTF-8 > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "localectl command failed."
            return 1
        fi

        return 0
    fi

    #SuSE 11.X
    if [ ! -f ${lang_file} ]; then
        echo "language file does not exist."
        return 1
    fi

    grep "^RC_LANG=" ${lang_file} > /dev/null
    if [ $? -ne 0 ]; then
        sed -i "1 a\RC_LANG=\"en_US.UTF-8\"" ${lang_file}
        if [ $? -ne 0 ]; then
            echo "Insert RC_LANG as UTF-8 failed."
            return 1
        fi
    else
        sed -i "s/^RC_LANG=.*/RC_LANG=\"en_US.UTF-8\"/g" ${lang_file}
        if [ $? -ne 0 ]; then
            echo "Instead RC_LANG as UTF-8 failed."
            return 1
        fi
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function replace_sysctl()
{
    local replace_path="$1"
    local target_path="/etc/sysctl.conf"

    #检查配置文件
    if [ ! -f ${replace_path} ]; then
        error "[${FUNCNAME}:${LINENO}]" "${replace_path} is not exist."
        return 1
    fi

    cat "$1" > ${target_path}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "cat $1 > ${target_path} failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function config_new_limits()
{
    local path="/etc/security/limits.d"
    local nproc="${path}/90-nproc.conf"
    local nofile="${path}/90-nofile.conf"
    local memlock="${path}/omm-memlock.conf"
    local omm_nproc="${path}/omm-nproc.conf"

    #创建limits.d目录
    if [ ! -d "${path}" ]; then
        mkdir -p ${path}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "mkdir -p ${path} failed"
            return 1
        fi
    fi

    echo "*          soft    nproc     60000" > ${nproc}
    echo "root       soft    nproc     unlimited">> ${nproc}

    info "[${FUNCNAME}:${LINENO}]" "configure nproc ok."

    echo "*          hard    nofile    640000" > ${nofile}
    echo "*          soft    nofile    640000" >> ${nofile}

    info "[${FUNCNAME}:${LINENO}]" "configure nofile ok."

    echo "omm        hard    memlock    unlimited" > ${memlock}
    echo "omm        soft    memlock    unlimited" >> ${memlock}

    info "[${FUNCNAME}:${LINENO}]" "configure memlock ok."

    echo "omm        hard    nproc     60000" > ${omm_nproc}
    echo "omm        soft    nproc     60000" >> ${omm_nproc}

    info "[${FUNCNAME}:${LINENO}]" "configure omm_nproc ok."

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function replace_fstab()
{
    local replace_path="$1"
    local target_path="/etc/fstab"

    #检查配置文件
    if [ ! -f ${replace_path} ]; then
        error "[${FUNCNAME}:${LINENO}]" "${replace_path} is not exist."
        return 1
    fi

    cat "${replace_path}" > "${target_path}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "cat ${replace_path} > ${target_path} failed"
        return 1
    fi

    return 0
}

##########################################################
#  修改 RC_LANG 为 en_US.UTF-8
##########################################################
function config_language ()
{
    ${CMD_SUSE_CONFIG_LANGUAGE}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "suse_config_language failed."
        return 1
    fi

    info "[${FUNCNAME}:${LINENO}]" "config language ok"

    return 0
}

##########################################################
#  配置服务
##########################################################
function config_service()
{
    local start_service_array=(cron sshd ipmi ipmievd nscd ntp ntpd rngd haveged)
    local stop_service_array=()
    local service_name=""

    for element in $(seq 0 $((${#start_service_array[@]} - 1)))
    do
        service_name="${start_service_array[$element]}"
        if [ -z "${service_name}" ]; then
            continue
        fi

        if [ -e "/usr/lib/systemd/system/${service_name}.service" ]; then
            ${CMD_SYSTEM_CTL} start "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl start ${service_name} failed."
            fi

            ${CMD_SYSTEM_CTL} enable "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl enable ${service_name} failed."
            fi
        fi

        if [ -e "/etc/init.d/${service_name}" ]; then
            ${CMD_SERVICES_CTL} "${service_name}" start > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute service ${service_name} start failed."
            fi

            ${CMD_CHKCONFIG_CTL} "${service_name}" on > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute chkconfig ${service_name} on failed."
            fi
        fi
    done


    for element in $(seq 0 $((${#stop_service_array[@]} - 1)))
    do
        service_name="${stop_service_array[$element]}"
        if [ -z "${service_name}" ]; then
            continue
        fi

        if [ -e "/usr/lib/systemd/system/${service_name}.service" ]; then
            ${CMD_SYSTEM_CTL} stop "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl stop ${service_name} failed."
            fi

            ${CMD_SYSTEM_CTL} disable "${service_name}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute systemctl disable ${service_name} failed."
            fi
        fi

        if [ -e "/etc/init.d/${service_name}" ]; then
            ${CMD_SERVICES_CTL} "${service_name}" stop > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute service ${service_name} stop failed."
            fi

            ${CMD_CHKCONFIG_CTL} "${service_name}" off > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                warning "[${FUNCNAME}:${LINENO}]" "execute chkconfig ${service_name} off failed."
            fi
        fi
    done

    return 0
}

##########################################################
#  实际修改配置
##########################################################
function configSysctlConf()
{
    local config_name=${1};
    local config_value=${2};
    local config_file=${3};
    grep "^[[:space:]]*${config_name}[[:space:]]*=" ${config_file} > /dev/null
    if [ $? -ne 0 ]; then
        echo "${config_name} = ${config_value}" >> ${config_file}
    else
        sed -i "s/^[[:space:]]*${config_name}[[:space:]]*=.*/${config_name} = ${config_value}/g" ${config_file}
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "Modify ${config_file} failed for ${config_name}."
            return 1
        fi
    fi
    info "[${FUNCNAME}:${LINENO}]" "config ${config_name} = ${config_value} ok"
    return 0
}

##########################################################
#  配置 /etc/sysctl.conf
##########################################################
function config_sysctl()
{
    local sysctl=/etc/sysctl.conf
    local tmpSysctlFile="${HOME}/sysctl.conf.$(date +%s%N)"
    local kernelPath="/proc/sys/kernel"

    #生成临时sysctl.conf文件
    ${CMD_CAT_SYSCTL} > "${tmpSysctlFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup sysctl to ${tmpSysctlFile} failed"
        return 1
    fi

    #设置 kernel.sysrq
    if [ -f "${kernelPath}/sysrq" ]; then
        if ! configSysctlConf "kernel.sysrq" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.sysrq is not support,skip it "
    fi

    #设置 kernel.kstack_depth_to_print
    if [ -f "${kernelPath}/kstack_depth_to_print" ]; then
        if ! configSysctlConf "kernel.kstack_depth_to_print" "20" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.kstack_depth_to_print is not support,skip it "
    fi

    #设置 kernel.panic_on_oops
    if [ -f "${kernelPath}/panic_on_oops" ]; then
        if ! configSysctlConf "kernel.panic_on_oops" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.panic_on_oops is not support,skip it "
    fi
    

    #设置 kernel.panic
    if [ -f "${kernelPath}/panic" ]; then
        if ! configSysctlConf "kernel.panic" "5" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.panic is not support,skip it "
    fi

    #设置 kernel.hung_task_timeout_secs
    if [ -f "${kernelPath}/hung_task_timeout_secs" ]; then
        if ! configSysctlConf "kernel.hung_task_timeout_secs" "3600" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.hung_task_timeout_secs is not support,skip it "
    fi

    #设置 kernel.hung_task_panic
    if [ -f "${kernelPath}/hung_task_panic" ]; then
        if ! configSysctlConf "kernel.hung_task_panic" "1" ${tmpSysctlFile};then
            return 1
        fi
    else
        info "[${FUNCNAME}:${LINENO}]" "kernel.hung_task_panic is not support,skip it "
    fi

    #设置 vm.panic_on_oom
    if ! configSysctlConf "vm.panic_on_oom" "2" ${tmpSysctlFile};then
        return 1
    fi

    #设置 vm.oom_dump_tasks
    if ! configSysctlConf "vm.oom_dump_tasks" "1" ${tmpSysctlFile};then
        return 1
    fi

    #设置 vm.swappiness
    if ! configSysctlConf "vm.swappiness" "1" ${tmpSysctlFile};then
        return 1
    fi

    #设置 fs.file-max
    if ! configSysctlConf "fs.file-max" "640000" ${tmpSysctlFile};then
        return 1
    fi
    
    #设置 net.ipv4.tcp_max_syn_backlog
    if ! configSysctlConf "net.ipv4.tcp_max_syn_backlog" "8192" ${tmpSysctlFile};then
        return 1
    fi
    
    #设置 net.core.somaxconn
    if ! configSysctlConf "net.core.somaxconn" "2048" ${tmpSysctlFile};then
        return 1
    fi

    #设置 fs.inotify.max_user_watches
    if ! configSysctlConf "fs.inotify.max_user_watches" "100000" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh1
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh1" "4096" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh2
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh2" "8192" ${tmpSysctlFile};then
        return 1
    fi

    #设置 net.ipv4.neigh.default.gc_thresh3
    if ! configSysctlConf "net.ipv4.neigh.default.gc_thresh3" "20480" ${tmpSysctlFile};then
        return 1
    fi

    ${CMD_REPLACE_SYSCTL} "${tmpSysctlFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "replace sysctl ${tmpSysctlFile} failed"
        rm "${tmpSysctlFile}" -f
        return 1
    fi

    rm "${tmpSysctlFile}" -f

    #使上面的设置直接生效,不用定向到null
    ${CMD_EFFECTIVE_SYSCTL}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "run command sysctl -q -p failed"
        return 1
    fi

    return 0
}

##########################################################
#  配置 /etc/security/limits.d/*
#  说明:
#       SuSE 中同样可以使用 limits.d
#  优先级:
#       当 同时配置了 limits.conf 和 limits.d/*
#       以 limits.d/* 为准
##########################################################
function config_limits()
{
    ${CMD_CONFIG_LIMITS}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "config limits failed"
        return 1
    fi

    return 0
}

##########################################################
# 更新fstab
##########################################################
function update_fstab()
{
    local update=0
    local content=""
    local content2=""
    local newCont=""
    local newCont2=""
    local option=""
    local tmpFile="${HOME}/fstab.$(date +%s%N)"

    #生成临时fstab文件
    ${CMD_CAT_FSTAB} > "${tmpFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup fstab failed"
        return 1
    fi

    #只更新特定目录
    for point in / /boot /home /opt /tmp /var /var/log
    do
        if [ -z "${point}" ]; then
            continue
        fi

        content=$(${CMD_CAT_FSTAB} | grep -vw '^[ ]*#' | grep " ${point} ")
        if [ -z "${content}" ]; then
            point="${point}/"
            content=$(${CMD_CAT_FSTAB} | grep -vw '^[ ]*#' | grep " ${point} ")
            if [ -z "${content}" ]; then
                continue
            fi
        fi

        if ! echo "${content}" | grep -q -E " ext3 | ext4 "; then
            continue
        fi

        echo "${content}" | grep "errors=panic" > /dev/null
        if [ $? -eq 0 ]; then
            continue
        fi

        option=$(echo "${content}" | awk '{print $4}')
        newCont="$(echo "${content}" | sed "s/${option}/${option},errors=panic/g")"

        #把 / 替换为 \/ 即增加转义字符
        content2="$(echo "${content}" | sed 's/\//\\\//g')"
        newCont2="$(echo "${newCont}" | sed 's/\//\\\//g')"

        sed -i "s/${content2}/${newCont2}/g" "${tmpFile}"
        if [ $? -ne 0 ]; then
            rm "${tmpFile}" -f
            error "[${FUNCNAME}:${LINENO}]" "sed -i s/${content2}/${newCont2}/g ${tmpFile} failed"
            error "[${FUNCNAME}:${LINENO}]" "update /etc/fstab failed"
            return 1
        fi

        info "[${FUNCNAME}:${LINENO}]" "OLD:${content}"
        info "[${FUNCNAME}:${LINENO}]" "NEW:${newCont}"

        mount | grep " ${point} " | grep "errors=panic" > /dev/null
        if [ $? -ne 0 ]; then
            #确保实时生效
            ${CMD_REMOUNT_CTL} ${point}
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "mount -o errors=panic,remount ${point} failed"
                rm "${tmpFile}" -f
                return 1
            fi
            info "[${FUNCNAME}:${LINENO}]" "mount -o errors=panic,remount ${point}"
        fi

        ((update++))
    done

    if [ ${update} -ne 0 ]; then
        ${CMD_REPLACE_FSTAB} "${tmpFile}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "replace fstab ${tmpFile} failed"
            rm "${tmpFile}" -f
            return 1
        fi
    fi

    rm "${tmpFile}" -f

    return 0
}


##########################################################
#  sudo封装函数
##########################################################
function preinstall_update_etc_hosts()
{
    local ipaddr="$1"
    local hostname="$2"

    #检查配置文件
    if [ -z "${ipaddr}" -o -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr=${ipaddr} or hostname=${hostname} is null."
        return 1
    fi

    echo "${ipaddr} ${hostname}" >> /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "echo ${ipaddr} ${hostname} >> /etc/hosts failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function preinstall_remove_etc_hosts()
{
    local ipaddr="$1"
    local hostname="$2"

    #检查配置文件
    if [ -z "${ipaddr}" -o -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "ipaddr=${ipaddr} or hostname=${hostname} is null."
        return 1
    fi

    sed -i '/^[[:space:]]*'$ipaddr' /d' /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    sed -i '/ '$hostname'[[:space:]]*$/d' /etc/hosts
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    return 0
}

##########################################################
#  sudo封装函数
##########################################################
function preinstall_suse_update_etc_hostname()
{
    local hostname="$1"

    if [ -z "${hostname}" ]; then
        error "[${FUNCNAME}:${LINENO}]" "args is null"
        return 1
    fi

    yast dns edit hostname="${hostname}"
    return $?
}


################################################################################
# Function: preinstall_remove_hostname_etc_hosts
# Description: add config /etc/hosts
# Parameter:
#   input:
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function preinstall_remove_hostname_etc_hosts()
{
    local hostname="$1"

    #检查配置文件
    if [ -z "${hostname}" ]; then
        echo "hostname=${hostname} is null."
        return 1
    fi

    sed -i '/ '$hostname'/d' /etc/hosts
    if [ $? -ne 0 ]; then
        echo "remove $ipaddr from /etc/hosts failed"
        return 1
    fi

    return 0
}

################################################################################
# Function: expand_ipv6
# Description: expand ipv6 address
# Parameter:
#   input:
#   $1   压缩的ipv6地址
#   output:完整的ipv6地址
#   N/A
# Others: N/A
################################################################################
function expand_ipv6() {
    if [ -z "${1}" ]; then
        echo "The argument is empty of expand_ipv6: arg1=${1}"
        return 1
    fi
    local ipv6=$(echo "${1}" | tr 'A-Z' 'a-z')
    local colon_num=$(echo ${ipv6} | awk '{print gsub(/:/, "")}')
    local replace_str=""
    for (( i = 0; i <= $(( 7 - ${colon_num} )); ++ i )); do
        replace_str="${replace_str}:0"
    done
    replace_str="${replace_str}:"
    local ipv6_expanded=${ipv6/::/$replace_str}
    [[ ${ipv6_expanded} == *: ]] && ipv6_expanded="${ipv6_expanded}0"
    [[ ${ipv6_expanded} == :* ]] && ipv6_expanded="0${ipv6_expanded}"
    echo ${ipv6_expanded}
}


function hex2dec(){
    [ "$1" != "" ] && printf "%d" "$(( 0x$1 ))"
}

##########################################################
#    Description:compress_ipv6
#    Parameter:  $1 ipv6 address
#    Return:
##########################################################
function compress_ipv6() {
    if [ -z "$1" ]; then
        echo "Param is empty in DBServerValidateFloatIp."
        return 1
    fi
    local tmpaddr=$(echo "${1}" | tr 'A-Z' 'a-z')
    # 先扩展，以防入参是压缩地址
    ip=$(expand_ipv6 "${tmpaddr}")
    blocks=$(echo $ip | grep -o "[0-9a-f]\+")
    set $blocks
    # 压缩前导0 0000->0
    ip=$(printf "%x:%x:%x:%x:%x:%x:%x:%x\n" \
        $(hex2dec $1) \
        $(hex2dec $2) \
        $(hex2dec $3) \
        $(hex2dec $4) \
        $(hex2dec $5) \
        $(hex2dec $6) \
        $(hex2dec $7) \
        $(hex2dec $8)
    )
    # 加上前缀：，易于匹配
    ip=:"$ip"
    # :: 压缩最长连续0，若有相同长度的，压缩左边的
    for pattern in :0:0:0:0:0:0:0:0 \
            :0:0:0:0:0:0:0 \
            :0:0:0:0:0:0 \
            :0:0:0:0:0 \
            :0:0:0:0 \
            :0:0:0 \
            :0:0; do
        if echo $ip | grep -qs $pattern; then
            ip=$(echo $ip | sed "s/$pattern/::/")
            ip=$(echo $ip | sed 's/:::/::/')
            break
        fi
    done
    echo $ip | grep -qs "^:[^:]" && ip=$(echo $ip | sed 's/://')
    echo $ip
}

##########################################################
#  配置 /etc/hosts
##########################################################
function config_hostname()
{
    local check_result=""
    local check_host_ip=""
    local services_ip="${GC_SERVICES_IP}"
    local hostname="${GC_HOSTNAME}"

    local hostname_ip=""
    local ip_count=0
    local hostname_count=0

    if [ ${GC_IP_MODEL} == "IPv6" ]; then
         services_ip=$(${CMD_COMPRESS_IPV6} "${GC_SERVICES_IP}")
    fi

    #检查/etc/hosts文件内容是否正确，配置不正确导致安装失败。现在LLD已经提供了hostname的配置，需要根据配置项检查
    #增加检查（LLD工具配置了就做到节点ip和主机名对应检查，做自动填写hosts文件，主机名hostname，检测未填写时不进行补充,也不进行提示,在preinstall配置，如果已经填写则只是提示对错）
    if [ -n "${services_ip}" ] && [ -n "${hostname}" ]; then
        #检查/etc/hosts中是否配置127.0.0.1 localhost
        check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "127.0.0.1" && ($2 == "localhost" || $3 == "localhost")){print $0}}')
        if [ -z "${check_result}" ]; then
            error "[${FUNCNAME}:${LINENO}]" "'127.0.0.1 localhost' is not configured in '/etc/hosts'."
            return 1
        fi

        #主机名不一致的场景,找到任意一项
        check_host_ip=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" || ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
        if [ -n "${check_host_ip}" ];then
            #检查是否和期望的一致
            check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" && ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
            if [ -z "${check_result}" ];then
                #不一致，需要删除，包括冲突项
                ${CMD_REMOVE_ETC_HOSTS} "${services_ip}" "${hostname}"
                if [ $? -ne 0 ]; then
                    error "[${FUNCNAME}:${LINENO}]" "${CMD_REMOVE_ETC_HOSTS} ${services_ip} failed."
                    return 1
                fi
                ${CMD_REMOVE_HOSTNAME_ETC_HOSTS} "${hostname}"
            fi
        fi

        check_result=""
        check_host_ip=""
        check_result=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" && ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
        if [ -z "${check_result}" ];then
            #IP地址和主机名都没有配置的场景
            check_host_ip=$(cat /etc/hosts | grep -vw '^#' | awk '{if ($1 == "'${services_ip}'" || ($2 == "'${hostname}'" || $3 == "'${hostname}'")){print $0}}')
            if [ -z "${check_host_ip}" ];then
                info "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts'."
                ${CMD_UPDATE_ETC_HOSTS} "${services_ip}" "${hostname}"
                if [ $? -eq 0 ];then
                    #配置成功，后面等待检查
                    info "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts',added automatically."
                else
                    error "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is not configured in '/etc/hosts',add failed."
                    return 1
                fi
            else
                error "[${FUNCNAME}:${LINENO}]" "'${services_ip} ${hostname}' is mistakenly configured as '${check_host_ip}' in '/etc/hosts'."
                return 1
            fi
        fi

        #永久设置hostname
        ${CMD_UPDATE_ETC_HOSTNAME} "${hostname}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "${CMD_UPDATE_ETC_HOSTNAME} ${hostname} cmd failed."
            return 1
        fi
        #设置完成之后对hostname进行检查
        #检查hostname是否已经配置
        hostname -i &> /dev/null
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the hostname is not configured in '/etc/hosts'. "
            return 1
        fi

        # 检查hostname对应的ip值是否唯一
        hostname_ip=$(hostname -i)
        ip_count=$(echo ${hostname_ip} | wc -w)
        if [ "${ip_count}" -ne 1 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the hostname corresponds to more than one ip in '/etc/hosts'."
            return 1
        fi

        #检查hostname对应的ip值是否存在于ifconfig列表中
        /sbin/ip addr list | grep "inet ${hostname_ip}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            /sbin/ip addr list | grep "inet6 ${hostname_ip}" > /dev/null 2>&1
            if [ $? -ne 0 ]; then
                error "[${FUNCNAME}:${LINENO}]" "the ip ${hostname_ip} in /etc/hosts is not in ifconfig list."
                return 1
            fi
        fi

        # 检查/etc/hosts中是否存在一个IP对应多个主机的情况
        hostname_count=$(cat /etc/hosts | egrep "^[[:blank:]]*$(hostname -i)[[:blank:]]" | wc -l)
        if [ "${hostname_count}" -ne 1 ]; then
            error "[${FUNCNAME}:${LINENO}]" "the ip ${hostname_ip} corresponds to more than one hostname in '/etc/hosts'."
            return 1
        fi
    fi

    return 0
}

##########################################################
#  配置关闭swap
##########################################################
function config_swap_off() {
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
    CMD_BLKID="blkid"

    local line=""
    local update=0
    local tmpFile="${HOME}/fstab.$(date +%s%N)"

    if [ "${GC_SWAP_OFF}" -eq 0 ]; then
        return 0
    fi

    #使用swapoff命令将swap全部关闭
    ${CMD_SWAPON_S} | awk '{if(NR != 1)print $1}' | while read line
    do
        ${CMD_SWAP_OFF} "${line}" > /dev/null 2>&1
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "swapoff ${line} failed."
            return 1
        fi
    done

    #生成临时fstab文件
    ${CMD_CAT_FSTAB} > "${tmpFile}"
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "backup fstab failed"
        return 1
    fi

    while read line
    do
        sed -i "\#${line}#d" "${tmpFile}"
        ((update++))
    done < <(${CMD_CAT_FSTAB} | grep -v "^[[:space:]]*#" | grep "[[:space:]]swap[[:space:]]" | awk '{print $1}')

    #存在fstab文件更新的情况，需要更新系统的crontab文件
    if [ ${update} -ne 0 ]; then
        ${CMD_REPLACE_FSTAB} "${tmpFile}"
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "replace fstab ${tmpFile} failed"
            #如果失败了，需要将swap重新打开，还原swap配置
            swapon -a > /dev/null 2>&1
            rm "${tmpFile}" -f
            return 1
        fi
    fi

    rm "${tmpFile}" -f
    
    #如果存在systemd，那么需要重新加载配置文件
    which "systemctl" > /dev/null 2>&1
    if [ $? -eq 0 -a -e "/usr/lib/systemd/system/" ]; then
        ${CMD_SYSTEM_CTL} daemon-reload
        if [ $? -ne 0 ]; then
            error "[${FUNCNAME}:${LINENO}]" "${CMD_SYSTEM_CTL} daemon-reload failed"
            return 1
        fi
    fi

    info "[${FUNCNAME}:${LINENO}]" "turn off swap ok."
    return 0
}

##########################################################
#  配置关闭selinux
##########################################################
function disable_selinux()
{
    #检查SELinux是否存在
    which sestatus > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "SELinux does not exist. Skip shutting down selinux."
        return 0
    fi
    
    #临时关闭SELinux（不用重启）
    which getenforce > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Get current selinux mode failed."
        return 1
    fi
    
    local currentMode=$(getenforce)
    if [ X$currentMode ==  XDisabled ]; then
        info "[${FUNCNAME}:${LINENO}]" "SELinux had been disabled. Skip turning off it."
    else
        setenforce 0
        if [ $? -ne 0 ];then
            error "[${FUNCNAME}:${LINENO}]" "Turn off SELinux failed."
            return 1
        fi
    fi

    #修改SELinux配置文件，永久生效
    local CONFIG_FILE="/etc/selinux/config"
    if [ ! -f $CONFIG_FILE ]; then
        error "[${FUNCNAME}:${LINENO}]" "SELinux config file /etc/selinux/config doesn't exist."
        return 1 
    fi
    
    sed -i 's/^SELINUX=.*$/SELINUX=disabled/g' $CONFIG_FILE
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to modify $CONFIG_FILE."
        return 1
    fi
       
    info "[${FUNCNAME}:${LINENO}]" "Disable SELinux ok."
    return 0
}

function preinstall_copy_libboost()
{
    local libboost_file=${1}
    cp -f ${libboost_file} /usr/lib64/
    if [ ! -f /usr/lib64/${libboost_file##*/} ]; then
        error "[${FUNCNAME}:${LINENO}]" "/usr/lib64/${libboost_file##*/} is not exist."
        return 1
    fi
    chmod 755 /usr/lib64/${libboost_file##*/}
    return $?
}


####################################################################
#  拷贝/usr/lib64/libboost_system.so.1.54.0解决suse15.1安装失败问题
####################################################################
function copy_libboost() 
{
    local libboost_dir=$(dirname $(readlink -f "$0"))
    local libboost_file=${libboost_dir}/libboost_system.so.1.54.0
    local libboost_atomic=${libboost_dir}/libboost_atomic.so.1.54.0

    ${CMD_COPY_LIBBOOST} ${libboost_file}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to overwrite /usr/lib64/libboost_system.so.1.54.0."
        return 1
    fi

    ${CMD_COPY_LIBBOOST} ${libboost_atomic}
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "Failed to overwrite /usr/lib64/libboost_atomic.so.1.54.0."
        return 1
    fi
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-s                                 Notes: services ip."
    echo "-n                                 Notes: host name."
    echo "-w                                 Notes: configuration turn off swap flag."
    echo "-h                                 Notes: show help."
    echo
    echo "Example:"
    echo "`basename $0` -s 192.168.45.11 -n 192-168-45-11"
    echo "`basename $0` -h"
    echo
    return 0
}

###################################################
#
# 获取OS的名称
#
###################################################
function get_osname_c()
{
    local sles_release="/etc/SuSE-release"
    local os_release="/etc/os-release"
    local name=""

    #获取SuSE的版本
    if [ -f "${sles_release}" ]; then
        name="suse-$(cat "${sles_release}" | grep "^VERSION" | awk '{print $3}')"
        name="${name}.$(cat "${sles_release}" | grep "^PATCHLEVEL" | awk '{print $3}')"
        echo "$name"
        return 0
    fi

    #获取SuSE15的版本
    if [ -f "${os_release}" ]; then
        if [ "suse" = "$(cat "${os_release}" | grep "^ID_LIKE=" | awk -F "\"" '{print $2}')" ]; then
            name="suse-$(cat "${os_release}" | grep "^VERSION_ID=" | awk -F "\"" '{print $2}')"
            echo "$name"
            return 0
        fi
    fi

    return 1
}

########################################################################
# 函数描述： SuSE 12.2以上系统，需要修改参数 TasksMax=20480
# 配置文件对象： /etc/systemd/system.conf
#
# 返回值  ： 0 成功
########################################################################
function config_TasksMax()
{
    info "[${FUNCNAME}:${LINENO}]" "TasksMax start config."

    check_suse_version
    if [ $? -ne 0 ]; then
        info "[${FUNCNAME}:${LINENO}]" "System version is not suitable.Do not configure TasksMax."
        return 0
    fi

    sed -i '/DefaultTasksMax/d' /etc/systemd/system.conf >/dev/null 2>&1
    echo 'DefaultTasksMax=20480' >>/etc/systemd/system.conf

    systemctl daemon-reload >/dev/null 2>&1

    info "[${FUNCNAME}:${LINENO}]" "/etc/systemd/system.conf config TasksMax=20480 ok."
    return 0
}

###################################################
#
#  检查SuSE系统版本在12.2以上
#
###################################################
function check_suse_version()
{
    info "[${FUNCNAME}:${LINENO}]" "Check that the suse system version is above 12.2"
    local current_system_version=""
    local current_VERSION=""
    local current_PATCHLEVEL=""

    #限定suse系统最低的VERSION
    local lowest_VERSION="12"
    #限定suse系统最低的PATCHLEVEL
    local lowest_PATCHLEVEL="2"

    #当前suse系统的版本
    current_system_version=$(get_osname_c | awk -F '-' '{print $2}')
    
    #解析获得系统VERSION和PATCHLEVEL
    current_VERSION=$(echo ${current_system_version} | awk -F '.' '{print $1}')
    current_PATCHLEVEL=$(echo ${current_system_version} | awk -F '.' '{print $2}')

    if [ "${current_VERSION}" == "" ] || [ "${current_PATCHLEVEL}" == "" ];then
        info "[${FUNCNAME}:${LINENO}]" "get current suse system VERSION PATCHLEVEL failed."
        return 1
    fi

    if [ "${current_VERSION}" -lt "${lowest_VERSION}" ]; then
        info "[${FUNCNAME}:${LINENO}]" "the current system VERSION (${current_VERSION}) is earlier than ${lowest_VERSION}."
        return 1
    elif [ "${current_VERSION}" -gt "${lowest_VERSION}" ] ; then
        return 0
    elif [ "${current_VERSION}" -eq "${lowest_VERSION}" ] ; then
    
        #当VERSION相等时，PATCHLEVEL不满足的情况，只有小于的场合
        if [ "${current_PATCHLEVEL}" -lt "${lowest_PATCHLEVEL}" ]; then
            info "[${FUNCNAME}:${LINENO}]" "the current system PATCHLEVEL (${current_PATCHLEVEL}) is earlier than ${lowest_PATCHLEVEL}."
            return 1
        fi

    fi

    return 0
}

##########################################################
# 入口函数
##########################################################
function main ()
{
    #脚本输入所有参数的处理流程
    while getopts "n:s:w:v:h" options;
    do
        case ${options} in
            n)
                GC_HOSTNAME="${OPTARG}"      #获取配置的本机的主机名
                ;;

            s)
                GC_SERVICES_IP="${OPTARG}"   #获取本机配置的service ip
                ;;

            w)
                GC_SWAP_OFF="${OPTARG}"      #获取是否关闭swap的标记
                ;;
            v)
                GC_IP_MODEL="${OPTARG}"      #获取ip地址格式
                ;;
            h)
                show_help                    #显示帮助
                exit $?
                ;;
            *)
                echo "invalid input!!"       #非法参数输入
                show_help
                exit 1
                ;;
        esac
    done

    if [ -z "${LOCAL_IPADDR}" ]; then
        gc_ipaddr=$(hostname -i)
    else
        gc_ipaddr="${LOCAL_IPADDR}"
    fi

    config_service
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_language
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_sysctl
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_limits
    if [ $? -ne 0 ]; then
        return 1
    fi

    update_fstab
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_hostname
    if [ $? -ne 0 ]; then
        return 1
    fi

    config_swap_off
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "turn off swap failed."
        return 1
    fi

    config_TasksMax
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "set TasksMax failed."
        return 1
    fi
    
    copy_libboost
    if [ $? -ne 0 ]; then
        error "[${FUNCNAME}:${LINENO}]" "copy libboost failed."
        return 1
    fi

    $CMD_DISABLE_SELINUX
    if [ $? -ne 0 ]; then
        return 1
    fi

    return 0
}

main "$@"

exit $?