#!/bin/bash

g_currentPath=""    #当前路径
g_localFile=""      #操作系统的LOCAL文件

g_logFile=""    #安装日志文件


###################################################
# 输出error日志
###################################################
function error()
{
    #直接使用echo 输出error日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [ERROR]" "$@" >> ${g_logFile}
}

###################################################
# 输出info日志
###################################################
function info()
{
    #直接使用echo 输出info日志 服务端会重定向该日志
    echo "[$(date)][${gc_ipaddr}] [INFO]" "$@" >> ${g_logFile}
}

###################################################
# 输出debug日志
###################################################
function debug()
{
    #直接使用echo 输出debug日志 服务端会重定向该日志
    if [ "${gc_debug}" -eq 1 ]; then
        echo "[$(date)][${gc_ipaddr}] [DEBUG]" "$@" >> ${g_logFile}
    fi
}

######################################################################
#   DESCRIPTION: 初始化日志文件
######################################################################
function init_logFile()
{       
    local log_dir=$(dirname "${g_logFile}")
    
    if [ -z "${log_dir}" ]; then
        log_dir="/var/log/osperf"
    fi
    
    #检查日志目录
    m_init_logFile "$log_dir"
	if [ $? -eq 0 ]; then
		return 0
	fi
	
	return 1
}

######################################################################
#   DESCRIPTION: 日志打印接口
######################################################################
function log()
{
    local logBuffer=""  #打印内容

    logBuffer="$1"       
    echo "[$(date)] ${logBuffer}" >> "${g_logFile}"    
}

######################################################################
#   DESCRIPTION: 获取当前目录
######################################################################
function get_currentPath()
{       
    #获取当前目录
    pushd $(dirname "${BASH_SOURCE:-$0}") > /dev/null
    g_currentPath="$(readlink -e "$(pwd)")"        
    popd > /dev/null
    
    if [ -z "${g_currentPath}" ]; then
        return 1
    fi
    
    return 0
}

######################################################################
#   DESCRIPTION: 获取操作系统LOCAL文件
######################################################################
function get_localFile()
{        
    #Red Hat版本
    if [ -f "/etc/redhat-release" ]; then  
        #版本号打印到运行日志,方便定位
        log "$(cat /etc/redhat-release)"
        
        g_localFile="/etc/rc.local"         #redhat的local目录
        return 0
    fi    
       
    #匹配SuSE11SP1
    if [ -f "/etc/SuSE-release" ]; then   
        #版本号打印到运行日志,方便定位
        log "$(cat /etc/SuSE-release)"
        
        g_localFile="/etc/rc.d/after.local" #suse的local目录
        return 0
    fi    
     
    #CentOS版本
    if [ -f "/etc/centos-release" ]; then  
        #版本号打印到运行日志,方便定位
        log "$(cat /etc/centos-release)"
        
        g_localFile="/etc/rc.local"         #CentOS的local目录
        return 0
    fi    
    
    #不可识别版本(只打日志,不影响安装)
    log "ERROR : The os version does not match."
    return 1
}

#########################################osperf################################
################################################################################
# Function: m_init_logFile
# Description: mkdir
# Parameter:
#   input:
#   $1        dir path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_init_logFile()
{
    if [ $# -eq 0 ]; then
        error "chown arguments ($1) is null."
        return 1
    fi
    
    #检查日志目录
    if [ ! -d "$1" ]; then
        mkdir -p "$1"
    fi
}

################################################################################
# Function: m_chown_log
# Description: change own
# Parameter:
#   input:
#   $1    own flag
#   $2    path name
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_chown_log()
{
    if [[ -z "$1" || -z "$2" ]]; then
        error "chown arguments ($1) ($2) is null."
        return 1
    fi
    
    if [ ! -e "$2" ]; then
        error "chown path does not exist."
        return 1
    fi
    
    #检查日志目录
    chown "$1" "$2" -R
    return $?
}

################################################################################
# Function: m_copy_scrpitToLocal
# Description: copy scrpit to /usr/local/
# Parameter:
#   input:
#   $1    local file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_copy_scrpitToLocal()
{
    local copyPath="$1"
    
    if [ -z "${copyPath}" ]; then
        error "copy source path is null."
        return 1
    fi
    
    if [ ! -e "${copyPath}" ]; then
        error "copy source path does not exist."
        return 1
    fi
    
    #拷贝脚本修改权限
    mkdir -p /usr/local/osperf/
    cp ${copyPath}/* /usr/local/osperf/ -arf
    chmod 750 /usr/local/osperf/ -R
    
    #转换格式并执行    
    hash dos2unix > /dev/null 2>&1
    if [ $? -eq 0 ]; then 
        find /usr/local/osperf/ -maxdepth 5 -mindepth 1 -type f -name "*" | while read line
        do    
            dos2unix "$line" > /dev/null 2>&1
        done
    fi
    
    chown 0:0 /usr/local/osperf -R
    return $?
}

################################################################################
# Function: m_init_localFile
# Description: create local file
# Parameter:
#   input:
#   $1    local file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_init_localFile()
{
    local pathname=""
    local filename=""
    local localFile="$1"
    
    #空串时退出(不影响安装)
    if [ -z "${localFile}" ]; then
        info  "The g_localFile is null."
        return 0
    fi
    
    pathname=$(dirname "${localFile}")
    filename=$(basename "${localFile}")
    
    #目录是否存在
    if [ ! -d $pathname ]; then
        return 1
    fi    
    
    #找到链接路径，重新组装LOCAL文件路径
    localFile="$(readlink -e $pathname)/$filename" 
  
    #文件不存在时创建
    if [ ! -f "${localFile}" ]; then 
        touch "${localFile}"
        if [ $? -ne 0 ]; then        
            return 1
        fi
    fi

    chmod 755 "${localFile}"        
    if [ $? -ne 0 ]; then        
        return 1
    fi
    
    return 0    
}

################################################################################
# Function: m_exec_osperf
# Description: create local file
# Parameter:
#   input:
#   $1    local file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_exec_osperf()
{
    local exec_file="/usr/local/osperf/script/setup.sh"
    
    if [ -f "${exec_file}" ]; then
        $(${exec_file})
        return $? 
    fi
    
    return 0
}

################################################################################
# Function: m_save_setupScriptToLocal
# Description: save osperf file to os local file
# Parameter:
#   input:
#   $1    local file path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function m_save_setupScriptToLocal()
{    
    local os_local_file="$1"
    local perf_file="/usr/local/osperf/script/setup.sh"
    
    if [ -z "${os_local_file}" ]; then
        error "os local file path is null."
        return 1
    fi
    
    if [ ! -e "${os_local_file}" ]; then
        error "os local file path does not exist."
        return 1
    fi
    
    #往LOCAL文件登记REBOOT时执行的OS调优脚本
    sed -n '1,$p' ${os_local_file} | grep 'osperf' | grep 'script' > /dev/null
    if [ $? -ne 0 ]; then         
        echo "${perf_file}" >> ${os_local_file} 
    else
        echo "The script has been written down in the local file."
    fi
    
    info "The setup script executing success."
    return 0
}

###############################################################################
# Function: system_ctl
# Description: start or stop some services
# Parameter:
#   input:
#   $1     services name
#   $2     start or stop
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function system_ctl()
{
    local services_contrl="$1"
    local services_name="$2"
    local services_array=(crond sshd iptables cgconfig cron statmon ipmi ipmievd nscd sssd ntp ntpd rngd haveged osperf firewalld diskmgt)
    
    if ! which systemctl > /dev/null 2>&1; then
        error "can not find systemctl command."
        return 1
    fi
    
    if [[ -z "${services_name}" ]]; then
        systemctl "${services_contrl}"
        return $?
    fi
    
    for element in $(seq 0 $((${#services_array[@]} - 1)))
    do
        if [ "X$services_name" == "X${services_array[$element]}" ]; then
            systemctl "${services_contrl}" "${services_name}"
            return $?
        fi
    done
    
    echo "unknow services name."
    return 1
}

################################################################################
# Function: systemd_chmod
# Description: change mod systemd dir
# Parameter:
#   input:
#   $1      mod flag
#   $2      target path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function systemd_chmod ()
{
    local paths_array=("/usr/lib/systemd/system//osperf.service" "/usr/lib/systemd/system/osperf.service" \
                       "/usr/sbin//statmon" "/usr/sbin/statmon" \
                       "/etc/init.d//statmon" "/etc/init.d/statmon" "/etc/rc.d/init.d/statmon" \
                       "/usr/sbin/rcstatmon" "/usr/sbin//rcstatmon" \
                       "/usr/lib/systemd/system//statmon.service" "/usr/lib/systemd/system/statmon.service" \
                       "/etc/sysconfig/rngd" "/usr/local/rngd")

    if [[ -z "$1" ]]; then
        error "arguments is null."
        return 1
    fi

    if [ ! -e "$1" ]; then
        error "path $1 does not exist."
        return 1
    fi

    local file=$(readlink -f $1)
    echo "${paths_array[@]}" | grep -q -E "^${file}[[:space:]]|[[:space:]]${file}[[:space:]]|[[:space:]]${file}$"
    if [ $? -eq 0 ]; then
        chmod "644" "${file}"
        return $?
    fi

    error "chmod 644 $file : Permission denied"
    return 1
}

################################################################################
# Function: systemd_chown
# Description: change own systemd dir
# Parameter:
#   input:
#   $1      user and group
#   $2      target path
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
function systemd_chown ()
{
    local paths_array=("/usr/lib/systemd/system//osperf.service" "/usr/lib/systemd/system/osperf.service" \
                       "/usr/sbin//statmon" "/usr/sbin/statmon" \
                       "/etc/init.d//statmon" "/etc/init.d/statmon" "/etc/rc.d/init.d/statmon" \
                       "/usr/sbin/rcstatmon" "/usr/sbin//rcstatmon" \
                       "/usr/lib/systemd/system//statmon.service" "/usr/lib/systemd/system/statmon.service" \
                       "/etc/sysconfig/rngd" "/usr/local/rngd")

    if [[ -z "$1" ]]; then
        error "path is null."
        return 1
    fi
    local file=$(readlink -f $1)
    echo "${paths_array[@]}" | grep -q -E "^${file}[[:space:]]|[[:space:]]${file}[[:space:]]|[[:space:]]${file}$"
    if [ $? -eq 0 ]; then
        chown "0:0" "${file}"
        return $?
    fi

    error "chown 0:0 $file: Permission denied"
    return 1
}
######################################################################
#   DESCRIPTION: os性能调优主执行函数 
######################################################################
function main()
{              
    #设置当前的umask
    UMASK=0027
    umask "$UMASK"
    
    local systemd_system_dir="/usr/lib/systemd/system/"
    local systemd_unit_file="osperf.service"
    local cmd_name="osperf"
    
    
    g_logFile="${HOME}/osperf/osperf.log"
    g_sudoPath="${INSTALL_SUDO_PATH}"
    

	if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
        CMD_CHOWN="sudo ${g_sudoPath} m_chown_log"
        CMD_COPY_SCRIPT_TO_LOCAL="sudo ${g_sudoPath} m_copy_scrpitToLocal"
        CMD_INIT_LOCAL_FILE="sudo ${g_sudoPath} m_init_localFile"
        CMD_EXEC_OSPERF="sudo ${g_sudoPath} m_exec_osperf"
        CMD_SAVE_OSPERF_TO_LOCAL="sudo ${g_sudoPath} m_save_setupScriptToLocal"
        CMD_SYSTEMCTL="sudo ${g_sudoPath} system_ctl"
        CMD_PREINSTALL_CP="sudo ${g_sudoPath} preinstall_cp"
        CMD_PREINSTALL_CHMOD="sudo ${g_sudoPath} systemd_chmod"
        CMD_PREINSTALL_CHOWN="sudo ${g_sudoPath} systemd_chown"
	else
        CMD_CHOWN="m_chown_log"
        CMD_COPY_SCRIPT_TO_LOCAL="m_copy_scrpitToLocal"
        CMD_INIT_LOCAL_FILE="m_init_localFile"
        CMD_EXEC_OSPERF="m_exec_osperf"
        CMD_SAVE_OSPERF_TO_LOCAL="m_save_setupScriptToLocal"
        CMD_SYSTEMCTL="system_ctl"
        CMD_PREINSTALL_CP="cp -arf"
        CMD_PREINSTALL_CHMOD="systemd_chmod"
        CMD_PREINSTALL_CHOWN="systemd_chown"
	fi
            
    #初始化安装日志文件 
    init_logFile
    
    log "INFO : Begin to installing the osperf scripts."
             
    #获取当前目录
    get_currentPath
    if [ $? -ne 0 ]; then 
        log "ERROR : Exception occurs when getting the current path."
        return 1
    fi
    
    if [ ! -d "${systemd_system_dir}" ]; then
        #获取LOCAL文件
        get_localFile
        if [ $? -ne 0 ]; then 
            #不可识别版本(不安装调优脚本,但其他安装程序仍然继续)
            log "ERROR : The os version does not match."
            return 0
        fi    
    
        #初始化LOCAL文件
        ${CMD_INIT_LOCAL_FILE} "${g_localFile}"
        if [ $? -ne 0 ]; then 
            log "ERROR : Init the local file fails."
            return 0
        fi
    fi        
    
    #拷贝脚本(同时进行格式化转换)
    ${CMD_COPY_SCRIPT_TO_LOCAL} "${g_currentPath}"
    
    #执行升级脚本
    ${CMD_EXEC_OSPERF}
        
    #脚本执行失败处理
    if [ $? -ne 0 ]; then        
        log "ERROR : The setup script executing failed."
        return 0 #执行失败不影响集群安装
    fi
    
    #往LOCAL文件登记REBOOT时执行的OS调优脚本
    if [ -d "${systemd_system_dir}" ]; then
        ${CMD_PREINSTALL_CP} "${g_currentPath}/script/${systemd_unit_file}" "${systemd_system_dir}/${systemd_unit_file}"
        
        ${CMD_PREINSTALL_CHMOD} "${systemd_system_dir}/${systemd_unit_file}"
        if [ $? -ne 0 ]; then
            echo "chmod 644 ${systemd_system_dir}/${systemd_unit_file} failed."
            return 1  
        fi
    
        ${CMD_PREINSTALL_CHOWN} "${systemd_system_dir}/${systemd_unit_file}"
        if [ $? -ne 0 ]; then
            echo "chown 0:0 ${systemd_system_dir}/${systemd_unit_file} failed."
            return 1  
        fi
        
        ${CMD_SYSTEMCTL} daemon-reload
        if [ $? -ne 0 ]; then
            echo "${CMD_SYSTEMCTL} daemon-reload failed."
            return 1  
        fi
        
        ${CMD_SYSTEMCTL} enable "$cmd_name"
        ${CMD_SYSTEMCTL} stop "$cmd_name"
        ${CMD_SYSTEMCTL} start "$cmd_name"
    else
        ${CMD_SAVE_OSPERF_TO_LOCAL} "${g_localFile}"
    fi
    
    #安装成功
    log "INFO : All of the osperf script executed successfully."
    
    return 0
}

main "$@"
exit $?

