#!/bin/bash

######################################################################
#   DESCRIPTION: 调整网口缓存区大小，把当前的缓存区大小调整至最大值，提升网络吞吐量
######################################################################
function ethtool_config()
{
    local ARM_MAX=1024
	
    #获取物理网口进行循环判断
    cat /proc/net/dev | awk '{if(NR > 2) print $1}' | cut -d : -f 1 | while read DEV
    do      
        ethtool -g $DEV > /dev/null 2>&1
        if [ $? -ne 0 ]; then    
            echo "[$(date)] DEBUG : Skip to set the $DEV,because it is not the physical network card." 
            continue
        fi              

        #最大RX
        MAXRX=$(ethtool -g $DEV | grep RX: | sed -n 1p | awk '{print $NF}')
        #当前RX
        CURRX=$(ethtool -g $DEV | grep RX: | sed -n 2p | awk '{print $NF}')
		
        if [ `uname -p` == "aarch64" ];then		    
            ethtool -G $DEV rx $ARM_MAX
            if [ $? -ne 0 ]; then    
                echo "[$(date)] ERROR : Setting the rx of the $DEV fails."                
            fi
        else
            if [ $MAXRX -gt $CURRX ]; then
                ethtool -G $DEV rx $MAXRX
                if [ $? -ne 0 ]; then    
                    echo "[$(date)] ERROR : Setting the rx of the $DEV fails."
                    return 1
                fi
            fi 
        fi     
       
        
        #最大TX
        MAXTX=$(ethtool -g $DEV | grep TX: | sed -n 1p | awk '{print $NF}')
        #当前TX
        CURTX=$(ethtool -g $DEV | grep TX: | sed -n 2p | awk '{print $NF}')
		
        if [ `uname -p` == "aarch64" ];then	
            ethtool -G $DEV tx $ARM_MAX
            if [ $? -ne 0 ]; then    
                echo "[$(date)] ERROR : Setting the tx of the $DEV fails."                
            fi
        else
            if [ $MAXTX -gt $CURTX ]; then
                ethtool -G $DEV tx $MAXTX
                if [ $? -ne 0 ]; then    
                    echo "[$(date)] ERROR : Setting the tx of the $DEV fails."
                    return 1
                fi
            fi 
        fi 
        
    done
    
    return 0
}

######################################################################
#   DESCRIPTION: 网络缓存调优主执行函数
######################################################################
function main()
{    
    #调整网口缓存区大小提升网络吞吐量
    ethtool_config
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Exception occurs when Setting the ethtool options."
        return 1
    fi

    echo "[$(date)] INFO : Setting the ethtool options successfully."
    return 0
}

main
exit $?