#!/bin/bash

######################################################################
#   DESCRIPTION: 关闭内存申请过程中的碎片理整功能
######################################################################
function hugepage_config()
{
    local hugePage="/sys/kernel/mm/transparent_hugepage/defrag"
    
    if [ ! -f "${hugePage}" ]; then
        echo "[$(date)] DEBUG : Skip to set the ${hugePage},because of it does not exist."
        return 0
    fi
        
    echo never > "${hugePage}"
    return $?
}

######################################################################
#   DESCRIPTION: 关闭THP功能
######################################################################
function transparent_hugepage_config()
{
    local thp_defrag="/sys/kernel/mm/transparent_hugepage/defrag"
    local thp_disable="/sys/kernel/mm/transparent_hugepage/enabled"
    
    local thp_redhat_defrag="/sys/kernel/mm/transparent_hugepage/defrag"
    local thp_redhat_disable="/sys/kernel/mm/transparent_hugepage/enabled"
    
    local ktune_script="/etc/tune-profiles/throughput-performance-no-thp/ktune.sh"
    
    #关闭THP内存整理
    if [ -f "${thp_defrag}" ]; then
        echo never > "${thp_defrag}"
        echo "[$(date)] INFO : Set the ${thp_defrag} to never."
    else
        echo "[$(date)] DEBUG : Skip to set the ${thp_defrag},because of it does not exist."
    fi
    
    #关闭THP
    if [ -f "${thp_disable}" ]; then
        echo never > "${thp_disable}"
        echo "[$(date)] INFO : Set the ${thp_disable} to never."
    else
        echo "[$(date)] DEBUG : Skip to set the ${thp_disable},because of it does not exist."
    fi
    
     #关闭redhat THP内存整理
    if [ -f "${thp_redhat_defrag}" ]; then
        echo never > "${thp_redhat_defrag}"
        echo "[$(date)] INFO : Set the ${thp_redhat_defrag} to never."
    else
        echo "[$(date)] DEBUG : Skip to set the ${thp_redhat_defrag},because of it does not exist."
    fi
    
    #关闭redhat THP内存
    if [ -f "${thp_redhat_disable}" ]; then
        echo never > "${thp_redhat_disable}"
        echo "[$(date)] INFO : Set the ${thp_redhat_disable} to never."
    else
        echo "[$(date)] DEBUG : Skip to set the ${thp_redhat_disable},because of it does not exist."
    fi
    
    #设置ktune_script文件中的transparent_hugepages，防止其修改，详见redhat 422283
    if [ -f "${ktune_script}" ]; then
        sed -ie 's,set_transparent_hugepages always,set_transparent_hugepages never,' "${ktune_script}"
    fi
        
    return $?
}

######################################################################
#   DESCRIPTION: 碎片整理功能主执行函数
######################################################################
function main()
{
    local os_name=""
    local base_dir=$(dirname $0)
    local adapter_script="${base_dir}/adapter.sh"
    local osperf_conf="${base_dir}/osperf.conf"
    
    . "${osperf_conf}"
    source "${adapter_script}"
    os_name=$(get_osname)
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Don't support [$(cat /proc/version)]"
        return 1
    fi
    
    #关闭碎片整理功能
    hugepage_config
    if [ $? -ne 0 ]; then
        echo "[$(date)] ERROR : Exception occurs when turning off the switch of hugepage."
        return 1
    fi
    
    echo "[$(date)] INFO : Turning off the switch of hugepage successfully."
    
    #关闭THP
    if [ "${osperf_disable_transparent_hugepage}" -eq 1 ]; then
        transparent_hugepage_config
        if [ $? -ne 0 ]; then
            echo "[$(date)] ERROR : Exception occurs when disable transparent_hugepage."
            return 1
        fi
        
        echo "[$(date)] INFO : Disable transparent_hugepage successfully."
    fi
    
    return 0
}

main
exit $?


