#/bin/bash
MASK=''
######################################################################
#   函数说明: 计算2的n次方
#   入参: $1   2的次方数
#   返回值: 无
#   作用: 更新全局变量MASK，把2的n次方数以16进制的方式赋值给MASK
######################################################################
function moveLeft()
{
    #拼接16进制数的头
    local head=''
    #拼接16进制数的0的个数
    local zero=''
    #2的n次方16进制数头只会出现1,2,4,8四种
    local -a arryhead=(1 2 4 8)
    local pow=$1

    #num1对4取模结果只有 0,1,2,3  
    num1=$(expr $pow % 4)
    #num2除以4得到16进制数中数字0的个数
    num2=$(expr $pow / 4)

    #用num1在数组中取出16进制的头
    head=${arryhead[${num1}]}

    #用num2构造16进制0的个数
    for((i=0;i<$num2;i++));
    do
        zero=${zero}'0'
    done

    #拼接16进制，更新MASK的值
    MASK=''
    MASK=${head}${zero}
}

######################################################################
#   DESCRIPTION: 设置网络中断绑定到CPU核
######################################################################
function set_irq_affinity()
{
    #检查irqbalance是否在运行，如果在运行那么直接返回，因为设置了也没什么效果
    ps ax | grep -v grep | grep -q irqbalance > /dev/null
    if [ $? -eq 0 ] ; then
        echo "[$(date)] WARN : irqbalance is running and will likely override this script's affinitization, please stop the irqbalance service."
        return 0
    fi
    
    #获取当前节点所有的eth设备
    eth_devices=$(cat /proc/net/dev | tr -d " " | awk -F: '{if (NR > 2) print $1}')
    if [ "X$eth_devices" == "X" ]; then
        echo "[$(date)] INFO : can not find any eth devices."
        return 0
    fi
    
    #变量设备对设备进行设置
    for DEV in $eth_devices
    do
        for DIR in rx tx TxRx fp "[0-9]"
        do
            #获取设备所分配的中断个数
            MAX=`grep $DEV-$DIR /proc/interrupts | wc -l`
            if [ "$MAX" == "0" ] ; then
                MAX=`egrep -i "$DEV:.*$DIR" /proc/interrupts | wc -l`
            fi
            
            if [ "$MAX" == "0" ] ; then
                echo "[$(date)] INFO : no $DIR vectors found on $DEV."
                continue
            fi
            
            ((MAX=MAX-1))
            #对每一个中断进行处理 
            for VEC in `seq 0 1 $MAX`
            do
                #获取中断号，获取失败直接返回
                if [ "X$DIR" == "X[0-9]" ]; then
                    IRQ=`cat /proc/interrupts | grep -i $DEV-$VEC"$"  | cut  -d:  -f1 | sed "s/ //g"`
                    if [ -z "$IRQ" ]; then
                        IRQ=`cat /proc/interrupts | egrep -i $DEV:v$VEC"$"  | cut  -d:  -f1 | sed "s/ //g"`
                    fi
                else
                    IRQ=`cat /proc/interrupts | grep -i $DEV-$DIR-$VEC"$"  | cut  -d:  -f1 | sed "s/ //g"`
                    if [ -z "$IRQ" ]; then
                        IRQ=`cat /proc/interrupts | egrep -i $DEV:v$VEC-$DIR"$"  | cut  -d:  -f1 | sed "s/ //g"`
                    fi
                fi
                
                if [ -z "$IRQ" ]; then
                    continue
                fi
                
                moveLeft $VEC
                printf "[$(date)] INFO : %s mask=%s for /proc/irq/%d/smp_affinity\n" $DEV $MASK $IRQ
                echo $MASK|sed -e ':a' -e 's/\(.*[0-9]\)\([0-9]\{8\}\)/\1,\2/;ta' > /proc/irq/$IRQ/smp_affinity 
            done
        done
    done
    
    return 0
}


function find_dev_node_id()
{
    if [ $# -ne 1 ]
    then
        echo "input parameter is wrong, please input the right ethx"
        return ${ret_err}
    fi

    local ethx=$1
    local eth_bus=`ethtool -i  ethx | grep bus-info | awk ' {print $2}'`        

    local dev_dir_in_sysfs=`find /sys/devices/ -name "${eth_bus}" | grep pci`  
    if [ $? -ne 0 ]
    then
        echo "not find the eth bus, so it is not a physical machine!"
        return ${ret_err}
    fi

    g_node_id=""    
    g_node_id=`cat ${dev_dir_in_sysfs}/numa_node`
}

function get_sorted_cpu_list()
{
    if [ $# -ne 1 ]
    then
        echo "input parameter is wrong, please input the right ethx"
        return ${ret_err}
    fi
 
    local ethx=$1
    
    find_dev_node_id ${ethx}
    if [ $? -eq 0 ]
    then
        echo "err: not find node id for ${ethx}"
        return ${ret_err}
    fi

    # sort nodes
    local node_id=${g_node_id}
    local let node_id_column=${node_id}+2
    local node_num=`lscpu | grep -E "NUMA node\(s\)" | sed 's/[ ]\+//g' | cut -d':' -f2`  
    local node_sorted=`numactl -H | tail -${node_num} | sed 's/[ ]\+/ /g' 
               | sed 's/://g' | sed 's/^ //g' 
               | cut -d' ' -f1,${node_id_column} | sort -k 2 
               | cut -d' ' -f1`    

    # sort cpus
    g_cpus=""
    for node_id in ${node_sorted} 
    do 
        g_cpus+=`numactl -H | grep -E "node ${node_id} cpus" | cut -d':' -f2`
    done
}

function get_dev_irqs()
{
    if [ $# -ne 1 ]
    then
        echo "input parameter is wrong, please input the right ethx"
        return ${ret_err}
    fi

    g_irqs=""
    local ethx=$1
    g_irqs=`cat /proc/interrupts | grep ${ethx} | cut -d':' -f1`
}

function bind_interrupt_to_cpu()
{
    if [ $# -ne 2 ]
    then
        echo "err: input parameters is wrong!"
        return ${ret_err}
    fi
    
    local result=0
    local irqs=($1)
    local cpus=($2)

    irq_count=${#irqs[@]}
    cpu_count=${#cpus[@]}
    if [ ${cpu_count} -lt ${irq_count} ]
    then
        echo "err: cpu count is less than irq count"
        return ${ret_err}
    fi

    echo "irq_count is ${irq_count}"

    i=0
    while [ $i -lt ${irq_count} ]
    do
        local irq_num=${irqs[${i}]}
        local cpu_num=${cpus[${i}]}
        let i=${i}+1

        echo "begin to bind interrupt ${irq_num} to cpu ${cpu_num}" 

        # bind 
        # if n < 63; 2^n = let y=2**n; echo "obase=16;ibase=10;$y" | bc
        #echo 2^n > /proc/irq/${irq_num}/smp_affinity   #   
        echo ${cpu_num} > /proc/irq/${irq_num}/smp_affinity_list

        # check 
        local cpu_num_back=`cat /proc/irq/${irq_num}/smp_affinity_list`
        if [ "${cpu_num}" != "${cpu_num_back}" ]
        then
            echo "fail to bind interrupt to cpu"
            result=${ret_err}
            break
        else
            echo "successfully to bind interrupt ${irq_num} to cpu ${cpu_num}"
        fi
    done 

    return ${result}
}


function set_physical_machine_cpu_affinity()
{
    local eth_devices=`ls /sys/class/net/  | grep -v lo`
    if [ "X$eth_devices" == "X" ]; then
        echo "can not find any eth devices."
        return ${ret_ok}
    fi

    for ethx in ${eth_devices}
    do
        get_sorted_cpu_list ${ethx}
        if [ $? -eq 0 ]
        then
            echo "err: not find cpus for ${ethx}"
            continue
        fi
        
        get_dev_irqs ${ethx}
        if [ $? -eq 0 ]
        then
            echo "err: not find irqs for ${ethx}"
            continue
        fi

        bind_interrupt_to_cpu ${g_irqs}, ${g_cpus}
        if [ $? -eq 0 ]
        then
            echo "fail to bind irq for ${ethx}!"
        fi
    done
}

function is_physical_machine()
{
    local bus_info=`ls /sys/class/net/ | grep -v lo \
                  | head -n1 | xargs -n 1 -i ethtool -i {} \
                  | grep bus-info | cut -d' ' -f2 \
                  | grep "[0-f]\{4\}:[0-f]\{2\}:[0-f]\{2\}.[0-f]"`

    if [ "Y${bus_info}" != "Y" ]
    then
        echo "this is a physical machine"
        return ${ret_ok}
    fi

    echo "this is a virtual mathine"
    return ${ret_err}
}

############################
ret_ok=0
ret_err=1
g_node_id=""
g_cpus=""
g_irqs=""

######################################################################
#   DESCRIPTION:设置网络中断绑定到CPU核主执行函数   
######################################################################
function main()
{
    local base_dir=$(dirname $0)
    local osperf_conf="${base_dir}/osperf.conf"

    . "${osperf_conf}"
    
    if [ "${osperf_net_irq_affinity}" -eq 1 ]; then
        is_physical_machine
        if [ $? -eq 0 ]
        then
            service irqbalance stop > /dev/null
            ps ax | grep -v grep | grep -q irqbalance > /dev/null
            if [ $? -eq 0 ] ; then
                echo "[$(date)] WARN : irqbalance is running and will likely override this script's affinitization, please stop the irqbalance service."
                return 0
            fi

            echo "set cpu affinity for physical machine using new method"
            set_physical_machine_cpu_affinity 
        else
            echo "set cpu affinity using original method!" 
            set_irq_affinity 
        fi

        if [ $? -ne 0 ]; then
           echo "[$(date)] ERROR : Exception occurs when set affinity."
           return 1
        fi
    fi

    echo "[$(date)] INFO : Setting irq affinity successfully."     
    return 0
}

main
exit $?

