#!/bin/bash

function config_para()
{
    sed -i -e '/kernel.core_pattern/d' /etc/sysctl.conf
    mkdir -p ${core_dir}
    chmod 1777 ${core_dir}
    echo "kernel.core_pattern=${core_dir}/core-%t-%u-%e-%p" >> /etc/sysctl.conf
    
    sed -i -e '/omm[ ]\+soft[ ]\+core/d' /etc/security/limits.d/omm-ncore.conf
    sed -i -e '/omm[ ]\+hard[ ]\+core/d' /etc/security/limits.d/omm-ncore.conf
    echo "omm   soft    core     10240" >> /etc/security/limits.d/omm-ncore.conf
    echo "omm   hard    core     10240" >> /etc/security/limits.d/omm-ncore.conf

    sysctl -p >/dev/null
}

function process_core()
{        
    local files=""
    local first_time=""
    local now_time=0
    local time_diff=0
    local total_size=0

    while [ 1 ]
    do
        total_size=`du -m ${core_dir} | awk '{print $1}'`
        if [ ${total_size} -lt ${max_space} ]
        then
            sleep ${timeout}
            continue
        fi
       
        files=`ls ${core_dir}/core-* -l 2>/dev/null` 
        if [ $? -ne 0 ]
        then
            sleep ${timeout}
            continue
        fi
    
        first_time=`echo ${files} | head -n1 | cut -d' ' -f9 | cut -d'-' -f2`
        now_time=`date +%s`
        let time_diff=${now_time}-${first_time}
 
        if [ ${total_size} -gt ${max_space} ] || [ ${time_diff} -gt ${max_keep_time} ]
        then
            rm -rf ${core_dir}/core-${first_time}*
        fi  

    done
}
 
function main()
{
    config_para
    process_core & 
}

timeout=30
max_space=512 #MB
let max_keep_time=7*24*3600  # a week
core_dir="/var/log/coredump"

main
