#!/bin/bash

function min()
{
    if [ $# -ne 2 ]
    then
        echo ${oom_err}
        return
    fi

    local value1=$1
    local value2=$2
    if [ $1 -lt $2 ]
    then
        echo $1
    else
        echo $2
    fi
}


function set_min_free_kbytes()
{
    local total_mem=$1
    let local cal_min_free_mem=${total_mem}*1024*2/100 # total * 2%
    let local default_value=2*1024*1024 #unit:KB ==> 2GB
    
    local value=$(min ${cal_min_free_mem} ${default_value})
    if [ ${value} -eq ${oom_err} ]
    then
        echo "err: min value is not right from cal"
        return 1 
    fi

    local cur_free=`cat /proc/sys/vm/min_free_kbytes`
    if [ ${cur_free} -lt ${value} ]
    then
        echo ${value} > /proc/sys/vm/min_free_kbytes
    fi
}

function adjust_mem_para()
{
    local total_mem=$1

    set_min_free_kbytes ${total_mem}
    if [ $? -ne 0 ]
    then
        echo "err fail to set min free"
        return 1 
    fi
}

function dynamic_prevent_oom()
{
    local total_mem=$1
    
    local default_min_free_mem=1024 #MB
    let local cal_min_free_mem=${total_mem}/100

    local min_free_mem_1=768 #MB
    local min_free_mem_2=$(min ${default_min_free_mem} ${cal_min_free_mem})
    if [ ${min_free_mem_2} -le ${min_free_mem_2} ] 
    then
        min_free_mem_2=${default_min_free_mem}
    fi

    log_dir="/var/log/osinfo/statistics"
    log_file=${log_dir}/oom.log
    mkdir -p ${log_dir}

    while [ 1 ]
    do
        local free_mem=`free -m  | grep Mem | sed 's/[ ]\+/ /g' | cut -d' ' -f4`
        if [ ${free_mem} -lt ${min_free_mem_1} ]
        then
            echo "`date`: begin to free pagecache, inode and dentry" >> ${log_file}
            sync
            echo 3 > /proc/sys/vm/drop_caches
            echo "`date`: end to free pagecache, inode and dentry" >> ${log_file}
        elif [ ${free_mem} -lt ${min_free_mem_2} ]
        then
            echo "`date`: begin to free pagecache" >> ${log_file}
            sync
            echo 1 > /proc/sys/vm/drop_caches 
            echo "`date`: end to free pagecache" >> ${log_file}
        else
            :  #nop
        fi

        sleep ${oom_timeout}
    done
}

function prevent_oom()
{
    local total_mem=$1

    adjust_mem_para ${total_mem}
    if [ $? -ne 0 ]
    then
        echo "err fail to adjust mem parameter"
        return 1 
    fi 
 
    dynamic_prevent_oom ${total_mem} &
    if [ $? -ne 0 ]
    then
        echo "err fail to adjust mem parameter"
        return 1 
    fi 
}

oom_err=-1
oom_timeout=5

function main()
{
    total=`free -m  | grep Mem | sed 's/[ ]\+/ /g' | cut -d' ' -f2`
    prevent_oom ${total}
}

main
