#!/bin/bash

g_currentPath=""                                #当前目录
g_logFile="/var/log/osperf/osperf.log"          #日志文件

######################################################################
#   DESCRIPTION: 日志打印接口
######################################################################
function log()
{
    local logBuffer=""  #打印内容

    logBuffer="$1"       
    echo "[$(date)] ${logBuffer}" >> "${g_logFile}"    
}

######################################################################
#   DESCRIPTION: 初始化日志文件
######################################################################
function init_logFile()
{       
    #检查日志目录
    if [ ! -d $(dirname "${g_logFile}") ]; then
        mkdir -p $(dirname "${g_logFile}")
    fi        
}

######################################################################
#   DESCRIPTION: 获取当前目录
######################################################################
function get_currentPath()
{       
    #获取当前目录
    pushd $(dirname "${BASH_SOURCE:-$0}") > /dev/null
    g_currentPath="$(readlink -e "$(pwd)")"        
    popd > /dev/null
    
    if [ -z "${g_currentPath}" ]; then
        return 1
    fi
    
    return 0
}

######################################################################
#   DESCRIPTION: os性能调优脚本主执行函数      
######################################################################
function main()
{    
    #设置当前的umask
    UMASK=0027
    umask "$UMASK"
    
    #初始化日志文件
    init_logFile
    
    log "INFO : Begins to executing the setup.sh"

    #获取当前目录
    get_currentPath
    if [ $? -ne 0 ]; then 
        log "ERROR : Exception occurs when getting the current path."
        return 1
    fi
    
    #日志压缩
    ${g_currentPath}/log/scriptlog.sh "${g_logFile}" 
        
    #日志压缩失败
    if [ $? -ne 0 ]; then
        #失败不退出安装
        log "ERROR : Exception occurs when Compress the log file."
    fi         
                    
    #不是一个目录
    if [ ! -d "${g_currentPath}/optimize/" ]; then
        log "ERROR : ${g_currentPath}/optimize/ is not a valid directory."
        return 1
    fi      
    
    #遍历目录下脚本并执行
    pushd "${g_currentPath}/optimize/" > /dev/null    
    find "${g_currentPath}/optimize/" -maxdepth 2 -mindepth 1 -name "[0-9][0-9][0-9]*.sh" | sort | while read line
    do      
        $line >> "${g_logFile}" 2>&1  #执行并重定向输出到日志文件
        if [ $? -ne 0 ]; then
            log "ERROR : Execute $line fails."
        fi     
    done
    popd > /dev/null

    sysctl -q -p
    log "INFO : Finish executing the setup.sh"
    return 0
}

main
exit $?
