#!/bin/bash

g_localFile=""      #操作系统的LOCAL文件

g_logFile=""        #安装日志文件

if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo
    CMD_UNINSTALL="sudo ${INSTALL_SUDO_PATH} m_uninstallOsperf"
else
    CMD_UNINSTALL="m_uninstallOsperf"
fi

######################################################################
#   DESCRIPTION: 初始化日志文件
######################################################################
function init_logFile()
{       
    #检查日志目录
    if [ ! -d $(dirname "${g_logFile}") ]; then
        mkdir -p $(dirname "${g_logFile}")
    fi    
}

######################################################################
#   DESCRIPTION: 日志打印接口
######################################################################
function log()
{
    local logBuffer=""  #打印内容

    logBuffer="$1"       
    echo "[$(date)] ${logBuffer}" >> "${g_logFile}"    
}

function m_uninstallOsperf()
{
    local g_localFile=''
    #Red Hat版本,且版本号为6.4
    if [ -f "/etc/redhat-release" ]; then  
        #版本号打印到运行日志,方便定位
        log "INFO" "$(cat /etc/redhat-release)"
        
        g_localFile="/etc/rc.local"         #redhat的local目录
    fi    
       
    #匹配SuSE11SP1
    if [ -f "/etc/SuSE-release" ]; then   
        #版本号打印到运行日志,方便定位
        log "INFO" "$(cat /etc/SuSE-release)"
    
        g_localFile="/etc/rc.d/after.local" #suse的local目录
    fi    
    
    #CentOS版本
    if [ -f "/etc/centos-release" ]; then  
        #版本号打印到运行日志,方便定位
        log "INFO" "$(cat /etc/centos-release)"
        
        g_localFile="/etc/rc.local"         #CentOS的local目录
    fi    
    
    if [ -z "${g_localFile}" ]; then
        log "ERROR" "Uninstall osperf failed, The os version does not match."
        return 0
    fi

    g_localFile=$(readlink -e "$g_localFile")
    
    #删除LOCAL文件登记的REBOOT时执行的OS调优脚本  
    if [ -f "${g_localFile}" ]; then 
        sed -i '/osperf\/script/d' ${g_localFile}
        if [ $? -ne 0 ]; then 
            log "ERROR" "Modify the g_localFile configure fails."
        fi    
    else
        log "INFO" "The g_localFile does not exist."
    fi
    
    #删除拷贝目录
    if [ -n "/usr/local/osperf" ]; then
        rm -rf "/usr/local/osperf"        
        if [ $? -ne 0 ]; then 
            log "ERROR" "Delete the directory[/usr/local/osperf/] fails."
        fi    
    fi 
    
    return 0
}

######################################################################
#   DESCRIPTION: os性能调优主执行函数 
######################################################################
function main()
{         
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"
    
    g_logFile="/var/log/osperf/uninstall.log"

    #初始化安装日志文件 
    init_logFile      

    #SUDO_PATH_NODEAGENT=$(su - "${OMM_USER}" -c 'echo ${SUDO_PATH_NODEAGENT}')
    ${CMD_UNINSTALL} >> ${g_logFile} 2>&1
    if [ $? -ne 0 ]; then
        log "ERROR" "uninstall osperf fails."
        return 1
    fi

    #卸载成功
    log "INFO : All of the uninstall script executed successfully." 
    
    return 0
}

main "$@"
exit $?