#!/bin/sh

###############################################################################
#author:   none
#date:     2014/02/16
#function: 硬盘管理服务监控脚本
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#设置当前的umask
UMASK=0077
umask "$UMASK"

#日志目录和文件
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"

#日志目录和文件
LOGPATH="/var/log/diskmgt"
LOGFILE="/var/log/diskmgt/diskmgt.log"
LOGSYSFILE="/var/log/messages"

#运行的用户角色
RUN_UID=0
RUN_GID=0
RUN_USER=""
RUN_GROUP=""


########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] diskmgtmodule cron: ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] diskmgtmodule cron: ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] diskmgtmodule cron: ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 设置运行的用户属性
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_run_belong()
{
    if [ ! -f ${SERVICERUN} ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${SERVICERUN} is not exist."
        return 1
    fi
    
    RUN_UID=$(stat -c %u ${SERVICERUN} 2>/dev/null)
    if [ -z "${RUN_UID}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_UID is null."
        stat ${SERVICERUN} >>"${LOGFILE}" 2>&1
        return 1
    fi
    
    RUN_GID=$(stat -c %g ${SERVICERUN} 2>/dev/null)
    if [ -z "${RUN_GID}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_GID is null."
        stat ${SERVICERUN} >>"${LOGFILE}" 2>&1
        return 1
    fi
    
    RUN_USER="$(stat -c %U ${SERVICERUN} 2>/dev/null)"
    if [ -z "${RUN_USER}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_USER is null."
        stat ${SERVICERUN} >>"${LOGFILE}" 2>&1
        return 1
    fi
    getent passwd | grep "^${RUN_USER}:" >/dev/null
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_USER(${RUN_USER}) is not exist in system."
        return 1
    fi
    
    RUN_GROUP="$(stat -c %G ${SERVICERUN} 2>/dev/null)"
    if [ -z "${RUN_GROUP}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_GROUP is null."
        stat ${SERVICERUN} >>"${LOGFILE}" 2>&1
        return 1
    fi
    getent group | grep "^${RUN_GROUP}:" >/dev/null
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] RUN_GROUP(${RUN_GROUP}) is not exist in system."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 监控过程中判断日志目录是否被移除，如果被删除则进行创建
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function make_sure_dir_exist()
{
    if [ -d "${LOGPATH}" ]; then
        return 0
    fi
    
    #目录不存在，打印日志
    echo `date` "diskmgtmodule cron: ${LOGPATH} is not exist, it was deleted by someone!" >>"${LOGSYSFILE}"
    mkdir -p "${LOGPATH}" >>"${LOGSYSFILE}" 2>&1
    if [ $? -ne 0 ]; then
        echo `date` "diskmgtmodule cron: mkdir -p ${LOGPATH} failed." >>"${LOGSYSFILE}"
        return 1
    fi
    echo `date` "diskmgtmodule cron: mkdir -p ${LOGPATH} successfully." >>"${LOGSYSFILE}"
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] mkdir -p ${LOGPATH} successfully."
    
    #获取用户属性，这样可以后面为新建的日志路径设置同样的属性
    set_run_belong
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] set run belong failed."
        return 1
    fi
    
    if [ ${RUN_UID} -ne 0 ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] need chown ${RUN_USER}:${RUN_GROUP} ${LOGPATH}."
        chown ${RUN_USER}:${RUN_GROUP} "${LOGPATH}" -R
        chmod 755 "${LOGPATH}" -R
    fi
    
    return 0
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    local run_user=""
    local run_group=""
    local is_newlog=0

    #判断日志路径是否存在，如果不存在，则创建并设置对应用户属性，否则可能运行的用户无法写入日志
    make_sure_dir_exist
    if [ $? -ne 0 ]; then
        return 1
    fi

    #已经运行了，则直接退出，这也是大部分的场景
    pid_d=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
    if [ ! -z "${pid_d}" ]; then
        return 0
    fi
    
    #服务异常停止了，下面进行拉起操作
    logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${SERVICERUN} disappears in ps."

    #服务主目录被删除，则打印错误后直接退出，无法进行任何有效操作了
    if [ ! -f "${SERVICERUN}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] run-script ${SERVICERUN} is not exist."
        return 1
    fi
    if [ ! -x "${SERVICERUN}" ]; then
        chmod 755 "${SERVICERUN}" >>"${LOGFILE}" 2>&1
        if [ $? -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 755 ${SERVICERUN} failed."
            return 1
        fi
    fi
    
    #获取用户属性，这样可以后面为新建的日志路径设置同样的属性
    set_run_belong
    if [ $? -ne 0 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] set run belong failed."
        return 1
    fi
    
    #判断用户属性，进行对应的后台启动控制
    if [ ${RUN_UID} -eq 0 ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] root resume ${SERVICERUN}."
        nohup ${SERVICERUN} >/dev/null 2>&1 &
    else
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ${RUN_USER}:${RUN_GROUP} resume ${SERVICERUN}."
        nohup ${SERVICERUN} >/dev/null 2>&1 &
    fi
    
    return $?
}

#调用主程序
main
exit $?

