#!/bin/bash


###############################################################################
#author:   none
#date:     2014/02/12
#function: 安装硬盘管理服务
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#主机名或IP地址
IPADDR="$(echo ${SSH_CONNECTION} | awk '{print $3}')"

#设置当前的umask
UMASK=0077
umask "$UMASK"

#运行的用户/组
RUN_USER=2000
RUN_GROUP=wheel

#设置当前路径
CURRPATH=""

#设置临时路径，所有的临时文件都放在此目录下
TMPDIR=""

#输入的挂载文件参数
INIFILE=""

#对挂载文件参数处理后的文件名
ININEWFILE=""

#是否强制安装
#0不强制安装（如果之前安装过，那么这次安装就失败），1强制安装
IS_FORCE=0

#检查系统之前是否安装过硬盘管理服务，判断依据是/usr/local/diskmgt目录是否存在
IS_EMPTY=0

#文件系统配置文件
FSTABFILE="/etc/fstab"

#根分区设备变量
ROOTDEV=""

#根分区文件系统
ROOTFS=""

#根分区格式化工具
ROOTMKFS=""

#系统中存在此脚本执行的总进程数
#主要用于此脚本如果被强杀那么就遗留锁文件，等待下一次进来此脚本被锁定判断其实是之前被强杀，就返回特定错误码告知此事
#并删除遗留锁
CMDCNT=0
TMPFILE=""

#服务目录和文件
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"
SCC_INIT_INSTALL_PATH="${HOME}/scc"
SCC_INSTALL_PATH="${SERVICEPATH}/scc"

#磁盘加密的口令文件
KEY_FILE_PATH="$SERVICEPATH/ini-plugin/keys"
KEY_FILE_1="$SERVICEPATH/ini-plugin/keys/fi_disk_partition_key_1"
KEY_FILE_2="$SERVICEPATH/ini-plugin/keys/fi_disk_partition_key_2"

#锁文件
LOCKFILE="${HOME}/insdmgt.lock"

#日志目录
LOGPATH="${HOME}/diskmgt/"

#日志文件
LOGFILE="${LOGPATH}/insdmgt.log"
RESULT=0
###############################################################################


#############################################################
#下面为非root用户切换root用户需要执行的命令
if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo    
    CMD_WHICH="sudo ${INSTALL_SUDO_PATH} preinstall_which"
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
    CMD_READLINK_E="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_e"
    CMD_READLINK_M="sudo ${INSTALL_SUDO_PATH} preinstall_readlink_m"
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
    CMD_FINDFS="sudo ${INSTALL_SUDO_PATH} preinstall_findfs"
    CMD_FILE_S="sudo ${INSTALL_SUDO_PATH} preinstall_file_s"
    CMD_CHOWN="sudo ${INSTALL_SUDO_PATH} preinstall_chown"
    CMD_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_CHMOD_R="sudo ${INSTALL_SUDO_PATH} preinstall_chmod_r"
    CMD_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_INSTALL="sudo ${INSTALL_SUDO_PATH} preinstall_install"
    CMD_DOS2UNIX="sudo ${INSTALL_SUDO_PATH} preinstall_dos2unix"
    CMD_IS_DIR="sudo ${INSTALL_SUDO_PATH} preinstall_is_dir"
    CMD_IS_FILE="sudo ${INSTALL_SUDO_PATH} preinstall_is_file"
    CMD_SWAPON_S="sudo ${INSTALL_SUDO_PATH} preinstall_swapon_s"
    CMD_RM_F="sudo ${INSTALL_SUDO_PATH} preinstall_rm_f"
else
    CMD_WHICH="which"
    CMD_MKDIR_P="mkdir -p"
    CMD_READLINK_E="readlink -e"
    CMD_READLINK_M="readlink -m"
    CMD_CAT_FSTAB=" cat /etc/fstab"
    CMD_FINDFS="findfs"
    CMD_FILE_S="file -s"
    CMD_CHOWN="chown"
    CMD_CHOWN_R="chown -R"
    CMD_CHMOD="chmod"
    CMD_CHMOD_R="chmod -R"
    CMD_CP="cp -arf"
    CMD_INSTALL="preinstall_install"
    CMD_DOS2UNIX="preinstall_dos2unix"
    CMD_IS_DIR="preinstall_is_dir"
    CMD_IS_FILE="preinstall_is_file"
    CMD_SWAPON_S="swapon -s"
    CMD_RM_F="rm -rf"
fi
################################################################


########################################################################
#                           sudo封装开始
########################################################################
########################################################################
# 函数描述： 判断路径是否是一个目录
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_dir ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -d "$1" ]; then
                 return 0
        fi

    return 1
}

########################################################################
# 函数描述： 判断路径是否是一个文件
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_file ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -f "$1" ]; then
                 return 0
        fi

    return 1
}


########################################################################
# 函数描述： 调用install命令
# 参数1说明：安装目录权限
# 参数2说明：安装路径
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_install()
{
    if [[ -z "$1" || -z "$2" ]]; then
        echo "arguments is null."
        return 1
    fi
    
        install -m "$1" -d "$2"
    return $?
}

########################################################################
# 函数描述： 将指定路径的文件全部执行dos2unix命令
# 参数1说明：待转换的路径
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_dos2unix()
{
        if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    find "$1" -type f | xargs dos2unix >/dev/null 2>&1
    return $?
}

########################################################################
#                           sudo封装结束
########################################################################

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi
    
    chmod 700 "${LOGPATH}"

    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
    fi
    
    return 0
}

########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************INSTALL-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    get_call_stack
    logme 1 "************************INSTALL-END**************************"
    
    return 0
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-n XXX.ini [-f]       Notes: install diskmgt by ini file(force)."
    echo "-h                    Notes: show help."
    echo
    echo "Example:"
    echo "`basename $0` -n master.ini"
    echo "`basename $0` -n master.ini -f"
    echo "`basename $0` -h"
    echo
    return 0
}

########################################################################
# 函数描述： 初始化路径变量
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_path()
{
    #初始化当前路径，为其他变量赋值做准备
    pushd $(dirname "$0") >/dev/null 2>&1
    CURRPATH="$(readlink -e "`pwd`")"
    popd >/dev/null 2>&1
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    
    #设置全局保存设备变量
    set_temp_file
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] set temp file failed."
        return 1
    fi
    
    #压缩日志，具体压缩规格见scriptlog.sh脚本中描述
    compress_log
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress log failed."
        return 1
    fi

    return 0
}

########################################################################
# 函数描述： 设置系统中的设备的保存文件绝对路径
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function set_temp_file()
{
    TMPDIR="${CURRPATH}/tmpdir"
    
    if [ ! -d "${TMPDIR}" ]; then
        mkdir -p "${TMPDIR}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkdir -p ${TMPDIR} failed."
            return 1
        fi
    fi
    
    return 0
}

########################################################################
# 函数描述： 压缩日志
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function compress_log()
{
    script_log="${CURRPATH}/inner/scriptlog.sh"
    
    if [ ! -f "${script_log}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] compress script(${script_log}) is not exist."
        return 1
    fi
    
    if [ ! -x "${script_log}" ]; then
        chmod 755 "${script_log}"
    fi
    
    #压缩日志
    if [ -f "${LOGFILE}" ]; then
        "${script_log}" "${LOGFILE}"
    fi
    
    return 0
}

########################################################################
# 函数描述： 获取脚本执行的调用栈
# 参数1说明：进程pid号
#
# 返回值  ： 0 成功
########################################################################
function get_call_stack()
{
    local pid_val=$1
    local line=""

    if [ -z "${pid_val}" ]; then
        pid_val=$$
    fi
    
    logme 1 "==stack-begin=="
    
    #获取调用栈
    while [[ ${pid_val} != 1 && ${pid_val} != 0 ]]
    do
        line=$(ps -wweo pid,ppid,uname,tname,args | grep "^[[:space:]]*${pid_val}[[:space:]]")
        if [ -z "${line}" ]; then
           break
        fi
        pid_val=$(echo ${line} | awk '{print $2}')
        logme 1 "${line}"
    done
    
    logme 1 "==stack-end=="
    
    return 0
}


########################################################################
# 函数描述： 检查系统的有效信息，包括用户、操作系统、命令等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_system()
{
    #检查操作系统
    check_osversion
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check os version failed."
        return 1
    fi
    
    #检查命令
    check_command
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check command failed."
        return 1
    fi
    
    #检查根分区信息
    check_rootinfo
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check root info failed."
        return 1
    fi

    #检查挂载项和fstab是否一致，如果不一致可能本来重启就会挂住，我们不会再冒险做任何操作
    check_mount
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check mount failed."
        return 1
    fi
    
    #检查swap挂载项和fstab是否一致
    check_swap
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check swap failed."
        return 1
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 检查执行操作系统版本，目前支持redhat/suse/centos
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_osversion()
{
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local centos_release="/etc/centos-release"
    local neokylin_relase="/etc/neokylin-release"
    local bclinux_release="neokylin_relase"
    
    #支持红帽/suse/centos
    #为了最大程度上的支持，都不会异常退出，但是会打印告警，表示可能不太支持，毕竟没有在其他操作系统上做过测试
    if [ -f "${centos_release}" ]; then
        cat "${centos_release}" | grep -i "CentOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is centos."
    #获取Euler的版本
    elif [ -f "${neokylin_relase}" ];then
        cat "${neokylin_relase}" | grep -i "NeoKylin" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not NeoKylin, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is NeoKylin."
    elif [ -f "${bclinux_release}" ];then
        cat "${bclinux_release}" | grep -i "BigCloud" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not bclinux, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is bclinux."        
    elif [ -f "${euler_release}" ]; then
        cat "${euler_release}" | grep -i "EulerOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not euleros, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is euleros."
    elif [ -f "${rhel_release}" ]; then
        cat "${rhel_release}" | grep -i "Red Hat" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not redhat, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is redhat."
    elif [ -f "${sles_release}" ]; then
        cat "${sles_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse."
    elif [ -f "${sles15_release}" ]; then
        cat "${sles15_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            cat "${sles15_release}" | grep -i "openEuler " >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is openEuler."
                return 0
            fi
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse-15, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse-15."
    else
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos/redhat/suse, maybe it will cause unknown result!"
        return 0
    fi
    
    
    return 0
}

########################################################################
# 函数描述： 检查系统包含的有效命令，如lsscsi/fdisk等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_command()
{
    #要求系统中存在lsscsi命令，因为是我们自己适配的os，这些命令都可以保证有，下同
    which lsscsi >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] lsscsi command not found."
        return 1
    fi

    ${CMD_WHICH} fdisk >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fdisk command not found."
        return 1
    fi

    ${CMD_WHICH} sfdisk >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sfdisk command not found."
        return 1
    fi
    
    ${CMD_WHICH} parted >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] parted command not found."
        return 1
    fi

    which free >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] free command not found."
        return 1
    fi
    
    ${CMD_WHICH} mkfs.ext3 >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs.ext3 command not found."
        return 1
    fi
    
    ${CMD_WHICH} mkfs.ext4 >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mkfs.ext4 command not found."
        return 1
    fi
    
    ${CMD_WHICH} swapon >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] swapon command not found."
        return 1
    fi
    
    which getent >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] getent command not found."
        return 1
    fi

    ${CMD_WHICH} blkid >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] blkid command not found."
        return 1
    fi

    ${CMD_WHICH} findfs >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] findfs command not found."
        return 1
    fi

    which readlink >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] readlink command not found."
        return 1
    fi

    which file >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] file command not found."
        return 1
    fi
    
    which ln >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ln command not found."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查操作系统根分区信息，包括文件系统、分区等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_rootinfo()
{
    if [ ! -f "${FSTABFILE}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab(${FSTABFILE}) is not exist."
        return 1
    fi
    
    #获取根分区
    ROOTDEV=$(mount 2>/dev/null | awk '{if( $3=="/" ) print $1}' | sed 's/\/dev\///' | sed 's/[0-9]//g' | sed 's/[[:space:]]//g')
    if [ -z "${ROOTDEV}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get root device failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTDEV is:${ROOTDEV}"
    
    #获取根分区的文件系统
    ROOTFS=$(mount 2>/dev/null | awk '{if( $3=="/" ) print $5}')
    if [ -z "${ROOTFS}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get root filesystem failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTFS is:${ROOTFS}"
    
    #获取根分区对应的格式化工具
    if [[ "${ROOTFS}" == "ext3" ]]; then
        ROOTMKFS="mkfs.ext3"
    elif [[ "${ROOTFS}" == "ext4" ]]; then
        ROOTMKFS="mkfs.ext4"
    elif [ "${ROOTFS}" == "btrfs" -o "${ROOTFS}" == "xfs" ]; then
        ROOTFS=xfs
        ROOTMKFS="mkfs.xfs"
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] root filesystem(${ROOTFS}) is not ext3/ext4/btrfs/xfs."
        return 1    
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ROOTMKFS is:${ROOTMKFS}"
    
    
    return 0
}

########################################################################
# 函数描述： 检查操作系统目前的挂载信息，是否和/etc/fstab一致
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_mount()
{
    local line=""
    local mnt_dev=""
    local mnt_link_dev=""
    local mnt_dir=""
    local mnt_link_dir=""
    local kylin_relase="/etc/kylin-release"
    #临时保存文件
    local devlist_mnt_sys="${TMPDIR}"/install_devlist_mnt_sys
    local devlist_mnt_sys_tmp="${TMPDIR}"/install_devlist_mnt_sys.tmp
    local devlist_mnt_fstab="${TMPDIR}"/install_devlist_mnt_fstab
    local devlist_mnt_fstab_tmp="${TMPDIR}"/install_devlist_mnt_fstab.tmp

    #先清空，后面会再生成
    rm -rf "${devlist_mnt_sys}"
    rm -rf "${devlist_mnt_sys_tmp}"
    rm -rf "${devlist_mnt_fstab}"
    rm -rf "${devlist_mnt_fstab_tmp}"
    
    
    #获取当前系统中的挂载设备分区信息
    mount | grep "^/dev/" | awk '{print $1,$3}' >"${devlist_mnt_sys}"
    if [ ! -s "${devlist_mnt_sys}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get system mnt failed."
        return 1
    fi
    
    while read line
    do
        mnt_dev="$(echo ${line} | awk '{print $1}')"
        if [ -z "${mnt_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) get first column dev failed."
            return 1
        fi
        mnt_link_dev="$($CMD_READLINK_E ${mnt_dev})"
        if [ -z "${mnt_link_dev}" ]; then     
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) $CMD_READLINK_E ${mnt_dev} failed."
            return 1
        fi
        
        mnt_dir="$(echo ${line} | awk '{print $2}')"
        if [ -z "${mnt_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] system mnt(${line}) get second column dir failed."
            return 1
        fi
        
        echo "${mnt_link_dev} ${mnt_dir}" >>"${devlist_mnt_sys_tmp}"
    done <"${devlist_mnt_sys}"
    
    
    
    #获取fstab里配置的分区信息
    $CMD_CAT_FSTAB | grep -v "^[[:space:]]*#" | grep -v "[[:space:]]swap[[:space:]]" | grep -v -E "cdrom|sr[0-9]|iso9660" | grep -E "^[[:space:]]*/dev/|^[[:space:]]*UUID=|[[:space:]]ext3[[:space:]]|[[:space:]]ext4[[:space:]]" | awk '{print $1,$2}' >"${devlist_mnt_fstab}"
    if [ ! -s "${devlist_mnt_fstab}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get fstab mnt failed."
        return 1
    fi

    #对每一行挂载信息进行判断，只有都合法才能继续执行，否则就失败：原因是本来系统里就存在不一致，为了安全起见我们不会进行任何操作，否则后面更不一致，原因就更难定位了
    #要求用户必须先排除这些异常因素
    while read line
    do
        if [ -f "${kylin_relase}" ] && [ "KylinSec" = "$(cat "${kylin_relase}" |awk -F " " '{print $1}')" ]; then
            if grep -q "/data" "/etc/fstab";then
               continue
            fi
        fi
        mnt_dev="$(echo ${line} | awk '{print $1}')"
        if [ -z "${mnt_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) get first column dev failed."
            return 1
        fi
        
        if [[ ${mnt_dev} =~ ^UUID=.* ]]; then
            mnt_link_dev="${mnt_dev}"
            mnt_dev=$(echo ${mnt_dev} | xargs ${CMD_FINDFS} 2>/dev/null)
            if [ -z "${mnt_dev}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) $CMD_FINDFS ${mnt_link_dev} failed."
                return 1
            fi
        fi
        
        mnt_link_dev="$($CMD_READLINK_E ${mnt_dev})"
        if [ -z "${mnt_link_dev}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) $CMD_READLINK_E ${mnt_dev} failed."
            return 1
        fi

        mnt_dir="$(echo ${line} | awk '{print $2}')"
        if [ -z "${mnt_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) get second column dir failed."
            return 1
        fi
        
        #包括的格式可能有：/// 或者/opt/ 或者/opt/// 或者//opt/abc//ad// 等，所以要通过readlink来获取绝对路径，这样统一
        mnt_link_dir="$($CMD_READLINK_M ${mnt_dir})"
        if [ -z "${mnt_link_dir}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) $CMD_READLINK_M ${mnt_dir} failed."
            return 1
        fi
        
        echo "${mnt_link_dev} ${mnt_link_dir}" >>"${devlist_mnt_fstab_tmp}"
        
        cat "${devlist_mnt_sys_tmp}" | grep "^${mnt_link_dev} ${mnt_link_dir}$" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            #光驱挂载项要忽略，比如：/dev/cdrom #/dev/sr0 #/dev/loop0
            #或者如下信息：/dev/sr0: ISO 9660 CD-ROM filesystem data 'HUAWEIOS ' (bootable) 
            #或者如下信息：/dev/loop0: ISO 9660 CD-ROM filesystem data 'SLE-11-SP1-SDK-DVD-x86_64.02.00'
            $CMD_FILE_S "${mnt_link_dev}" | grep -i " ISO " >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) dev(${mnt_link_dev}) is iso."
                continue
            fi
            
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab mnt(${line}) dev(${mnt_link_dev}) dir(${mnt_link_dir}) can not find in system mnt."
            return 1
        fi
    done <"${devlist_mnt_fstab}"
    
    
    return 0
}


########################################################################
# 函数描述： 记录操作系统目前的swap使用信息，用于定位失败原因
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function record_swap_info()
{
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info start=="
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info in sys=="
    cat "${1}" >>"${LOGFILE}"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info in fstab=="
    cat "${2}" >>"${LOGFILE}"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ==swap info end=="
    
    return 0
}

########################################################################
# 函数描述： 检查操作系统目前的swap使用信息，是否和/etc/fstab一致
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_swap()
{
    local line=""
    local line_link=""

    #临时保存文件
    local swap_dev_sys="${TMPDIR}"/install_devlist_swp_sys
    local swap_dev_sys_tmp="${TMPDIR}"/install_devlist_swp_sys.tmp
    local swap_dev_sys_tmp_sort="${TMPDIR}"/install_devlist_swp_sys.tmp.sort
    local swap_dev_fstab="${TMPDIR}"/install_devlist_swp_fstab
    local swap_dev_fstab_tmp="${TMPDIR}"/install_devlist_swp_fstab.tmp
    local swap_dev_fstab_tmp_sort="${TMPDIR}"/install_devlist_swp_fstab.tmp.sort
    
    #清空临时配置文件，后面会重新生成，否则会对后面可能有影响
    rm -rf "${swap_dev_sys}"
    rm -rf "${swap_dev_sys_tmp}"
    rm -rf "${swap_dev_sys_tmp_sort}"
    rm -rf "${swap_dev_fstab}"
    rm -rf "${swap_dev_fstab_tmp}"
    rm -rf "${swap_dev_fstab_tmp_sort}"
    
    
    #获取系统中目前挂载的swap信息
    $CMD_SWAPON_S | grep "^/dev/" | awk '{print $1}' >${swap_dev_sys}
    while read line
    do
        line_link="$($CMD_READLINK_E ${line})"
        if [ -z "${line_link}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sys swap(${line}) $CMD_READLINK_E ${line} failed."
            return 1
        fi
        echo ${line_link} >>"${swap_dev_sys_tmp}"
    done <"${swap_dev_sys}"
    cat "${swap_dev_sys_tmp}" 2>/dev/null | sort >"${swap_dev_sys_tmp_sort}"
    
    
    #获取fstab里配置的swap信息
    $CMD_CAT_FSTAB | grep -v "^[[:space:]]*#" | grep "[[:space:]]swap[[:space:]]" | grep -E "^[[:space:]]*/dev/|^[[:space:]]*UUID=" | awk '{print $1}' >${swap_dev_fstab}
    while read line
    do
        if [[ ${line} =~ ^UUID=.* ]]; then
            line_link="${line}"
            line="$($CMD_FINDFS ${line} 2>/dev/null)"
            if [ -z "${line}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab swap(${line}) $CMD_FINDFS ${line_link} failed."
                return 1
            fi
        fi
        
        line_link="$($CMD_READLINK_E ${line})"
        if [ -z "${line_link}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] fstab swap(${line}) $CMD_READLINK_E ${line} failed."
            return 1
        fi
        echo "${line_link}" >>"${swap_dev_fstab_tmp}"
    done <"${swap_dev_fstab}"
    cat "${swap_dev_fstab_tmp}" 2>/dev/null | sort >"${swap_dev_fstab_tmp_sort}"

    
    #对比
    diff "${swap_dev_sys_tmp_sort}" "${swap_dev_fstab_tmp_sort}" >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] sys swap and fstab swap are different."
        record_swap_info "${swap_dev_sys_tmp_sort}" "${swap_dev_fstab_tmp_sort}"
        return 1
    fi

    #记录方便后续定位
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] check swap ok."
    record_swap_info "${swap_dev_sys}" "${swap_dev_fstab}" 
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的item列有效性
# 参数1说明：配置项的item列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_item()
{
    local item=$1
    
    local item_i_dir=""
    local item_i_size=0
    local item_i_fs=""
    local item_i_user=""
    local item_i_group=""
    local item_i_permission=0
    local item_i_notify=""
    local item_i_ntytimeout=0
    local item_i_raid=""
    
    #最大通知脚本的执行时间
    local max_ntftimeout=1800
    local dir_link=""
    
    #合法性判断
    if [ -z "${item}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item para is null."
        return 1
    fi
    
    if [ ! -f "${CURRPATH}/ini-plugin/conf/${item}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${CURRPATH}/ini-plugin/conf/${item}) is not exist."
        return 1
    fi
    
    
    #挂载目录
    item_i_dir="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_dir}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get dir failed."
        return 1
    fi
    
    dir_link="$($CMD_READLINK_M ${item_i_dir})"
    if [ -f "${dir_link}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) dir(${dir_link}) is regular file."
        return 1
    fi
    mount | grep " ${dir_link} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] item(${item}) dir(${dir_link}) need mount."
    fi
    
    #设备大小要求
    item_i_size="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*size=" | awk '{print $1}' | sed 's/size=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_size}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get size failed."
        return 1
    fi
    if [ "${item_i_size}" != "max" ]; then
        if [[ ${item_i_size} =~ ^[0-9]+$ ]]; then
            if [ ${item_i_size} -le 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) size(${item_i_size}) is invalid."
                return 1
            fi
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) size(${item_i_size}) is not max/digital."
            return 1
        fi
    fi
    
    
    #文件系统
    item_i_fs="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*filesystem=" | awk '{print $1}' | sed 's/filesystem=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_fs}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get filesystem failed."
        return 1
    fi
    if [ "${item_i_fs}" != "ext3" -a "${item_i_fs}" != "ext4" -a "${item_i_fs}" != "xfs" -a "${item_i_fs}" != "default" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) filesystem(${item_i_fs}) is not ext3/ext4/xfs/default."
        return 1
    fi
    
    
    #用户
    item_i_user="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*user=" | awk '{print $1}' | sed 's/user=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_user}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get user failed."
        return 1
    fi
    

    #群主
    item_i_group="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*group=" | awk '{print $1}' | sed 's/group=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_group}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get group failed."
        return 1
    fi
    getent group | grep "^${item_i_group}:" >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) group(${item_i_group}) is not exist."
        return 1
    fi
    
    
    #权限
    item_i_permission="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*permission=" | awk '{print $1}' | sed 's/permission=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_permission}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get permission failed."
        return 1
    fi
    if [ "${item_i_permission}" != "default" ]; then
        if [[ ${item_i_permission} =~ ^[0-9]+$ ]]; then
            local tmpfile="${HOME}/installdiskmgt.permission.tmp_`date +%s`"
            touch "${tmpfile}"
            while [ ! -f "${tmpfile}" ]
            do
                logme 2 "[WARN] [${FUNCNAME}:${LINENO}] ${tmpfile} is not exist still."
                touch "${tmpfile}"
                sleep 1
            done
            chmod ${item_i_permission} "${tmpfile}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) permission(${item_i_permission}) is invalid."
                rm -f "${tmpfile}"
                return 1
            fi
            rm -f "${tmpfile}"
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) permission(${item_i_permission}) is not default/digital."
            return 1
        fi
    fi


    #通知脚本的判断
    item_i_notify="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*notify=" | awk '{print $1}' | sed 's/notify=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_notify}" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] item(${item}) notify is not set."
    else
        if [ ! -f "${CURRPATH}/ini-plugin/notify/${item_i_notify}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notify(${CURRPATH}/ini-plugin/notify/${item_i_notify}) is not exist."
            return 1
        fi
    fi
    

    #通知脚本的超时时间判断
    item_i_ntytimeout="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*notifytimeout=" | awk '{print $1}' | sed 's/notifytimeout=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_ntytimeout}" ]; then
        if [ ! -z "${item_i_notify}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notify(${item_i_notify}) get timeout failed."
            return 1
        fi
    else
        if [[ ${item_i_ntytimeout} =~ ^[0-9]+$ ]]; then
            if [ ${item_i_ntytimeout} -le 0 -o ${item_i_ntytimeout} -gt ${max_ntftimeout} ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notifytimeout(${item_i_ntytimeout}) is invalid, it should between[1,1800]."
                return 1
            fi
        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) notifytimeout(${item_i_ntytimeout}) is not digital."
            return 1
        fi
    fi
    
    #RAID 级别检查
    item_i_raid="$(cat ${CURRPATH}/ini-plugin/conf/${item} | grep "^[[:space:]]*raid=" | awk '{print $1}' | sed 's/raid=//g' | head -n 1 2>/dev/null)"
    if [ -z "${item_i_raid}" ]; then
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] item(${item}) raid is not set, default is non raid."
        item_i_raid=nonraid
    else
        if [ "${item_i_raid}" != "nonraid" -a "${item_i_raid}" != "raid0" -a "${item_i_raid}" != "raid1" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] raid(${item_i_raid}) is not nonraid/raid0/raid1."
            return 1
        fi
    fi
    
    #保证该节点的所有的RAID的级别都是一致的
    if [ -z "${RAID_LEVEL}" ]; then
        RAID_LEVEL="${item_i_raid}"
    fi
    
    if [ "X${item_i_raid}" != "X${RAID_LEVEL}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] raid(${item_i_raid}) is not ${RAID_LEVEL}."
        return 1
    fi
    
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的conf列有效性
# 参数1说明：配置项的conf列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_conf()
{
    local conf=$1
    local item=""
    local item_nf=0
    local index=1
    
    
    if [ -z "${conf}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column conf para is null."
        return 1
    fi
    
    #对conf列进行检查，可能一个设备分为多个分区，要对多个conf进行分别检查
    item_nf=$(echo "${conf}" | awk -F '#' '{print NF}')
    for ((index=1; index<=${item_nf}; index++))
    do
        item=$(echo "${conf}" | awk -F '#' "{print \$${index}}")
        check_column_item "${item}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check item(${item}) invalid."
            return 1
        fi
    done
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的required列有效性
# 参数1说明：配置项的required列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_required()
{
    local required=$1
    
    if [ -z "${required}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column required para is null."
        return 1
    fi

    #只能配置为n或者y
    #为n表示此挂载项不是必须挂载的
    #为y表示此挂载项必须挂载的，否则整个挂载流程失败
    if [ "${required}" != "y" -a "${required}" != "n" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] required(${required}) is not y/n."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的care列有效性
# 参数1说明：配置项的care列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_care()
{
    local care=$1
    
    if [ -z "${care}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column care para is null."
        return 1
    fi

    #只能配置为n或者y
    #为n表示后面安装后，监控流程中不监控他
    #为y表示后面安装后，监控流程中监控他，包括盘拔出等会进行处理和通知
    if [ "${care}" != "y" -a "${care}" != "n" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] care(${care}) is not y/n."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项的condition列有效性
# 参数1说明：配置项的condition列
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_column_condition()
{
    local condition=$1
    local condition_file=""
    
    if [ -z "${condition}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column condition para is null."
        return 1
    fi

    #如果为n表示对设备没有任何要求，包括大小、盘符等
    #否则就必须指定脚本，而且这个脚本必须存在
    if [ "${condition}" != "n" ]; then
        condition_file="${CURRPATH}/ini-plugin/condition/${condition}"
        if [ ! -f "${condition_file}" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] condition(${condition_file}) is not exist."
            return 1
        fi
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查配置项文件的每一行有效性
# 参数1说明：配置项的每一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_line_inifile()
{
    local line=$1
    local line_column_num=4
    local line_column_nf=0
    
    local column_conf=""
    local column_required=""
    local column_care=""
    local column_condition=""
    
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] iniline para is null."
        return 1
    fi
    
    line_column_nf=$(echo ${line} | awk '{print NF}')
    if [ ${line_column_nf} -ne ${line_column_num} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] column num(${line_column_nf}) is not ${line_column_num}."
        return 1
    fi
    
    
    #判断每个列的合法性
    column_conf="$(echo ${line} | awk '{print $1}')"
    column_required="$(echo ${line} | awk '{print $2}')"
    column_care="$(echo ${line} | awk '{print $3}')"
    column_condition="$(echo ${line} | awk '{print $4}')"
    
    #判断conf列
    check_column_conf "${column_conf}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column conf(${column_conf}) invalid."
        return 1
    fi
    
    #判断required列
    check_column_required "${column_required}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column required(${column_required}) invalid."
        return 1
    fi
    
    #判断care列
    check_column_care "${column_care}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column care(${column_care}) invalid."
        return 1
    fi
    
    #判断condition列
    check_column_condition "${column_condition}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check column condition(${column_condition}) invalid."
        return 1
    fi
    
    
    return 0   
}


########################################################################
# 函数描述： 检查配置项文件的所有行有效性
# 参数1说明：配置项文件名
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_inifile()
{
    local line=""
    local ini_file="$1"
    local ini_base=""
    local ini_tmp_file=""
    
    
    #合法性判断
    if [ -z "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile para is null."
        show_help
        return 1
    fi
    echo ${ini_file} | grep '\.ini$' >/dev/null
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile(${ini_file}) end without .ini"
        return 1
    fi
    if [ ! -f "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] inifile(${ini_file}) is not exist."
        return 1
    fi
    
    ini_base="$(basename ${ini_file})"
    if [ -z "${ini_base}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] basename ${ini_file} failed."
        show_help
        return 1
    fi
    
    #判断配置文件有效性
    ini_tmp_file="${TMPDIR}/${ini_base}.instmp"
    cat "${ini_file}" | grep -v "^[[:space:]]*#" | grep -v "^[[:space:]]*$" >"${ini_tmp_file}"
    while read line
    do
        check_line_inifile "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check inifile line(${line}) invalid."
            return 1
        fi
    done <"${ini_tmp_file}"
    
    
    #如果挂载配置文件为空，也是合法的
    #把有条件要求的放在前面首先满足
    ININEWFILE="${TMPDIR}/${ini_base}.ins"
    awk '$2 == "y" && $4 != "n" {print $0}' "${ini_tmp_file}" >"${ININEWFILE}"
    awk '$2 == "y" && $4 == "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    awk '$2 == "n" && $4 != "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    awk '$2 == "n" && $4 == "n" {print $0}' "${ini_tmp_file}" >>"${ININEWFILE}"
    
    
    return 0
}

function setup_scc_tool() {
    #SCC 加解密工具拷贝
    if [ ! -d "${SCC_INIT_INSTALL_PATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] scc-tool dir[${SCC_INIT_INSTALL_PATH}] is not exist."
        return 1
    fi
    sed -i "s|${SCC_INIT_INSTALL_PATH}|${SCC_INSTALL_PATH}|g" "${SCC_INIT_INSTALL_PATH}/config/logger.conf"
    sed -i "s|${SCC_INIT_INSTALL_PATH}|${SCC_INSTALL_PATH}|g" "${SCC_INIT_INSTALL_PATH}/config/scc.conf"

    ${CMD_RM_F} ${SCC_INSTALL_PATH}
    ${CMD_MKDIR_P} ${SCC_INSTALL_PATH}
    local result=0
    ${CMD_CP} "${SCC_INIT_INSTALL_PATH}" "${SERVICEPATH}/"
    result=$?

    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${SCC_INIT_INSTALL_PATH}/* ${SCC_INSTALL_PATH} -arf failed."
    fi

    ${CMD_RM_F} ${SCC_INIT_INSTALL_PATH}
    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to delete scc_init_install_path[${SCC_INIT_INSTALL_PATH}]."
    fi
}

########################################################################
# 函数描述： 安装硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function setup_service()
{
    local pid_d=0
    
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        IS_EMPTY=1
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] diskmgt new."
    fi
    
    #既然确定要安装了，那么安装之前首先要清空之前可能存在的安装文件
    rollback
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] roll back failed."
        return 1
    fi
    
    #如果后台有硬盘管理服务在运行，那么失败
    pid_d=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
    if [ ! -z "${pid_d}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] diskmgtd is running already."
        return 1
    fi
    
    #前面无法清空，那么失败
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -eq 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] diskmgt has been installed already."
        return 1
    fi
    
    #开始安装了，首先创建目录
    ${CMD_INSTALL} 750 "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install -m 750 -d ${SERVICEPATH} failed."
        return 1
    fi
    
    
    #挂载配置文件设定
    ${CMD_INSTALL} 750 "${SERVICEPATH}/ini"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install -m 750 -d ${SERVICEPATH}/ini failed."
        return 1
    fi
    if [ ! -d "${CURRPATH}/ini" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/ini is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/ini/${INIFILE}" "${SERVICEPATH}/ini"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp {CURRPATH}/ini/${INIFILE} ${SERVICEPATH}/ini/ -arf failed."
        return 1
    fi

    #拷贝wce配置文件
    ${CMD_CP} "${CURRPATH}/ini/wce" "${SERVICEPATH}/ini/wce"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp {CURRPATH}/ini/wce ${SERVICEPATH}/ini/ -arf failed."
        return 1
    fi
    
    
    #插件拷贝
    if [ ! -d "${CURRPATH}/ini-plugin" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/ini-plugin is not exist."
        return 1
    fi

    ${CMD_CP} "${CURRPATH}/ini-plugin" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/ini-plugin ${SERVICEPATH} -arf failed."
        return 1
    fi

    
    #内部脚本拷贝
    if [ ! -d "${CURRPATH}/inner" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/inner is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/inner" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/inner ${SERVICEPATH} -arf failed."
        return 1
    fi
    
    
    #外部脚本拷贝
    if [ ! -d "${CURRPATH}/script" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/script is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/script" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/script ${SERVICEPATH} -arf failed."
        return 1
    fi
    
    #RAID插件脚本拷贝
    if [ ! -d "${CURRPATH}/raidadapter" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] dir ${CURRPATH}/raidadapter is not exist."
        return 1
    fi
    ${CMD_CP} "${CURRPATH}/raidadapter" "${SERVICEPATH}/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${CURRPATH}/raidadapter ${SERVICEPATH} -arf failed."
        return 1
    fi

    #设置用户、群主、权限
    ${CMD_CHOWN_R} 0:0 "${SERVICEPATH}" 
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown 0:0 ${SERVICEPATH} -R failed."
        return 1
    fi
    
    ${CMD_CHMOD_R} 750 "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 755 ${SERVICEPATH} -R failed."
        return 1
    fi

    setup_scc_tool
    
    #安全起见，所有的配置文件和脚本都转为unix格式
    ${CMD_DOS2UNIX} "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] find ${SERVICEPATH} -type f | xargs dos2unix >/dev/null 2>&1 failed."
        return 1
    fi

    #把diskmgt做成系统服务，加入关机流程
    ${CMD_CONFIG_SERVICE}
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] config diskmgt service failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 停止硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function kill_diskmgtd()
{
    local run_pids=0
    local count=0
    local count_max=60

    ${CMD_IS_FILE} "/etc/cron.d/diskmgtd.cron"
    if [ $? -eq 0 ]; then
        rm -rf "/etc/cron.d/diskmgtd.cron"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf /etc/cron.d/diskmgtd.cron failed."
            return 1
        fi
    fi
    
    #循环停止服务，如果超过30秒则停止失败
    while [ ${count} -le ${count_max} ]
    do
        run_pids=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
        if [ -z "${run_pids}" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] kill deamon ok."
            return 0
        fi
        
        logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill deamon pid(${run_pids})."
        echo ${run_pids} | xargs kill >>"${LOGFILE}" 2>&1
        
        ((count++))
        if [ ${count} -eq ${count_max} ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] kill force deamon pid(${run_pids})."
            echo ${run_pids} | xargs kill -9 >>"${LOGFILE}" 2>&1
        fi
        
        sleep 2
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] kill deamon failed after ${count_max} times."
    return 1
}


########################################################################
# 函数描述： 回滚安装流程
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback()
{
    #如果没有主目录，杀死进程也是合理的
    if [ ${IS_EMPTY} -eq 1 -o ${IS_FORCE} -eq 1 ]; then
        kill_diskmgtd
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] kill diskmgtd failed."
            return 1
        fi
        
        #回退首先把服务的主目录删除干净
        ${CMD_IS_DIR} "${SERVICEPATH}"
        if [ $? -eq 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] uninstall old."
            rm -rf "${SERVICEPATH}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICEPATH} failed."
                return 1
            fi
        fi
    fi
    
    return 0
}

########################################################################
# 函数描述： 修改秘钥文件权限为400
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function modify_keys_authority()
{
    if [ -d "$KEY_FILE_PATH" ]; then
        ${CMD_CHOWN_R} root:root "$KEY_FILE_PATH"
        ${CMD_CHMOD} 700 "$KEY_FILE_PATH"
    fi
    
    [ -f "$KEY_FILE_1" ] && ${CMD_CHMOD} 400 "$KEY_FILE_1"
    [ -f "$KEY_FILE_2" ] && ${CMD_CHMOD} 400 "$KEY_FILE_2"
}

########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    #检查操作系统有效性
    check_system
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check system failed."
        return 1
    fi
    
    #检查挂载配置文件有效性
    check_inifile "${CURRPATH}/ini/${INIFILE}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check inifile failed."
        return 1
    fi
    
    #安装硬盘管理服务
    setup_service
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] setup service failed."
        return 1
    fi
    
    
    #修改秘钥文件权限
    modify_keys_authority
    
    return 0
}


########################################################################
# 函数描述： 清除锁函数
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function clean_lock()
{
    rm -rf "${LOCKFILE}"
}


########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    rollback
    clean_lock
    exit 2
}

#捕获信号
trap "sig_catch" INT TERM QUIT


########################################################################
# 函数描述： 以下为最外层函数，在锁保护的范围内
# 参数1说明：脚本的所有输入参数
#
# 返回值  ： 0 成功
#            1 普通失败错误码
#            2 返回信号处理特殊错误码
#            3 锁保护获取失败错误码
#            4 上次被强杀进程导致的锁遗留错误
#            5 后台正在运行硬盘管理服务错误码
########################################################################
log_begin "$@"

{
    flock -n 3
    #如果获取锁失败，要判断之前是否是被强杀进程导致锁遗留
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] `basename $0` locked by another user."
        
        #这里判断到之前是否是被强杀进程导致锁遗留，就把遗留锁清理干净
        TMPFILE="${HOME}/insdmgttmpfile_`date +%s`.txt"
        ps -wweo pid,ppid,uname,tname,args | grep -v grep | grep "installdmgt.sh" | grep "/bin/" >"${TMPFILE}"
        CMDCNT=$(cat "${TMPFILE}" | wc -l)
        if [ ${CMDCNT} -eq 1 ]; then
            logme 2 "[ERROR] [none:${LINENO}] `basename $0` kill force last time, clean lock now."
            clean_lock
            exit 4
        fi
        
        exit 3
    fi
    
    #初始化路径全局变量
    init_path
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] init path failed."
        exit 1
    fi
    
    #配置diskmgt服务命令
    CMD_CONFIG_SERVICE="${CURRPATH}/script/config_diskmgt_service.sh"

    
    #脚本输入所有参数的处理流程
    while getopts "n:fh" options;
    do
        case ${options} in
            n)
                INIFILE="${OPTARG}"  #输入挂载配置文件
                ;;
                
            f)
                IS_FORCE=1  #是否强制安装硬盘管理服务
                ;;
                
            h)
                show_help   #显示帮助
                exit $?
                ;;
                
            *)
                logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
                show_help
                exit 1
                ;;
        esac
    done
    
    #执行主函数
    main
    RESULT=$?
    if [ ${RESULT} -ne 0 ]; then
        #只要失败就需要进行回退操作
        rollback
        logme 2 "[ERROR] [none:${LINENO}] execute `basename $0` failed."
    else
        logme 2 "[INFO] [none:${LINENO}] execute `basename $0` successfully."
    fi
    
    #清理锁并返回特定返回码
    clean_lock
    exit "${RESULT}"
} 3<>"${LOCKFILE}"

