#!/bin/bash

#新的秘钥文件
OLD_USER_NAME=""
NEW_USER_NAME="$1"

#磁盘管理服务安装路径
LUKSMGTPATH="/usr/local/kmsLuksMgt"

#连接kms服务配置文件
KMS_CONF_FILE="${LUKSMGTPATH}/ini/kms_info"
KMS_CONF_FILE_BAK="${LUKSMGTPATH}/ini/kms_info.bak"

#kmc工具路径
kmc_tool_dir="${LUKSMGTPATH}/kmc"

#日志文件
LOGPATH="/var/log/kmsLuksMgt/"
LOGFILE="/var/log/kmsLuksMgt/modifyUserInfo.log"


###################################################
# 显示帮助
###################################################
function show_help()
{
    echo "Usage: $0 1 new version name"
}



########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    mkdir -p "${LOGPATH}"
    if [ $? -ne 0 ]; then
        echo "[$(date)] [EMER] mkdir -p ${LOGPATH} failed."
        echo "[$(date)] ${log_info}"
        return 1
    fi
    
    chmod 700 "${LOGPATH}"
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

###################################################
# 初始化
###################################################
function init()
{
    
    kms_server=$(cat ${KMS_CONF_FILE} | grep "^[[:space:]]*kms_server=" | awk '{print $1}' | sed 's/kms_server=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_server" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms server from $KMS_CONF_FILE."
        return 1
    fi
    
    OLD_USER_NAME=$(cat ${KMS_CONF_FILE} | grep "^[[:space:]]*kms_user_info=" | awk '{print $1}' | sed 's/kms_user_info=//g' | head -n 1 2>/dev/null)
    if [ -z "$OLD_USER_NAME" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $kms_user_info."
        return 1
    fi
    
    kms_key_version_name=$(cat ${KMS_CONF_FILE} | grep "^[[:space:]]*kms_key_version_name=" | awk '{print $1}' | sed 's/kms_key_version_name=//g' | head -n 1 2>/dev/null)
    if [ -z "$kms_key_version_name" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $KMS_CONF_FILE."
        return 1
    fi
    
    kms_key_name=$(echo ${kms_key_version_name} | awk -F '@' '{print $1}')
    if [ -z "$kms_key_name" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get kms key version name from $kms_key_name."
        return 1
    fi
}

###################################################
# 备份旧信息配置
###################################################
function backup()
{
    if [ -f ${KMS_CONF_FILE_BAK} ]; then
        rm -rf ${KMS_CONF_FILE_BAK}
    fi
    
    cp -pf ${KMS_CONF_FILE} ${KMS_CONF_FILE_BAK}
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${KMS_CONF_FILE}."
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Backup ${KMS_CONF_FILE} to ${KMS_CONF_FILE_BAK}."
    return 0
}

###################################################
#
# 设置用户输入的集群服务器密码
#
###################################################
function set_kms_passwd()
{
    local empty=""   #密码是否为空
    local i=0        #循环控制变量
    local max=3      #最大的循环次数
    local show="Please enter kms password for $NEW_USER_NAME: "

    logme 2 "[DEBUG] Enter [${FUNCNAME}:${LINENO}]"

    #如果输入了无效字符,可以使用Ctrl+Backspace键删除这些字符
    #说明:执行 stty erase '^H' 后,可以使用Backspace键删除字符
    #     但这改变了默认的行为,有可能会对其他程序产生影响
    #echo "Tips:Press the Ctrl+Backspace key to delete wrong characters."
    while true
    do
        
        #超过最大次数后,退出
        if [ ${i} -ge ${max} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Entered password more than 3 entres."
            return 1
        fi
        ((i++))
        
        #输入超时时间设置为60秒
        #说明:使用系统内置REPLY来保存输入信息,
        #     可以完全保留输入信息中的空格
        #如果使用自定义变量,输入信息的最前端和最后端的空格将会被丢弃
        #每次read都会重新初始化REPLY内置变量
        read -t 60 -r -s -p "$show"
        echo ""
        if [ -n "$REPLY" ]; then
            new_user_info=$(echo -n "$NEW_USER_NAME:$REPLY" | base64)
            curl -s  -i -H "Authorization: Basic ${new_user_info}" -k -X GET "https://${kms_server}/kms/v1/keys/names" | grep -w ${kms_key_name} >/dev/null 2>&1
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Entered password is invalid"
                continue
            fi
            cd $kmc_tool_dir > /dev/null 2>&1
            ./kmc_tool -e -i "$NEW_USER_NAME:$REPLY"
            if [ $? -ne 0 ]; then
                cd - > /dev/null 2>&1
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] password encrypt failed"
                return 1
            fi
            cd - > /dev/null 2>&1
            break
        else
            
            logme 2 "[DEBUG] Again[${i}/$((${max}-1))] [${FUNCNAME}:${LINENO}]"
            continue
        fi
        show="Please enter kms password for $NEW_USER_NAME again[${i}/$((${max}-1))]: "
    done
    
    logme 2 "[DEBUG] Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
# 更新kms_info文件中的用户信息
###################################################
function update_kms_info()
{
    KMS_CONF_FILE_TMP="${KMS_CONF_FILE}.tmp"
    
    cp -pf "$KMS_CONF_FILE" "$KMS_CONF_FILE_TMP"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup ${KMS_CONF_FILE}."
        return 1
    fi
    
    #如果替换的秘钥为当前秘钥，将备用秘钥更换为当前秘钥
    sed -i "s#${OLD_USER_NAME}#${NEW_USER_NAME}#g" "$KMS_CONF_FILE_TMP"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace ${OLD_USER_NAME} with ${NEW_USER_NAME} for ${KMS_CONF_FILE_TMP}."
        return 1
    fi
    
    mv -f "$KMS_CONF_FILE_TMP" "$KMS_CONF_FILE"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to mv -f $crypttab_tmp $KMS_CONF_FILE_TMP."
        return 1
    fi
    
    return 0
}


###################################################
# 校验旧密钥
###################################################
function chk_old_info()
{
    local empty=""   #密码是否为空
    local i=0        #循环控制变量
    local max=3      #最大的循环次数
    local show="Please enter old kms password for $OLD_USER_NAME: "

    logme 2 "[DEBUG] Enter [${FUNCNAME}:${LINENO}]"

    #如果输入了无效字符,可以使用Ctrl+Backspace键删除这些字符
    #说明:执行 stty erase '^H' 后,可以使用Backspace键删除字符
    #     但这改变了默认的行为,有可能会对其他程序产生影响
    #echo "Tips:Press the Ctrl+Backspace key to delete wrong characters."
    while true
    do
        #超过最大次数后,退出
        if [ ${i} -ge ${max} ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Entered password more than 3 entres."
            return 1
        fi
        ((i++))
        #输入超时时间设置为60秒
        #说明:使用系统内置REPLY来保存输入信息,
        #     可以完全保留输入信息中的空格
        #如果使用自定义变量,输入信息的最前端和最后端的空格将会被丢弃
        #每次read都会重新初始化REPLY内置变量
        read -t 60 -r -s -p "$show"
        echo ""
        if [ -n "$REPLY" ]; then
            old_user_info=$(echo -n "$OLD_USER_NAME:$REPLY" | base64)
            curl -s  -i -H "Authorization: Basic ${old_user_info}" -k -X GET "https://${kms_server}/kms/v1/keys/names" | grep -w ${kms_key_name} >/dev/null 2>&1
            if [ $? -eq 0 ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] validata old key successed"
                break
            else
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Entered password is invalid"
                return 1
            fi
        else
                
                logme 2 "[DEBUG] Again[${i}/$((${max}-1))] [${FUNCNAME}:${LINENO}]"
                continue
        fi
        show="Please enter kms password for $OLD_USER_NAME again[${i}/$((${max}-1))]: "
    done

    logme 2 "[DEBUG] Leave [${FUNCNAME}:${LINENO}]"

    return 0
    
}

###################################################
# 主函数
###################################################
function main()
{
    #初始化配置信息
    init
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to init."
        return 1
    fi
    
    #备份配置文件
    backup
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to backup."
        return 1
    fi
    
    #校验旧密码
    chk_old_info
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] the password is invalid."
        return 1
    fi
    
    #设置新密码
    set_kms_passwd
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] set new password error."
        return 1
    fi
    
    #将新更换的秘钥设置成当前秘钥
    update_kms_info
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to update kms_info for $KMS_CONF_FILE."
        return 1
    fi
    
    rm -rf ${KMS_CONF_FILE_BAK}
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to replace $OLD_USER_NAME with $NEW_USER_NAME." 
    return 0
}

main "$@"
exit $?
