#!/bin/bash

#日志目录和文件
LOGPATH="/var/log/kmsLuksMgt/"
LOGFILE="/var/log/kmsLuksMgt/uninstall.log"

#文件系统挂载记录文件
PROCMOUNTFILE="/proc/mounts"

IS_FORCE=0
IS_UMOUNT=0

CMD_UNINSTALL="m_uninstallluksmgt"
CMD_CRYPTSETUP="cryptsetup"
CMD_UMOUNT="umount"
CMD_CAT="cat"


########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] [EMER] luksmgtmgtmodule mkdir -p ${LOGPATH} failed." >>/var/log/messages
            echo "[$(date)] ${log_info}" >>/var/log/messages
            return 1
        fi
    fi
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "                      Notes: Uninstall kmsLuksMgt."
    echo "-u                    Notes: Uninstall kmsLuksMgt, umount the partions monitored by kmsLuksMgt, and delete these items from mount."
    echo "-f                    Notes: Uninstall kmsLuksMgt, umount the partions monitored by kmsLuksMgt(compulsively), and delete these items from mount."
    echo "-h                    Notes: Show help."
    echo
    echo "Example:"
    echo "`basename $0` -f"
    echo
    return 0
}

########################################################################
# 函数描述： 对kmsluks加密分区目录进行umount，并将记录从关闭加密映射
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function umount_encrypt_dir()
{
    local ini_file="/usr/local/kmsLuksMgt/ini/kms_encrypt_devs"
    if [ ! -f "${ini_file}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get inifile in ${SERVICEPATH}/ini failed."
        return 1
    fi

    #对每一个挂载配置项进行监控
    while read line
    do
        echo ${line} | grep -e "^UUID="
        if [ $? -ne 0 ]; then
            continue
        fi
        
        if [ -z "${line}" ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] line para is null."
            continue
        fi

        clean_line "${line}"
        if [ $? -ne 0 ];then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] clean line ${line} failed."
            return 1
        fi
    done <"${ini_file}"
}

########################################################################
# 函数描述： 卸载需要回退的挂载目录
# 参数1说明：需要回退的挂载目录信息
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function umount_dir()
{
    local line="$1"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} now."
    
    #第1次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed once?"
        return 0
    fi
    
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    else
        if [ ${IS_FORCE} -eq 0 ];then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount ${line} failed."
            return 1
        fi
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed once."
    
    
    #第2次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1 
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed twice?"
        return 0
    fi
    
    fuser -mk "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] fuser -mk ${line} failed."
    fi
    
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed twice."
    
    
    
    #第3次卸载
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed third times?"
        return 0
    fi
    
    kill -9 $(lsof | grep -w " ${line}" | awk '{print $2}') >/dev/null 2>&1
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed after kill?"
        return 0
    fi
    
    fuser -mk "${line}"
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] fuser -mk ${line} failed."
    fi
    eval ${CMD_UMOUNT} "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} failed third times, need to umount force."
    
    
    eval ${CMD_UMOUNT} -l "${line}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] umount -l ${line} ok."
        return 0
    fi
    logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount -l ${line} failed."
    
    
    #最后再确认下是否卸载成功
    eval ${CMD_CAT} ${PROCMOUNTFILE} | grep " ${line} " >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] umount ${line} needed fourth times?"
        return 0
    fi
    
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount ${line} failed finally."
    return 1
}

########################################################################
# 函数描述： 定期监控挂载配置文件的目录
# 参数1说明：指定的配置文件的每一行
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function clean_line()
{
    local line="$1"
    
    #参数判断
    if [ -z "${line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] clean line para is null."
        return 1
    fi
    #获取映射
    dev_mapper=$(echo ${line} | awk '{print $2}')
    
    #获取挂载目录
    dev_parted=$(echo ${line} | awk '{print $3}')
    
    #先取消挂载
    umount_dir "${dev_parted}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount dir(${dev_parted}) failed! You may need to do it manually."
        return 1
    fi
    
    #关闭映射
    ${CMD_CRYPTSETUP} luksClose "${dev_mapper}" >/dev/null 2>&1
    
    return 0
}

function m_uninstallluksmgt()
{
    local g_localFile=''
    #Red Hat版本
    if [ -f "/etc/redhat-release" ]; then  
        #版本号打印到运行日志,方便定位
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] $(cat /etc/redhat-release)"
        
        g_localFile="/etc/rc.local"         #redhat的local目录
    fi    
       
    #CentOS版本
    if [ -f "/etc/centos-release" ]; then  
        #版本号打印到运行日志,方便定位
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] $(cat /etc/centos-release)"
        
        g_localFile="/etc/rc.local"         #CentOS的local目录
    fi
    
    if [ -z "${g_localFile}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Uninstall kmsLuksMgt failed, The os version does not match."
        return 0
    fi

    g_localFile=$(readlink -e "$g_localFile")
    
    #删除LOCAL文件登记的REBOOT时执行的OS调优脚本  
    if [ -f "${g_localFile}" ]; then 
        sed -i '/kmsLuksMgt\/script/d' ${g_localFile}
        if [ $? -ne 0 ]; then 
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Modify the ${g_localFile} configure fails."
        fi    
    else
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] The ${g_localFile} does not exist."
    fi
    
    #删除拷贝目录
    if [ -n "/usr/local/kmsLuksMgt" ]; then
        rm -rf "/usr/local/kmsLuksMgt"        
        if [ $? -ne 0 ]; then 
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Delete the directory[/usr/local/kmsLuksMgt/] fails."
        fi    
    fi 
    
    return 0
}

######################################################################
#   DESCRIPTION: os性能调优主执行函数 
######################################################################
function main()
{         
    #设置当前的umask
    UMASK=0077
    umask "$UMASK"
    
    #是否需要umount掉kmsluks加密分区所监控的目录   
    if [ ${IS_FORCE} -eq 1 -o ${IS_UMOUNT} -eq 1 ]; then
        umount_encrypt_dir
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] umount monitored dir failed."
            return 1
        fi
    fi
    

    ${CMD_UNINSTALL} >> ${LOGFILE} 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] uninstall kmsLuksMgt fails."
        return 1
    fi

    #卸载成功
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] uninstall kmsLuksMgt successfully."
    
    return 0
}

#脚本输入所有参数的处理流程
while getopts "ufh" options;
do
    case ${options} in
        u)
            IS_UMOUNT=1  #是否umount磁盘管理服务所监控的分区
            ;;

        f)
            IS_FORCE=1  #是否强制umount磁盘管理服务所监控的分区
            ;;
            
        h)
            show_help   #显示帮助
            exit $?
            ;;
            
        *)
            logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
            show_help
            exit 1
            ;;
    esac
done

main "$@"
exit $?