#!/bin/bash


###############################################################################
#author:   none
#date:     2014/02/12
#function: 安装硬盘管理服务
###############################################################################

#此环境变量为设置系统字体等信息，防止用户通过不同的终端登录显示信息为乱码
export LC_ALL=C

#主机名或IP地址
IPADDR="$(echo ${SSH_CONNECTION} | awk '{print $3}')"

#设置当前的umask
UMASK=0077
umask "$UMASK"

#设置当前路径
CURRPATH=""

#设置临时路径，所有的临时文件都放在此目录下
TMPDIR=""


#是否强制安装
#0不强制安装（如果之前安装过，那么这次安装就失败），1强制安装
IS_FORCE=0


#系统中存在此脚本执行的总进程数
#主要用于此脚本如果被强杀那么就遗留锁文件，等待下一次进来此脚本被锁定判断其实是之前被强杀，就返回特定错误码告知此事
#并删除遗留锁
CMDCNT=0
TMPFILE=""

#服务目录和文件
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"

SERVICE_TMP_PATH="${HOME}/diskmgt_tmp"
SERVICE_BACKUP_PATH="/usr/local/diskmgt_bak"
SERVICE_INI_PATH="${SERVICEPATH}/ini/"
SERVICE_INI_PLUGIN_PATH="${SERVICEPATH}/ini-plugin"
SERVICE_SCRIPT_PATH="${SERVICEPATH}/script/"
SERVICE_SCRIPT="diskmgt.sh"

SCC_INSTALL_PATH="${SERVICEPATH}/scc"
SCC_INIT_INSTALL_PATH="${HOME}/scc"
SCC_INIT_CONF_DIR="${SCC_INIT_INSTALL_PATH}/config"

SERVICE_KEYS_PATH="$SERVICE_INI_PLUGIN_PATH/keys"

KEY_FILE_1="$SERVICE_KEYS_PATH/fi_disk_partition_key_1"
KEY_FILE_2="$SERVICE_KEYS_PATH/fi_disk_partition_key_2"
CRYPT_TAB_INFO="/etc/crypttab"
CRYPT_TAB_INFO_UPGRADE_TMP="/etc/crypttab.diskmgt.upgrade.tmp"
CRYPT_TAB_INFO_UPGRADE_BAK="/etc/crypttab.diskmgt.upgrade.bak"
ENCRYPT_PARTITION_INFO="$SERVICE_KEYS_PATH/encrypt_partition_info.lst"
ENCRYPT_PARTITION_INFO_UPGRADE_BAK="$SERVICE_KEYS_PATH/encrypt_partition_info.lst.diskmgt.upgrade.bak"

PLAIN_PWD_FILE_SUFFIX="plain"

#suse12.X和redhat7.X中diskmgt服务的配置文件
DISKMGT_UNIT_FILE="/usr/lib/systemd/system/diskmgt.service"

ETC_FSTAB="/etc/fstab"
ETC_FSTAB_BAK="/etc/fstab.bak.diskmgt.upgrade"

#只读告警保存文件
ALARMROTXT="/usr/local/diskmgt/inner/alarmro.txt"

#只读告警保存文件的备份
ALARMREPEATTXT="/usr/local/diskmgt/inner/alarmrepeat.txt"

#分区丢失告警保存文件
ALARMLOSTTXT="/usr/local/diskmgt/inner/alarmlost.txt"

#控制debug日志打印的开关
DEBUGSWITCH="/usr/local/diskmgt/inner/debugswc"

#当前操作的标志文件
COMMIT=0
START=0
ROLL_BACK=0

#锁文件
LOCKFILE="${HOME}/upgradedmgt.lock"

#日志目录
LOGPATH="${HOME}/diskmgt/"

#日志文件
LOGFILE="${LOGPATH}/upgradedmgt.log"
RESULT=0

#返回值
g_GENERAL_ERROR=1
g_UPGRADE_GENERAL_ERROR=11
g_UPGRADE_STOP_DSKMGT_ERROR=12
g_UPGRADE_START_DSKMGT_ERROR=13
g_COMMIT_GENERAL_ERROR=21
g_COMMIT_NOT_UPGRADED_ERROR=22
g_ROLL_GENERAL_ERROR=31
g_ROLL_STOP_DSKMGT_ERROR=32
g_ROLL_START_DSKMGT_ERROR=33

g_UPGRADE_UPGRADED_WARN=101
g_UPGRADE_NOT_INSTALLED_WARN=102
g_UPGRADE_IS_LATEST_VERSION_WARN=107
g_COMMIT_COMMITTED_WARN=103
g_COMMIT_NOT_INSTALLED_WARN=104
g_ROLL_ROLLED_WARN=105
g_ROLL_NOT_INSTALLED_WARN=106

###############################################################################


#############################################################
#下面为非root用户切换root用户需要执行的命令
if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    #非root用户，使用sudo    
    CMD_WHICH="sudo ${INSTALL_SUDO_PATH} preinstall_which"
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
    CMD_CHOWN="sudo ${INSTALL_SUDO_PATH} preinstall_chown"
    CMD_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
    CMD_CHMOD="sudo ${INSTALL_SUDO_PATH} preinstall_chmod"
    CMD_CHMOD_R="sudo ${INSTALL_SUDO_PATH} preinstall_chmod_r"
    CMD_CP="sudo ${INSTALL_SUDO_PATH} preinstall_cp"
    CMD_DOS2UNIX="sudo ${INSTALL_SUDO_PATH} preinstall_dos2unix"
    CMD_IS_DIR="sudo ${INSTALL_SUDO_PATH} preinstall_is_dir"
    CMD_IS_FILE="sudo ${INSTALL_SUDO_PATH} preinstall_is_file"
    CMD_RM_RF="sudo ${INSTALL_SUDO_PATH} preinstall_rm_rf"
    CMD_MV="sudo ${INSTALL_SUDO_PATH} preinstall_move"
    CMD_CAT="sudo ${INSTALL_SUDO_PATH} preinstall_cat"
    CMD_STOP_DISKMGT="sudo ${INSTALL_SUDO_PATH} preinstall_stop_diskmgt"
    CMD_START_DISKMGT="sudo ${INSTALL_SUDO_PATH} preinstall_start_diskmgt"
    CMD_SYSTEMCTL="sudo ${INSTALL_SUDO_PATH} system_ctl"
    CMD_UPDATE_RPM_PACKAGE="sudo ${INSTALL_SUDO_PATH} update_rpm_package"
    CMD_CRYPTSETUP="sudo ${INSTALL_SUDO_PATH} preinstall_cryptsetup"
    CMD_BLKID="sudo ${INSTALL_SUDO_PATH} preinstall_blkid"
else
    CMD_WHICH="which"
    CMD_MKDIR_P="mkdir -p"
    CMD_CHOWN="chown"
    CMD_CHOWN_R="chown -R"
    CMD_CHMOD="chmod"
    CMD_CHMOD_R="chmod -R"
    CMD_CP="cp -arf"
    CMD_DOS2UNIX="preinstall_dos2unix"
    CMD_IS_DIR="preinstall_is_dir"
    CMD_IS_FILE="preinstall_is_file"
    CMD_RM_RF="rm -rf"
    CMD_MV="mv"
    CMD_CAT="cat"
    CMD_STOP_DISKMGT="preinstall_stop_diskmgt"
    CMD_START_DISKMGT="preinstall_start_diskmgt"
    CMD_SYSTEMCTL="systemctl"
    CMD_UPDATE_RPM_PACKAGE="update_rpm_package"
    CMD_CRYPTSETUP="cryptsetup"
    CMD_BLKID="blkid"
fi
################################################################


########################################################################
#                           sudo封装开始
########################################################################
########################################################################
# 函数描述： 判断路径是否是一个目录
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_dir ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -d "$1" ]; then
        return 0
    fi

    return 1
}

########################################################################
# 函数描述： 判断路径是否是一个文件
# 参数1说明：路径
#
# 返回值  ：0 成功
#           1 失败
########################################################################
function preinstall_is_file ()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ -f "$1" ]; then
        return 0
    fi

    return 1
}


########################################################################
# 函数描述： 将指定路径的文件全部执行dos2unix命令
# 参数1说明：待转换的路径
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_dos2unix()
{
    if [ -z "$1" ]; then
        echo "arguments is null."
        return 1
    fi
    
    if [ ! -e "$1" ]; then
        echo "dev $1 does not exist."
        return 1
    fi
    
    find "$1" -type f | xargs dos2unix >/dev/null 2>&1
    return $?
}


########################################################################
# 函数描述： 停止硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_stop_diskmgt()
{
    local count=0
    local count_max=60
    
    ${CMD_IS_FILE} "${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not find file ${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}, failed."
        return 1
    fi
        
    #循环停止服务，如果超过30秒则停止失败
    while [ ${count} -le ${count_max} ]
    do
        bash "${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}" -k >>"${LOGFILE}" 2>&1
        if [ $? -ne 0 ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] ${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT} -k failed."
            ((count++))
        else
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] stop diskmgt successful."
            return 0
        fi
        
        sleep 2
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] stop diskmgt failed after ${count_max} times."
    return 1
}


########################################################################
# 函数描述： 启动硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function preinstall_start_diskmgt()
{
    local count=0
    local count_max=10
    local ret=0
    
    ${CMD_IS_FILE} "${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not find file ${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}, failed."
        return 1
    fi
        
    #启动diskmgt
    while [ ${count} -le ${count_max} ]
    do
        if [ -f "${DISKMGT_UNIT_FILE}" ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] start-diskmgt: ${CMD_SYSTEMCTL} restart diskmgt"
            ${CMD_SYSTEMCTL} restart diskmgt >>"${LOGFILE}" 2>&1
            ret=$?
        else
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] start-diskmgt: bash ${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT} -r"
            bash "${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}" -r >>"${LOGFILE}" 2>&1
            ret=$?
        fi
        
        if [ $ret -ne 0 ]; then
            logme 1 "[WARN] [${FUNCNAME}:${LINENO}] start diskmgt failed."
            ((count++))
        else
            #启动成功
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start diskmgt successful."
            return 0
        fi
        
        sleep 2
    done
    
    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] start diskmgt failed after ${count_max} times."
    return 1
}

########################################################################
#                           sudo封装结束
########################################################################

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode=$1
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    #root安装，所以没有路径要实时创建
    if [ ! -d "${LOGPATH}" ]; then
        mkdir -p "${LOGPATH}"
        if [ $? -ne 0 ]; then
            echo "[$(date)] [EMER] diskmgtmodule ins: mkdir -p ${LOGPATH} failed."
            echo "[$(date)] ${log_info}"
            return 1
        fi
    fi
    
    chmod 700 "${LOGPATH}"
    
    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)][${IPADDR}] ${log_info}" | tee -a "${LOGFILE}"  #通过tee追加显示
    else
        echo "logme invalid mode para."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 记录脚本执行的上下文，主要用于定位用户的执行历史明细
# 参数1说明：此脚本的所有输入参数
#
# 返回值  ：0 成功
########################################################################
function log_begin()
{
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 ""
    logme 1 "************************INSTALL-START************************"
    logme 1 "==$0 $*=="
    logme 1 "==who=="
    logme 1 "$(who)"
    get_call_stack
    logme 1 "************************INSTALL-END**************************"
    
    return 0
}

########################################################################
# 函数描述： 显示帮助信息
# 参数1说明：无
#
# 返回值  ： 0 成功
########################################################################
function show_help()
{
    echo
    echo "Usage: `basename $0` [option]"
    echo "options:"
    echo "-n XXX.ini [-f]       Notes: install diskmgt by ini file(force)."
    echo "-h                    Notes: show help."
    echo
    echo "Example:"
    echo "`basename $0` -n master.ini"
    echo "`basename $0` -n master.ini -f"
    echo "`basename $0` -h"
    echo
    return 0
}

########################################################################
# 函数描述： 初始化路径变量
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_path()
{
    #初始化当前路径，为其他变量赋值做准备
    pushd $(dirname "$0") >/dev/null 2>&1
    CURRPATH="$(readlink -e "`pwd`")"
    popd >/dev/null 2>&1
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    return 0
}


########################################################################
# 函数描述： 获取脚本执行的调用栈
# 参数1说明：进程pid号
#
# 返回值  ： 0 成功
########################################################################
function get_call_stack()
{
    local pid_val=$1
    local line=""

    if [ -z "${pid_val}" ]; then
        pid_val=$$
    fi
    
    logme 1 "==stack-begin=="
    
    #获取调用栈
    while [[ ${pid_val} != 1 && ${pid_val} != 0 ]]
    do
        line=$(ps -wweo pid,ppid,uname,tname,args | grep "^[[:space:]]*${pid_val}[[:space:]]")
        if [ -z "${line}" ]; then
           break
        fi
        pid_val=$(echo ${line} | awk '{print $2}')
        logme 1 "${line}"
    done
    
    logme 1 "==stack-end=="
    
    return 0
}


########################################################################
# 函数描述： 检查系统的有效信息，包括用户、操作系统、命令等
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_system()
{  
    #检查操作系统
    check_osversion
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check os version failed."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 检查执行操作系统版本，目前支持redhat/suse/centos
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function check_osversion()
{
    local rhel_release="/etc/redhat-release"
    local sles_release="/etc/SuSE-release"
    local sles15_release="/etc/os-release"
    local centos_release="/etc/centos-release"
    local neokylin_relase="/etc/neokylin-release"
    local bclinux_release="/etc/bclinux-release"
    
    #支持红帽/suse/centos
    #为了最大程度上的支持，都不会异常退出，但是会打印告警，表示可能不太支持，毕竟没有在其他操作系统上做过测试
    if [ -f "${centos_release}" ]; then
        cat "${centos_release}" | grep -i "CentOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is centos."
    elif [ -f "${neokylin_relase}" ];then
        cat "${neokylin_relase}" | grep -i "NeoKylin" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not NeoKylin, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is NeoKylin."
    elif [ -f "${bclinux_release}" ];then
        cat "${bclinux_release}" | grep -i "BigCloud" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not bclinux, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is NeoKylin."         
    #获取Euler的版本
    elif [ -f "${euler_release}" ]; then
        cat "${euler_release}" | grep -i "EulerOS" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not euleros, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is euleros."
    elif [ -f "${rhel_release}" ]; then
        cat "${rhel_release}" | grep -i "Red Hat" >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not redhat, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is redhat."
    elif [ -f "${sles_release}" ]; then
        cat "${sles_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse."
    elif [ -f "${sles15_release}" ]; then
        cat "${sles15_release}" | grep -i "SUSE " >/dev/null 2>&1
        if [ $? -ne 0 ]; then
            logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not suse-15, maybe it will cause unknown result!"
            return 0
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] os version is suse-15."
    else
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] os version is not centos/redhat/suse, maybe it will cause unknown result!"
        return 0
    fi
    
    return 0
}

########################################################################
# 函数描述： 版本号比较
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function compare_version()
{
    local old_version_file="${SERVICE_SCRIPT_PATH}/VERSION"
    local new_version_file="${CURRPATH}/script/VERSION"
    local old_version=""
    local new_version=""
    
    ${CMD_IS_FILE} "${old_version_file}"
    if [ $? -ne 0 ]; then
        #当前的环境中没有VERSION文件，需要升级
        return 1
    fi
    
    old_version="$(${CMD_CAT} ${old_version_file})"
    new_version="$(${CMD_CAT} ${new_version_file})"
    
    #当前的版本有问题
    if [ -z "${old_version}" ]; then
        return 1
    fi
    
    # 比较版本号
    for ((index=1; index<=4; index++))
    do
        old_version_compare=$(echo "${old_version}" | awk -F '.' "{print \$${index}}")
        new_version_compare=$(echo "${new_version}" | awk -F '.' "{print \$${index}}")

        if [ -z $old_version_compare ]; then
            old_version_compare=0
        fi
        if [ -z $new_version_compare ]; then
            new_version_compare=0
        fi

        # 旧版本比新版本高，则不升级
        if [ ${old_version_compare} -gt ${new_version_compare} ]; then
            return 0
        fi

        # 新版本比旧版本高，升级
        if [ ${new_version_compare} -gt ${old_version_compare} ]; then
            return 1
        fi
    done

    # 版本号一致，不升级
    return 0
}

########################################################################
# 函数描述： 更新fstab文件
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function upgrade_fstab()
{
    local fstab_tmp_file="/etc/fstab.tmp.diskmgt.upgrade"
    local ini_tmp_dir=""
    local item_nf=0
    local index=1
    
    ${CMD_CP} "${ETC_FSTAB}" "${ETC_FSTAB_BAK}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${ETC_FSTAB} ${ETC_FSTAB_BAK} failed."
        return 1
    fi
    
    ${CMD_CP} "${ETC_FSTAB}" "${fstab_tmp_file}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${ETC_FSTAB} ${fstab_tmp_file} failed."
        return 1
    fi
    
    ini_tmp_dir="${HOME}/tmp_$(date +%s)"
    ${CMD_CP} "${SERVICEPATH}/ini/" "$ini_tmp_dir"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${CMD_CP} ${SERVICEPATH}/ini/ $ini_tmp_dir failed."
        return 1
    fi

    ${CMD_CHOWN_R} omm:wheel "$ini_tmp_dir"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${CMD_CHOWN_R} omm:wheel $ini_tmp_dir failed."
        return 1
    fi

    while read line
    do
        item_nf=$(echo "${line}" | awk -F '#' '{print NF}')
        for ((index=1; index<=${item_nf}; index++))
        do
            item=$(echo "${line}" | awk -F '#' "{print \$${index}}")
            item_file="${SERVICEPATH}/ini-plugin/conf/${item}"
            ${CMD_IS_FILE} "${item_file}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not find ${item_file}."
                rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
                return 1
            fi
        
            item_i_dir="$(${CMD_CAT} ${item_file} | grep "^[[:space:]]*dir=" | awk '{print $1}' | sed 's/dir=//g' | head -n 1 2>/dev/null)"
            if [ -z "${item_i_dir}" ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] item(${item}) get dir failed."
                rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
                return 1
            fi
        
            itme_fstab="$(cat "${fstab_tmp_file}" | grep -v "^[[:space:]]*#" | grep -v "grep" | grep " ${item_i_dir} ")"
            if [ -z "${itme_fstab}" ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] item(${item}) item_i_dir(${item_i_dir}) is not in fstab."
                continue
            fi
        
            new_itme_fstab="$(echo ${itme_fstab} | sed 's# 2# 0#g')"
            if [ "${itme_fstab}" == "${new_itme_fstab}" ]; then
                logme 2 "[INFO] [${FUNCNAME}:${LINENO}] itme_fstab(${itme_fstab}) new_itme_fstab(${new_itme_fstab}) is same."
                continue
            fi
        
            sed -i 's# '${item_i_dir}' # EXCLUSIVE_IMPOSSIBLE #g;/ EXCLUSIVE_IMPOSSIBLE /d' "${fstab_tmp_file}"
            if [ $? -eq 0 ]; then
                logme 1 "[INFO] [${FUNCNAME}:${LINENO}] DELFSTAB ${itme_fstab} for item(${item}) ok."
                sed -i "\$a ${new_itme_fstab}" "${fstab_tmp_file}"
                if [ $? -eq 0 ]; then
                    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ADDFSTAB ${new_itme_fstab} for item(${item}) ok."
                else
                    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ADDFSTAB ${new_itme_fstab} for item(${item}) ok."
                    rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
                    return 1
                fi
            else
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] DELFSTAB ${new_itme_fstab} for item(${item}) failed."
                rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
                return 1
            fi
        done
    done < <(cat ${ini_tmp_dir}/*ini | grep -v "^[[:space:]]*#" | awk '{print $1}')
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${ETC_FSTAB} do while failed."
        rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
        return 1
    fi

    # /etc/fstab 挂载项添加 nofail
    ${CMD_CAT} "${fstab_tmp_file}" | sed "/^#/d" | sed "/^$/d" | while read line; do
        echo ${line} | grep -e "^UUID" | grep "nofail"
        if [ $? -eq 0 ]; then
            continue
        fi

        local mount_config=$(echo "${line}" | awk -F '[ ]+' '{print $4}')
        if [ ! -z "${mount_config}" ]; then
            local new_config=$(echo "${line}" | sed "s%${mount_config}%${mount_config},nofail%")
            sed -i "s%${line}%${new_config}%" "${fstab_tmp_file}"
        fi
    done

    ${CMD_CP} "${fstab_tmp_file}" "${ETC_FSTAB}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${fstab_tmp_file} ${ETC_FSTAB} failed."
        rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
        return 1
    fi

    ${CMD_CHOWN_R} 0:0 "${ETC_FSTAB}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown 0:0 ${ETC_FSTAB} -R failed."
        rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
        return 1
    fi
    
    rm -rf "${fstab_tmp_file}" "$ini_tmp_dir"
    return 0
}

function init_install_scc_tool() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to init install scc tool[$SCC_INIT_INSTALL_PATH]"
    ${CMD_RM_RF} "$SCC_INIT_INSTALL_PATH"
    ${CMD_MKDIR_P} "$SCC_INIT_INSTALL_PATH"
    ${CMD_CP} ${CURRPATH}/scc/* "$SCC_INIT_INSTALL_PATH"
    sed -i "s|#SCC_ROOT_DIR#|$SCC_INIT_INSTALL_PATH|g" "${SCC_INIT_CONF_DIR}/logger.conf"
    sed -i "s|#SCC_ROOT_DIR#|$SCC_INIT_INSTALL_PATH|g" "${SCC_INIT_CONF_DIR}/scc.conf"

    local cpuType=$(arch)
    local retVal=0
    ${CMD_UPDATE_RPM_PACKAGE} "$SECCOMPONENT_PACKAGE_NAME" "$SCC_INIT_INSTALL_PATH/package/${SECCOMPONENT_PACKAGE_NAME}.${cpuType}.rpm"
    retVal=$?
    if [ $retVal -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to update seccomponent rpm package."
        return 1
    fi

    encrypt_password "$RANDOM" "${SCC_INIT_CONF_DIR}/scc.conf" > /dev/null 2>&1
    retVal=$?
    if [ $retVal -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to init scc tool, return-code[$retVal]"
        return 1
    fi

    find ${SCC_INIT_INSTALL_PATH} -type f | xargs -I{} chmod 600 "{}" >/dev/null 2>&1
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to init install scc tool"
}

########################################################################
# 函数描述： 初始化分区加密秘钥
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init_partition_encryption_key() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to init disk partition encryption key[$SERVICE_KEYS_PATH]"
    if [ ! -d "${SERVICE_KEYS_PATH}" ]; then
        ${CMD_MKDIR_P} ${SERVICE_KEYS_PATH}
    fi
    local diskPartitionKey1=$(generate_random_pwd)
    local diskPartitionKey2=$(generate_random_pwd)
    echo "${diskPartitionKey1}" > ${KEY_FILE_1}
    echo "${diskPartitionKey2}" > ${KEY_FILE_2}

    local result=0
    local sccConf="${SCC_INIT_CONF_DIR}/scc.conf"
    local diskPartitionKey1Cipher=$(encrypt_password $(cat ${KEY_FILE_1}) ${sccConf})
    result=$?
    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to init partition encryption key-1, return-code[$result]"
        return 1
    fi

    local diskPartitionKey2Cipher=$(encrypt_password $(cat ${KEY_FILE_2}) ${sccConf})
    result=$?
    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to init partition encryption key-2, return-code[$result]"
        return 1
    fi

    if [ -z "${diskPartitionKey1Cipher}" ] || [ -z "${diskPartitionKey2Cipher}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] partition encryption key is empty."
        return 1
    fi

    echo "${diskPartitionKey1Cipher}" > ${KEY_FILE_1}
    echo "${diskPartitionKey2Cipher}" > ${KEY_FILE_2}
    ${CMD_CHOWN_R} 0:0 "${KEY_FILE_1}"
    ${CMD_CHOWN_R} 0:0 "${KEY_FILE_2}"
    ${CMD_CHMOD_R} 400 "${KEY_FILE_1}"
    ${CMD_CHMOD_R} 400 "${KEY_FILE_2}"
    mv ${KEY_FILE_1} "${SERVICE_TMP_PATH}/ini-plugin/keys"/
    mv ${KEY_FILE_2} "${SERVICE_TMP_PATH}/ini-plugin/keys"/
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to init disk partition encryption key"
}

########################################################################
# 函数描述： 磁盘分区加密工具升级适配
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function upgrade_partition_encryption_tool() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to upgrade and adapt the disk partition encryption tool."
    local result=0
    if [ -d "${SCC_INSTALL_PATH}" ]; then
        # scc工具已存在，则只需更新scc rpm包
        local cpuType=$(arch)
        ${CMD_UPDATE_RPM_PACKAGE} "$SECCOMPONENT_PACKAGE_NAME" "$SCC_INIT_INSTALL_PATH/package/${SECCOMPONENT_PACKAGE_NAME}.${cpuType}.rpm"
        result=$?
        if [ $result -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to update seccomponent rpm package."
            return 1
        fi
    else
        # scc工具不存在，则需要安装和初始化scc工具
        init_install_scc_tool
        result=$?
        if [ $result -ne 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] failed to install scc-tool and init scc-tool"
            return 1
        fi

        # 初始化分区加密秘钥
        init_partition_encryption_key
        result=$?
        if [ $result -ne 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] failed to init partition encryption keys"
            return 1
        fi

        # 拷贝scc工具(diskmgt_install_tmp_path -> diskmgt_install_path)
        ${CMD_MKDIR_P} ${SERVICE_TMP_PATH}/scc
        sed -i "s|${SCC_INIT_INSTALL_PATH}|${SCC_INSTALL_PATH}|g" "${SCC_INIT_INSTALL_PATH}/config/logger.conf"
        sed -i "s|${SCC_INIT_INSTALL_PATH}|${SCC_INSTALL_PATH}|g" "${SCC_INIT_INSTALL_PATH}/config/scc.conf"
        ${CMD_CP} ${SCC_INIT_INSTALL_PATH}/* "${SERVICE_TMP_PATH}/scc/"
        result=$?
        if [ $result -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${SCC_INIT_INSTALL_PATH} ${SERVICE_TMP_PATH}/scc failed"
            return 1
        fi
    fi
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to upgrade and adapt the disk partition encryption tool."
}

###################################################
# 获取秘钥文件(解密后)名称
###################################################
function build_key_plain_text_file_name() {
    local keyFile="$1"
    local dev_mapper="$2"
    echo "${keyFile}.${dev_mapper}.${PLAIN_PWD_FILE_SUFFIX}"
}

###################################################
# 获取解密后的秘钥
###################################################
function get_key_plain_text() {
    local keyFile="$1"
    local keyPlainTextFile="$2"
    ${CMD_RM_RF} "${keyPlainTextFile}"

    if [ ! -s "${keyFile}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] The key file[${keyFile}] is empty or it doesn't exist"
        return 1
    fi

    local sccConf="${SCC_INSTALL_PATH}/config/scc.conf"
    local keyPlainText=$(decrypt_password $(cat "${keyFile}") "${sccConf}")
    local result=$?
    if [ $result -ne 0 ] || [ -z ${keyPlainText} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed decrypt password with key_file[${keyFile}], result-code[$result]"
        return 1
    fi

    echo "${keyPlainText}" >"${keyPlainTextFile}"
    return 0
}

########################################################################
# 函数描述： 添加新秘钥至加密的磁盘分区
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function add_new_key_to_disk_partition() {
    local mapper="$1"
    local old_key="$2"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to add new-key to disk partition[mapper[$mapper], old-key[$old_key]]"
    if [ ! -b "/dev/mapper/$mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] /dev/mapper/$mapper is not a block device."
        return 1
    fi

    ${CMD_CRYPTSETUP} status "$mapper" >>"${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get status for $mapper."
        return 1
    fi

    local dev=$(${CMD_CRYPTSETUP} status "$mapper" | grep 'device:' | awk '{print $2}')
    if [ -z "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get device for $mapper."
        return 1
    elif [ ! -b "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $dev is not a block device."
        return 1
    fi

    local newKey1PlainTextFile=$(build_key_plain_text_file_name "${KEY_FILE_1}" "${mapper}")
    get_key_plain_text "${KEY_FILE_1}" "${newKey1PlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${KEY_FILE_1}]."
        return 1
    fi

    local newKey2PlainTextFile=$(build_key_plain_text_file_name "${KEY_FILE_2}" "${mapper}")
    get_key_plain_text "${KEY_FILE_2}" "${newKey2PlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${KEY_FILE_2}]."
        return 1
    fi

    #添加新秘钥(key-1)
    ${CMD_CRYPTSETUP} --key-file "${old_key}" luksAddKey "$dev" "$newKey1PlainTextFile" >>"${LOGFILE}" 2>&1
    local result=$?
    if [ ${result} -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key ${KEY_FILE_1} for $mapper($dev) with ${old_key}."
        return 1
    fi

     #添加新秘钥(key-2)
    ${CMD_CRYPTSETUP} --key-file "${old_key}" luksAddKey "$dev" "$newKey2PlainTextFile" >>"${LOGFILE}" 2>&1
    result=$?
    if [ $? -ne 0 ]; then
        #删除新秘钥(key-1)
        ${CMD_CRYPTSETUP} luksRemoveKey "$dev" "$newKey1PlainTextFile" >>"${LOGFILE}" 2>&1
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to add key ${KEY_FILE_2} for $mapper($dev) with ${old_key}."
        return 1
    fi

    sed -i "/$mapper/d" "${ENCRYPT_PARTITION_INFO}"
    local dev_uuid=$(${CMD_BLKID} | grep "${mapper}" | awk '{print $2}' | sed 's/\"//g')
    local fs_type=$(${CMD_BLKID} | grep "${mapper}" | awk '{print $3}' | sed 's/\"//g' | awk -F '=' '{print $2}')
    local mount_dir=$(echo $mapper| sed 's/-/\//g'| awk -F "encrypted" '{print $2}' 2>/dev/null)
    if [ ! -z "$mapper" ] && [ ! -z "$dev_uuid" ] && [ ! -z "$mount_dir" ] && [ ! -z "$fs_type" ]; then
        echo "$dev_mapper $dev_uuid $mount_dir $fs_type" >> "${ENCRYPT_PARTITION_INFO}" 2>>"$LOGFILE"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to add item [$dev_mapper $dev_uuid $mount_dir $fs_type] to $ENCRYPT_PARTITION_INFO."
        fi
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Invalid partition encryption information[dev_mapper[$dev_mapper],dev_uuid[$dev_uuid],mount_dir[$mount_dir],fs_type[$fs_type]]"
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to add new-key to disk partition[mapper[$mapper],dev[$dev]]."
}

########################################################################
# 函数描述： 磁盘分区加密秘钥升级处理(单个分区)
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function upgrade_disk_partition_encryption_key() {
    local dev_partition_info="$1"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] the dev partition encryption info:$dev_partition_info"
    local key_file=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $3}')
    if [ -z "${key_file}" ] || [ ! -s "${key_file}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] failed to get key_file from ${CRYPT_TAB_INFO}."
        return 0
    fi

    if [ "${key_file}" != "$SERVICE_KEYS_PATH/fi_disk_key_1" ] && [ "${key_file}" != "$SERVICE_KEYS_PATH/fi_disk_key_2" ]; then
        # 821版本(含821) 使用新秘钥文件(fi_disk_partition_key_1|fi_disk_partition_key_1)
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] the new key[${key_file}] file has been used."
        return 0
    fi

    local dev_mapper=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $1}')
    if [ -z ${dev_mapper} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper from ${CRYPT_TAB_INFO}."
        return 1
    fi

    local dev_mapper_uuid=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $2}')
    if [ -z ${dev_mapper_uuid} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper_uuid from ${CRYPT_TAB_INFO}."
        return 1
    fi

    # 821版本(不含821)之前的秘钥文件(fi_disk_key_1|fi_disk_key_2)需收编处理(追加秘钥)
    add_new_key_to_disk_partition "$dev_mapper" "$key_file"
    local result=$?
    ${CMD_RM_RF} "$(build_key_plain_text_file_name "${KEY_FILE_1}" "${dev_mapper}")"
    ${CMD_RM_RF} "$(build_key_plain_text_file_name "${KEY_FILE_2}" "${dev_mapper}")"
    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to add new key to disk partition[dev_mapper[$dev_mapper], old_key_file[$key_file]]."
        return 1
    fi

    sed -i "/$dev_mapper/d" "${CRYPT_TAB_INFO_UPGRADE_TMP}"
    echo "$dev_mapper $dev_mapper_uuid $KEY_FILE_1" >> "${CRYPT_TAB_INFO_UPGRADE_TMP}" 2>>"$LOGFILE"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to add item [$dev_mapper $dev_mapper_uuid $KEY_FILE_1] to ${CRYPT_TAB_INFO_UPGRADE_TMP}."
    fi

    return 0
}

########################################################################
# 函数描述： 磁盘分区加密秘钥升级处理
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function upgrade_disk_partitions_encryption_keys() {
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to upgrade the disk partition encryption key."
     #记录加密分区列表，用于更换秘钥
    if [ ! -s "${CRYPT_TAB_INFO}" ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to upgrade the disk partition encryption key(No partition encryption)."
        return 0
    fi

    # 备份文件
    ${CMD_CAT} "${CRYPT_TAB_INFO}" > "${CRYPT_TAB_INFO_UPGRADE_BAK}"
    ${CMD_CAT} "${CRYPT_TAB_INFO}" > "${CRYPT_TAB_INFO_UPGRADE_TMP}"
    ${CMD_IS_FILE} "${ENCRYPT_PARTITION_INFO}"
    if [ $? -ne 0 ]; then
        # 初始化文件
        touch "${ENCRYPT_PARTITION_INFO}"
    fi

    ${CMD_CAT} "${ENCRYPT_PARTITION_INFO}" > "${ENCRYPT_PARTITION_INFO_UPGRADE_BAK}"

    #读取配置并挂载
    ${CMD_CAT} "${CRYPT_TAB_INFO}" | sed "/^#/d" | sed "/^$/d" | while read line; do
        echo ${line} | grep -e "^encrypted"
        if [ $? -ne 0 ]; then
            continue
        fi
        upgrade_disk_partition_encryption_key "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to upgrade the disk partition encryption key[${line}]"
        fi
    done

    ${CMD_CAT} "${CRYPT_TAB_INFO_UPGRADE_TMP}" > "${CRYPT_TAB_INFO}"
    ${CMD_RM_RF} "${CRYPT_TAB_INFO_UPGRADE_TMP}"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] end to upgrade the disk partition encryption key."
}

function adjust_diskmgt_service_file_permission() {
    #调整diskmgt服务安装目录下文件权限
    find "${SERVICEPATH}" -type f | xargs -I{} "${CMD_CHMOD}" 640 "{}" >/dev/null 2>&1
    find "${SERVICEPATH}" -name "*.sh" | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    find "${SERVICEPATH}" -name "diskmgt*" -type f | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    find "${SERVICEPATH}" -name "kmsLuksMgt*" -type f | xargs -I{} "${CMD_CHMOD}" 500 "{}" >/dev/null 2>&1
    ${CMD_CHMOD} 500 "${SERVICEPATH}/ini-plugin/notify/clickhouse_get_replica.py"
    find "${SERVICEPATH}/ini-plugin/keys" -name "fi_disk*" | xargs -I{} "${CMD_CHMOD}" 600 "{}" >/dev/null 2>&1
    find "${SERVICEPATH}/scc/ks" -type f | xargs -I{} "${CMD_CHMOD}" 600 "{}" >/dev/null 2>&1
}

########################################################################
# 函数描述： 安装硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function upgrade_service()
{
    #检查是否安装了diskmgt，没有安装什么都不做
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] diskmgt is not been install, return succeed."
        return ${g_UPGRADE_NOT_INSTALLED_WARN}
    fi
    
    ${CMD_IS_DIR} "${SERVICE_BACKUP_PATH}"
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] upgrade may be done."
        return ${g_UPGRADE_UPGRADED_WARN}
    fi
    
    #比较当前的版本是不是最新的。
    compare_version
    if [ $? -eq 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] diskmgt have the latest version, skiped."
        return ${g_UPGRADE_IS_LATEST_VERSION_WARN}
    fi
        
    #停止diskmgt
    ${CMD_STOP_DISKMGT}
    ret=$?
    if [ $ret -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] stop diskmgt failed."
        return ${g_UPGRADE_STOP_DSKMGT_ERROR}
    fi
    
    #####################################修改FSTABE文件BEGIN#######################################
    upgrade_fstab
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] upgrade fstab failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi    
    #####################################修改FSTABE文件END#######################################
        
    #创建临时目录
    ${CMD_RM_RF} "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICE_TMP_PATH} failed."
        return 11
    fi
            
    #拷贝新的diskmgt目录
    ${CMD_CP} "${SERVICEPATH}" "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp ${SERVICEPATH} ${SERVICE_TMP_PATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
        
    #####################################拷贝最新文件BEGIN#######################################  
    #设置用户、群主、权限
    ${CMD_CHMOD_R} 750 "${CURRPATH}/inner"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 750 -R ${SERVICEPATH}/inner failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CP} "${CURRPATH}/inner" "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp -rf ${CURRPATH}/inner ${SERVICE_TMP_PATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CHMOD_R} 750 "${CURRPATH}/script"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 750 -R ${SERVICEPATH}/script failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CP} "${CURRPATH}/script" "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp -rf ${CURRPATH}/script ${SERVICE_TMP_PATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CHMOD_R} 750 "${CURRPATH}/raidadapter"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 750 -R ${SERVICEPATH}/raidadapter failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CP} "${CURRPATH}/raidadapter" "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp -rf ${CURRPATH}/raidadapter ${SERVICE_TMP_PATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    #更新notify文件
    ${CMD_CHMOD_R} 750 "${CURRPATH}/ini-plugin/notify"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 750 -R ${SERVICEPATH}/ini-plugin/notify failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    ${CMD_CP} "${CURRPATH}/ini-plugin/notify" "${SERVICE_TMP_PATH}/ini-plugin/"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] cp -rf ${CURRPATH}/ini-plugin/notify ${SERVICE_TMP_PATH}/ini-plugin/ failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi

    upgrade_partition_encryption_tool
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to adapt to the partition encryption upgrade."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi

    #设置用户、群主、权限
    ${CMD_CHOWN_R} 0:0 "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown 0:0 ${SERVICE_TMP_PATH} -R failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    #安全起见，所有的配置文件和脚本都转为unix格式
    ${CMD_DOS2UNIX} "${SERVICE_TMP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] find ${SERVICE_TMP_PATH} -type f | xargs dos2unix >/dev/null 2>&1 failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    #####################################拷贝最新文件END#######################################  
    
    
    #####################################拷贝最新文件BEGIN#######################################    
    #备份原diskmgt目录
    ${CMD_MV} "${SERVICEPATH}" "${SERVICE_BACKUP_PATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${SERVICEPATH} ${SERVICE_BACKUP_PATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
        
    #拷贝新的diskmgt目录
    ${CMD_MV} "${SERVICE_TMP_PATH}" "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${SERVICE_TMP_PATH} ${SERVICEPATH} failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    #####################################修改特殊文件的权限END#######################################

    # 升级磁盘分区加密密钥
    upgrade_disk_partitions_encryption_keys
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to upgrade the disk partition encryption key."
        # 失败，回退
        ${CMD_CAT} "${CRYPT_TAB_INFO_UPGRADE_BAK}" > "${CRYPT_TAB_INFO}"
        return ${g_UPGRADE_GENERAL_ERROR}
    fi
    
    #配置diskmgt服务，加入开关机流程
    CMD_CONFIG_SERVICE="${CURRPATH}/script/config_diskmgt_service.sh"
    ${CMD_CONFIG_SERVICE}
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] config diskmgt service failed."
        return ${g_UPGRADE_GENERAL_ERROR}
    fi

    adjust_diskmgt_service_file_permission

    #启动diskmgt
    ${CMD_START_DISKMGT}
    ret=$?
    if [ $ret -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] start_diskmgt failed."
        return ${g_UPGRADE_START_DSKMGT_ERROR}
    fi
    
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] upgrade successful"
    return 0
}


########################################################################
# 函数描述： 回滚安装流程
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function commit_upgrade_service()
{
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -eq 0 ]; then
        ${CMD_IS_DIR} "${SERVICE_BACKUP_PATH}"
        if [ $? -eq 0 ]; then   
            ${CMD_RM_RF} "${SERVICE_BACKUP_PATH}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICE_BACKUP_PATH} failed."
                return ${g_COMMIT_GENERAL_ERROR}
            fi
            
            ${CMD_RM_RF} "${ETC_FSTAB_BAK}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${ETC_FSTAB_BAK} failed."
                return ${g_COMMIT_GENERAL_ERROR}
            fi

            ${CMD_IS_FILE} "${CRYPT_TAB_INFO_UPGRADE_BAK}"
            if [ $? -eq 0 ]; then
                ${CMD_RM_RF} "${CRYPT_TAB_INFO_UPGRADE_BAK}"
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${CRYPT_TAB_INFO_UPGRADE_BAK} failed"
                    return ${g_COMMIT_GENERAL_ERROR}
                fi
            fi

            ${CMD_IS_FILE} "${ENCRYPT_PARTITION_INFO_UPGRADE_BAK}"
            if [ $? -eq 0 ]; then
                ${CMD_RM_RF} "${ENCRYPT_PARTITION_INFO_UPGRADE_BAK}"
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${ENCRYPT_PARTITION_INFO_UPGRADE_BAK} failed"
                    return ${g_COMMIT_GENERAL_ERROR}
                fi
            fi

        else
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not commit diskmgt service."
            return ${g_COMMIT_COMMITTED_WARN}
        fi
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] diskmgt service has not been installed."
        return ${g_COMMIT_NOT_INSTALLED_WARN}
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback successful."
    return 0
}

########################################################################
# 函数描述： 删除已追加的分区加密key(单个分区)
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function remove_keys_from_disk_partition() {
    local mapper="$1"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] start to remove keys to from disk partition[mapper[$mapper]]"
    if [ ! -b "/dev/mapper/$mapper" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] /dev/mapper/$mapper is not a block device."
        return 1
    fi

    ${CMD_CRYPTSETUP} status "$mapper" >>"${LOGFILE}" 2>&1
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get status for $mapper."
        return 1
    fi

    local dev=$(${CMD_CRYPTSETUP} status "$mapper" | grep 'device:' | awk '{print $2}')
    if [ -z "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get device for $mapper."
        return 1
    elif [ ! -b "$dev" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $dev is not a block device."
        return 1
    fi

    local newKey1PlainTextFile=$(build_key_plain_text_file_name "${KEY_FILE_1}" "${mapper}")
    get_key_plain_text "${KEY_FILE_1}" "${newKey1PlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${KEY_FILE_1}]."
        return 1
    fi

    local newKey2PlainTextFile=$(build_key_plain_text_file_name "${KEY_FILE_2}" "${mapper}")
    get_key_plain_text "${KEY_FILE_2}" "${newKey2PlainTextFile}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get key plain text from key-file[${KEY_FILE_2}]."
        return 1
    fi

    #删除旧秘钥
    cryptsetup luksRemoveKey "$dev" "$newKey1PlainTextFile" >>"${LOGFILE}" 2>&1
    local result=$?
    if [ ${result} -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Failed to delete key[$KEY_FILE_1] for $mapper($dev)."
    fi

    #删除旧秘钥
    cryptsetup luksRemoveKey "$dev" "$newKey2PlainTextFile" >>"${LOGFILE}" 2>&1
    retVal=$?

    if [ ${result} -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Failed to delete key[$KEY_FILE_2] for $mapper($dev)."
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Success to delete keys for $mapper($dev)."
    return 0
}

########################################################################
# 函数描述： 回滚已追加的分区加密key(单个分区)
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback_disk_partition_encryption_keys() {
    local dev_partition_info="$1"
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] the dev partition encryption info:$dev_partition_info"

    local key_file=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $3}')
    if [ -z "${key_file}" ] || [ ! -s "${key_file}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] failed to get key_file from ${CRYPT_TAB_INFO_UPGRADE_BAK}."
        return 0
    fi

    if [ "${key_file}" != "$SERVICE_KEYS_PATH/fi_disk_key_1" ] && [ "${key_file}" != "$SERVICE_KEYS_PATH/fi_disk_key_2" ]; then
        # 821版本(含821) 使用新秘钥文件(fi_disk_partition_key_1|fi_disk_partition_key_1)
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] the new key[${key_file}] file has been used. Therefore, rollback is not required."
        return 0
    fi

    local dev_mapper=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $1}')
    if [ -z ${dev_mapper} ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper from ${CRYPT_TAB_INFO_UPGRADE_BAK}."
        return 1
    fi

    local dev_mapper_uuid=$(echo ${dev_partition_info} | awk -F '[ ]+' '{print $2}')
    if [ -z ${dev_mapper_uuid} ]; then
        log 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to get dev_mapper_uuid from ${CRYPT_TAB_INFO_UPGRADE_BAK}."
        return 1
    fi

    # 821版本(不含821)之前的秘钥文件(fi_disk_key_1|fi_disk_key_2),回滚时需删除追加的新秘钥(fi_disk_partition_key_1|fi_disk_partition_key_1)
    remove_keys_from_disk_partition "$dev_mapper"
    local result=$?
    ${CMD_RM_RF} "$(build_key_plain_text_file_name "${KEY_FILE_1}" "${dev_mapper}")"
    ${CMD_RM_RF} "$(build_key_plain_text_file_name "${KEY_FILE_2}" "${dev_mapper}")"
    if [ $result -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to  removew key to disk partition[dev_mapper[$dev_mapper], old_key_file[$key_file]]."
        return 1
    fi
}

########################################################################
# 函数描述： 回滚已追加的分区加密key
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback_disk_partitions_encryption_keys() {
    ${CMD_IS_FILE} "${CRYPT_TAB_INFO_UPGRADE_BAK}"
    if [ $? -ne 0 ]; then
        logme 2 "[INFO] [${FUNCNAME}:${LINENO}] The disk partition encryption information is not changed. Therefore, rollback is not required."
        result 0
    fi

    #读取配置并挂载
    ${CMD_CAT} "${CRYPT_TAB_INFO_UPGRADE_BAK}" | sed "/^#/d" | sed "/^$/d" | while read line; do
        echo ${line} | grep -e "^encrypted"
        if [ $? -ne 0 ]; then
            continue
        fi
        rollback_disk_partition_encryption_keys "${line}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] failed to rollback the disk partition encryption key[${line}]"
            return 1
        fi
    done

    ${CMD_CAT} "${CRYPT_TAB_INFO_UPGRADE_BAK}" > "${CRYPT_TAB_INFO}"
    ${CMD_RM_RF} "${CRYPT_TAB_INFO_UPGRADE_BAK}"
    return 0

}

########################################################################
# 函数描述： 回滚安装流程
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function rollback()
{   
    #检查是否安装了diskmgt，没有安装什么都不做
    ${CMD_IS_DIR} "${SERVICEPATH}"
    if [ $? -ne 0 ]; then
        ${CMD_IS_DIR} "${SERVICE_BACKUP_PATH}"
        if [ $? -ne 0 ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] diskmgt is not been install, return succeed."
            return ${g_ROLL_NOT_INSTALLED_WARN}
        fi
    fi
    
    ${CMD_IS_DIR} "${SERVICE_BACKUP_PATH}"
    if [ $? -eq 0 ]; then
        ${CMD_IS_DIR} "${SERVICEPATH}"
        if [ $? -eq 0 ]; then
            ${CMD_IS_FILE} "${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT}"
            if [ $? -eq 0 ]; then
                ${CMD_STOP_DISKMGT}
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] stop_diskmgt failed."
                    return ${g_ROLL_STOP_DSKMGT_ERROR}
                fi
            fi

            rollback_disk_partitions_encryption_keys
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rollback disk partitions encryption keys failed."
                return ${g_ROLL_GENERAL_ERROR}
            fi
            
            ${CMD_RM_RF} "${SERVICEPATH}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICEPATH} failed."
                return ${g_ROLL_GENERAL_ERROR}
            fi
                        
            ${CMD_MV} "${SERVICE_BACKUP_PATH}" "${SERVICEPATH}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${SERVICE_BACKUP_PATH} ${SERVICEPATH} failed."
                return ${g_ROLL_GENERAL_ERROR}
            fi
            
            ${CMD_MV} "${ETC_FSTAB_BAK}" "${ETC_FSTAB}"
            if [ $? -ne 0 ]; then
                logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${ETC_FSTAB_BAK} ${ETC_FSTAB} failed."
                return ${g_ROLL_GENERAL_ERROR}
            fi

            ${CMD_IS_FILE} "${CRYPT_TAB_INFO_UPGRADE_BAK}"
            if [ $? -eq 0 ]; then
                ${CMD_MV} "${CRYPT_TAB_INFO_UPGRADE_BAK}" "${CRYPT_TAB_INFO}"
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${CRYPT_TAB_INFO_UPGRADE_BAK} ${CRYPT_TAB_INFO} failed"
                    return ${g_ROLL_GENERAL_ERROR}
                fi
            fi

            ${CMD_IS_FILE} "${ENCRYPT_PARTITION_INFO_UPGRADE_BAK}"
            if [ $? -eq 0 ]; then
                ${CMD_MV} "${ENCRYPT_PARTITION_INFO_UPGRADE_BAK}" "${ENCRYPT_PARTITION_INFO}"
                if [ $? -ne 0 ]; then
                    logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] mv ${ENCRYPT_PARTITION_INFO_UPGRADE_BAK} ${ENCRYPT_PARTITION_INFO} failed"
                    return ${g_ROLL_GENERAL_ERROR}
                fi
            fi
        fi
    else
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] can not find ${SERVICE_BACKUP_PATH}."
        return ${g_ROLL_ROLLED_WARN}
    fi
        
    ${CMD_IS_DIR} "${SERVICE_TMP_PATH}"
    if [ $? -eq 0 ]; then
        ${CMD_RM_RF} "${SERVICE_TMP_PATH}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] rm -rf ${SERVICE_TMP_PATH} failed."
            return ${g_ROLL_GENERAL_ERROR}
        fi
    fi
        
    #启动diskmgt
    ${CMD_START_DISKMGT}
    ret=$?
    if [ $ret -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] start_diskmgt failed."
        return ${g_ROLL_START_DSKMGT_ERROR}
    fi

    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] rollback successful"
    return 0
}


########################################################################
# 函数描述： 主入口函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{
    #检查操作系统有效性
    check_system
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] check system failed."
        return 1
    fi
    
    if [ ${COMMIT} -eq 1 ]; then
        #提交硬盘管理服务
        commit_upgrade_service
        ret=$?
    elif [ ${START} -eq 1 ]; then
        #升级硬盘管理服务
        if [ -f ${SERVICE_SCRIPT_PATH}/${SERVICE_SCRIPT} ]; then
            upgrade_service
            ret=$?
        fi
    elif [ ${ROLL_BACK} -eq 1 ]; then
        #回滚升级硬盘管理服务
        rollback
        ret=$?
    fi
    
    if [ $ret -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] do upgrade service failed."            
        return $ret
    fi
    
    return 0
}


########################################################################
# 函数描述： 清除锁函数
# 参数1说明：无
#
# 返回值  ： 无
########################################################################
function clean_lock()
{
    rm -rf "${LOCKFILE}"
}


########################################################################
# 函数描述： 信号处理函数
# 参数1说明：无
#
# 返回值  ： 2 返回信号处理特殊错误码
########################################################################
function sig_catch()
{
    logme 2 "[EMER] [${FUNCNAME}:${LINENO}] catch signal!!"
    rollback
    clean_lock
    exit 2
}

#捕获信号
trap "sig_catch" INT TERM QUIT


########################################################################
# 函数描述： 以下为最外层函数，在锁保护的范围内
# 参数1说明：脚本的所有输入参数
#
# 返回值  ： 0 成功
#            1 普通失败错误码
#            2 返回信号处理特殊错误码
#            3 锁保护获取失败错误码
#            4 上次被强杀进程导致的锁遗留错误
#            5 后台正在运行硬盘管理服务错误码
########################################################################
log_begin "$@"

{
    flock -n 3
    #如果获取锁失败，要判断之前是否是被强杀进程导致锁遗留
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] `basename $0` locked by another user."
        
        #这里判断到之前是否是被强杀进程导致锁遗留，就把遗留锁清理干净
        TMPFILE="${HOME}/upgdmgttmpfile_`date +%s`.txt"
        ps -wweo pid,ppid,uname,tname,args | grep -v grep | grep "upgradedmgt.sh" | grep "/bin/" > "${TMPFILE}"
        CMDCNT=$(cat "${TMPFILE}" | wc -l)
        if [ ${CMDCNT} -eq 1 ]; then
            logme 2 "[ERROR] [none:${LINENO}] `basename $0` kill force last time, clean lock now."
            clean_lock
            exit 4
        fi
        
        exit 3
    fi
    
    #初始化路径全局变量
    init_path
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [none:${LINENO}] init path failed."
        exit 1
    fi

    source ${CURRPATH}/script/scc_tool.sh
    
    #脚本输入所有参数的处理流程
    while getopts "csrh" options;
    do
        case ${options} in
            c)
                COMMIT=1  #输入挂载配置文件
                ;;
                
            s)
                START=1  #是否强制安装硬盘管理服务
                ;;
                                
                        r)
                ROLL_BACK=1  #是否强制安装硬盘管理服务
                ;;
                
            h)
                show_help   #显示帮助
                exit $?
                ;;
                
            *)
                logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
                show_help
                exit 1
                ;;
        esac
    done
    
    #执行主函数
    args=$((COMMIT+START+ROLL_BACK))
    if [ $args -ne 1 ]; then
        logme 2 "[ERROR] [none:${LINENO}] invalid input!!"  #非法参数输入
        show_help
        RESULT=1
    else
        main
        RESULT=$?
    fi
    
    if [ ${RESULT} -ne 0 ]; then
        #只要失败就需要进行回退操作
        logme 2 "[ERROR] [none:${LINENO}] execute `basename $0` failed."
    else
        logme 2 "[INFO] [none:${LINENO}] execute `basename $0` successfully."
    fi
    
    #清理锁并返回特定返回码
    clean_lock
    exit "${RESULT}"
} 3<>"${LOCKFILE}"

