#!/bin/bash

###################################################
# 功能: 完成preinstall之后,进行检查
# 返回值范围: 101-120
# 101  : swap is on
# 102  : 
###################################################

#系统启用了swap分区
gc_SWAP_IS_ON=101

if [ -f "$HOME/.bash_profile" ]; then
    . $HOME/.bash_profile  
fi
if [[ $UID -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    CMD_CAT_FSTAB="sudo ${INSTALL_SUDO_PATH} cat_fstab"
else
    CMD_CAT_FSTAB=" cat /etc/fstab"
fi

###################################################
# 功能: 检查是否有swap分区
###################################################
function check_swap()
{
    #判断系统中是否有swap分区
    #说明: 如果系统的swap分区已经 swapoff, 
    #      则认为系统无swap分区
    cat /proc/swaps | grep -v "^File" > /dev/null 
    if [ $? -eq 0 ]; then
        #有 swap 分区
        echo "[$(date)] [WARN] [${FUNCNAME}:${LINENO}] swap is on "
        return $gc_SWAP_IS_ON
    fi
    
    ${CMD_CAT_FSTAB} | grep "[[:space:]]swap[[:space:]]*swap[[:space:]]" > /dev/null
    if [ $? -eq 0 ]; then
        echo "[$(date)] [WARN] [${FUNCNAME}:${LINENO}] swap is off "
    else
        echo "[$(date)] [INFO] [${FUNCNAME}:${LINENO}] no swap "
    fi

    return 0    
}

##########################################################
# 入口函数
##########################################################
function main ()
{   
    local ret=0
    
    #检查是否有swap分区
    check_swap
    ret=$?
    if [ $ret -ne 0 ]; then
        return $ret
    fi

    return 0
}

main

exit $?