#!/bin/bash

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   bash $0 "$@"
   exit $?
fi

g_pre_home="../../"                         #preinstall主目录
g_script_lib_dir="script/function/"         #脚本库路径
g_aes_rand_key="$(date +%s%N)"              #用于保存加密密码的随机key
g_passwd_aes_file="${HOME}/pre_in_pw.bin"      #用于保存密码密钥的文件
g_remote="remote.sh"                        #运行命令或拷贝文件
g_work_space="${HOME}/diskmgt_home"            #各节点的工作目录
g_src_partition="partition"
g_dst_partition="script/modules/070.autopart/autopart"
g_install_sh="${g_work_space}/autopart/installdmgt.sh"
g_uninstall_sh="/usr/local/diskmgt/script/uninstall.sh"
g_diskmgt_sh="/usr/local/diskmgt/script/diskmgt.sh"
g_user="root"
g_group="root"
g_result=0

#############################################################
#
# 获取绝对路径
#
#############################################################
function get_absolute_path()
{
    local path=""
    
    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi
    
    #获取 g_pre_home 的绝对路径
    g_pre_home="${path}/${g_pre_home}"   
    pushd "${g_pre_home}" > /dev/null
    g_pre_home="$(readlink -e "$(pwd)")"
    popd >/dev/null    
    
    g_script_lib_dir="${g_pre_home}/${g_script_lib_dir}"    
    g_src_partition="${g_pre_home}/${g_src_partition}"
    g_dst_partition="${g_pre_home}/${g_dst_partition}"
    g_remote="${g_script_lib_dir}/${g_remote}"

    return 0
}

###################################################
#
# 引入配置文件 及 脚本
# 
###################################################
function source_file()
{
    export g_aes_rand_key="${g_aes_rand_key}"
    
    #引入配置文件
    source "${g_pre_home}/preinstall.ini"
    if [ $? -ne 0 ]; then
        echo "source ${g_pre_home} failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/001.log.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/001.log.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/002.parse_host.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}}/002.parse_host.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/003.parse_parted_conf.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/003.parse_parted_conf.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/006.set_passwd.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/006.set_passwd.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/008.run_in_remote.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/008.run_in_remote.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/009.ctrl_workspace.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/009.ctrl_workspace.sh failed"
        return 1
    fi
    
    source "${g_script_lib_dir}/014.check_login.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/014.check_login.sh failed"
        return 1
    fi
    
    return 0
} 

###################################################
#
# 更新partition配置文件
# 
###################################################
function update_partition()
{
    
    #删除源文件
    rm "${g_dst_partition}"/ini/* -rf
    rm "${g_dst_partition}"/ini-plugin/* -rf
    
    #拷贝partition ini配置文件
    cp "${g_src_partition}"/ini  "${g_dst_partition}" -arf
    if [ $? -ne 0 ]; then
        echo "Copy partition configuration failed."
        return 1
    fi
    
    #拷贝partition ini配置文件
    cp "${g_src_partition}"/ini-plugin  "${g_dst_partition}" -arf
    if [ $? -ne 0 ]; then
        echo "Copy partition configuration failed."
        return 1
    fi
    
    return 0
}

###################################################
#
# 适配各节点
#
###################################################
function install_node()
{
    local ipaddr="$1"     #主机名或ip地址
    local index="$2"      #数组索引
    local parted_conf=""  #节点对应的格式化配置
    local ret=0
    
    debug "[${ipaddr}:${index}]Enter [${FUNCNAME}:${LINENO}]"
    
    echo "Install diskmgt in ${ipaddr}"

    #入参检查
    if [ -z "$ipaddr" ] || [ -z "$index" ]; then
        error "Parameter error."
        error "Install failed in ${ipaddr}"
        return 1
    fi

    #在各个节点创建工作空间
    gen_workspace "$g_work_space" "${ipaddr}"
    if [ $? -ne 0 ]; then
        error "Gen workspace failed on ${ipaddr}"
        error "Install failed in ${ipaddr}"
        return 1
    fi

    #把适配脚本拷贝至远端服务器
    cp_file_to_r "$ipaddr" "240" "${g_dst_partition}" "${g_work_space}" "0" > /dev/null
    if [ $? -ne 0 ]; then
        del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
        error "Copy files to ${ipaddr} failed"
        error "Install failed in ${ipaddr}"
        return 1
    fi

    parted_conf="${g_parted_array[${index}]}"

    #在各节点上执行安装
    exec_cmd_r "$ipaddr" "360" "bash ${g_install_sh} -n ${parted_conf} -f"
    if [ $? -ne 0 ]; then
        del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
        error "Run install in ${ipaddr} failed"
        error "Install failed in ${ipaddr}"
        return 1
    fi
    
    #判断目标节点是否有 ${g_user} 用户
    exec_cmd_r "$ipaddr" "180" "id ${g_user}"
    if [ $? -eq 0 ]; then
        #在各节点上执行命令
        exec_cmd_r "$ipaddr" "180" "chown ${g_user}:${g_group} /usr/local/diskmgt /var/log/diskmgt -R"
        if [ $? -ne 0 ]; then
            del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1
            error "Chown failed in ${ipaddr}"
            error "Install failed in ${ipaddr}"
            return 1
        fi
    fi
    
    #删除节点上的工作空间
    del_workspace "$g_work_space" "${ipaddr}" > /dev/null 2>&1

    debug "Leave [${FUNCNAME}:${LINENO}]"
    
    echo "Install successfully in ${ipaddr}"

    return 0
}

###################################################
#
# 安装 磁盘监控工具
#
###################################################
function install()
{
    local i=""       #循环变量
    local ipaddr=""    
    
    pre_stage "install"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    #更新分区配置文件
    update_partition 
    if [ $? -ne 0 ]; then
        return 1
    fi

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        #在各节点上执行命令
        install_node "${ipaddr}" "${i}" &

        #休眠0.1秒
        sleep .1

    done
    
    wait $(jobs -p)

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}


###################################################
#
# 卸载 磁盘监控工具
#
###################################################
function uninstall()
{
    local i=""       #循环变量
    local ret=0
    local ipaddr=""    
    
    pre_stage
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        echo "Uninstall diskmgt in ${ipaddr}" 
       
        #在各节点上执行命令
        exec_cmd_r "$ipaddr" "180" "bash ${g_uninstall_sh}"
        ret=$?
        if [ $ret -eq 0 ]; then
            echo "Uninstall successfully in ${ipaddr}"
        elif [ $ret -eq 127 ]; then
            echo "No diskmgt in ${ipaddr}, don't need to uninstall."
        else
            echo "Uninstall failed in ${ipaddr}, Code:${ret}"
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 启动 磁盘监控工具
#
###################################################
function start()
{
    local i=""       #循环变量
    local ret=0
    local ipaddr=""    
    
    pre_stage
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        echo "Start diskmgt in ${ipaddr}" 
       
        #判断目标节点是否有 ${g_user} 用户
        exec_cmd_r "$ipaddr" "180" "id ${g_user}"
        if [ $? -eq 0 -a "X${g_user}" != "Xroot" ]; then   
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "su - ${g_user} -c \"bash ${g_diskmgt_sh} -s\""
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Start successfully in ${ipaddr} for ${g_user}"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to start."
            else
                echo "Start failed in ${ipaddr}, Code:${ret}"
            fi
        else
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "bash ${g_diskmgt_sh} -s"
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Start successfully in ${ipaddr} for root"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to start."
            else
                echo "Start failed in ${ipaddr}, Code:${ret}"
            fi
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 停止 磁盘监控工具
#
###################################################
function stop()
{
    local i=""       #循环变量
    local ret=0
    local ipaddr=""    
    
    pre_stage
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        echo "Stop diskmgt in ${ipaddr}" 
       
        #判断目标节点是否有 ${g_user} 用户
        exec_cmd_r "$ipaddr" "180" "id ${g_user}"
        if [ $? -eq 0 -a "X${g_user}" != "Xroot" ]; then  
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "su - ${g_user} -c \"bash ${g_diskmgt_sh} -k\""
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Stop successfully in ${ipaddr} for ${g_user}"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to stop."
            else
                echo "Stop failed in ${ipaddr}, Code:${ret}"
            fi
        else
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "bash ${g_diskmgt_sh} -k"
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Stop successfully in ${ipaddr} for root"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to stop."
            else
                echo "Stop failed in ${ipaddr}, Code:${ret}"
            fi
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 重启 磁盘监控工具
#
###################################################
function restart()
{
    local i=""       #循环变量
    local ret=0
    local ipaddr=""   
    
    pre_stage
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    debug "Enter [${FUNCNAME}:${LINENO}]"
    
    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"
        
        echo "Restart diskmgt in ${ipaddr}" 
        
        #判断目标节点是否有 ${g_user} 用户
        exec_cmd_r "$ipaddr" "180" "id ${g_user}"
        if [ $? -eq 0 -a "X${g_user}" != "Xroot" ]; then  
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "su - ${g_user} -c \"bash ${g_diskmgt_sh} -r\""
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Restart successfully in ${ipaddr}  for ${g_user}"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to restart."
            else
                echo "Restart failed in ${ipaddr}, Code:${ret}"
            fi
        else
            #在各节点上执行命令
            exec_cmd_r "$ipaddr" "180" "bash ${g_diskmgt_sh} -r"
            ret=$?
            if [ $ret -eq 0 ]; then
                echo "Restart successfully in ${ipaddr} for root"
            elif [ $ret -eq 127 ]; then
                echo "No diskmgt in ${ipaddr}, unable to restart."
            else
                echo "Restart failed in ${ipaddr}, Code:${ret}"
            fi
        fi
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 预处理阶段
#
###################################################
function pre_stage()
{
    local hosts=""
    
    which expect > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Please Install expect."
        return 1
    fi
    
    #获取绝对路径
    get_absolute_path
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    #引入配置文件 及 脚本
    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    if [ "$1" = "install" ]; then
        #在安装 磁盘监控工具时, g_parted 必须为1
        g_parted=1
    fi
    
    #解析HOST列表
    g_hosts="$(parse_host "$g_hosts")"
    if [ $? -ne 0 ]; then
        error "Please check g_hosts in ${g_pre_home}/preinstall.ini"
        return 1
    fi
    
    #解析并检查 g_parted_conf 配置项
    if [ -z "$g_parted_conf" ]; then
        error "Please check g_parted_conf in ${g_pre_home}/preinstall.ini"
        return 1
    fi

    #扩展配置中的主机名
    g_parted_conf="$(extend_parted_conf "$g_parted_conf")"
    if [ $? -ne 0 ]; then
        error "g_parted_conf invalid in ${g_pre_home}/preinstall.ini"
        return 1
    fi

    #合并配置中的主机名
    hosts="$(union_hosts "$g_parted_conf")"
    if [ $? -ne 0 ]; then
        error "g_parted_conf invalid in ${g_pre_home}/preinstall.ini"
        return 1
    fi

    #检查g_parted_conf和g_hosts中的主机名是否一致
    if [ "$g_hosts" != "$hosts" ]; then
        error "hosts in g_parted_conf don't match g_hosts in ${g_pre_home}/preinstall.ini"
        return 1
    fi    
    
    #输入集群的密码(各节点密码是一致的)
    set_passwd "$g_passwd_aes_file" 
    if [ $? -ne 0 ]; then
        return 1
    fi 
    
    #检查各节点是否可以联通
    check_login "$g_hosts"
    if [ $? -ne 0 ]; then
        return 1
    fi
    
    return 0
}

###################################################
#
# 入口函数
#
###################################################
function main()
{
    case "$1" in
        install)
                install "$@"
                return $?
                ;;
        uninstall)
                uninstall
                return $?
                ;;
        start)
                start
                return $?
                ;;
        stop)
                stop
                return $?
                ;;
        restart)
                restart
                return $?
                ;; 
        *)
                echo $"Usage: $0 {install|uninstall|start|stop|restart}"
                return 1
    esac
    
    return 1
}

main "$@"
g_result=$?

#删除临时文件
rm "${g_passwd_aes_file}" -f
rm "${g_known_hosts}" -f

exit $g_result
