#!/bin/bash

declare BASE_CHARS="abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVXYZ01223456789~@#%^&*-_=+|[];<.>/?"
declare BASE_SPEC_CHARS="~@#%^&*-_=+|[];<.>/?"
declare DIGIT_CHARS="0123456789"
declare ALP_CHARS="abcdefghijklmnopqrstuvwxyz"
declare PWD_LEN=16
declare RANDOMPWD=""
declare g_logFile="${HOME}/mrs_cacert.log"
declare -r  VERSION_LESS=0
declare -r  VERSION_EQUAL=1
declare -r  VERSION_GREATER=2
declare -r  VERSION_ERROR=255

if [[ "$UID" -ne 0 && -n "${INSTALL_SUDO_PATH}" ]]; then
    CMD_MKDIR_P="sudo ${INSTALL_SUDO_PATH} preinstall_mkdir_p"
    CMD_PREINSTALL_CHOWN_R="sudo ${INSTALL_SUDO_PATH} preinstall_chown_r"
else
    CMD_MKDIR_P="mkdir -p"
    CMD_PREINSTALL_CHOWN_R="chown -R"
fi

LOG()
{
    currentTime=$(date '+%Y-%m-%d %H:%M:%S')
    # 日志工具： 日志文件 日志级别 日志内容 日志标识
    echo "[${currentTime}] $2 $3($1)" >> "${g_logFile}"
}

####################################################################
#  FUNCTION     : creat_log_dir
#  DESCRIPTION  : 创建/var/log/Bigdata/executor/logs
#  INPUT        :
#  OUTPUT       :
#  RETURN       :
####################################################################
creat_log_dir()
{
    LOG "${LINENO}" "INFO" "Start creat and chown repare_oms logs dir."
    touch ${g_logFile}
    chmod 640 ${g_logFile} || return 1
    LOG "${LINENO}" "INFO" "End creat and chown repare_oms logs dir."
    return 0
}
rand(){

    min=$1

    max=$(($2-$min+1))

    num=$(cat /proc/sys/kernel/random/uuid | cksum | awk -F ' ' '{print $1}')

    echo $(($num%$max+$min))

}

genRandPWD()
{
    local len=${#BASE_CHARS}
    local randpwd=""
    for((i=0; i<=$PWD_LEN; i=i+1))
    do
        if [ $i -eq 3 ];then
            len=${#ALP_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${ALP_CHARS:$rnd:1}
        elif [ $i -eq 6 ];then
            len=${#BASE_SPEC_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${BASE_SPEC_CHARS:$rnd:1}
        elif [ $i -eq 9 ];then
            len=${#DIGIT_CHARS}
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${DIGIT_CHARS:$rnd:1}
        else
            let len=len-1
            rnd=$(rand 1 $len)
            randpwd="$randpwd"${BASE_CHARS:$rnd:1}
        fi
    done

    echo "$randpwd"
}

validatePwd()
{
    local inpwd="$1"
    if [ -z "${inpwd}" ] ; then
        LOG "${LINENO}" "ERROR" "the passwd is null."
        return 1
    fi

    #字符串长度
    echo "${inpwd}" | grep -E "^.{8,32}$" 1>/dev/null
    if [ $? -ne 0 ] ; then
       LOG  "${LINENO}" "ERROR" "the passwd does not meet the password policy,length not engouth."
       return 1
    fi

    #判断是否包含' \ 或 " 当前不支持
    echo "${inpwd}" | grep -P "[\\\'\"]" 1>/dev/null
    if [ $? -eq 0 ];then
       LOG  "${LINENO}" "ERROR"  "the passwd may contains unsupported char, such as ' \\ or \""
       return 1
    fi

    #判断是否包含特殊字符
    echo "${inpwd}" | grep -P "[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-]" 1>/dev/null
    local charvalue=$?
    if [ $charvalue -ne 0 ];then
       LOG  "${LINENO}" "ERROR"  "the passwd must contains special char"
       return 1
    fi

    #判断是否包数字
    local temp_pwd=($(echo "${inpwd}" | sed "s/[\x21-\x2F\x3A-\x40\x5B-\x60\x7B-\x7E _-|]//g"))
    echo "${temp_pwd}" | grep -E "[0-9]{1,}" 1>/dev/null
    local digitvalue=$?
    if [ $digitvalue -ne 0 ];then
       LOG  "${LINENO}" "ERROR"  "the passwd must contains digit char"
       return 1
    fi

    #判断是否包小写字母
    echo "${inpwd}" | grep -E "[a-z]{1,}" 1>/dev/null
    local lowercasevalue=$?
    if [ $lowercasevalue -ne 0 ];then
       LOG "${LINENO}" "ERROR"  "the passwd must contains lowercase char"
       return 1
    fi

    #判断是否包大写字母
    echo "${temp_pwd}" | grep -E "[A-Z]{1,}" 1>/dev/null
    local uppercasevalue=$?

    #至少包含大写字母，小写字母，数字，特殊字符中三种，如果四个返回值的和大于等于2，则说明不满足至少包含三种，返回失败。
    local valid_value=`expr $digitvalue + $uppercasevalue + $lowercasevalue + $charvalue`
    if [ $valid_value -ge 2 ];then
       LOG "${LINENO}" "ERROR" "the new passwd does not meet the password policy,digitvalue=$digitvalue,uppercasevalue=$uppercasevalue,lowercasevalue=$lowercasevalue,charvalue=$charvalue."
       return 1
    fi

    return 0
}

getRandpwd()
{
    local rndpwd=""
    for((i=0; i<=1000; i=i+1))
    do
        rndpwd=$(genRandPWD)
        validatePwd "$rndpwd"
        if [ $? -ne 0 ];then
            continue
        else
            RANDOMPWD="${rndpwd}"
            return 0
        fi
    done

    return 1
}

generate_cacert()
{
    #获取密码
    cacertpsswd_file="${1}"
    if [ -f "${cacertpsswd_file}" ];then
        RANDOMPWD=$(cat ${cacertpsswd_file} |cut -d '=' -f2-)
        validatePwd ${RANDOMPWD}
    else
        getRandpwd
    fi
    if [ $? -ne 0 ];then
        LOG "${LINENO}" "ERROR" "the new passwd does not meet the password policy"
        return 1
    fi
    
    LOG "${LINENO}" "INFO" "Start generate cacert."
    ${CMD_MKDIR_P} /opt/cacert
    if [[ "$UID" -ne 0 ]]; then
        ${CMD_PREINSTALL_CHOWN_R} 'omm:wheel' '/opt/cacert' 
    fi    
    local ssl_bin="/usr/bin/openssl"
    local sslversion=$(${ssl_bin} version 2>&1 | grep "OpenSSL" | tr -cd "[0-9 .]" | awk '{print $1}' | sed -e "s/ //g")
    version_compare "${sslversion}" "1.1.0"
    local compareRst=$? 
    if [ ${compareRst} -eq ${VERSION_EQUAL} -o ${compareRst} -eq ${VERSION_GREATER} ];then
        openssl genrsa -out /opt/cacert/ca.key -passout stdin -aes256 3072 <<EOF
${RANDOMPWD}
EOF
    else
        openssl genrsa -out /opt/cacert/ca.key -passout stdin -aes256 3072 -sha256 <<EOF
${RANDOMPWD}
EOF
    fi
    if [ $? -ne 0 ]; then
      LOG "${LINENO}" "ERROR" "generate ca.key file failed."
      return 1
    fi
    openssl req -new -x509 -days 3650 -key /opt/cacert/ca.key -out /opt/cacert/ca.crt -subj "/C=cn/ST=guangdong/L=shenzhen/O=huawei/OU=huawei/CN=huawei" -passin stdin -sha256 <<EOF
${RANDOMPWD}
EOF
    if [ $? -ne 0 ]; then
      LOG "${LINENO}" "ERROR" "generate ca.crt file failed."
      return 1
    fi
    echo password=${RANDOMPWD} > /opt/cacert/cacert.ini
    chmod 644 /opt/cacert/* -R
    return 0
}
version_compare()
{
    LOG  "${LINENO}" "INFO" "Start to compare $@ ."
    if [ $# -ne 2 ]; then
        LOG  "${LINENO}" "ERROR" "version_compare need two parameter"
        return ${VERSION_ERROR}
    fi

    if [ "$1" = "$2" ]; then
        return ${VERSION_EQUAL}
    fi

    old_version=$( echo "$1" | sed -e "s/ //g")
    new_version=$( echo "$2" | sed -e "s/ //g")
    if [ "${old_version}" != "" ] && [ "${new_version}" != "" ]; then
        old_version_array=( $(echo "${old_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        new_version_array=( $(echo "${new_version}" | cut -d "." --output-delimiter=" " -f 1-) )
        old_version_array_length=${#old_version_array[@]}
        new_version_array_length=${#new_version_array[@]}

        length=$((${new_version_array_length}>${old_version_array_length}?${old_version_array_length}:${new_version_array_length}))
        index=0
        eq_tag=${VERSION_ERROR}
        while [ $index -lt "${length}" ]
        do
            v1=${old_version_array[$index]}
            v2=${new_version_array[$index]}
            if [ "${v1}" -lt "${v2}" ]; then
                return ${VERSION_LESS}
            elif [ "${v1}" -gt "${v2}" ] ; then
                return ${VERSION_GREATER}
            elif [ "${v1}" -eq "${v2}" ] ; then
                eq_tag=${VERSION_EQUAL}
                let index+=1
                continue
            else
                return ${VERSION_ERROR}
            fi
        done
        return ${eq_tag}
    else
        LOG  "${LINENO}" "ERROR" "parameters can not be null"
        return ${VERSION_ERROR}
    fi
}
###################################################
#
# 入口函数
#
###################################################
function main()
{
    creat_log_dir
    #随机数生成证书
    g_certpasswd_file=${1}
    generate_cacert ${g_certpasswd_file} >> ${g_logFile} 2>&1
    if [ $? -ne 0 ];then
      LOG "${LINENO}" "ERROR" "generate cacert faild."
      return 1
    fi
    LOG "${LINENO}" "INFO" "Random password generate cacert SUCCESS."
    #生成成功后删除日志文件
    rm -rf ${g_logFile}
  }
main "$@"