#!/bin/bash

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   bash $0 "$@"
   exit $?
fi

#文件系统挂载记录文件
PROCMOUNTFILE="/proc/mounts"

#默认用户，主要是为大数据服务
DFL_USER="omm"
DFL_GROUP="wheel"

#FI数目目录根目录
FI_DATA_HOME=""

#脚本日志目录
LOGPATH="/var/log/diskmgt/"
LOGFILE="/var/log/diskmgt/genConfigAndInstallDiskmgt.log"

#标记文件，用于标记diskmgt是使用genConfigAndInstallDiskmgt.sh工具部署的
TAG_FILE="/usr/local/diskmgt/inner/genConfigAndInstallDiskmgt"

#diskmgt运行目录和脚本
SERVICEPATH="/usr/local/diskmgt"
SERVICERUN="/usr/local/diskmgt/inner/diskmgtd"

#当前脚本路径
CURRPATH=""
#配置文件生成规则
RULE_FIlE=""
#Preinstall脚本路径
PREINSTALL_PATH=""
#diskmgt安装脚本路径
AUTOPART_PATH=""
#diskmgt配置文件路径
INI_DIR=""
INI_PLUGIN_CONF_DIR=""
INI_PLUGIN_NOTIFY_DIR=""

#FI数目目录挂载信息
FI_MOUNT_INFO_FILE=""

#FI数目目录所在磁盘列表
DATA_DEV_LIST=""

#指定的配置文件（由用户自行指定配置文件）
INPUT_INI_FILE=""
#FI节点分区配置文件
INI_FILE=""

#FI数目目录分区配置文件模板
TEMPLET_INI_PLUGIN=""

########################################################################
# 函数描述： 记录日志封装函数
# 参数1说明：0 只显示终端
#            1 只记录日志文件
#            2 显示终端，并记录日志文件（一般用于主要输出信息）
# 参数2说明：日志内容
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function logme()
{
    local log_mode="$1"
    local log_info="$2"
    
    
    if [ $# -ne 2 ]; then
        echo "logme para invalid."
        return 1
    fi
    
    if [ ! -f "${LOGFILE}" ]; then
        if [ $(id -u) -eq 0 ]; then
            if [ ! -e "${LOGPATH}" ]; then
                mkdir -p "${LOGPATH}"
            fi
            touch "${LOGFILE}"
            echo "[$(date)] [WARN] [${FUNCNAME}:${LINENO}] diskmgt module root need chown omm:wheel ${LOGFILE} where first create it." >>"${LOGFILE}"
            chown omm:wheel "${LOGFILE}" >> "${LOGFILE}" 2>&1
        fi
    fi

    if [ ${log_mode} -eq 0 ]; then
        echo "[$(date)] ${log_info}"
    elif [ ${log_mode} -eq 1 ]; then
        echo "[$(date)] ${log_info}" >>"${LOGFILE}"
    elif [ ${log_mode} -eq 2 ]; then
        echo "[$(date)] ${log_info}" | tee -a "${LOGFILE}"
    else
        echo "logme invalid mode para."
        return 1
    fi

    if [ -f "${LOGFILE}" ]; then
        chmod 640 "${LOGFILE}"
    fi
    
    return 0
}

########################################################################
# 函数描述： 校验diskmgt是否已安装
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function verify()
{    
    #diskmgtd进程已经在运行了，直接退出
    local pid_d=$(ps -ww -eo pid,cmd | grep -v grep | grep -w "${SERVICERUN}" | grep "/bin/" | awk '{print $1}')
    if [ ! -z "${pid_d}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] ${SERVICERUN} is running now."
        return 1
    fi
    
    #diskmgtd安装目录已经存在了，直接退出
    if [ -d "${SERVICEPATH}" ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Diskmgt run path ${SERVICEPATH} already exists."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 初始化函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function init()
{
    #初始化当前路径，为其他变量赋值做准备
    CURRPATH="$(dirname "$0")"
    CURRPATH="$(readlink -e "$CURRPATH")"
    if [ -z "${CURRPATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] get current path failed."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] CURRPATH is:${CURRPATH}"
    
    PREINSTALL_PATH="${CURRPATH}/../.."
    PREINSTALL_PATH="$(readlink -e "$PREINSTALL_PATH")"
    if [ ! -d "${PREINSTALL_PATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Directory $PREINSTALL_PATH does not exist."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] PREINSTALL_PATH is: ${PREINSTALL_PATH}"
    
    if [ ! -z "$INPUT_INI_FILE" ]; then
        INPUT_INI_FILE="$PREINSTALL_PATH/partition/ini/$INPUT_INI_FILE"
        if [ ! -f "$INPUT_INI_FILE" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $INPUT_INI_FILE does not exist."
            return 1
        fi
    fi
    
    AUTOPART_PATH="${CURRPATH}/../../script/modules/070.autopart/autopart"
    AUTOPART_PATH="$(readlink -e "$AUTOPART_PATH")"
    if [ ! -d "${AUTOPART_PATH}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Directory $AUTOPART_PATH does not exist."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] AUTOPART_PATH is: ${AUTOPART_PATH}"
    
    rm -rf "$AUTOPART_PATH"/ini/*
    rm -rf "$AUTOPART_PATH"/ini-plugin/*
    cp -rpf "$PREINSTALL_PATH/partition/ini" "$AUTOPART_PATH"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to cp -rpf $PREINSTALL_PATH/partition/ini $AUTOPART_PATH."
        return 1
    fi
    
    cp -rpf "$PREINSTALL_PATH/partition/ini-plugin" "$AUTOPART_PATH"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to cp -rpf $PREINSTALL_PATH/partition/ini-plugin $AUTOPART_PATH."
        return 1
    fi
    
    pushd "${AUTOPART_PATH}" >/dev/null 2>&1
    find ./ini -type f | xargs dos2unix >/dev/null 2>&1
    find ./ini-plugin -type f | xargs dos2unix >/dev/null 2>&1
    find ./inner -type f | xargs dos2unix >/dev/null 2>&1
    find ./script -type f | xargs dos2unix >/dev/null 2>&1
    popd >/dev/null 2>&1
    
    INI_DIR="${AUTOPART_PATH}/ini"
    INI_PLUGIN_CONF_DIR="${AUTOPART_PATH}/ini-plugin/conf"
    INI_PLUGIN_NOTIFY_DIR="${AUTOPART_PATH}/ini-plugin/notify"
    TEMPLET_INI_PLUGIN="${CURRPATH}/template.conf"
    FI_MOUNT_INFO_FILE="${CURRPATH}/fi_mount_info.txt"
    DATA_DEV_LIST="${CURRPATH}/fi_data_disk.lst"

    local host_name=$(hostname)
    if [ ! -z "$host_name" ]; then
        INI_FILE="${INI_DIR}/${host_name}.ini"
    else
        INI_FILE="${INI_DIR}/localhost.ini"
    fi
    
    rm -rf "${FI_MOUNT_INFO_FILE}"
    rm -rf "${DATA_DEV_LIST}"
    rm -rf "${INI_FILE}"
    
    if [ -f "${FI_MOUNT_INFO_FILE}" -o -f "${DATA_DEV_LIST}" -o -f "${INI_FILE}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to delete ${FI_MOUNT_INFO_FILE} ${DATA_DEV_LIST} ${INI_FILE}."
        return 1
    fi
    
    FI_DATA_HOME=$(su - omm -c 'echo $BIGDATA_DATA_HOME') >> "${LOGFILE}" 2>&1
    if [ -z "$FI_DATA_HOME" -o ! -d "$FI_DATA_HOME" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get BIGDATA_DATA_HOME."
        return 1
    fi
    
    RULE_FIlE="${CURRPATH}/rule.conf"
    if [ ! -f "$RULE_FIlE" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] $RULE_FIlE doesn't exist."
        return 1
    fi
    
    rm -f "${CURRPATH}/rule.conf.new"
    cat ${RULE_FIlE} | grep -v "^#" | grep -v "^[[:space:]]*$" >> "${CURRPATH}/rule.conf.new"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate ${CURRPATH}/rule.conf.new."
        return 1
    fi
    
    sed -i "s#\${BIGDATA_DATA_HOME}#${FI_DATA_HOME}#g" "${CURRPATH}/rule.conf.new"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to replace BIGDATA_DATA_HOME with FI_DATA_HOME."
        return 1
    fi
    
    RULE_FIlE="${CURRPATH}/rule.conf.new"
    return 0
}

########################################################################
# 函数描述： 获取当前FI数据目录的挂载信息
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function get_data_dir_mount_info()
{
    cat ${PROCMOUNTFILE} | grep -v '/dev/mapper' | grep "${FI_DATA_HOME}" >> "${FI_MOUNT_INFO_FILE}"

    return 0
}

########################################################################
# 函数描述： 获取当前FI数据目录所在磁盘列表
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function gen_dev_list()
{
    #获取FI数据目录中非nvme盘列表
    cat ${PROCMOUNTFILE} | grep -v '/dev/mapper' | grep "${FI_DATA_HOME}" | awk '{print $1}' | grep -v nvme | sed 's/[0-9]\{0,\}$//g' | sort | uniq >> "${DATA_DEV_LIST}"
    
    #获取FI数据目录中nvme盘列表
    cat ${PROCMOUNTFILE} | grep -v '/dev/mapper' | grep "${FI_DATA_HOME}" | awk '{print $1}' | grep nvme | sed 's/p[0-9]\{0,\}$//g' | sort | uniq >> "${DATA_DEV_LIST}"
    
    return 0    
}

########################################################################
# 函数描述： 获取当前FI数据目录的挂载信息
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function verify_ini_line()
{
    local ini_line="$1"
    local dev="$2"
    
    if [ -z "${ini_line}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate mount info for disk ${dev}."
        return 1
    fi

    if [ -z "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Disk name is empty."
        return 1
    fi
    
    #/dev/${dev}盘上数据目录的实际分区数
    local dev_partation_num=$(grep "${dev}" "${FI_MOUNT_INFO_FILE}" | wc -l)
    #生成的节点分区配置信息中的分区数    
    local ini_line_column_num=$(echo "${ini_line}" | awk -F '#' '{print NF}')
    
    #如果不相等，说明生成的配置文件信息和数据目录的实际挂载情况不一致，退出
    if [ "${dev_partation_num}" -ne "${ini_line_column_num}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] The partition number of disk $dev is inconsistent with ${ini_line}."
        return 1
    fi
    
    return 0
}


########################################################################
# 函数描述： 生成某个数据目录的配置文件(/usr/local/diskmgt/ini-plugin/conf/下的配置文件)
# 参数1说明：无
#
# 入参    ： $1 /proc/mount中某行的挂载信息
# 返回值  ： 0 成功
#            1 失败
########################################################################
function gen_ini_plugin()
{
    local line="$1"
    local ini_plugin_conf="${INI_PLUGIN_CONF_DIR}/$2"
    local rule_line="$3"
    local name=""
    local dir=""
    local size=""
    local filesystem=""
    local user=""
    local group=""
    local permission=""
    local notify=""
    local partition=""
    
    rm -rf "$ini_plugin_conf"
    cp -rf "$TEMPLET_INI_PLUGIN" "$ini_plugin_conf"
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to generate $ini_plugin_conf depends on info: [ $line ]."
    
    dir=$(echo $line | awk '{print $2}')
    if [ ! -d "$dir" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Directory ${dir} does not exist."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start generate $ini_plugin_conf for dir $dir."
    
    sed -i "s#^dir=.*#dir=${dir}#" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^dir=.*/dir=${dir}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^dir=.*/dir=${dir}/ $ini_plugin_conf."
    
    name=$(echo $rule_line | awk -F':' '{print $1}')
    if [ $? -ne 0 -o -z "${name}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get name for $dir from $rule_line."
        return 1
    fi
    sed -i "s/^name=.*/name=${name}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^name=.*/name=${name}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^name=.*/name=${name}/ $ini_plugin_conf."
    
    size=$(df -P -B G "$dir" | tail -n 1 | awk '{print $2}' | tr -d [:alpha:])
    if ! grep -q '^[[:digit:]]*$' <<< "$size"; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get partition size for $dir."
        return 1    
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Size of directory $dir is $size."
    
    sed -i "s/^size=.*/size=${size}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^size=.*/size=${size}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^size=.*/size=${size}/ $ini_plugin_conf."
    
    filesystem=$(echo $line | awk '{print $3}')
    sed -i "s/^filesystem=.*/filesystem=${filesystem}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^filesystem=.*/filesystem=${filesystem}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^filesystem=.*/filesystem=${filesystem}/ $ini_plugin_conf."
    
    user=$(stat -c %u ${dir})
    sed -i "s/^user=.*/user=${user}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^user=.*/user=${user}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^user=.*/user=${user}/ $ini_plugin_conf."
    
    group=$(stat -c %G ${dir})
    sed -i "s/^group=.*/group=${group}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^group=.*/group=${group}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^group=.*/group=${group}/ $ini_plugin_conf."
    
    permission=$(stat -c %a ${dir})
    sed -i "s/^permission=.*/permission=${permission}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^permission=.*/permission=${permission}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^permission=.*/permission=${permission}/ $ini_plugin_conf."
    
    notify=$(echo $rule_line | awk -F':' '{print $3}')
    if [ $? -ne 0 -o -z "${notify}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get notify script for $dir from $rule_line."
        return 1
    fi
    if [ ! -f "${INI_PLUGIN_NOTIFY_DIR}/${notify}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] ${INI_PLUGIN_NOTIFY_DIR}/${notify} doesn't exist."
        return 1
    fi
    
    sed -i "s/^notify=.*/notify=${notify}/" "$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^notify=.*/notify=${notify}/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^notify=.*/notify=${notify}/ $ini_plugin_conf."
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] End to generate $ini_plugin_conf depends on info: $line."
    return 0
}

############################################################################
# 函数描述： 生成某个数据目录的配置文件名称
#            (/usr/local/diskmgt/ini-plugin/conf/下的配置文件)
# 生成规则： 取rule.conf中第一列的配置项名；如果存在多个数据目录，
#            如datanode，则在配置项名后增加对应的数字编号，如datanode6.conf
# 参数1说明：无
#
# 入参    ： $1 数据目录名
# 返回值  ： 0 成功
#            1 失败
############################################################################
function get_conf_name()
{
    local dir="$1"
    local rule_line="$2"
    local name=""
    local base_dir=""
    local conf_name=""
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to generate ini conf name for: $dir."
    
    name=$(echo $rule_line | awk -F':' '{print $1}')
    if [ $? -ne 0 -o -z "${name}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get name for $dir from $rule_line."
        return 1
    fi
    
    base_dir=$(basename ${dir})
    if [ $? -ne 0 -o -z "${base_dir}" ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get base dir for $dir."
        return 1
    fi
    
    #获取目录名中最后的数字编号
    order_number=${base_dir##*[a-zA-Z-_]}
    if ! grep -q '^[[:digit:]]*$' <<< "$order_number"; then
        if [ ! -z $order_number ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get partition size for $dir."
            return 1
        fi
    fi

    conf_name="${name}${order_number}.conf"
    echo ${conf_name}
    return 0
}

############################################################################
# 函数描述： 根据rule.conf中配置信息第二列匹配目录对应的配置规则
# 参数1说明：无
#
# 入参    ： $1 数据目录名
# 返回值  ： 0 成功
#            1 失败
############################################################################
function get_rule_line()
{
    local dir="$1"
    local line=""
    local segments_number=0
    local regular=""
    local match_line=""
    local match_number=0
    
    while read line
    do
        segments_number=$(echo "$line" | awk -F':' '{print NF}')
        if [ $segments_number -ne 3 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Wrong format! Line($line) is not divided into 3 segments by ':'."
            return 1
        fi
        
        regular=$(echo "$line" | awk -F':' '{print $2}')
        echo "$dir" | grep -q -E "$regular"
        if [ $? -eq 0 ]; then
            match_line="$line"
            ((match_number++))
            logme 1 "[DEBUG] [${FUNCNAME}:${LINENO}] Find matching line($line) for $dir."
        fi
    done <"$RULE_FIlE"
    
    if [ $match_number -ne 1 ]; then
        logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Items which can match $dir is not uniq."
        return 1
    fi
    
    echo $match_line
    return 0
}

########################################################################
# 函数描述： 生成某个数据目录的配置文件(/usr/local/diskmgt/ini-plugin/下的配置文件)
# 参数1说明：无
#
# 入参    ： $1 数据盘盘符
# 返回值  ： 0 成功
#            1 失败
########################################################################
function gen_ini_line()
{
    local dev="$1"
    local index=1
    local partition=""
    local ini_line=""           #ini文件中某一行的信息，如"zookeeper.conf#dbdata_service.conf  y  y  n"
    local dir=""                #目录名称
    local rule_line=""          #配置规则信息
    local rule_name=""          #规则名
    local rule_regular=""       #目录的正则表达式
    local rule_notify=""        #notify脚本名
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to generate ini line for dev: $dev."
    
    if [ ! -b "${dev}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Disk ${dev} does not exist."
        return 1
    fi
    
    while true
    do
        echo "${dev}" | grep 'nvme' >> /dev/null
        if [ $? -ne 0 ]; then
            partition="${dev}${index}"
        else
            partition="${dev}p${index}"
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to generate config for partition: $partition."
        
        #如果分区不存在，则退出循环
        if [ ! -b "${partition}" ]; then
            logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Partition ${partition} does not exist. End generate partition config file for disk ${dev}."
            break
        fi
        
        mount_line_num=$(grep -w "${partition}" "${FI_MOUNT_INFO_FILE}" | wc -l)
        if [ "${mount_line_num}" -ne 1 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Partition ${partition} is not uniq in /proc/mounts. Or it is not mounted on FI data directory."
            return 1
        fi
        
        local mount_line_info=$(grep -w "${partition}" "${FI_MOUNT_INFO_FILE}")
        dir=$(echo "$mount_line_info" | awk '{print $2}')
        if [ ! -d "$dir" ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Directory $dir does not exist."
            return 1
        fi
        
        #获取配置规则信息
        rule_line=$(get_rule_line ${dir})
        #生成数据目录对应的分区配置文件，如/srv/BigData/hadoop/data12的名字应该为datanode12.conf
        local ini_plugin_conf=$(get_conf_name ${dir} ${rule_line})
        if [ $? -ne 0 -o -z "${ini_plugin_conf}" ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to get plugin conf file name for $dir."
            return 1
        fi
                
        #生成数据目录的分区配置文件
        gen_ini_plugin "$mount_line_info" "$ini_plugin_conf" "$rule_line"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate ini plugin config file ${ini_plugin_conf} for ${dir}."
            return 1
        fi
        
        #如果ini_line不为空，说明是并区的情况，需要在分区配置文件之间加"#"
        if [ -z "$ini_line" ]; then
            ini_line="${ini_plugin_conf}"
        else
            ini_line="${ini_line}#${ini_plugin_conf}"
        fi
        
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] End generating config for partition: $partition."
        logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Current ini_line for dev $dev is: $ini_line."
        
        #下一个分区编号
        index=$(( $index + 1 ))
    done
    
    sed -i "s/^size=.*/size=max/" "${INI_PLUGIN_CONF_DIR}/$ini_plugin_conf"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to sed -i s/^size=.*/size=max/ $ini_plugin_conf."
        return 1
    fi
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to sed -i s/^size=.*/size=max/ $ini_plugin_conf."
    
    #校验生成的ini_line是否合法
    verify_ini_line "${ini_line}" "${dev}"
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${ini_line} is invalid."
        return 1
    fi
    
    #把磁盘的挂载配置信息写入节点分区配置文件
    ini_line="${ini_line}  y  y  n"
    echo "${ini_line}" >> "$INI_FILE"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Echo [ ${ini_line} ] into ini file $INI_FILE."
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] End generating ini line for dev: $dev."
    
    return 0
}

###################################################################################
# 函数描述： 生成当前节点上的总的分区配置文件(/usr/local/diskmgt/ini/下的配置文件)
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
###################################################################################
function gen_bigdata_partation_conf()
{
    if [ ! -f "${DATA_DEV_LIST}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] DATA_DEV_LIST ${DATA_DEV_LIST} is not a regular file."
        return 1
    fi
    
    if [ ! -s "${DATA_DEV_LIST}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] DATA_DEV_LIST is empty."
        return 1
    fi
    
    if [ ! -d "${INI_PLUGIN_CONF_DIR}" -o ! -d "${INI_PLUGIN_NOTIFY_DIR}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] ${INI_PLUGIN_CONF_DIR} or ${INI_PLUGIN_NOTIFY_DIR} does not exist."
        return 1
    fi
    
    local disk=""
    for disk in $(cat "${DATA_DEV_LIST}")
    do
        gen_ini_line "${disk}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate partition info for ${disk}."
            return 1
        fi
    done
    
    return 0
}

########################################################################
# 函数描述： 生成diskmgt所需配置文件主函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function gen_config()
{
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Start to generate bigdata partition config files."

    get_data_dir_mount_info
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate data_dir_mount_info."
        return 1
    fi
    
    gen_dev_list
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate dev_list."
        return 1
    fi
    
    gen_bigdata_partation_conf
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate bigdata partition config files."
        return 1
    fi
    
    return 0
}

########################################################################
# 函数描述： 安装硬盘管理服务
# 参数1说明：无
#
# 返回值  ： 0 成功
#            其他返回值 失败
########################################################################
function ins_dmgt()
{
    local install_script="${AUTOPART_PATH}/installdmgt.sh"
    
    #合法性判断
    if [ ! -f "${install_script}" ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] install ${install_script} is not exist."
        return 1
    fi
    
    #开始安装，调用同目录下的installdmgt.sh脚本
    #所以installdmgt.sh脚本也可以单独调用，指定相应的参数即可，具体执行-h查询
    if [ ! -x "${install_script}" ]; then
        chmod 755 "${install_script}"
    fi

    INI_FILE_NAME=$(basename ${INI_FILE})
    "${install_script}" -n "${INI_FILE_NAME}"
    
    return $?
}

########################################################################
# 函数描述： 主函数
# 参数1说明：无
#
# 返回值  ： 0 成功
#            1 失败
########################################################################
function main()
{    
    INPUT_INI_FILE="$1"
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] ============Start to excute script $0 $1============."
    
    verify
    if [ $? -ne 0 ]; then
        logme 2 "[WARN] [${FUNCNAME}:${LINENO}] Failed to verify. May be diskmgt have ben installed."
        return 0
    fi
    
    init
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to init."
        return 1
    fi
    
    #没有手动指定配置文件的情况下需要自动生成
    if [ -z "$INPUT_INI_FILE" ]; then
        gen_config
        if [ "$?" -eq 0 ]; then
            logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to generate bigdata partition config files."
        else
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to generate bigdata partition config files."
            return 1
        fi
    else
        dos2unix "$INPUT_INI_FILE"
        cp -pf "$INPUT_INI_FILE" "$INI_FILE"
        if [ "$?" -ne 0 ]; then
            logme 1 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to cp -pf $INPUT_INI_FILE $INI_FILE."
            return 1
        fi
        INI_FILE="$INPUT_INI_FILE"
    fi

    #安装硬盘管理服务
    ins_dmgt
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to install diskmgt."
        return 1
    else
        cp -arf "${AUTOPART_PATH}/ini/${INI_FILE_NAME}" /usr/local/diskmgt/inner/ >> "${LOGFILE}" 2>&1
        chmod 644 "/usr/local/diskmgt/inner/${INI_FILE_NAME}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chmod 644 /usr/local/diskmgt/inner/${INI_FILE_NAME} failed."
            return 1
        fi
        
        chown 0:0 "/usr/local/diskmgt/inner/${INI_FILE_NAME}"
        if [ $? -ne 0 ]; then
            logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] chown root:root /usr/local/diskmgt/inner/${INI_FILE_NAME} failed."
            return 1
        fi
        
        #创建标记文件，用于后续识别diskmgt是由genConfigAndInstallDiskmgt.sh工具补充部署的
        date >> "${TAG_FILE}"
    fi

    "$SERVICEPATH"/script/diskmgt.sh -s
    if [ $? -ne 0 ]; then
        logme 2 "[ERROR] [${FUNCNAME}:${LINENO}] Failed to start diskmgt."
        return 1
    fi
    
    logme 1 "[INFO] [${FUNCNAME}:${LINENO}] Success to start diskmgt."
    logme 2 "[INFO] [${FUNCNAME}:${LINENO}] Execute `basename $0` successfully."
    
    return 0
}


########################################################################
#脚本开始执行
########################################################################
main "$@"
exit $?

