#!/bin/bash

NEW_OS="redhat-7.5_must-rpms.lst"           #需要更新的版本
OS_ADPTER_PATH=$(cd $(dirname $0); pwd)'/'  #执行脚本的路径
declare -a g_rpmlistArray                   #存放rpmlst文件名

#FusionInsight_SetupTool的路径
SETUPTOOL_PATH=$(echo ${OS_ADPTER_PATH} | awk -F'FusionInsight_SetupTool/' '{print $1"FusionInsight_SetupTool/"}')

#成功标志文件
ADAPT_OK="/preinstall/adaptOK"

LOG_FILE="${HOME}/adapterNewOS.log"

touch "${LOG_FILE}"
echo "" > "${LOG_FILE}"
chmod 600 "${LOG_FILE}"

function LOG()
{
    # 打印的日志级别
    local logLevel="$1"
    shift

    # 外部调用日志打印函数时所在的行号
    local lineNO="$1"
    shift

    # 打印时间、日志级别、日志内容、脚本名称、调用日志打印函数的函数、打印时的行号及脚本的进程号
    local logTime="$(date -d today +'%Y-%m-%d %H:%M:%S')"
    
    # 调用日志打印函数的函数名称
    local funcName="${FUNCNAME[1]}"
    
    echo "[${logTime}] ${logLevel} $* [${SCRIPT_NAME}(${funcName}):${lineNO}]($$)" | tee -a "$LOG_FILE"
    
    return 0
}

#############################################################
#
# 检查同一目录下rpmlst文件
#
#############################################################
function checkUpdataItem()
{
    unset g_rpmlistArray
    g_rpmlistArray=$(echo ${NEW_OS} | awk -F ';' '{for(i=1;i<=NF;i++){print $i}}')
    if [ $? -ne 0 ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: Parse rpmlst name error."
        return 1
    fi
    LOG "[INFO]" "${LINENO}" "localhost: Parsing rpmlst name successfully."

    #检查同一目录下的rpmlst文件，是不是指定的版本 
    for j in "${!g_rpmlistArray[@]}"
    do
        if [ ! -e ${OS_ADPTER_PATH}${g_rpmlistArray[${j}]} ]; then
            LOG "[ERROR]" "${LINENO}" "localhost: ${g_rpmlistArray[${j}]} does not exist in the directory."
            unset g_rpmlistArray[${j}]
        fi
    done
    return 0
}

#############################################################
#
# 复制rpm-lst文件
#
#############################################################
function cpRpmlst()
{
    local precheck_path="precheck/scriptFI/rpm_lists/"
    local preinstall_path="preinstall/script/function/add_packages/conf/"
    #取得rpmlst名
    for j in "${!g_rpmlistArray[@]}"
    do
        if [ ! -e "${SETUPTOOL_PATH}/${precheck_path}/${g_rpmlistArray[${j}]}" ]; then
            cp ${OS_ADPTER_PATH}"${g_rpmlistArray[${j}]}" "${SETUPTOOL_PATH}/${precheck_path}"
            if [ $? -eq 0 ]; then
                chmod 644 "${SETUPTOOL_PATH}/${precheck_path}/${g_rpmlistArray[${j}]}"
                LOG "[INFO]" "${LINENO}" "localhost: Copy the ${SETUPTOOL_PATH}${precheck_path}${g_rpmlistArray[${j}]} file successfully."
            fi
        fi
        
        if [ ! -e "${SETUPTOOL_PATH}/${preinstall_path}/${g_rpmlistArray[${j}]}" ]; then
            cp ${OS_ADPTER_PATH}"${g_rpmlistArray[${j}]}" "${SETUPTOOL_PATH}/${preinstall_path}"
            if [ $? -eq 0 ]; then
                chmod 644 "${SETUPTOOL_PATH}/${preinstall_path}/${g_rpmlistArray[${j}]}"
                LOG "[INFO]" "${LINENO}" "localhost: Copy the ${SETUPTOOL_PATH}${preinstall_path}${g_rpmlistArray[${j}]} file successfully."
            fi
        fi
    done
}
#############################################################
#
# 在osList.ini中添加字段
#
#############################################################
function addOSForPreinstall()
{
    local changed=${1}
    local replaced=""
    majorVersion=$(echo ${changed} | awk -F'.' '{print $1}')
  
    #替换的osList.ini文件路径
    local preinstall_script="preinstall/osList.ini"
    if [ ! -e "${SETUPTOOL_PATH}/${preinstall_script}" ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: osList.ini file does not exist."
        return 1
    fi

    #如果已经加过、就不再加了
    readLine=$(grep ${changed}';' "${SETUPTOOL_PATH}/${preinstall_script}")
    if [ -n "${readLine}" ]; then
        LOG "[WARN]" "${LINENO}" "localhost: The ${changed} has been adapted for osList.ini and does not need to be added."
        return 0
    fi
    
    #替换前内容
    modifyBefore=$(grep "${majorVersion}" "${SETUPTOOL_PATH}/${preinstall_script}")
    #替换后的内容
    modifyAfter="$(echo ${modifyBefore})${changed};"

    sed -i "s/${modifyBefore}/${modifyAfter}/g" "${SETUPTOOL_PATH}/${preinstall_script}"

    if [ $? -ne 0 ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: Replace osList.ini failed."
        return 1
    else
        LOG "[INFO]" "${LINENO}" "localhost: osList.ini added field successfully."
    fi
}

#############################################################
#
# 在checkNSS.sh中添加字段
#
#############################################################
function addRpmNameForNSS()
{
    local changed=${1}
    local replaced=""
    
    #替换的checkNSS.sh文件路径
    local tmpPathNSS="precheck/scriptFI/modules/OMSServer/checkItems/checkNSS.sh"
    if [ ! -e "${SETUPTOOL_PATH}/${tmpPathNSS}" ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: checkNSS.sh file does not exist."
        return 1
    fi
    
    #如果已经加过、就不再加了
    readLine=$(grep "check_os_type " "${SETUPTOOL_PATH}/${tmpPathNSS}" | grep ${changed})
    if [ -n "${readLine}" ]; then
        LOG "[WARN]" "${LINENO}" "localhost: The ${changed} has been adapted for checkNSS.sh and does not need to be added."
        return 0
    fi

    #替换前的内容
    modifyBefore="$(grep "check_os_type " "${SETUPTOOL_PATH}/${tmpPathNSS}")"
    #替换后的内容
    modifyAfter="$(echo ${modifyBefore%\"*});${changed}\""

    sed -i "s/${modifyBefore}/${modifyAfter}/g" "${SETUPTOOL_PATH}/${tmpPathNSS}"

    if [ $? -ne 0 ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: Replace checkNSS.sh failed."
        return 1
    else
        LOG "[INFO]" "${LINENO}" "localhost: checkNSS.sh added field successfully."
    fi
}

main() 
{
    #执行前准备检查
    checkUpdataItem
    if [ $? -ne 0 ]; then
        LOG "[ERROR]" "${LINENO}" "localhost: The preparation condition fails and the preparation items are checked."
        return 1
    fi
    
    #复制rpm-lst文件
    cpRpmlst

    for s in "${!g_rpmlistArray[@]}"
    do
        tempName=$(echo ${g_rpmlistArray[${s}]} | awk -F '_' '{print $1}')
        #在osList.ini中添加字段
        addOSForPreinstall ${tempName}
        if [ $? -ne 0 ]; then
            LOG "[ERROR]" "${LINENO}" "localhost: osList.ini failed to add a field."
            return 1
        fi

        #在checkNSS.sh中添加字段
        addRpmNameForNSS ${tempName}
        if [ $? -ne 0 ]; then
            LOG "[ERROR]" "${LINENO}" "localhost: checkNSS.sh failed to add a field."
            return 1
        fi

    done

    ret=$?

    #生成执行成功的标志文件
    if [ $ret -eq 0 ]; then

        if [ ! -f "${SETUPTOOL_PATH}/${ADAPT_OK}" ]; then
            touch "${SETUPTOOL_PATH}/${ADAPT_OK}"
        fi
        
    fi

    return $ret
}

main


