#!/bin/bash

#脚本解释器 强制设置为 bash
if [ "$BASH" != "/bin/bash" -a "$BASH" != "/usr/bin/bash" ]; then
   bash $0 "$@"
   exit $?
fi

g_pre_home="../../"                         #preinstall主目录
g_script_lib_dir="script/function/"         #脚本库路径
g_aes_rand_key="$(date +%s%N)"              #用于保存加密密码的随机key
g_passwd_aes_file="${HOME}/pre_in_pw.bin"      #用于保存密码密钥的文件
g_remote="remote.sh"                        #运行命令或拷贝文件
g_diskmgt_sh="/usr/local/diskmgt/script/diskmgt.sh"
g_result=0

#############################################################
#
# 获取绝对路径
#
#############################################################
function get_absolute_path()
{
    local path=""

    pushd $(dirname "$0") > /dev/null
    path="$(readlink -e "$(pwd)")"
    popd >/dev/null
    if [ -z "${path}" ]; then
        echo "current path is null."
        return 1
    fi

    #获取 g_pre_home 的绝对路径
    g_pre_home="${path}/${g_pre_home}"
    pushd "${g_pre_home}" > /dev/null
    g_pre_home="$(readlink -e "$(pwd)")"
    popd >/dev/null

    g_script_lib_dir="${g_pre_home}/${g_script_lib_dir}"
    g_remote="${g_script_lib_dir}/${g_remote}"

    return 0
}

###################################################
#
# 引入配置文件 及 脚本
#
###################################################
function source_file()
{
    export g_aes_rand_key="${g_aes_rand_key}"
    
    #引入配置文件
    source "${g_pre_home}/preinstall.ini"
    if [ $? -ne 0 ]; then
        echo "source ${g_pre_home} failed"
        return 1
    fi

    source "${g_script_lib_dir}/001.log.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/001.log.sh failed"
        return 1
    fi

    source "${g_script_lib_dir}/002.parse_host.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}}/002.parse_host.sh failed"
        return 1
    fi

    source "${g_script_lib_dir}/006.set_passwd.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/006.set_passwd.sh failed"
        return 1
    fi

    source "${g_script_lib_dir}/008.run_in_remote.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/008.run_in_remote.sh failed"
        return 1
    fi

    source "${g_script_lib_dir}/014.check_login.sh"
    if [ $? -ne 0 ]; then
        echo "source ${g_script_lib_dir}/014.check_login.sh failed"
        return 1
    fi

    return 0
}

###################################################
#
# 设置/清除 硬盘 缓存
#
###################################################
function set_wce()
{
    local wce="$1"
    local info="$2"
    local i=""       #循环变量
    local ret=0
    local ipaddr=""

    if [ "$wce" != "0" ] && [ "$wce" != "1" ]; then
        echo $"Usage: $0 {set|clear}"
        return 1
    fi

    pre_stage
    if [ $? -ne 0 ]; then
        return 1
    fi

    debug "Enter [${FUNCNAME}:${LINENO}]"

    for ((i=0;i<${#g_ipaddr_array[@]};i++))
    do
        ipaddr="${g_ipaddr_array[${i}]}"

        #在各节点上执行命令
        exec_cmd_r "$ipaddr" "180" "bash ${g_diskmgt_sh} -w ${wce}"
        ret=$?
        if [ $ret -ne 0 ]; then
            error "${info} Writeback Cache Enable in ${ipaddr} failed."
            continue
        fi

        echo "${info} Writeback Cache Enable in ${ipaddr} successfully."
    done

    debug "Leave [${FUNCNAME}:${LINENO}]"

    return 0
}

###################################################
#
# 预处理阶段
#
###################################################
function pre_stage()
{
    local hosts=""

    which expect > /dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Please Install expect."
        return 1
    fi

    #获取绝对路径
    get_absolute_path
    if [ $? -ne 0 ]; then
        return 1
    fi

    #引入配置文件 及 脚本
    source_file
    if [ $? -ne 0 ]; then
        return 1
    fi

    #解析HOST列表
    g_hosts="$(parse_host "$g_hosts")"
    if [ $? -ne 0 ]; then
        error "Please check g_hosts in ${g_pre_home}/preinstall.ini"
        return 1
    fi

    #输入集群的密码(各节点密码是一致的)
    set_passwd "$g_passwd_aes_file"
    if [ $? -ne 0 ]; then
        return 1
    fi

    #检查各节点是否可以连通
    only_check_login "$g_hosts"
    if [ $? -ne 0 ]; then
        return 1
    fi

    return 0
}

###################################################
#
# 入口函数
#
###################################################
function main()
{
    case "$1" in
        set)
                set_wce "1" "Set"
                return $?
                ;;
        clear)
                set_wce "0" "Clear"
                return $?
                ;;
        *)
                echo $"Usage: $0 {set|clear}"
                return 1
    esac

    return 1
}

main "$@"
g_result=$?

#删除临时文件
rm "${g_passwd_aes_file}" -f
rm "${g_known_hosts}" -f

exit $g_result
