#!/bin/bash

declare g_osType=""                                 # 当前节点os类型，仅限euleros-2.0sp5和euleros-2.0sp8
declare g_curPath=""                                # 当前脚本所在的目录
declare g_logPath="/var/log/Bigdata/sudo"           # 日志路径
declare g_logFile="preset.log"                      # 日志文件
declare g_must_rpms=(boost-atomic boost-system libverto libverto-libevent libverto-devel)

################################################################################
# Function: init_path
# Description: 初始化变量
################################################################################
init_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_curPath="${PWD}"
    g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_path
# Description: 初始化日志
################################################################################
init_log()
{
    [ -d ${g_logPath} ] || mkdir -p ${g_logPath}
    [ -f ${g_logPath}/${g_logFile} ] || touch "${g_logPath}/${g_logFile}"
    chmod 600 "${g_logPath}/${g_logFile}"
}

################################################################################
# Function: main
# Description: 主函数
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{  
    # 如果不是root用户，则退出
    if [ $(id -u) -ne 0 ] ; then
        echo  "ERROR: Only super user can execute this script."
        return 1
    fi
    
    if [ ! -f /etc/euleros-release ]; then
        echo "Not Euler OS, skip."
        return 0
    fi

    local euler_release_version=$(cat /etc/euleros-release | tr -d '()' | awk '{print $4}' | tr [A-Z] [a-z])
    if [ "$euler_release_version" != "sp5" -a "$euler_release_version" != "sp8" ] ;  then
        echo "No Need adapter."
        return 0
    fi
    
    showLog_info "${LINENO}" "start to add rpm pkg for mrs cluster."
    g_osType="euleros-2.0$euler_release_version"
    
    rpm_list=$(ls -l ${g_curPath}/mrs_rpm_pkg_list/${g_osType} | grep -v total | awk '{print $9}')
    
    for must_rpm in ${g_must_rpms[@]}
    do
        rpm -q $must_rpm
        if [ $? -eq 0 ] ; then
            showLog_info "${LINENO}" "rpm pkg $must_rpm was already installed,skip."
            continue
        fi
        
        rpm_pkg=$(ls -l ${g_curPath}/mrs_rpm_pkg_list/${g_osType} | grep "$must_rpm" | awk '{print $9}' |  awk 'NR==1')
        if [ -z "$rpm_pkg" ] ; then 
            showLog_error "${LINENO}" "cannot find $must_rpm in ${g_curPath}/mrs_rpm_pkg_list."
            return 1
        fi
    
        showLog_info "${LINENO}" "installing rpm $rpm_pkg."
        rpm -ivh ${g_curPath}/mrs_rpm_pkg_list/${g_osType}/${rpm_pkg}
        if [ $? -ne 0 ]; then
            showLog_error "${LINENO}" "failed to execute rpm -ivh ${g_curPath}/mrs_rpm_pkg_list/${g_osType}/$rpm_pkg."
            return 1
        fi
    done
    
    showLog_info "${LINENO}" "success to add rpm pkgs for mrs cluster."
    return 0
}

# ---------------------------------------------------------------------------- #
#                        获取当前路径,初始化日志文件                           #
# ---------------------------------------------------------------------------- #
init_path
init_log

# ---------------------------------------------------------------------------- #
#                                   导入头文件                                 #
# ---------------------------------------------------------------------------- #
. "${g_curPath}/func/log.sh" || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${g_curPath}/log.sh."; exit 1;}

# ---------------------------------------------------------------------------- #
#                                 脚本开始运行                                 #
# ---------------------------------------------------------------------------- #
main "$@"
ret=$?

#在操作系统日志中记录审计日志
syslog "manager" "add_pkgs_for_mrs.sh" "$ret" "Install boost-atomic boost-system rpm pkgs."
exit $ret
