#!/bin/bash

###################################################
# Usage: setuptool.sh
# function:  setup tool entry
###################################################

declare g_cur_path=""                                # script path
declare g_script_name=""                             # script name
declare g_version_script=""                         # version script
declare -r g_fuctionArray=(precheck/precheck.sh preinstall/preinstall.sh preset/preset.sh preset/postset.sh)

########################################################################
# Function: show_help
# Description: show help
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
########################################################################
function show_help()
{
    echo
    echo "Usage: setuptool [function]"
    echo "function:"
    echo "precheck"
    echo "preinstall"
    echo "preset"
    echo "postset"
    echo
    return 0
}

###################################################
#
# record audit log
#
###################################################
function syslog()
{
    local component=$1
    local filename=$2
    local status=$3
    local message=$4

    which logger >/dev/null 2>&1
    [ "$?" -ne "0" ] && return 0;

    login_user_ip="$(who -m | sed 's/.*(//g;s/)*$//g')"
    execute_user_name="$(whoami)"
    logger -p local0.notice -i "FusionInsight;${component};[${filename}];${status};${login_user_ip};${execute_user_name};${message}"
    return 0
}

################################################################################
# Function: get_cur_path
# Description: Get script path and name
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    g_cur_path="${PWD}"
    g_script_name="$(basename "${BASH_SOURCE-$0}")"
    cd - >/dev/null
}

################################################################################
# Function: init_global_param
# Description: init golbal paremeters
# Parameter:
#   input:
#   N/A
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
init_global_param()
{
    get_cur_path
    g_version_script="${g_cur_path}/script/version.sh"
}

################################################################################
# Function: main
# Description: Entrance of the script
# Parameter:
#   input:
#   $1  function's name & arguments
#   output:
#   N/A
# Return: 0 -- success; not 0 -- failure
# Others: N/A
################################################################################
main()
{
    if [ $# -lt 1 ]; then
        show_help
        return 1
    fi
    init_global_param
    local function_name=$1
    shift

    function_scrip=${function_name}.sh
    function_scrip_path=""
    for function in "${g_fuctionArray[@]}"
    do
        if echo "${function}" | grep "${function_scrip}" &>/dev/null; then
            function_scrip_path=${function}
            break;
        fi
    done

    if [ -z ${function_scrip_path} ] ; then
        show_help
        return 1
    fi


    ${g_cur_path}/${function_scrip_path} "$@"
}

PATH="$PATH:/sbin/:/usr/sbin/"


umask 027
main "$@"
exit $?