#!/bin/bash
# --------------------------  variables  ------------------------------------- #


#----------------global variable, control outprint style-----------------------#
CURRENT_PATH=""
LOG_FILE=""
UPD_PREINSTALL_CFG=""
MANAGE_IP_LIST=""
AGENT_PRESET_PATH="/opt/preset"
ROOT_PASSWD=""
G_REMOTE_SCRIPT=""

# -------------------------------  retcode  ---------------------------------- #
declare -ri RET_OK=0                            # operate successfully
declare -ri RET_FAIL=1                          # operate failed
declare -ri INPUT_PARAM_ERROR=2                 # received invalid param



# ----------------------------  functions  ----------------------------------- #

# **************************************************************************** #
# Function Name: get_cur_path
# Description: get current script path
# Parameter: none
# Return: 
# **************************************************************************** #
function get_cur_path()
{
    cd "$(dirname "${BASH_SOURCE-$0}")"
    local curPath="${PWD}"
    local g_scriptName="$(basename "${BASH_SOURCE-$0}")"
    echo "$curPath"
    cd - > /dev/null
}

function PrintHelp()
{
    echo "Useage:" 
    echo -e "\t upgrade tomcat srcfilepath[/opt/xxx.tar.gz] ."    
    echo -e "\t rollback tomcat ."
    echo -e "\t clean tomcat ."    
}

sys_log()
{
    local time=$(date)
    local content=${1}
    echo "${time} ${content}"    
    echo "${time} ${content}"  >> "${G_LOG_FILE}"  
}

init_env_variable()
{
    source /home/omm/.profile || return 1    
    
    G_LOG_FILE="$(echo ${BIGDATA_LOG_HOME})/upgrade/setup_tool.log_$(date +%F)"
    G_MESSAGE="$(echo ${BIGDATA_LOG_HOME})/upgrade/mess_info"
    mkdir -p "$(dirname ${G_LOG_FILE})"
    touch "${G_LOG_FILE}"
    touch "${G_MESSAGE}"
    echo "" > "${G_MESSAGE}"
    chown  -R omm:wheel "$(dirname ${G_LOG_FILE})"
    chmod 600 "${G_LOG_FILE}"
    chmod 600 "${G_MESSAGE}"
    chmod 700 "$(dirname ${G_LOG_FILE})"
}

function do_prepare_jdk()
{
    local component_package_name="${1}"
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"

    local component_package_full_name="${compoent_package_name}"
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/install_jre.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}" >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put install_jre.sh failed." 
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/install_jre.sh \"${component_package_name}\" \"${JRE_INSTALL_DIR}\" \"${OLD_JRE_INSTALL_DIR}\"" "${G_MESSAGE}"  >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run install_jre.sh failed." 
        return 1
    fi
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/update_jre.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put update_jre.sh failed." 
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/update_jre.sh \"${JRE_INSTALL_DIR}\" \"${OLD_JRE_INSTALL_DIR}\"" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run update_jre.sh failed." 
        return 1
    fi
    return 0
}

function do_rolllback_tomcat()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    sh ${preinstall_cmd_tool} "mkdir -p /tmp/upgrade_work_tmp/"  >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} mkdir -p /tmp/upgrade_work_tmp/ failed."         
    fi
    
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/rollback_tomcat.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}" >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put tomcat upgrade failed." 
        return 1
    fi
    
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/rollback_tomcat.sh \"${backup_dir}\" \"${G_LOG_FILE}\" \"${CURRENT_TOMCAT_NAME}\"" "${G_MESSAGE}"  >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run tomcat upgrade failed." 
        return 1
    fi
    
    return 0
}

function do_prepare_tomcat()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    local backup_dir="/tmp/upgrade_tomcat_back/"    
    
    sh ${preinstall_cmd_tool} "mkdir -p /tmp/upgrade_tomcat_back/"  >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} mkdir -p /tmp/upgrade_tomcat_back/ failed."         
    fi
    
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/upgrade_tomcat.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}" >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put tomcat upgrade failed." 
        return 1
    fi
    
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/upgrade_tomcat.sh \"${backup_dir}\" \"${G_LOG_FILE}\" \"${CURRENT_TOMCAT_NAME}\" \"${NEW_TOMCAT_NAME}\"" "${G_MESSAGE}"  >> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run tomcat upgrade failed." 
        return 1
    fi
    
    return 0
}

function do_clean_jdk()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/clean_jre.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put clean_jre.sh failed."  
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/clean_jre.sh \"${OLD_JRE_INSTALL_DIR}\"" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run clean_jre.sh failed."  
        return 1
    fi
    return 0
}

function do_clean_tomcat()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/clean_tomcat.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put clean_jre.sh failed."  
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/clean_tomcat.sh" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run clean_jre.sh failed."  
        return 1
    fi
    return 0
}

function do_rollabck_jdk()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/rollback_jre.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put rollback_jre.sh failed."  
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/rollback_jre.sh  \"${JRE_INSTALL_DIR}\" \"${OLD_JRE_INSTALL_DIR}\"" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run rollback_jre.sh failed."  
        return 1
    fi
    return 0
}

function do_commit_jdk()
{
    local preinstall_scp_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clusterscp.sh"
    local preinstall_cmd_tool="${CURRENT_PATH}/../preinstall/tools/cluster/clustercmd.sh"
    
    sh ${preinstall_cmd_tool} "mkdir -p /tmp/upgrade_work_tmp/" >> "${G_LOG_FILE}"
    sh ${preinstall_scp_tool} put "${CURRENT_PATH}/tool/commit_jre.sh" "/tmp/upgrade_work_tmp/" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} put commit_jre.sh failed."  
        return 1
    fi
    sh ${preinstall_cmd_tool} "/tmp/upgrade_work_tmp/commit_jre.sh \"${OLD_JRE_INSTALL_DIR}\" \"${JRE_INSTALL_DIR}\"" "${G_MESSAGE}">> "${G_LOG_FILE}"
    if [ $? -ne 0 ];then
        sys_log "${LINENO} run commit_jre.sh failed."  
        return 1
    fi
    return 0
}

function upgrade_jdk()
{
    local component_package_name="${1}"
    local component="${2}"
    local component_src_fie="${3}"
    
    sh "${CURRENT_PATH}/tool/do_prepare.sh" "${component_package_name}" "${component_src_fie}" "${G_MESSAGE}">> "${G_LOG_FILE}"
       
    if [ "${component}" == "jdk"  ];then
        do_prepare_jdk "${component_package_name}"
        if [ $? -ne 0 ];then
            sys_log "upgrade jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} failed."                
            return 1
        fi

        return 0
    fi
}

function rollback_tomcat_ext()
{
    local component="${1}"
       
    if [ "${component}" == "tomcat" ];then
        do_rolllback_tomcat
        if [ $? -ne 0 ];then
            sys_log "rollback tomcat failed."                
            return 1
        fi

        return 0
    fi
}

function upgrade_tomcat_ext()
{
    local component_package_name="${1}"
    local component="${2}"
    local component_src_fie="${3}"
    
    sh "${CURRENT_PATH}/tool/do_prepare.sh" "${component_package_name}" "${component_src_fie}" "${G_MESSAGE}">> "${G_LOG_FILE}"
       
    if [ "${component}" == "tomcat"  ];then
        do_prepare_tomcat "${component_package_name}"
        if [ $? -ne 0 ];then
            sys_log "upgrade jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} failed."                
            return 1
        fi

        return 0
    fi
}

function commit_jdk()
{
    local component="${1}"
    if [ "${component}" == "jdk" ];then
        do_commit_jdk
        if [ $? -ne 0 ];then
            sys_log "commit jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} failed."                
            return 1
        fi
        return 0
    else
        echo "invalid argument."
        PrintHelp
        return 1
    fi
}

function rollback_jdk()
{
    local component="${1}"
    if [ "${component}" == "jdk" ];then
        do_rollabck_jdk
        if [ $? -ne 0 ];then
            sys_log "rollback jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} failed."                
            return 1
        fi
        return 0
    else
        echo "invalid argument."
        PrintHelp
        return 1
    fi  
}

function clean_jdk()
{
    local component="${1}"
    if [ "${component}" == "jdk" ];then
        do_clean_jdk
        if [ $? -ne 0 ];then
            sys_log "clean jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} failed."                
            return 1
        fi
        sys_log "clean jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} end." 
        return 0
    else
        echo "invalid argument."
        PrintHelp
        return 1
    fi
}

function upgrade_tomcat()
{
    sys_log "upgrade tomcat from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."        
    local component="${1}"
    local component_src_fie="${2}"
        
    if [ -z "${component}" ];then        
        sys_log "${LINENO} main invalid argument."
        PrintHelp
        return 1
    fi
        
    if [ -z "${component_src_fie}" ];then
        sys_log "${LINENO} main invalid argument." 
        PrintHelp
        return 1
    fi
        
    local component_package_name=$(echo ${component_src_fie##*/})
    upgrade_tomcat_ext "${component_package_name}" "${component}" "${component_src_fie}" &
    spin_wait 'upgrading tomcat.' "$!"
    if [ $? -ne 0 ]; then
        sys_log "[ERROR] upgrade JDK failed."
        
        
        cat "${G_MESSAGE}"
        echo "see full log in ${G_LOG_FILE}"
        return 1
    fi
    return 0
}


function rollback_tomcat()
{
    sys_log "upgrade tomcat from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."        
    local component="${1}"
    local component_src_fie="${2}"
        
    if [ -z "${component}" ];then        
        sys_log "${LINENO} main invalid argument."
        PrintHelp
        return 1
    fi
        
    local component_package_name=$(echo ${component_src_fie##*/})
    rollback_tomcat_ext "${component}" &
    spin_wait 'rollback tomcat.' "$!"
    if [ $? -ne 0 ]; then
        sys_log "[ERROR] rollback tomcat failed."
        
        cat "${G_MESSAGE}"
        echo "see full log in ${G_LOG_FILE}"
        return 1
    fi
    return 0
}
function main()
{
    init_env_variable
    
    if [ $# -eq 0 ]; then
        PrintHelp
        return 1
    fi
    
    local operator="${1}"
    local component="${2}"
    local component_src_fie="${3}"
    if [ "${operator}" == "upgrade" ];then
        if [ "${component}" == "tomcat" ];then
            upgrade_tomcat "${component}" "${component_src_fie}"
            if [ $? -ne 0 ];then
                 return 1
            fi
            return 0
        fi 
        sys_log "upgrade jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."        
        
        if [ -z "${component}" ];then        
            sys_log "${LINENO} main invalid argument."
            PrintHelp
            return 1
        fi
        
        if [ -z "${component_src_fie}" ];then
            sys_log "${LINENO} main invalid argument." 
            PrintHelp
            return 1
        fi
        
        local component_package_name=$(echo ${component_src_fie##*/})
        upgrade_jdk "${component_package_name}" "${component}" "${component_src_fie}" &
        spin_wait 'upgrading JDK.' "$!"
        if [ $? -ne 0 ]; then
            sys_log "[ERROR] upgrade JDK failed."
            cat "${G_MESSAGE}"
            echo "see full log in ${G_LOG_FILE}"
            return 1
        fi
    elif [ "${operator}" == "commit" ];then
        
        sys_log "commit jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."
     
        commit_jdk "${component}" &
        spin_wait 'committing JDK.' "$!"
        if [ $? -ne 0 ]; then
            sys_log "[ERROR] commit JDK failed."
            cat "${G_MESSAGE}"
            echo "see full log in ${G_LOG_FILE}"
            return 1
        fi
    elif [ "${operator}" == "rollback" ];then
        if [ "${component}" == "tomcat" ];then
            rollback_tomcat "${component}"
            if [ $? -ne 0 ];then
                 return 1
            fi
            return 0
        fi 
        sys_log "rollback jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."       
    
        rollback_jdk "${component}" &
        spin_wait 'rollbacking JDK.' "$!"
        if [ $? -ne 0 ]; then
            sys_log "[ERROR] rollback JDK failed."
            cat "${G_MESSAGE}"
            echo "see full log in ${G_LOG_FILE}"
            return 1
        fi  
    elif [ "${operator}" == "clean" ];then
        if [ "${component}" == "tomcat" ];then
            do_clean_tomcat
            if [ $? -ne 0 ];then
                 return 1
            fi
            return 0
        fi 
    
        sys_log "clean jdk from ${OLD_JRE_INSTALL_DIR} to ${JRE_INSTALL_DIR} start."   
        
        clean_jdk "${component}" &
        spin_wait 'cleanning JDK.' "$!"
        if [ $? -ne 0 ]; then
            sys_log "[ERROR] clean JDK failed."
            cat "${G_MESSAGE}"
            echo "see full log in ${G_LOG_FILE}"
            return 1
        fi
    fi
    cat "${G_MESSAGE}"
    echo "see full log in ${G_LOG_FILE}"
    sys_log "run ${operator} ${component} End."    
    return 0
}

# set local current path
CURRENT_PATH=$(get_cur_path)

. "${CURRENT_PATH}/tool/config.ini"  || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${CURRENT_PATH}/tool/config.ini."; exit 1; }
. "${CURRENT_PATH}/func/functions.sh"  || { echo "[${g_scriptName}:${LINENO}] ERROR: Failed to load ${CURRENT_PATH}/func/functions.sh."; exit 1; }

main "$@"
exit $?
